#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/async.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <linux/kthread.h>
#include <linux/miscdevice.h>
#include <linux/uaccess.h>
#include <linux/version.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <linux/io.h>
#include <linux/ioctl.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/syscalls.h>
#include <linux/err.h>
#include <linux/cdev.h>
#include <linux/types.h>
#include <linux/pm_wakeup.h>
#include <linux/sched.h>
#include <linux/mutex.h>
#include <linux/pm.h>
#include <linux/time.h>
#include <linux/namei.h>
#include <linux/mount.h>
#include <linux/poll.h>
#include <linux/completion.h>
#include <linux/of.h>
#include <linux/of_irq.h>
#include <linux/of_gpio.h>
#include <linux/notifier.h>
#include <linux/fb.h>
#include <linux/clk.h>

#include "elan_fp_tee.h"


#define _ELAN_DEBUG_
#define POWER_NOTIFY
//#define CUSTOM_LDO_VDD
#define MTK_SPI_CLK_CTRL
#define COMPATIBLE_BY_ID_PIN

#define ELAN_DRIVER_VERSION		"ELAN FINGER PRINT V1.6.4.2"
#define ELAN_FP_NAME 			"elan_fp"
#define ELAN_FP_IRQ_NODE_NAME	"mediatek,fingerprint_irq"
#define FTM_NORMAL_STATUS 		0
#define FTM_START_STATUS 		2
#define FTM_END_STATUS  		4
#define TRUE_IRQ_NOTIFICATION  	1
#define ELAN_ID_PIN_VALUE		0	//low level
#define MAX_LIB_BUF 			256


#ifdef _ELAN_DEBUG_
	static int elan_debug = 1;
	#define ELAN_DEBUG(format, args ...) \
		do { \
			if (elan_debug) \
				printk(KERN_ERR "[ELAN] " format, ##args); \
		} while (0)
#else
	#define ELAN_DEBUG(format, args ...)
#endif

typedef struct efsa120s_data {
	int 					irq_gpio;
	int 					rst_gpio;
	#ifdef CUSTOM_LDO_VDD
	int 					pwr_gpio;
	#endif
	int						isr;
	int						irq_is_disable;
	int						efsa120s_init_step;
	struct miscdevice		efsa120_dev; /* char device for ioctl */
	struct platform_device	*pdev;
	spinlock_t				irq_lock;
	wait_queue_head_t		efsa_wait;
	struct wakeup_source	*wake_lock;
	struct wakeup_source	*hal_wake_lock;
	struct notifier_block	notifier;
	#ifdef MTK_SPI_CLK_CTRL
	struct clk 				*parent_clk, *sel_clk, *spi_clk;
	#endif
} efsa120s_data;


struct efsa120s_data *elan_fp = NULL;
static int irq_status = 0;
static int irq_status_buffer = 0;
static int elan_work_flag = 0;
static int factory_status = FTM_NORMAL_STATUS;;
static struct fasync_struct *fasync_queue = NULL;
static DEFINE_MUTEX(elan_factory_mutex);
static char lib_ver_buf[MAX_LIB_BUF] = "unknow";
static int is_screen_poweroff = 0;
static int key_status = 0;


static void elan_spi_clk_enable(struct efsa120s_data *fp, u8 bonoff);
static int elan_request_irq(struct efsa120s_data *pdata);


void efsa120s_irq_enable(void *_fp)
{
	struct efsa120s_data *fp = _fp;
	unsigned long irqflags = 0;
	ELAN_DEBUG("IRQ Enable = %d.\n", fp->isr);

	spin_lock_irqsave(&fp->irq_lock, irqflags);
	if (fp->irq_is_disable)
	{
		enable_irq(fp->isr);
		fp->irq_is_disable = 0;
	}
	spin_unlock_irqrestore(&fp->irq_lock, irqflags);
}

void efsa120s_irq_disable(void *_fp)
{
	struct efsa120s_data *fp = _fp;
	unsigned long irqflags;
	ELAN_DEBUG("IRQ Disable = %d.\n", fp->isr);

	spin_lock_irqsave(&fp->irq_lock, irqflags);
	if (!fp->irq_is_disable)
	{
		fp->irq_is_disable = 1;
		disable_irq_nosync(fp->isr);
	}
	spin_unlock_irqrestore(&fp->irq_lock, irqflags);
}

static ssize_t show_drv_version_value(struct device *dev, struct device_attribute *attr, char *buf)
{
	return sprintf(buf, "%s\n", ELAN_DRIVER_VERSION);
}
static DEVICE_ATTR(drv_version, S_IRUGO, show_drv_version_value, NULL);

#ifdef _ELAN_DEBUG_
static ssize_t elan_debug_value(struct device *dev, struct device_attribute *attr, char *buf)
{
	if (elan_debug) {
		elan_debug = 0;
	} else {
		elan_debug = 1;
	}
	return sprintf(buf, "[ELAN] elan debug %d\n", elan_debug);
}
static DEVICE_ATTR(elan_debug, S_IRUGO, elan_debug_value, NULL);
#endif

static struct attribute *efsa120s_attributes[] = {
	&dev_attr_drv_version.attr,
#ifdef _ELAN_DEBUG_
	&dev_attr_elan_debug.attr,
#endif
	NULL
};

static struct attribute_group efsa120s_attr_group = {
	.attrs = efsa120s_attributes,
};

static void efsa120s_reset(struct efsa120s_data *fp)
{
	/* Developement platform */
	gpio_direction_output(fp->rst_gpio, 0);
	mdelay(5);
	gpio_direction_output(fp->rst_gpio, 1);
	mdelay(50);
}

static int efsa120s_open(struct inode *inode, struct file *filp)
{
	ELAN_DEBUG("%s()\n", __func__);
	return 0;
}

static int efsa120s_close(struct inode *inode, struct file *filp)
{
	ELAN_DEBUG("%s()\n", __func__);
	return 0;
}

static int elan_write_key_status(int key)
{
	key_status = key;

	return 0;
}

static int elan_read_key_status(void)
{
	return key_status;
}

static long efsa120s_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
	struct efsa120s_data *fp = elan_fp;
	int ret = 0;
	int key = 0;
	int wake_lock_arg;

	ELAN_DEBUG("%s() : cmd = [%04X]\n", __func__, cmd);

	switch (cmd)
	{
		case ID_IOCTL_RESET: //6
			efsa120s_reset(fp);
			ELAN_DEBUG("ID_IOCTL_RESET\n");
			break;
		case ID_IOCTL_POLL_INIT: //20
			//reinit_completion(&cmd_done_irq);
			elan_work_flag = 0;
			ELAN_DEBUG("ID_IOCTL_POLL_INIT\n");
			break;
		case ID_IOCTL_POLL_EXIT: //23
			elan_work_flag = TRUE_IRQ_NOTIFICATION;
			wake_up(&fp->efsa_wait);
			ELAN_DEBUG("ID_IOCTL_POLL_EXIT\n");
			break;
		case ID_IOCTL_WAKE_LOCK_UNLOCK: //41
		    wake_lock_arg = (int __user)arg;
		    if (!wake_lock_arg) {
		        __pm_relax(fp->hal_wake_lock);
		        ELAN_DEBUG("[IOCTL] HAL WAKE UNLOCK = %d\n", wake_lock_arg);
		    } else if (wake_lock_arg) {
		        __pm_stay_awake(fp->hal_wake_lock);
		        ELAN_DEBUG("[IOCTL] HAL WAKE LOCK = %d\n", wake_lock_arg);
		    } else {
		        ELAN_DEBUG("[IOCTL] ERROR WAKE LOCK ARGUMENT\n");
		    }
		    break;
		case IOCTL_READ_KEY_STATUS:
			ELAN_DEBUG("IOCTL_READ_KEY_STATUS, key_status = %d\n", key_status);
			return elan_read_key_status();
		case IOCTL_WRITE_KEY_STATUS:
			ELAN_DEBUG("IOCTL_WRITE_KEY_STATUS, arg = %ld\n", arg);
			ret = copy_from_user(&key, (int *)arg, sizeof(int));
		    if (!ret) {
				elan_write_key_status(key);
            }
			break;
        case ID_IOCTL_READ_FACTORY_STATUS:
            mutex_lock(&elan_factory_mutex);
            ELAN_DEBUG("READ_FACTORY_STATUS = %d", factory_status);
            mutex_unlock(&elan_factory_mutex);
            return factory_status;
            break;
        case ID_IOCTL_WRITE_FACTORY_STATUS:
            mutex_lock(&elan_factory_mutex);
            factory_status = (int __user)arg;
            if (factory_status == FTM_START_STATUS)
            {
                elan_work_flag = TRUE_IRQ_NOTIFICATION;
                wake_up(&fp->efsa_wait);
            }
            ELAN_DEBUG("WRITE_FACTORY_STATUS = %d\n", factory_status);
            mutex_unlock(&elan_factory_mutex);
            break;
        case ID_IOCTL_IOIRQ_STATUS:
			//mutex_lock(&elan_factory_mutex);
            irq_status_buffer = irq_status;
            ELAN_DEBUG("READ_IRQ_STATUS = %d", irq_status);
            irq_status = 2;
			//mutex_unlock(&elan_factory_mutex);
            return irq_status_buffer;
            break;
        case ID_IOCTL_SET_VERSION: //100
            ELAN_DEBUG("do nothing\n");
            break;
        case ID_IOCTL_SET_VERSION_0: //99
            ret = copy_from_user(lib_ver_buf, (char *)arg, MAX_LIB_BUF);
            if (!ret) {
				ELAN_DEBUG("lib_ver_buf= %s\n", lib_ver_buf);
            }
            break;
        case ID_IOCTL_GET_VERSION: //101
            ret = copy_to_user((char *)arg, lib_ver_buf, MAX_LIB_BUF);
            break;
        case ID_IOCTL_REQUSEST_IRQ: //95
            if (arg) {
				ELAN_DEBUG("ID_IOCTL_REQUSEST_IRQ\n");
                if (elan_request_irq(fp) == 0) {
					ELAN_DEBUG("elan request irq success.\n");
					fp->irq_is_disable = 1;
					efsa120s_irq_enable(fp);
                    break;
                }
            }
            break;
		case ID_IOCTL_EN_IRQ: //55
			efsa120s_irq_enable(fp);
			ELAN_DEBUG("ID_IOCTL_EN_IRQ\n");
			break;
		case ID_IOCTL_DIS_IRQ: //66
			efsa120s_irq_disable(fp);
			ELAN_DEBUG("ID_IOCTL_DIS_IRQ\n");
			break;
		case ID_IOCTL_ENABLE_SPI_CLK:
			ELAN_DEBUG("%s() : ELAN_IOC_ENABLE_SPI_CLK ======\n", __func__);
			elan_spi_clk_enable(fp, 1);
			break;
		case ID_IOCTL_DISABLE_SPI_CLK:
			ELAN_DEBUG("%s() : ID_IOCTL_DISABLE_SPI_CLK ======\n", __func__);
			elan_spi_clk_enable(fp, 0);
			break;
		case ID_IOCTL_GET_SCREEN_STATUS:
			return is_screen_poweroff;
		default:
			ELAN_DEBUG("INVALID COMMAND\n");
			break;
	}
	return 0;
}

static unsigned int efsa120s_poll(struct file *file, poll_table *wait)
{
	struct efsa120s_data *fp = elan_fp;
	int mask = 0;
    int ret = 0;

    ELAN_DEBUG("%s()\n", __func__);
	__pm_relax(fp->hal_wake_lock);
    ret = wait_event_interruptible(fp->efsa_wait, elan_work_flag > 0);
	if (ret != 0) {
        ELAN_DEBUG("%s() ret = %d, elan_work_flag = %d.\n", __func__, ret, elan_work_flag);
    }

	if(elan_work_flag > 0) {
		mask |= POLLIN | POLLRDNORM;
	}

	elan_work_flag = 0;

	return mask;
}

static int elan_fp_fasync(int fd, struct file * filp, int on)
{
	//elan_info("%s enter \n",__func__);
	return fasync_helper(fd, filp, on, &fasync_queue);
}

static const struct file_operations efsa120s_fops = {
	.owner 			= THIS_MODULE,
	.open 			= efsa120s_open,
	.unlocked_ioctl = efsa120s_ioctl,
	.poll			= efsa120s_poll,
	.release 		= efsa120s_close,
	.fasync 		= elan_fp_fasync,
};

static irqreturn_t efsa120s_irq_handler(int irq, void *_fp)
{
	struct efsa120s_data *fp = _fp;

	ELAN_DEBUG("%s()\n", __func__);

    __pm_wakeup_event(fp->wake_lock, msecs_to_jiffies(3000));
	elan_work_flag = TRUE_IRQ_NOTIFICATION;

	if(factory_status == FTM_NORMAL_STATUS) {
		wake_up(&fp->efsa_wait);
	} else {
		irq_status = 1;
	}

	return IRQ_HANDLED;
}

static int efsa120s_setup_cdev(struct efsa120s_data *fp)
{
	fp->efsa120_dev.minor = MISC_DYNAMIC_MINOR;
	fp->efsa120_dev.name = "elan_fp";
	fp->efsa120_dev.fops = &efsa120s_fops;
	fp->efsa120_dev.mode = S_IFREG|S_IRWXUGO;
	if (misc_register(&fp->efsa120_dev) < 0) {
		ELAN_DEBUG("misc_register failed!!");
		return -1;
	} else {
		ELAN_DEBUG("misc_register finished!!");
	}

	return 0;
}

static int efsa120s_sysfs_create(struct efsa120s_data *sysfs)
{
	struct efsa120s_data *fp = elan_fp;
	int error = 0;

	/* Register sysfs */
	error = sysfs_create_group(&fp->pdev->dev.kobj, &efsa120s_attr_group);
	if (error) {
		dev_err(&fp->pdev->dev, "[ELAN] Failed to create sysfs attributes, err: %d\n", error);
		goto fail_un;
	}
	return 0;
fail_un:
	/* Remove sysfs */
	sysfs_remove_group(&fp->pdev->dev.kobj, &efsa120s_attr_group);

	return error;
}

static char efsa120s_gpio_config(void *_fp)
{
	struct efsa120s_data *fp = _fp;
	struct device_node *dev_node = NULL;
	int ret;

	// Configure INT GPIO (Input)
	ret = gpio_request(fp->irq_gpio, "efsa120-irq");
	ELAN_DEBUG("irq gpio_request ret = %d", ret);
	if (ret < 0) {
		ELAN_DEBUG("%s() IRQ(%d) request fail, err=0x%x.\n", __func__, fp->irq_gpio, ret);
		ret = -ENODEV;
		return ret;
	} else {
		gpio_direction_input(fp->irq_gpio);
		//fp->isr = gpio_to_irq(fp->irq_gpio);

		dev_node = of_find_compatible_node(NULL, NULL, ELAN_FP_IRQ_NODE_NAME);
		fp->isr = irq_of_parse_and_map(dev_node, 0);
		if (fp->isr < 0) {
			ELAN_DEBUG("%s() irq of parse err=0x%x.\n", __func__, fp->isr);
			ret = -ENODEV;
			return ret;
		}
		ELAN_DEBUG("%s() IRQ(%d) = ISR(%d) request success, err=0x%x.\n", __func__, fp->irq_gpio, fp->isr, ret);
	}

	// Configure RST GPIO (Output)
	ret = gpio_request(fp->rst_gpio, "efsa120-reset");
	ELAN_DEBUG("reset gpio_request ret = %d\n", ret);
	if (ret < 0) {
		gpio_free(fp->irq_gpio);
		free_irq(fp->isr, fp);
		fp->irq_gpio = 0;
		fp->isr = 0;
		ELAN_DEBUG("%s() RST(%d) request fail, err=0x%x.\n", __func__, fp->rst_gpio, ret);
		ret = -ENODEV;
		return ret;
	} else {
		ELAN_DEBUG("%s() RST(%d) request success, err=0x%x.\n", __func__, fp->rst_gpio, ret);
	}

#ifdef CUSTOM_LDO_VDD
	// Configure Power GPIO (Output)
	ret = gpio_request(fp->pwr_gpio, "efsa120-irq");
	ELAN_DEBUG("pwr gpio_request ret = %d", ret);
	if (ret < 0) {
		ELAN_DEBUG("%s() PWR(%d) request fail, err=0x%x.\n", __func__, fp->pwr_gpio, ret);
		gpio_free(fp->irq_gpio);
		gpio_free(fp->rst_gpio);
		free_irq(fp->isr, fp);
		fp->irq_gpio = 0;
		fp->rst_gpio = 0;
		fp->pwr_gpio = 0;
		fp->isr = 0;
		ret = -ENODEV;
		return ret;
	} else {
		gpio_direction_output(fp->pwr_gpio, 1); //power on
		ELAN_DEBUG("%s() PWR(%d) request success, err=0x%x.\n", __func__, fp->pwr_gpio, ret);
	}
#endif

	return ret;
}

static void elan_spi_clk_enable(struct efsa120s_data *fp, u8 bonoff)
{
#ifdef MTK_SPI_CLK_CTRL
    int ret;
    static int count = 0;

    if (bonoff && (count == 0)) {
        ELAN_DEBUG("clk_prepare_enable\n");
        ret = clk_prepare_enable(fp->spi_clk);
        count = 1;
    } else if ((count > 0) && (bonoff == 0)) {
		ELAN_DEBUG("clk_disable_unprepare\n");
        clk_disable_unprepare(fp->spi_clk);
        count = 0;
    }
#endif
}

static int elan_parse_dt(struct device *dev, struct efsa120s_data *pdata)
{
	struct device_node *np = dev->of_node;
	int ret = 0;

#ifdef COMPATIBLE_BY_ID_PIN
	int fp_id_gpio = -1;

	fp_id_gpio = of_get_named_gpio(np, "elan,id-gpio", 0);
	ELAN_DEBUG("fp_id_gpio = %d\n", fp_id_gpio);
	if (fp_id_gpio > 0) {
		ret = gpio_request(fp_id_gpio, "efsa120-id");
		ELAN_DEBUG("fp id gpio request ret = %d", ret);
		if (ret >= 0) {
			ret = gpio_get_value(fp_id_gpio);
			ELAN_DEBUG("fingerprint id gpio, get value = %d\n", ret);
			gpio_free(fp_id_gpio);

			if (ret == ELAN_ID_PIN_VALUE) {
				ELAN_DEBUG("check elan fp id gpio OK.\n");
			} else {
				ELAN_DEBUG("check elan fp id gpio error.\n");
				return -2;
			}
		} else {
			ELAN_DEBUG("%s() id gpio(%d) request fail, err=0x%x.\n", __func__, fp_id_gpio, ret);
			ret = -ENODEV;
			return ret;
		}
	} else {
		ELAN_DEBUG("of get id gpio error(%d)\n", fp_id_gpio);
	}
#endif

	pdata->irq_gpio = of_get_named_gpio(np, "elan,irq-gpio", 0);
	ELAN_DEBUG("irq_gpio = %d\n", pdata->irq_gpio);
	if (pdata->irq_gpio < 0)
		return pdata->irq_gpio;

	pdata->rst_gpio = of_get_named_gpio(np, "elan,rst-gpio", 0);
	ELAN_DEBUG("rst_gpio = %d\n", pdata->rst_gpio);
	if (pdata->rst_gpio < 0)
		return pdata->rst_gpio;

#ifdef CUSTOM_LDO_VDD
	pdata->pwr_gpio = of_get_named_gpio(np, "elan,pwr-gpio", 0);
	ELAN_DEBUG("pwr_gpio = %d\n", pdata->pwr_gpio);
	if (pdata->pwr_gpio < 0)
		return pdata->pwr_gpio;

	ELAN_DEBUG("irq_gpio = %d, rst_gpio = %d, pwr_gpio = %d\n", pdata->irq_gpio, pdata->rst_gpio, pdata->pwr_gpio);
#else
	ELAN_DEBUG("irq_gpio = %d, rst_gpio = %d\n", pdata->irq_gpio, pdata->rst_gpio);
#endif

#ifdef MTK_SPI_CLK_CTRL
	pdata->parent_clk = devm_clk_get(dev, "parent-clk");
	if (IS_ERR(pdata->parent_clk)) {
		ret = PTR_ERR(pdata->parent_clk);
		dev_err(dev, "failed to get parent-clk: %d\n", ret);
		return ret;
	}

	pdata->sel_clk = devm_clk_get(dev, "sel-clk");
	if (IS_ERR(pdata->sel_clk)) {
		ret = PTR_ERR(pdata->sel_clk);
		dev_err(dev, "failed to get sel-clk: %d\n", ret);
		return ret;
	}

	pdata->spi_clk = devm_clk_get(dev, "spi-clk");
	if (IS_ERR(pdata->spi_clk)) {
		ret = PTR_ERR(pdata->spi_clk);
		dev_err(dev, "failed to get spi-clk: %d\n", ret);
		return ret;
	}

	ret = clk_prepare_enable(pdata->spi_clk);
	if (ret < 0) {
		dev_err(dev, "failed to enable spi_clk (%d)\n", ret);
		return ret;
	}

	ret = clk_set_parent(pdata->sel_clk, pdata->parent_clk);
	if (ret < 0) {
		dev_err(dev, "failed to clk_set_parent (%d)\n", ret);
		clk_disable_unprepare(pdata->spi_clk);
		return ret;
	}
	clk_disable_unprepare(pdata->spi_clk);

	ELAN_DEBUG("spi clk init OK.\n");
#endif

	return ret;
}

static int elan_request_irq(struct efsa120s_data *pdata)
{
    int err = -1;

	err = request_irq(pdata->isr, efsa120s_irq_handler, IRQF_NO_SUSPEND | IRQF_TRIGGER_RISING | IRQF_ONESHOT, pdata->pdev->dev.driver->name, pdata);
    if (err) {
        ELAN_DEBUG("Failed to request IRQ %d.\n", err);
        return -1;
    }

    irq_set_irq_wake(pdata->isr, 1);
    spin_lock_init(&pdata->irq_lock);
    return 0;
}

#ifdef POWER_NOTIFY
static int elan_fb_state_chg_callback(struct notifier_block *nb,
	unsigned long val, void *data)
{
	struct fb_event *evdata = data;
	unsigned int blank;
	struct efsa120s_data *fp = container_of(nb, struct efsa120s_data, notifier);

	ELAN_DEBUG("=====%s() Start=====\n", __func__);
	ELAN_DEBUG("TINNO: elan_fb_state_chg_callback FP\n");

	if (val != FB_EARLY_EVENT_BLANK) {
		return 0;
	}

	if (evdata && evdata->data && val == FB_EARLY_EVENT_BLANK && fp) {
		blank = *(int *)(evdata->data);
		switch (blank) {
			case FB_BLANK_POWERDOWN:
				is_screen_poweroff = 1;
                elan_work_flag = TRUE_IRQ_NOTIFICATION;
                wake_up(&fp->efsa_wait);
				//complete(&cmd_done_irq);
				ELAN_DEBUG("Screen Off !!!->1.wait_event_interruptible notify\n");
				//wake_up(&fp->efsa_wait);
				break;
			case FB_BLANK_UNBLANK:
				is_screen_poweroff = 0;
				ELAN_DEBUG("Screen On !!!\n");
				break;
			default:
				pr_info("%s defalut\n", __func__);
				break;
		}
	}
	ELAN_DEBUG("=====%s() END=====\n", __func__);
	return NOTIFY_OK;
}

static struct notifier_block elan_noti_block = {
	.notifier_call = elan_fb_state_chg_callback,
};
#endif

static int efsa120s_probe(struct platform_device *pdev)
{
    struct efsa120s_data *fp = NULL;
    int err = 0;

    ELAN_DEBUG("=====%s() Start=====\n", __func__);

    /* Allocate Device Data */
    fp = devm_kzalloc(&pdev->dev, sizeof(struct efsa120s_data), GFP_KERNEL);
    if (!fp) {
		ELAN_DEBUG("alloc efsa120s data fail.\n");
		return -ENOMEM;
	}

	elan_fp = fp;
	fp->pdev = pdev;
	fp->efsa120s_init_step = 1;

    /* Parse Device Tree */
    err = elan_parse_dt(&pdev->dev, fp);
	if (err < 0) {
		ELAN_DEBUG("device tree initial failed, err = %d\n", err);
		goto init_efsa120s_failed;
	}
	fp->efsa120s_init_step = 2;

    /* Init Char Device */
    err = efsa120s_setup_cdev(fp);
	if (err < 0) {
		ELAN_DEBUG("efsa120s setup device fail.\n");
		goto init_efsa120s_failed;
	}
	fp->efsa120s_init_step = 3;

    /* Init EFSA120S GPIO */
    err = efsa120s_gpio_config(fp);
    if(err < 0) {
		ELAN_DEBUG("GPIO request fail (%d).\n", err);
		goto init_efsa120s_failed;
	}
	fp->efsa120s_init_step = 4;

    /* Init Sysfs */
    err = efsa120s_sysfs_create(fp);
    if(err < 0) {
		ELAN_DEBUG("efsa120s sysfs fail.\n");
		goto init_efsa120s_failed;
    }
	fp->efsa120s_init_step = 5;

	platform_set_drvdata(pdev, fp);

    /* Init Poll Wait */
    init_waitqueue_head(&fp->efsa_wait);

	fp->wake_lock = wakeup_source_create("fp_wake_lock");
	wakeup_source_add(fp->wake_lock);
	fp->hal_wake_lock = wakeup_source_create("hal_fp_wake_lock");
	wakeup_source_add(fp->hal_wake_lock);

#ifdef POWER_NOTIFY
	fp->notifier = elan_noti_block;
	fb_register_client(&fp->notifier);
#endif

	fp->efsa120s_init_step = 6;

    ELAN_DEBUG("=====%s() End=====\n", __func__);

    return 0;

init_efsa120s_failed:
	if (fp != NULL) {
		switch (fp->efsa120s_init_step) {
			case 6:
				wakeup_source_remove(fp->wake_lock);
				wakeup_source_remove(fp->hal_wake_lock);
			case 5:
				sysfs_remove_group(&fp->pdev->dev.kobj, &efsa120s_attr_group);
			case 4:
				if (fp->irq_gpio > 0)
					gpio_free(fp->irq_gpio);
				if (fp->rst_gpio > 0) {
					gpio_direction_output(fp->rst_gpio, 0);
					gpio_free(fp->rst_gpio);
				}
				#ifdef CUSTOM_LDO_VDD
				if (fp->pwr_gpio > 0) {
					gpio_direction_output(fp->pwr_gpio, 0); //power off
					gpio_free(fp->pwr_gpio);
				}
				#endif
				if (fp->isr > 0)
					free_irq(fp->isr, fp);
				fp->irq_gpio = 0;
				fp->rst_gpio = 0;
				#ifdef CUSTOM_LDO_VDD
				fp->pwr_gpio = 0;
				#endif
				fp->isr = 0;
			case 3:
				misc_deregister(&fp->efsa120_dev);
			case 2:
			case 1:
				devm_kfree(&pdev->dev, fp);
				elan_fp = NULL;

			default:
				break;
		}
	}
	return -1;
}

static int efsa120s_remove(struct platform_device *pdev)
{
	if (elan_fp != NULL) {
		devm_kfree(&pdev->dev, elan_fp);
		elan_fp = NULL;
	}

	return 0;
}

#ifdef CONFIG_PM_SLEEP
static int efsa120s_suspend(struct device *dev)
{
	ELAN_DEBUG("efsa120s suspend!\n");
	return 0;
}

static int efsa120s_resume(struct device *dev)
{
	ELAN_DEBUG("efsa120s resume!\n");
	return 0;
}
#endif

static SIMPLE_DEV_PM_OPS(efsa120s_pm_ops, efsa120s_suspend, efsa120s_resume);

#ifdef CONFIG_OF
static struct of_device_id efsa120s_metallica_table[] = {
	{ .compatible = "elan,elan_fp",},
	{ },
};
#endif

static struct platform_driver efsa120s_driver = {
	.driver = {
		.name 	= "efsa120s",
		.owner = THIS_MODULE,
		.pm 	= &efsa120s_pm_ops,
		#ifdef CONFIG_OF
		.of_match_table = efsa120s_metallica_table,
		#endif
	},
	.probe 	= efsa120s_probe,
	.remove = efsa120s_remove,
};

static int __init efsa120s_init(void)
{
	int status = 0;
	ELAN_DEBUG("=====%s() Start=====\n", __func__);

	status = platform_driver_register(&efsa120s_driver);
	if (status < 0)
		ELAN_DEBUG("%s FAIL !\n", __func__);

	ELAN_DEBUG("=====%s() End=====\n", __func__);

	return status;
}

static void __exit efsa120s_exist(void)
{
	ELAN_DEBUG("=====%s() Start=====\n", __func__);
	platform_driver_unregister(&efsa120s_driver);
}

module_init(efsa120s_init);
module_exit(efsa120s_exist);

MODULE_AUTHOR("KennyKang <kenny.kang@emc.com.tw>");
MODULE_DESCRIPTION("ELAN SPI FingerPrint eFSA120S driver");
MODULE_VERSION(ELAN_DRIVER_VERSION);
MODULE_LICENSE("GPL");
