#ifndef _LINUX_ELANTS_KTD_SPI_H
#define _LINUX_ELANTS_KTD_SPI_H
#include <linux/spi/spi.h>
#include <linux/input.h>
#include <linux/miscdevice.h>
#include <linux/delay.h>

#ifdef SMARTISAN_RELEASE

#define ELAN_DEBUG(fmt, args...)
#define ELAN_FUNC_ENTER()
#define ELAN_FUNC_EXIT()

#else

#define ELAN_DEBUG(fmt, args...)                               \
	do                                                         \
	{                                                          \
		printk(KERN_INFO "[ELAN/D]%s:" fmt, __func__, ##args); \
	} while (0)

#define ELAN_FUNC_ENTER()                              \
	do                                                 \
	{                                                  \
		printk(KERN_INFO "[ELAN]%s: Enter", __func__); \
	} while (0)

#define ELAN_FUNC_EXIT()                                            \
	do                                                              \
	{                                                               \
		printk(KERN_INFO "[ELAN]%s: Exit(%d)", __func__, __LINE__); \
	} while (0)

#endif

#define ELAN_ERROR(fmt, args...)                              \
	do                                                        \
	{                                                         \
		printk(KERN_ERR "[ELAN/E]%s:" fmt, __func__, ##args); \
	} while (0)

#define ELAN_INFO(fmt, args...)                                \
	do                                                         \
	{                                                          \
		printk(KERN_INFO "[ELAN/I]%s:" fmt, __func__, ##args); \
	} while (0)

struct elan_spi_fw_info
{
	int fw_ver;
	int fw_id;
	int fw_bcv;
	int rx;
	int tx;
	int finger_osr;
	int x_res;
	int y_res;
	int testsolversion;
	int testversion;
	int solutionversion;
	int whck_ver;
};

struct elan_spi_operation
{
	int (*transfer)(struct spi_device *spi, const void *tx_buf, void *rx_buf, const size_t len);
	int (*poll)(int ms, int times);
};

struct elants_data
{
	struct spi_device *spi;
	struct input_dev *input_dev;
	struct input_dev *pen_input_dev;
	struct elan_spi_fw_info fw_info;
	int int_gpio;
	int irq;
	int irq_gpio;
	int rst_gpio;
	int resx_gpio;
	int vddi_gpio;
	int apqio1_gpio;
	int apqio2_gpio;
	bool force_updata;
	struct work_struct work;
	bool irq_status; // 1:enable, 0:disable
	wait_queue_head_t elan_wait;
	struct wakeup_source wake_lock;
	struct wakeup_source hal_wake_lock;
	struct miscdevice elan_dev; /* char device for ioctl */
	struct regulator *reg;
	struct regulator *vdd; /*tp vdd*/
	struct regulator *vcc; /*tp vio*/
	struct elan_spi_operation *ops;
};

//extern function
extern void elan_reset(struct elants_data *ts);
extern void elan_irq_enable(struct elants_data *ts);
extern void elan_irq_disable(struct elants_data *ts);
extern int elan_fw_packet_handler(struct elants_data *ts);
//extern int elan_module_test(struct elants_data *ts);
//extern int elan_mt_get_noise_test_data(struct elants_data *ts);

#endif /* _LINUX_ELANTS_KTD_SPI_H */