// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2020 MediaTek Inc.
 */

#include "apusys_power_cust.h"


//	| VPU0 |		| VPU1 |		| MDLA0 |  --> DVFS_USER
//	(VPU0_VPU)    (VPU1_VPU)     (VMDLA0_MDLA)
//	   |              |               |
//	   v	          v               v
//   |APU_CONN|     |APU_CONN|     |APU_CONN |
//   (VPU0_APU_CONN)(VPU1_APU_CONN)(VMDLA0_APU_CONN)

char *user_str[APUSYS_DVFS_USER_NUM] = {
	"VPU0",
	"VPU1",
	"VPU2",
	"MDLA0",
	"MDLA1",
};


char *buck_domain_str[APUSYS_BUCK_DOMAIN_NUM] = {
	"V_VPU0",
	"V_VPU1",
	"V_VPU2",
	"V_MDLA0",
	"V_MDLA1",
	"V_APU_CONN",
	"V_TOP_IOMMU",
	"V_VCORE",
};

char *buck_str[APUSYS_BUCK_NUM] = {
	"VPU_BUCK",
	"MDLA_BUCK",
	"VCORE_BUCK",
};


bool apusys_dvfs_user_support[APUSYS_DVFS_USER_NUM] = {
	true, true, true, true, true
};

#if VCORE_DVFS_SUPPORT
bool apusys_dvfs_buck_domain_support[APUSYS_BUCK_DOMAIN_NUM] = {
	true, true, true, true, true, true, true, true
};
#else
bool apusys_dvfs_buck_domain_support[APUSYS_BUCK_DOMAIN_NUM] = {
	true, true, true, true, true, true, true, false
};
#endif

enum DVFS_VOLTAGE_DOMAIN apusys_user_to_buck_domain[APUSYS_DVFS_USER_NUM] = {
	V_VPU0, V_VPU1, V_VPU2, V_MDLA0, V_MDLA1
};


enum DVFS_BUCK apusys_user_to_buck[APUSYS_DVFS_USER_NUM] = {
	VPU_BUCK, VPU_BUCK, VPU_BUCK, MDLA_BUCK, MDLA_BUCK
};


enum DVFS_USER apusys_buck_domain_to_user[APUSYS_BUCK_DOMAIN_NUM] = {
	VPU0, VPU1, VPU2, MDLA0, MDLA1,
	APUSYS_DVFS_USER_NUM,
	APUSYS_DVFS_USER_NUM,
	APUSYS_DVFS_USER_NUM  // APUSYS_DVFS_USER_NUM means invalid
};


enum DVFS_BUCK apusys_buck_domain_to_buck[APUSYS_BUCK_DOMAIN_NUM] = {
	VPU_BUCK, VPU_BUCK, VPU_BUCK, MDLA_BUCK, MDLA_BUCK,
	VPU_BUCK, VPU_BUCK, VCORE_BUCK
};


enum DVFS_VOLTAGE_DOMAIN apusys_buck_to_buck_domain[APUSYS_BUCK_NUM] = {
	V_VPU0, V_MDLA0, V_VCORE
};


// voltage for clk path
uint8_t dvfs_clk_path[APUSYS_DVFS_USER_NUM][APUSYS_PATH_USER_NUM] = {
	{VPU0_VPU, VPU0_APU_CONN, VPU0_TOP_IOMMU, VPU0_VCORE},
	{VPU1_VPU, VPU1_APU_CONN, VPU1_TOP_IOMMU, VPU1_VCORE},
	{VPU2_VPU, VPU2_APU_CONN, VPU2_TOP_IOMMU, VPU2_VCORE},
	{MDLA0_MDLA, MDLA0_APU_CONN, MDLA0_TOP_IOMMU, MDLA0_VCORE},
	{MDLA1_MDLA, MDLA1_APU_CONN, MDLA1_TOP_IOMMU, MDLA1_VCORE},
};

#if defined(CONFIG_MACH_MT6893)
enum DVFS_VOLTAGE
	dvfs_clk_path_max_vol[APUSYS_DVFS_USER_NUM][APUSYS_PATH_USER_NUM] = {
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_750000_V},
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_750000_V},
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_750000_V},
	{DVFS_VOLT_00_825000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_750000_V},
	{DVFS_VOLT_00_825000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_750000_V},
};
#else
enum DVFS_VOLTAGE
	dvfs_clk_path_max_vol[APUSYS_DVFS_USER_NUM][APUSYS_PATH_USER_NUM] = {
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_725000_V},
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_725000_V},
	{DVFS_VOLT_00_800000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_725000_V},
	{DVFS_VOLT_00_825000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_725000_V},
	{DVFS_VOLT_00_825000_V, DVFS_VOLT_00_800000_V,
		DVFS_VOLT_00_800000_V, DVFS_VOLT_00_725000_V},
};
#endif

#if 0
// buck for clk path
uint8_t dvfs_buck_for_clk_path[APUSYS_DVFS_USER_NUM][APUSYS_BUCK_NUM] = {
	{VPU_BUCK, MDLA_BUCK, VCORE_BUCK},
	{VPU_BUCK, MDLA_BUCK, VCORE_BUCK},
	{MDLA_BUCK, MDLA_BUCK, VCORE_BUCK},
};
#endif

// relation for dvfs_clk_path,
bool buck_shared[APUSYS_BUCK_NUM]
			[APUSYS_DVFS_USER_NUM][APUSYS_PATH_USER_NUM] = {
	// VPU BUCK
	{{true, true, true, false},
	{true, true, true, false},
	{true, true, true, false},
	{false, true, true, false},
	{false, true, true, false},
	},

	// MDLA BUCK
	{{false, false, false, false},
	{false, false, false, false},
	{false, false, false, false},
	{true, false, false, false},
	{true, false, false, false},
	},

	// VCORE
	{{false, false, false, true},
	 {false, false, false, true},
	 {false, false, false, true},
	 {false, false, false, true},
	 {false, false, false, true},
	},
};

// small voltage first
struct apusys_dvfs_constraint
	dvfs_constraint_table[APUSYS_DVFS_CONSTRAINT_NUM] = {
	{VPU_BUCK, DVFS_VOLT_00_575000_V, MDLA_BUCK, DVFS_VOLT_00_825000_V},
	{VPU_BUCK, DVFS_VOLT_00_575000_V, MDLA_BUCK, DVFS_VOLT_00_800000_V},
	{MDLA_BUCK, DVFS_VOLT_00_575000_V, VPU_BUCK, DVFS_VOLT_00_800000_V},
	{VCORE_BUCK, DVFS_VOLT_00_575000_V, VPU_BUCK, DVFS_VOLT_00_800000_V},
};


enum DVFS_VOLTAGE vcore_opp_mapping[] = {
#if defined(CONFIG_MACH_MT6893)
	DVFS_VOLT_00_750000_V,	// VCORE_OPP_0
#endif
	DVFS_VOLT_00_725000_V,	// VCORE_OPP_0
	DVFS_VOLT_00_650000_V,	// VCORE_OPP_1
	DVFS_VOLT_00_600000_V,  // VCORE_OPP_2
	DVFS_VOLT_00_575000_V   // VCORE_OPP_3
};

#if SUPPORT_VCORE_TO_IPUIF
/**************************************************
 * IPUIF OPP table definition
 **************************************************/
/*
 *#define NUM_OF_IPUIF_OPP (sizeof(g_ipuif_opp_table) / \
 *				sizeof(g_ipuif_opp_table[0]))
 */

#define IPUIFOP(khz, apu_vcore)	\
{							\
	.ipuif_khz = khz,				\
	.ipuif_vcore = apu_vcore,				\
}

struct ipuif_opp_table g_ipuif_opp_table[] = {
#if defined(CONFIG_MACH_MT6893)
	IPUIFOP(DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V), /* 0 */
#endif
	IPUIFOP(DVFS_FREQ_00_550000_F, DVFS_VOLT_00_725000_V), /* 0 */
	IPUIFOP(DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V), /* 1 */
	IPUIFOP(DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V), /* 2 */
	IPUIFOP(DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V), /* 3 */
};
#endif

#if !defined(CONFIG_MACH_MT6893)
// 5G-L
struct apusys_dvfs_steps
	dvfs_table_0[APUSYS_MAX_NUM_OPPS][APUSYS_BUCK_DOMAIN_NUM] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_725000_V} },

	// opp 1
	{{DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_725000_V} },

	// opp 2
	{{DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_725000_V} },

	// opp 3
	{{DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V} },

	// opp 4
	{{DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V} },

	// opp 5
	{{DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V} },

	// opp 6
	{{DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V} },

	// opp 7
	{{DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V} },

	// opp 8
	{{DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V} },

	// opp 9
	// set min 208Mhz as default freq
	{{DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_136500_F, DVFS_VOLT_00_575000_V} },

};

// 5G-H
struct apusys_dvfs_steps
	dvfs_table_1[APUSYS_MAX_NUM_OPPS][APUSYS_BUCK_DOMAIN_NUM] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_725000_V} },

	// opp 1
	{{DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_725000_V} },

	// opp 2
	{{DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_725000_V} },

	// opp 3
	{{DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V} },

	// opp 4
	{{DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V} },

	// opp 5
	{{DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V} },

	// opp 6
	{{DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V} },

	// opp 7
	{{DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V} },

	// opp 8
	{{DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V} },

	// opp 9
	// set min 208Mhz as default freq
	{{DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_136500_F, DVFS_VOLT_00_575000_V} },

};

// 5G-H+
struct apusys_dvfs_steps
	dvfs_table_2[APUSYS_MAX_NUM_OPPS][APUSYS_BUCK_DOMAIN_NUM] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_725000_V} },

	// opp 1
	{{DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_725000_V} },

	// opp 2
	{{DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_725000_V} },

	// opp 3
	{{DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_594000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V} },

	// opp 4
	{{DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V} },

	// opp 5
	{{DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_546000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V} },

	// opp 6
	{{DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V} },

	// opp 7
	{{DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_392857_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V} },

	// opp 8
	{{DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_364000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V} },

	// opp 9
	// set min 208Mhz as default freq
	{{DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_136500_F, DVFS_VOLT_00_575000_V} },
};

#else // defined(CONFIG_MACH_MT6893)
struct apusys_dvfs_steps
	dvfs_table_b0[APUSYS_MAX_NUM_OPPS][APUSYS_BUCK_DOMAIN_NUM] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V} },

	// opp 1
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_725000_V} },

	// opp 2
	{{DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_725000_V} },

	// opp 3
	{{DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V} },

	// opp 4
	{{DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V} },

	// opp 5
	{{DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V} },
};

struct apusys_dvfs_steps
	dvfs_table_b1[APUSYS_MAX_NUM_OPPS][APUSYS_BUCK_DOMAIN_NUM] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_900000_F, DVFS_VOLT_00_825000_V},
	 {DVFS_FREQ_00_900000_F, DVFS_VOLT_00_825000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V} },

	// opp 1
	{{DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_832000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_800000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_687500_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_800000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_725000_V} },

	// opp 2
	{{DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_728000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_700000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_550000_F, DVFS_VOLT_00_750000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_725000_V} },

	// opp 3
	{{DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_624000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_572000_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_458333_F, DVFS_VOLT_00_700000_V},
	 {DVFS_FREQ_00_416000_F, DVFS_VOLT_00_650000_V} },

	// opp 4
	{{DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_499200_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_457000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_650000_V},
	 {DVFS_FREQ_00_312000_F, DVFS_VOLT_00_600000_V} },

	// opp 5
	{{DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_280000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_208000_F, DVFS_VOLT_00_575000_V},
	 {DVFS_FREQ_00_273000_F, DVFS_VOLT_00_575000_V} },

};
#endif /* CONFIG_MACH_MT6893 */

#if defined(AGING_MARGIN) && defined(CONFIG_MACH_MT6893)
/* Below array define relation between different freq <--> aging voltage */
struct apusys_aging_steps aging_tbl_b0[APUSYS_MAX_NUM_OPPS][V_VCORE] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_687500_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_18750} },

	// opp 1
	{{DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_687500_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_18750} },

	// opp 2
	{{DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_700000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_700000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_550000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_550000_F, MG_VOLT_18750} },

	// opp 3
	{{DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_572000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_572000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_458333_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_458333_F, MG_VOLT_12500} },

	// opp 4
	{{DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_457000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_457000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_312000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_312000_F, MG_VOLT_12500} },

	// opp 5
	{{DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_280000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_280000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_208000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_208000_F, MG_VOLT_06250} },
};

/* Below array define relation between different freq <--> aging voltage */
struct apusys_aging_steps aging_tbl_b1[APUSYS_MAX_NUM_OPPS][V_VCORE] = {
	// opp 0
	{{DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_900000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_900000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_687500_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_18750} },

	// opp 1
	{{DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_832000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_800000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_687500_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_18750} },

	// opp 2
	{{DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_728000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_700000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_700000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_550000_F, MG_VOLT_18750},
	 {DVFS_FREQ_00_550000_F, MG_VOLT_18750} },

	// opp 3
	{{DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_624000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_572000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_572000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_458333_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_458333_F, MG_VOLT_12500} },

	// opp 4
	{{DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_499200_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_457000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_457000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_312000_F, MG_VOLT_12500},
	 {DVFS_FREQ_00_312000_F, MG_VOLT_12500} },

	// opp 5
	{{DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_273000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_280000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_280000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_208000_F, MG_VOLT_06250},
	 {DVFS_FREQ_00_208000_F, MG_VOLT_06250} },
};
#endif /* AGING_MARGIN && CONFIG_MACH_MT6893 */

