/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019 MediaTek Inc.
*/

/* Lafite_TPPA_FY_0817 */

/**********************************************
 * unified_power_data.h
 * This header file includes:
 * 1. Macros of SRAM related address
 * 2. Raw datas of unified power tables for each bank
 **********************************************/

#ifndef UNIFIED_POWER_DATA_H
#define UNIFIED_POWER_DATA_H

/* remember to sync to sspm upower */
#define UPOWER_CSRAM_BASE 0x0012a000
#define UPOWER_CSRAM_SIZE 0x3000 /* 12K bytes */
#define UPOWER_DVFS_OFF_BOTTOM 0x8 /* ignore the last 8 bytes */
/* limit should be at 0x12CFF4 */
#define UPOWER_TBL_LIMIT \
	((UPOWER_CSRAM_BASE)+(UPOWER_CSRAM_SIZE)-(UPOWER_DVFS_OFF_BOTTOM))

struct upower_tbl upower_tbl_l_6768 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 273, .volt = 70000, .dyn_pwr = 33217,
			.lkg_pwr = {45351, 45351, 45351, 45351, 45351, 45351} },
		{.cap = 284, .volt = 71875, .dyn_pwr = 37081,
			.lkg_pwr = {47037, 47037, 47037, 47037, 47037, 47037} },
		{.cap = 296, .volt = 73125, .dyn_pwr = 40514,
			.lkg_pwr = {48161, 48161, 48161, 48161, 48161, 48161} },
		{.cap = 308, .volt = 74375, .dyn_pwr = 44072,
			.lkg_pwr = {49285, 49285, 49285, 49285, 49285, 49285} },
		{.cap = 323, .volt = 76250, .dyn_pwr = 48687,
			.lkg_pwr = {50971, 50971, 50971, 50971, 50971, 50971} },
		{.cap = 338, .volt = 77500, .dyn_pwr = 52692,
			.lkg_pwr = {52095, 52095, 52095, 52095, 52095, 52095} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 59974,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 385, .volt = 83125, .dyn_pwr = 70262,
			.lkg_pwr = {57673, 57673, 57673, 57673, 57673, 57673} },
		{.cap = 399, .volt = 84375, .dyn_pwr = 75230,
			.lkg_pwr = {59005, 59005, 59005, 59005, 59005, 59005} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 80399,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 431, .volt = 88125, .dyn_pwr = 89808,
			.lkg_pwr = {63002, 63002, 63002, 63002, 63002, 63002} },
		{.cap = 443, .volt = 89375, .dyn_pwr = 95559,
			.lkg_pwr = {64334, 64334, 64334, 64334, 64334, 64334} },
		{.cap = 476, .volt = 93125, .dyn_pwr = 112391,
			.lkg_pwr = {68826, 68826, 68826, 68826, 68826, 68826} },
		{.cap = 495, .volt = 96250, .dyn_pwr = 125602,
			.lkg_pwr = {72653, 72653, 72653, 72653, 72653, 72653} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6768 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 273, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 284, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 296, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 308, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 323, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 338, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 385, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 399, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 431, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 443, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 476, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 495, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6768 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 128602,
		.lkg_pwr = {117734, 117734, 117734, 117734, 117734, 117734} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 153126,
		.lkg_pwr = {125658, 125658, 125658, 125658, 125658, 125658} },
		{.cap = 625, .volt = 76875, .dyn_pwr = 183177,
		.lkg_pwr = {135166, 135166, 135166, 135166, 135166, 135166} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 754, .volt = 86875, .dyn_pwr = 285909,
		.lkg_pwr = {161591, 161591, 161591, 161591, 161591, 161591} },
		{.cap = 796, .volt = 90000, .dyn_pwr = 324926,
		.lkg_pwr = {170000, 170000, 170000, 170000, 170000, 170000} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 869, .volt = 96875, .dyn_pwr = 417459,
		.lkg_pwr = {191620, 191620, 191620, 191620, 191620, 191620} },
		{.cap = 905, .volt = 100000, .dyn_pwr = 466190,
		.lkg_pwr = {201447, 201447, 201447, 201447, 201447, 201447} },
		{.cap = 941, .volt = 103125, .dyn_pwr = 518413,
		.lkg_pwr = {213105, 213105, 213105, 213105, 213105, 213105} },
		{.cap = 963, .volt = 105000, .dyn_pwr = 550314,
		.lkg_pwr = {220099, 220099, 220099, 220099, 220099, 220099} },
		{.cap = 984, .volt = 106250, .dyn_pwr = 576569,
		.lkg_pwr = {224762, 224762, 224762, 224762, 224762, 224762} },
		{.cap = 1004, .volt = 107500, .dyn_pwr = 603482,
		.lkg_pwr = {229425, 229425, 229425, 229425, 229425, 229425} },
		{.cap = 1024, .volt = 108750, .dyn_pwr = 631057,
		.lkg_pwr = {234089, 234089, 234089, 234089, 234089, 234089} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6768 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 625, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 754, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 796, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 869, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 905, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 941, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 963, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 984, .volt = 106250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1004, .volt = 107500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 108750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6768 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {46048, 46048, 46048, 46048, 46048, 46048} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48330, 48330, 48330, 48330, 48330, 48330} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 10697,
			.lkg_pwr = {50041, 50041, 50041, 50041, 50041, 50041} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 11741,
			.lkg_pwr = {51239, 51239, 51239, 51239, 51239, 51239} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 12838,
			.lkg_pwr = {52493, 52493, 52493, 52493, 52493, 52493} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 13990,
			.lkg_pwr = {53747, 53747, 53747, 53747, 53747, 53747} },
		{.cap = 0, .volt = 75000, .dyn_pwr = 15454,
			.lkg_pwr = {55629, 55629, 55629, 55629, 55629, 55629} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 16736,
			.lkg_pwr = {56883, 56883, 56883, 56883, 56883, 56883} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 18782,
			.lkg_pwr = {58765, 58765, 58765, 58765, 58765, 58765} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60646, 60646, 60646, 60646, 60646, 60646} },
		{.cap = 0, .volt = 81875, .dyn_pwr = 22839,
			.lkg_pwr = {62876, 62876, 62876, 62876, 62876, 62876} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 25264,
			.lkg_pwr = {65106, 65106, 65106, 65106, 65106, 65106} },
		{.cap = 0, .volt = 86250, .dyn_pwr = 28273,
			.lkg_pwr = {68080, 68080, 68080, 68080, 68080, 68080} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 30103,
			.lkg_pwr = {69567, 69567, 69567, 69567, 69567, 69567} },
		{.cap = 0, .volt = 90000, .dyn_pwr = 34003,
			.lkg_pwr = {72540, 72540, 72540, 72540, 72540, 72540} },
		{.cap = 0, .volt = 94375, .dyn_pwr = 39626,
			.lkg_pwr = {78518, 78518, 78518, 78518, 78518, 78518} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
	},
};

struct upower_tbl upower_tbl_l_6767 = {
	.row = {
		{.cap = 178, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 276, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 290, .volt = 69375, .dyn_pwr = 31667,
			.lkg_pwr = {44840, 44840, 44840, 44840, 44840, 44840} },
		{.cap = 303, .volt = 70625, .dyn_pwr = 34807,
			.lkg_pwr = {45913, 45913, 45913, 45913, 45913, 45913} },
		{.cap = 315, .volt = 72500, .dyn_pwr = 38776,
			.lkg_pwr = {47599, 47599, 47599, 47599, 47599, 47599} },
		{.cap = 328, .volt = 73750, .dyn_pwr = 42250,
			.lkg_pwr = {48723, 48723, 48723, 48723, 48723, 48723} },
		{.cap = 344, .volt = 75625, .dyn_pwr = 46752,
			.lkg_pwr = {50409, 50409, 50409, 50409, 50409, 50409} },
		{.cap = 360, .volt = 76875, .dyn_pwr = 50667,
			.lkg_pwr = {51533, 51533, 51533, 51533, 51533, 51533} },
		{.cap = 375, .volt = 78125, .dyn_pwr = 54762,
			.lkg_pwr = {52657, 52657, 52657, 52657, 52657, 52657} },
		{.cap = 390, .volt = 80000, .dyn_pwr = 59974,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 419, .volt = 83125, .dyn_pwr = 70262,
			.lkg_pwr = {57673, 57673, 57673, 57673, 57673, 57673} },
		{.cap = 434, .volt = 84375, .dyn_pwr = 75230,
			.lkg_pwr = {59005, 59005, 59005, 59005, 59005, 59005} },
		{.cap = 448, .volt = 85625, .dyn_pwr = 80399,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 462, .volt = 87500, .dyn_pwr = 87012,
			.lkg_pwr = {62336, 62336, 62336, 62336, 62336, 62336} },
		{.cap = 482, .volt = 89375, .dyn_pwr = 95559,
			.lkg_pwr = {64334, 64334, 64334, 64334, 64334, 64334} },
		{.cap = 517, .volt = 93125, .dyn_pwr = 112391,
			.lkg_pwr = {68826, 68826, 68826, 68826, 68826, 68826} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6767 = {
	.row = {
		{.cap = 178, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 276, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 290, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 303, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 315, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 328, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 344, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 360, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 375, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 390, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 419, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 434, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 448, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 462, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 482, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 517, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6767 = {
	.row = {
		{.cap = 548, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 564, .volt = 68750, .dyn_pwr = 120064,
		.lkg_pwr = {115015, 115015, 115015, 115015, 115015, 115015} },
		{.cap = 597, .volt = 71250, .dyn_pwr = 137345,
		.lkg_pwr = {120904, 120904, 120904, 120904, 120904, 120904} },
		{.cap = 631, .volt = 73125, .dyn_pwr = 153126,
		.lkg_pwr = {125658, 125658, 125658, 125658, 125658, 125658} },
		{.cap = 664, .volt = 75625, .dyn_pwr = 172741,
		.lkg_pwr = {131997, 131997, 131997, 131997, 131997, 131997} },
		{.cap = 696, .volt = 78125, .dyn_pwr = 193992,
		.lkg_pwr = {138336, 138336, 138336, 138336, 138336, 138336} },
		{.cap = 725, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 788, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 820, .volt = 86875, .dyn_pwr = 285909,
		.lkg_pwr = {161591, 161591, 161591, 161591, 161591, 161591} },
		{.cap = 853, .volt = 88750, .dyn_pwr = 310262,
		.lkg_pwr = {166636, 166636, 166636, 166636, 166636, 166636} },
		{.cap = 880, .volt = 91250, .dyn_pwr = 340219,
		.lkg_pwr = {173931, 173931, 173931, 173931, 173931, 173931} },
		{.cap = 907, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 945, .volt = 96875, .dyn_pwr = 417459,
		.lkg_pwr = {191620, 191620, 191620, 191620, 191620, 191620} },
		{.cap = 971, .volt = 98750, .dyn_pwr = 447854,
		.lkg_pwr = {197516, 197516, 197516, 197516, 197516, 197516} },
		{.cap = 998, .volt = 101250, .dyn_pwr = 485226,
		.lkg_pwr = {206110, 206110, 206110, 206110, 206110, 206110} },
		{.cap = 1024, .volt = 103125, .dyn_pwr = 518413,
		.lkg_pwr = {213105, 213105, 213105, 213105, 213105, 213105} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6767 = {
	.row = {
		{.cap = 548, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 564, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 597, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 631, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 664, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 696, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 725, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 788, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 820, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 853, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 880, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 907, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 945, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 971, .volt = 98750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 998, .volt = 101250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6767 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {45883, 45883, 45883, 45883, 45883, 45883} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48157, 48157, 48157, 48157, 48157, 48157} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 10187,
			.lkg_pwr = {49293, 49293, 49293, 49293, 49293, 49293} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 11203,
			.lkg_pwr = {50430, 50430, 50430, 50430, 50430, 50430} },
		{.cap = 0, .volt = 71250, .dyn_pwr = 12273,
			.lkg_pwr = {51680, 51680, 51680, 51680, 51680, 51680} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 13397,
			.lkg_pwr = {52930, 52930, 52930, 52930, 52930, 52930} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 14845,
			.lkg_pwr = {54805, 54805, 54805, 54805, 54805, 54805} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 16736,
			.lkg_pwr = {56679, 56679, 56679, 56679, 56679, 56679} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 18782,
			.lkg_pwr = {58554, 58554, 58554, 58554, 58554, 58554} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60429, 60429, 60429, 60429, 60429, 60429} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 22047,
			.lkg_pwr = {61910, 61910, 61910, 61910, 61910, 61910} },
		{.cap = 0, .volt = 82500, .dyn_pwr = 23623,
			.lkg_pwr = {63392, 63392, 63392, 63392, 63392, 63392} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 25264,
			.lkg_pwr = {64873, 64873, 64873, 64873, 64873, 64873} },
		{.cap = 0, .volt = 85625, .dyn_pwr = 27398,
			.lkg_pwr = {67095, 67095, 67095, 67095, 67095, 67095} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 30103,
			.lkg_pwr = {69317, 69317, 69317, 69317, 69317, 69317} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 34970,
			.lkg_pwr = {73131, 73131, 73131, 73131, 73131, 73131} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
	},
};

struct upower_tbl upower_tbl_l_PRO = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 236, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 259, .volt = 70625, .dyn_pwr = 34807,
			.lkg_pwr = {45913, 45913, 45913, 45913, 45913, 45913} },
		{.cap = 280, .volt = 73750, .dyn_pwr = 42250,
			.lkg_pwr = {48723, 48723, 48723, 48723, 48723, 48723} },
		{.cap = 307, .volt = 76875, .dyn_pwr = 50667,
			.lkg_pwr = {51533, 51533, 51533, 51533, 51533, 51533} },
		{.cap = 333, .volt = 80000, .dyn_pwr = 59974,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 358, .volt = 83125, .dyn_pwr = 70262,
			.lkg_pwr = {57673, 57673, 57673, 57673, 57673, 57673} },
		{.cap = 383, .volt = 85625, .dyn_pwr = 80399,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 412, .volt = 89375, .dyn_pwr = 95559,
			.lkg_pwr = {64334, 64334, 64334, 64334, 64334, 64334} },
		{.cap = 442, .volt = 93125, .dyn_pwr = 112391,
			.lkg_pwr = {68826, 68826, 68826, 68826, 68826, 68826} },
		{.cap = 460, .volt = 96250, .dyn_pwr = 125602,
			.lkg_pwr = {72653, 72653, 72653, 72653, 72653, 72653} },
		{.cap = 480, .volt = 100000, .dyn_pwr = 143556,
			.lkg_pwr = {77245, 77245, 77245, 77245, 77245, 77245} },
		{.cap = 492, .volt = 101875, .dyn_pwr = 153128,
			.lkg_pwr = {80561, 80561, 80561, 80561, 80561, 80561} },
		{.cap = 504, .volt = 103750, .dyn_pwr = 163109,
			.lkg_pwr = {83878, 83878, 83878, 83878, 83878, 83878} },
		{.cap = 514, .volt = 105625, .dyn_pwr = 173506,
			.lkg_pwr = {87195, 87195, 87195, 87195, 87195, 87195} },
		{.cap = 525, .volt = 107500, .dyn_pwr = 184329,
			.lkg_pwr = {90511, 90511, 90511, 90511, 90511, 90511} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_PRO = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 236, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 259, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 280, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 307, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 333, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 383, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 412, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 442, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 460, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 480, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 492, .volt = 101875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 504, .volt = 103750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 514, .volt = 105625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 525, .volt = 107500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_PRO = {
	.row = {
		{.cap = 468, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 510, .volt = 71250, .dyn_pwr = 137345,
		.lkg_pwr = {120904, 120904, 120904, 120904, 120904, 120904} },
		{.cap = 567, .volt = 75625, .dyn_pwr = 172741,
		.lkg_pwr = {131997, 131997, 131997, 131997, 131997, 131997} },
		{.cap = 619, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 673, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 714, .volt = 88125, .dyn_pwr = 300066,
		.lkg_pwr = {164954, 164954, 164954, 164954, 164954, 164954} },
		{.cap = 752, .volt = 91250, .dyn_pwr = 340219,
		.lkg_pwr = {173931, 173931, 173931, 173931, 173931, 173931} },
		{.cap = 774, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 830, .volt = 98750, .dyn_pwr = 447854,
		.lkg_pwr = {197516, 197516, 197516, 197516, 197516, 197516} },
		{.cap = 874, .volt = 103125, .dyn_pwr = 518413,
		.lkg_pwr = {213105, 213105, 213105, 213105, 213105, 213105} },
		{.cap = 901, .volt = 105000, .dyn_pwr = 554412,
		.lkg_pwr = {220099, 220099, 220099, 220099, 220099, 220099} },
		{.cap = 927, .volt = 106875, .dyn_pwr = 592040,
		.lkg_pwr = {227094, 227094, 227094, 227094, 227094, 227094} },
		{.cap = 951, .volt = 108750, .dyn_pwr = 631057,
		.lkg_pwr = {234089, 234089, 234089, 234089, 234089, 234089} },
		{.cap = 976, .volt = 110625, .dyn_pwr = 671200,
		.lkg_pwr = {241919, 241919, 241919, 241919, 241919, 241919} },
		{.cap = 1019, .volt = 111875, .dyn_pwr = 705120,
		.lkg_pwr = {248254, 248254, 248254, 248254, 248254, 248254} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 724108,
		.lkg_pwr = {248254, 248254, 248254, 248254, 248254, 248254} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_PRO = {
	.row = {
		{.cap = 468, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 510, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 567, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 619, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 673, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 714, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 752, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 774, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 830, .volt = 98750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 874, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 901, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 927, .volt = 106875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 951, .volt = 108750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 976, .volt = 110625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1019, .volt = 111875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_PRO = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {46048, 46048, 46048, 46048, 46048, 46048} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48330, 48330, 48330, 48330, 48330, 48330} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 11203,
			.lkg_pwr = {50612, 50612, 50612, 50612, 50612, 50612} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 13397,
			.lkg_pwr = {53120, 53120, 53120, 53120, 53120, 53120} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 16099,
			.lkg_pwr = {56256, 56256, 56256, 56256, 56256, 56256} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 18782,
			.lkg_pwr = {58765, 58765, 58765, 58765, 58765, 58765} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60646, 60646, 60646, 60646, 60646, 60646} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 24448,
			.lkg_pwr = {64363, 64363, 64363, 64363, 64363, 64363} },
		{.cap = 0, .volt = 85625, .dyn_pwr = 27398,
			.lkg_pwr = {67336, 67336, 67336, 67336, 67336, 67336} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 30103,
			.lkg_pwr = {69567, 69567, 69567, 69567, 69567, 69567} },
		{.cap = 0, .volt = 90000, .dyn_pwr = 34003,
			.lkg_pwr = {72540, 72540, 72540, 72540, 72540, 72540} },
		{.cap = 0, .volt = 94375, .dyn_pwr = 39626,
			.lkg_pwr = {78518, 78518, 78518, 78518, 78518, 78518} },
		{.cap = 0, .volt = 99375, .dyn_pwr = 46749,
			.lkg_pwr = {85351, 85351, 85351, 85351, 85351, 85351} },
		{.cap = 0, .volt = 101250, .dyn_pwr = 49567,
			.lkg_pwr = {88673, 88673, 88673, 88673, 88673, 88673} },
		{.cap = 0, .volt = 104375, .dyn_pwr = 54839,
			.lkg_pwr = {94842, 94842, 94842, 94842, 94842, 94842} },
		{.cap = 0, .volt = 107500, .dyn_pwr = 60467,
		.lkg_pwr = {101011, 101011, 101011, 101011, 101011, 101011} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
	},
};
struct upower_tbl upower_tbl_l_6768_v7 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 273, .volt = 69375, .dyn_pwr = 32627,
			.lkg_pwr = {44840, 44840, 44840, 44840, 44840, 44840} },
		{.cap = 284, .volt = 70625, .dyn_pwr = 35802,
			.lkg_pwr = {45913, 45913, 45913, 45913, 45913, 45913} },
		{.cap = 296, .volt = 71875, .dyn_pwr = 39141,
			.lkg_pwr = {47037, 47037, 47037, 47037, 47037, 47037} },
		{.cap = 308, .volt = 73125, .dyn_pwr = 42603,
			.lkg_pwr = {48161, 48161, 48161, 48161, 48161, 48161} },
		{.cap = 323, .volt = 74375, .dyn_pwr = 46322,
			.lkg_pwr = {49285, 49285, 49285, 49285, 49285, 49285} },
		{.cap = 338, .volt = 75625, .dyn_pwr = 50173,
			.lkg_pwr = {50409, 50409, 50409, 50409, 50409, 50409} },
		{.cap = 358, .volt = 77500, .dyn_pwr = 56284,
			.lkg_pwr = {52095, 52095, 52095, 52095, 52095, 52095} },
		{.cap = 385, .volt = 80000, .dyn_pwr = 65079,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 399, .volt = 81250, .dyn_pwr = 69761,
			.lkg_pwr = {55675, 55675, 55675, 55675, 55675, 55675} },
		{.cap = 412, .volt = 82500, .dyn_pwr = 74638,
			.lkg_pwr = {57007, 57007, 57007, 57007, 57007, 57007} },
		{.cap = 431, .volt = 84375, .dyn_pwr = 82327,
			.lkg_pwr = {59005, 59005, 59005, 59005, 59005, 59005} },
		{.cap = 443, .volt = 85625, .dyn_pwr = 87708,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 476, .volt = 88125, .dyn_pwr = 100647,
			.lkg_pwr = {63002, 63002, 63002, 63002, 63002, 63002} },
		{.cap = 495, .volt = 92500, .dyn_pwr = 116006,
			.lkg_pwr = {68061, 68061, 68061, 68061, 68061, 68061} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6768_v7 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 273, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 284, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 296, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 308, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 323, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 338, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 385, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 399, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 412, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 431, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 443, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 476, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 495, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6768_v7 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 128602,
		.lkg_pwr = {117734, 117734, 117734, 117734, 117734, 117734} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 153126,
		.lkg_pwr = {125658, 125658, 125658, 125658, 125658, 125658} },
		{.cap = 625, .volt = 76250, .dyn_pwr = 180211,
		.lkg_pwr = {133581, 133581, 133581, 133581, 133581, 133581} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 754, .volt = 86250, .dyn_pwr = 281810,
		.lkg_pwr = {159909, 159909, 159909, 159909, 159909, 159909} },
		{.cap = 796, .volt = 89375, .dyn_pwr = 320429,
		.lkg_pwr = {168318, 168318, 168318, 168318, 168318, 168318} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 869, .volt = 96250, .dyn_pwr = 412090,
		.lkg_pwr = {189655, 189655, 189655, 189655, 189655, 189655} },
		{.cap = 905, .volt = 99375, .dyn_pwr = 460381,
		.lkg_pwr = {199482, 199482, 199482, 199482, 199482, 199482} },
		{.cap = 941, .volt = 102500, .dyn_pwr = 512148,
		.lkg_pwr = {210773, 210773, 210773, 210773, 210773, 210773} },
		{.cap = 963, .volt = 103750, .dyn_pwr = 537290,
		.lkg_pwr = {215436, 215436, 215436, 215436, 215436, 215436} },
		{.cap = 984, .volt = 105000, .dyn_pwr = 563083,
		.lkg_pwr = {220099, 220099, 220099, 220099, 220099, 220099} },
		{.cap = 1004, .volt = 106250, .dyn_pwr = 589529,
		.lkg_pwr = {224762, 224762, 224762, 224762, 224762, 224762} },
		{.cap = 1024, .volt = 107500, .dyn_pwr = 616633,
		.lkg_pwr = {229425, 229425, 229425, 229425, 229425, 229425} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6768_v7 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 625, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 754, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 796, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 869, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 905, .volt = 99375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 941, .volt = 102500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 963, .volt = 103750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 984, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1004, .volt = 106250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 107500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6768_v7 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {46048, 46048, 46048, 46048, 46048, 46048} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48330, 48330, 48330, 48330, 48330, 48330} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 10697,
			.lkg_pwr = {50041, 50041, 50041, 50041, 50041, 50041} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 11741,
			.lkg_pwr = {51239, 51239, 51239, 51239, 51239, 51239} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 12838,
			.lkg_pwr = {52493, 52493, 52493, 52493, 52493, 52493} },
		{.cap = 0, .volt = 73750, .dyn_pwr = 14230,
			.lkg_pwr = {54375, 54375, 54375, 54375, 54375, 54375} },
		{.cap = 0, .volt = 75000, .dyn_pwr = 15454,
			.lkg_pwr = {55629, 55629, 55629, 55629, 55629, 55629} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 16736,
			.lkg_pwr = {56883, 56883, 56883, 56883, 56883, 56883} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 19084,
			.lkg_pwr = {59392, 59392, 59392, 59392, 59392, 59392} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60646, 60646, 60646, 60646, 60646, 60646} },
		{.cap = 0, .volt = 82500, .dyn_pwr = 23189,
			.lkg_pwr = {63620, 63620, 63620, 63620, 63620, 63620} },
		{.cap = 0, .volt = 84375, .dyn_pwr = 25643,
			.lkg_pwr = {65850, 65850, 65850, 65850, 65850, 65850} },
		{.cap = 0, .volt = 86875, .dyn_pwr = 28684,
			.lkg_pwr = {68823, 68823, 68823, 68823, 68823, 68823} },
		{.cap = 0, .volt = 88125, .dyn_pwr = 30534,
			.lkg_pwr = {70310, 70310, 70310, 70310, 70310, 70310} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 34477,
			.lkg_pwr = {73394, 73394, 73394, 73394, 73394, 73394} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 40153,
			.lkg_pwr = {79372, 79372, 79372, 79372, 79372, 79372} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
	},
};

struct upower_tbl upower_tbl_l_6767_v7 = {
	.row = {
		{.cap = 178, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 276, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 290, .volt = 68750, .dyn_pwr = 31099,
			.lkg_pwr = {44329, 44329, 44329, 44329, 44329, 44329} },
		{.cap = 303, .volt = 70000, .dyn_pwr = 34194,
			.lkg_pwr = {45351, 45351, 45351, 45351, 45351, 45351} },
		{.cap = 315, .volt = 71250, .dyn_pwr = 37451,
			.lkg_pwr = {46475, 46475, 46475, 46475, 46475, 46475} },
		{.cap = 328, .volt = 72500, .dyn_pwr = 40830,
			.lkg_pwr = {47599, 47599, 47599, 47599, 47599, 47599} },
		{.cap = 344, .volt = 73750, .dyn_pwr = 44463,
			.lkg_pwr = {48723, 48723, 48723, 48723, 48723, 48723} },
		{.cap = 360, .volt = 75000, .dyn_pwr = 48226,
			.lkg_pwr = {49847, 49847, 49847, 49847, 49847, 49847} },
		{.cap = 375, .volt = 76250, .dyn_pwr = 52165,
			.lkg_pwr = {50971, 50971, 50971, 50971, 50971, 50971} },
		{.cap = 390, .volt = 77500, .dyn_pwr = 56284,
			.lkg_pwr = {52095, 52095, 52095, 52095, 52095, 52095} },
		{.cap = 419, .volt = 80000, .dyn_pwr = 65079,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 434, .volt = 81250, .dyn_pwr = 69761,
			.lkg_pwr = {55675, 55675, 55675, 55675, 55675, 55675} },
		{.cap = 448, .volt = 82500, .dyn_pwr = 74638,
			.lkg_pwr = {57007, 57007, 57007, 57007, 57007, 57007} },
		{.cap = 462, .volt = 83750, .dyn_pwr = 79714,
			.lkg_pwr = {58339, 58339, 58339, 58339, 58339, 58339} },
		{.cap = 482, .volt = 85625, .dyn_pwr = 87708,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 517, .volt = 88125, .dyn_pwr = 100647,
			.lkg_pwr = {63002, 63002, 63002, 63002, 63002, 63002} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6767_v7 = {
	.row = {
		{.cap = 178, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 276, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 290, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 303, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 315, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 328, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 344, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 360, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 375, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 390, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 419, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 434, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 448, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 462, .volt = 83750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 482, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 517, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6767_v7 = {
	.row = {
		{.cap = 548, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 564, .volt = 68750, .dyn_pwr = 120064,
		.lkg_pwr = {115015, 115015, 115015, 115015, 115015, 115015} },
		{.cap = 597, .volt = 71250, .dyn_pwr = 137345,
		.lkg_pwr = {120904, 120904, 120904, 120904, 120904, 120904} },
		{.cap = 631, .volt = 73125, .dyn_pwr = 153126,
		.lkg_pwr = {125658, 125658, 125658, 125658, 125658, 125658} },
		{.cap = 664, .volt = 75625, .dyn_pwr = 172741,
		.lkg_pwr = {131997, 131997, 131997, 131997, 131997, 131997} },
		{.cap = 696, .volt = 77500, .dyn_pwr = 190901,
		.lkg_pwr = {136751, 136751, 136751, 136751, 136751, 136751} },
		{.cap = 725, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 788, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 820, .volt = 86250, .dyn_pwr = 281810,
		.lkg_pwr = {159909, 159909, 159909, 159909, 159909, 159909} },
		{.cap = 853, .volt = 88750, .dyn_pwr = 310262,
		.lkg_pwr = {166636, 166636, 166636, 166636, 166636, 166636} },
		{.cap = 880, .volt = 90625, .dyn_pwr = 335574,
		.lkg_pwr = {171965, 171965, 171965, 171965, 171965, 171965} },
		{.cap = 907, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 945, .volt = 96250, .dyn_pwr = 412090,
		.lkg_pwr = {189655, 189655, 189655, 189655, 189655, 189655} },
		{.cap = 971, .volt = 98125, .dyn_pwr = 442203,
		.lkg_pwr = {195551, 195551, 195551, 195551, 195551, 195551} },
		{.cap = 998, .volt = 100625, .dyn_pwr = 479254,
		.lkg_pwr = {203779, 203779, 203779, 203779, 203779, 203779} },
		{.cap = 1024, .volt = 102500, .dyn_pwr = 512148,
		.lkg_pwr = {210773, 210773, 210773, 210773, 210773, 210773} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6767_v7 = {
	.row = {
		{.cap = 548, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 564, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 597, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 631, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 664, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 696, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 725, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 788, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 820, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 853, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 880, .volt = 90625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 907, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 945, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 971, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 998, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 102500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6767_v7 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {45883, 45883, 45883, 45883, 45883, 45883} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48157, 48157, 48157, 48157, 48157, 48157} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 10187,
			.lkg_pwr = {49293, 49293, 49293, 49293, 49293, 49293} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 11203,
			.lkg_pwr = {50430, 50430, 50430, 50430, 50430, 50430} },
		{.cap = 0, .volt = 71250, .dyn_pwr = 12273,
			.lkg_pwr = {51680, 51680, 51680, 51680, 51680, 51680} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 13629,
			.lkg_pwr = {53555, 53555, 53555, 53555, 53555, 53555} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 14845,
			.lkg_pwr = {54805, 54805, 54805, 54805, 54805, 54805} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 16736,
			.lkg_pwr = {56679, 56679, 56679, 56679, 56679, 56679} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 19084,
			.lkg_pwr = {59179, 59179, 59179, 59179, 59179, 59179} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60429, 60429, 60429, 60429, 60429, 60429} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 22047,
			.lkg_pwr = {61910, 61910, 61910, 61910, 61910, 61910} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 23982,
			.lkg_pwr = {64132, 64132, 64132, 64132, 64132, 64132} },
		{.cap = 0, .volt = 84375, .dyn_pwr = 25643,
			.lkg_pwr = {65614, 65614, 65614, 65614, 65614, 65614} },
		{.cap = 0, .volt = 85625, .dyn_pwr = 27398,
			.lkg_pwr = {67095, 67095, 67095, 67095, 67095, 67095} },
		{.cap = 0, .volt = 88125, .dyn_pwr = 30534,
			.lkg_pwr = {70058, 70058, 70058, 70058, 70058, 70058} },
		{.cap = 0, .volt = 91250, .dyn_pwr = 35454,
			.lkg_pwr = {73982, 73982, 73982, 73982, 73982, 73982} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
		{{0}, {48157} },
	},
};

struct upower_tbl upower_tbl_l_PRO_v7 = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 236, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 259, .volt = 70000, .dyn_pwr = 34194,
			.lkg_pwr = {45351, 45351, 45351, 45351, 45351, 45351} },
		{.cap = 280, .volt = 72500, .dyn_pwr = 40830,
			.lkg_pwr = {47599, 47599, 47599, 47599, 47599, 47599} },
		{.cap = 307, .volt = 75000, .dyn_pwr = 48226,
			.lkg_pwr = {49847, 49847, 49847, 49847, 49847, 49847} },
		{.cap = 333, .volt = 77500, .dyn_pwr = 56284,
			.lkg_pwr = {52095, 52095, 52095, 52095, 52095, 52095} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 65079,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 383, .volt = 82500, .dyn_pwr = 74638,
			.lkg_pwr = {57007, 57007, 57007, 57007, 57007, 57007} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 87708,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 442, .volt = 88125, .dyn_pwr = 100647,
			.lkg_pwr = {63002, 63002, 63002, 63002, 63002, 63002} },
		{.cap = 460, .volt = 92500, .dyn_pwr = 116006,
			.lkg_pwr = {68061, 68061, 68061, 68061, 68061, 68061} },
		{.cap = 480, .volt = 97500, .dyn_pwr = 136467,
			.lkg_pwr = {74183, 74183, 74183, 74183, 74183, 74183} },
		{.cap = 492, .volt = 100625, .dyn_pwr = 149393,
			.lkg_pwr = {78350, 78350, 78350, 78350, 78350, 78350} },
		{.cap = 504, .volt = 103125, .dyn_pwr = 161150,
			.lkg_pwr = {82772, 82772, 82772, 82772, 82772, 82772} },
		{.cap = 514, .volt = 105625, .dyn_pwr = 173506,
			.lkg_pwr = {87195, 87195, 87195, 87195, 87195, 87195} },
		{.cap = 525, .volt = 108125, .dyn_pwr = 186479,
			.lkg_pwr = {91617, 91617, 91617, 91617, 91617, 91617} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_PRO_v7 = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 236, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 259, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 280, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 307, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 333, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 383, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 442, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 460, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 480, .volt = 97500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 492, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 504, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 514, .volt = 105625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 525, .volt = 108125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_PRO_v7 = {
	.row = {
		{.cap = 468, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 510, .volt = 71250, .dyn_pwr = 137345,
		.lkg_pwr = {120904, 120904, 120904, 120904, 120904, 120904} },
		{.cap = 567, .volt = 75625, .dyn_pwr = 172741,
		.lkg_pwr = {131997, 131997, 131997, 131997, 131997, 131997} },
		{.cap = 619, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 673, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 714, .volt = 87500, .dyn_pwr = 295825,
		.lkg_pwr = {163272, 163272, 163272, 163272, 163272, 163272} },
		{.cap = 752, .volt = 90625, .dyn_pwr = 335574,
		.lkg_pwr = {171965, 171965, 171965, 171965, 171965, 171965} },
		{.cap = 774, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 830, .volt = 98125, .dyn_pwr = 442203,
		.lkg_pwr = {195551, 195551, 195551, 195551, 195551, 195551} },
		{.cap = 874, .volt = 102500, .dyn_pwr = 512148,
		.lkg_pwr = {210773, 210773, 210773, 210773, 210773, 210773} },
		{.cap = 901, .volt = 104375, .dyn_pwr = 547832,
		.lkg_pwr = {217768, 217768, 217768, 217768, 217768, 217768} },
		{.cap = 927, .volt = 105625, .dyn_pwr = 578272,
		.lkg_pwr = {222431, 222431, 222431, 222431, 222431, 222431} },
		{.cap = 951, .volt = 107500, .dyn_pwr = 616633,
		.lkg_pwr = {229425, 229425, 229425, 229425, 229425, 229425} },
		{.cap = 976, .volt = 108750, .dyn_pwr = 648640,
		.lkg_pwr = {234089, 234089, 234089, 234089, 234089, 234089} },
		{.cap = 1019, .volt = 110625, .dyn_pwr = 689452,
		.lkg_pwr = {241919, 241919, 241919, 241919, 241919, 241919} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 724108,
		.lkg_pwr = {248254, 248254, 248254, 248254, 248254, 248254} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_PRO_v7 = {
	.row = {
		{.cap = 468, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 510, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 567, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 619, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 673, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 714, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 752, .volt = 90625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 774, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 830, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 874, .volt = 102500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 901, .volt = 104375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 927, .volt = 105625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 951, .volt = 107500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 976, .volt = 108750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1019, .volt = 110625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_PRO_v7 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {46048, 46048, 46048, 46048, 46048, 46048} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48330, 48330, 48330, 48330, 48330, 48330} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 11203,
			.lkg_pwr = {50612, 50612, 50612, 50612, 50612, 50612} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 13629,
			.lkg_pwr = {53747, 53747, 53747, 53747, 53747, 53747} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 16099,
			.lkg_pwr = {56256, 56256, 56256, 56256, 56256, 56256} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 19084,
			.lkg_pwr = {59392, 59392, 59392, 59392, 59392, 59392} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60646, 60646, 60646, 60646, 60646, 60646} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 24818,
			.lkg_pwr = {65106, 65106, 65106, 65106, 65106, 65106} },
		{.cap = 0, .volt = 85625, .dyn_pwr = 27398,
			.lkg_pwr = {67336, 67336, 67336, 67336, 67336, 67336} },
		{.cap = 0, .volt = 88125, .dyn_pwr = 30534,
			.lkg_pwr = {70310, 70310, 70310, 70310, 70310, 70310} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 34477,
			.lkg_pwr = {73394, 73394, 73394, 73394, 73394, 73394} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 40153,
			.lkg_pwr = {79372, 79372, 79372, 79372, 79372, 79372} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 47339,
			.lkg_pwr = {86205, 86205, 86205, 86205, 86205, 86205} },
		{.cap = 0, .volt = 101875, .dyn_pwr = 50181,
			.lkg_pwr = {89906, 89906, 89906, 89906, 89906, 89906} },
		{.cap = 0, .volt = 105000, .dyn_pwr = 55497,
			.lkg_pwr = {96076, 96076, 96076, 96076, 96076, 96076} },
		{.cap = 0, .volt = 108125, .dyn_pwr = 61172,
		.lkg_pwr = {102245, 102245, 102245, 102245, 102245, 102245} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
	},
};

struct upower_tbl upower_tbl_l_G75 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 16848,
			.lkg_pwr = {41262, 41262, 41262, 41262, 41262, 41262} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 28125,
			.lkg_pwr = {43306, 43306, 43306, 43306, 43306, 43306} },
		{.cap = 273, .volt = 70000, .dyn_pwr = 33217,
			.lkg_pwr = {45351, 45351, 45351, 45351, 45351, 45351} },
		{.cap = 284, .volt = 71875, .dyn_pwr = 37081,
			.lkg_pwr = {47037, 47037, 47037, 47037, 47037, 47037} },
		{.cap = 296, .volt = 73125, .dyn_pwr = 40514,
			.lkg_pwr = {48161, 48161, 48161, 48161, 48161, 48161} },
		{.cap = 308, .volt = 74375, .dyn_pwr = 44072,
			.lkg_pwr = {49285, 49285, 49285, 49285, 49285, 49285} },
		{.cap = 323, .volt = 76250, .dyn_pwr = 48687,
			.lkg_pwr = {50971, 50971, 50971, 50971, 50971, 50971} },
		{.cap = 338, .volt = 77500, .dyn_pwr = 52692,
			.lkg_pwr = {52095, 52095, 52095, 52095, 52095, 52095} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 59974,
			.lkg_pwr = {54343, 54343, 54343, 54343, 54343, 54343} },
		{.cap = 385, .volt = 83125, .dyn_pwr = 70262,
			.lkg_pwr = {57673, 57673, 57673, 57673, 57673, 57673} },
		{.cap = 399, .volt = 84375, .dyn_pwr = 75230,
			.lkg_pwr = {59005, 59005, 59005, 59005, 59005, 59005} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 80399,
			.lkg_pwr = {60337, 60337, 60337, 60337, 60337, 60337} },
		{.cap = 431, .volt = 88125, .dyn_pwr = 89808,
			.lkg_pwr = {63002, 63002, 63002, 63002, 63002, 63002} },
		{.cap = 443, .volt = 89375, .dyn_pwr = 95559,
			.lkg_pwr = {64334, 64334, 64334, 64334, 64334, 64334} },
		{.cap = 476, .volt = 93125, .dyn_pwr = 112391,
			.lkg_pwr = {68826, 68826, 68826, 68826, 68826, 68826} },
		{.cap = 516, .volt = 100625, .dyn_pwr = 145356,
			.lkg_pwr = {78350, 78350, 78350, 78350, 78350, 78350} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
		{{0}, {43306} },
	},
};

struct upower_tbl upower_tbl_cluster_l_G75 = {
	.row = {
		{.cap = 164, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 254, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 273, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 284, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 296, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 308, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 323, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 338, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 385, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 399, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 412, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 431, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 443, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 476, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 516, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_G75 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 111920,
		.lkg_pwr = {112296, 112296, 112296, 112296, 112296, 112296} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 128602,
		.lkg_pwr = {117734, 117734, 117734, 117734, 117734, 117734} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 153126,
		.lkg_pwr = {125658, 125658, 125658, 125658, 125658, 125658} },
		{.cap = 625, .volt = 76875, .dyn_pwr = 183177,
		.lkg_pwr = {135166, 135166, 135166, 135166, 135166, 135166} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 213268,
		.lkg_pwr = {143090, 143090, 143090, 143090, 143090, 143090} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 259032,
		.lkg_pwr = {154863, 154863, 154863, 154863, 154863, 154863} },
		{.cap = 754, .volt = 86875, .dyn_pwr = 285909,
		.lkg_pwr = {161591, 161591, 161591, 161591, 161591, 161591} },
		{.cap = 796, .volt = 90000, .dyn_pwr = 324926,
		.lkg_pwr = {170000, 170000, 170000, 170000, 170000, 170000} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 366962,
		.lkg_pwr = {179827, 179827, 179827, 179827, 179827, 179827} },
		{.cap = 869, .volt = 96875, .dyn_pwr = 417459,
		.lkg_pwr = {191620, 191620, 191620, 191620, 191620, 191620} },
		{.cap = 905, .volt = 100000, .dyn_pwr = 466190,
		.lkg_pwr = {201447, 201447, 201447, 201447, 201447, 201447} },
		{.cap = 941, .volt = 103125, .dyn_pwr = 518413,
		.lkg_pwr = {213105, 213105, 213105, 213105, 213105, 213105} },
		{.cap = 963, .volt = 105000, .dyn_pwr = 550314,
		.lkg_pwr = {220099, 220099, 220099, 220099, 220099, 220099} },
		{.cap = 984, .volt = 106250, .dyn_pwr = 576569,
		.lkg_pwr = {224762, 224762, 224762, 224762, 224762, 224762} },
		{.cap = 1004, .volt = 107500, .dyn_pwr = 603482,
		.lkg_pwr = {229425, 229425, 229425, 229425, 229425, 229425} },
		{.cap = 1024, .volt = 108750, .dyn_pwr = 631057,
		.lkg_pwr = {234089, 234089, 234089, 234089, 234089, 234089} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
		{{0}, {112296} },
	},
};

struct upower_tbl upower_tbl_cluster_b_G75 = {
	.row = {
		{.cap = 504, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 532, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 580, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 625, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 667, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 725, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 754, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 796, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 833, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 869, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 905, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 941, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 963, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 984, .volt = 106250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1004, .volt = 107500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 108750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_G75 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 7918,
			.lkg_pwr = {46048, 46048, 46048, 46048, 46048, 46048} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 9529,
			.lkg_pwr = {48330, 48330, 48330, 48330, 48330, 48330} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 10697,
			.lkg_pwr = {50041, 50041, 50041, 50041, 50041, 50041} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 11741,
			.lkg_pwr = {51239, 51239, 51239, 51239, 51239, 51239} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 12838,
			.lkg_pwr = {52493, 52493, 52493, 52493, 52493, 52493} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 13990,
			.lkg_pwr = {53747, 53747, 53747, 53747, 53747, 53747} },
		{.cap = 0, .volt = 75000, .dyn_pwr = 15454,
			.lkg_pwr = {55629, 55629, 55629, 55629, 55629, 55629} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 16736,
			.lkg_pwr = {56883, 56883, 56883, 56883, 56883, 56883} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 18782,
			.lkg_pwr = {58765, 58765, 58765, 58765, 58765, 58765} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 20534,
			.lkg_pwr = {60646, 60646, 60646, 60646, 60646, 60646} },
		{.cap = 0, .volt = 81875, .dyn_pwr = 22839,
			.lkg_pwr = {62876, 62876, 62876, 62876, 62876, 62876} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 25264,
			.lkg_pwr = {65106, 65106, 65106, 65106, 65106, 65106} },
		{.cap = 0, .volt = 86250, .dyn_pwr = 28273,
			.lkg_pwr = {68080, 68080, 68080, 68080, 68080, 68080} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 30103,
			.lkg_pwr = {69567, 69567, 69567, 69567, 69567, 69567} },
		{.cap = 0, .volt = 90000, .dyn_pwr = 34003,
			.lkg_pwr = {72540, 72540, 72540, 72540, 72540, 72540} },
		{.cap = 0, .volt = 100625, .dyn_pwr = 48464,
			.lkg_pwr = {87439, 87439, 87439, 87439, 87439, 87439} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
		{{0}, {48330} },
	},
};


#endif /* UNIFIED_POWER_DATA_H */



