/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019 MediaTek Inc.
*/

/* Lafite_TPPA_FY_0817 */

/**********************************************
 * unified_power_data.h
 * This header file includes:
 * 1. Macros of SRAM related address
 * 2. Raw datas of unified power tables for each bank
 **********************************************/

#ifndef UNIFIED_POWER_DATA_H
#define UNIFIED_POWER_DATA_H

/* remember to sync to sspm upower */
#define UPOWER_CSRAM_BASE 0x0012a000
#define UPOWER_CSRAM_SIZE 0x3000 /* 12K bytes */
#define UPOWER_DVFS_OFF_BOTTOM 0x8 /* ignore the last 8 bytes */
/* limit should be at 0x12CFF4 */
#define UPOWER_TBL_LIMIT \
	((UPOWER_CSRAM_BASE)+(UPOWER_CSRAM_SIZE)-(UPOWER_DVFS_OFF_BOTTOM))

struct upower_tbl upower_tbl_l_6785 = {
	.row = {
		{.cap = 126, .volt = 60000, .dyn_pwr = 14733,
			.lkg_pwr = {34199, 34199, 34199, 34199, 34199, 34199} },
		{.cap = 195, .volt = 67500, .dyn_pwr = 28865,
			.lkg_pwr = {39842, 39842, 39842, 39842, 39842, 39842} },
		{.cap = 215, .volt = 70000, .dyn_pwr = 35094,
			.lkg_pwr = {41723, 41723, 41723, 41723, 41723, 41723} },
		{.cap = 233, .volt = 72500, .dyn_pwr = 41948,
			.lkg_pwr = {43791, 43791, 43791, 43791, 43791, 43791} },
		{.cap = 255, .volt = 75000, .dyn_pwr = 49495,
			.lkg_pwr = {45859, 45859, 45859, 45859, 45859, 45859} },
		{.cap = 276, .volt = 77500, .dyn_pwr = 57766,
			.lkg_pwr = {47927, 47927, 47927, 47927, 47927, 47927} },
		{.cap = 297, .volt = 80000, .dyn_pwr = 66791,
			.lkg_pwr = {49995, 49995, 49995, 49995, 49995, 49995} },
		{.cap = 317, .volt = 82500, .dyn_pwr = 76602,
			.lkg_pwr = {52446, 52446, 52446, 52446, 52446, 52446} },
		{.cap = 341, .volt = 85625, .dyn_pwr = 90016,
			.lkg_pwr = {55510, 55510, 55510, 55510, 55510, 55510} },
		{.cap = 365, .volt = 87500, .dyn_pwr = 101397,
			.lkg_pwr = {57349, 57349, 57349, 57349, 57349, 57349} },
		{.cap = 375, .volt = 90000, .dyn_pwr = 110456,
			.lkg_pwr = {59800, 59800, 59800, 59800, 59800, 59800} },
		{.cap = 387, .volt = 92500, .dyn_pwr = 121370,
			.lkg_pwr = {62616, 62616, 62616, 62616, 62616, 62616} },
		{.cap = 397, .volt = 95000, .dyn_pwr = 132968,
			.lkg_pwr = {65432, 65432, 65432, 65432, 65432, 65432} },
		{.cap = 411, .volt = 98125, .dyn_pwr = 147062,
			.lkg_pwr = {68953, 68953, 68953, 68953, 68953, 68953} },
		{.cap = 423, .volt = 100625, .dyn_pwr = 160204,
			.lkg_pwr = {72082, 72082, 72082, 72082, 72082, 72082} },
		{.cap = 435, .volt = 103125, .dyn_pwr = 174095,
			.lkg_pwr = {76151, 76151, 76151, 76151, 76151, 76151} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6785 = {
	.row = {
		{.cap = 126, .volt = 60000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 195, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 215, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 233, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 255, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 276, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 297, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 317, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 341, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 365, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 375, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 387, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 397, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 411, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 423, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 435, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6785 = {
	.row = {
		{.cap = 459, .volt = 67500, .dyn_pwr = 126720,
		.lkg_pwr = {177757, 177757, 177757, 177757, 177757, 177757} },
		{.cap = 495, .volt = 69375, .dyn_pwr = 144408,
		.lkg_pwr = {184214, 184214, 184214, 184214, 184214, 184214} },
		{.cap = 537, .volt = 71875, .dyn_pwr = 170323,
		.lkg_pwr = {193892, 193892, 193892, 193892, 193892, 193892} },
		{.cap = 582, .volt = 74375, .dyn_pwr = 197699,
		.lkg_pwr = {203926, 203926, 203926, 203926, 203926, 203926} },
		{.cap = 624, .volt = 77500, .dyn_pwr = 231091,
		.lkg_pwr = {216469, 216469, 216469, 216469, 216469, 216469} },
		{.cap = 663, .volt = 80000, .dyn_pwr = 263734,
		.lkg_pwr = {226503, 226503, 226503, 226503, 226503, 226503} },
		{.cap = 731, .volt = 84375, .dyn_pwr = 325017,
		.lkg_pwr = {245139, 245139, 245139, 245139, 245139, 245139} },
		{.cap = 786, .volt = 87500, .dyn_pwr = 376186,
		.lkg_pwr = {258451, 258451, 258451, 258451, 258451, 258451} },
		{.cap = 832, .volt = 91250, .dyn_pwr = 437591,
		.lkg_pwr = {275322, 275322, 275322, 275322, 275322, 275322} },
		{.cap = 888, .volt = 95625, .dyn_pwr = 519186,
		.lkg_pwr = {297101, 297101, 297101, 297101, 297101, 297101} },
		{.cap = 914, .volt = 98125, .dyn_pwr = 564684,
		.lkg_pwr = {309546, 309546, 309546, 309546, 309546, 309546} },
		{.cap = 939, .volt = 101250, .dyn_pwr = 620182,
		.lkg_pwr = {326261, 326261, 326261, 326261, 326261, 326261} },
		{.cap = 966, .volt = 103750, .dyn_pwr = 671188,
		.lkg_pwr = {341023, 341023, 341023, 341023, 341023, 341023} },
		{.cap = 986, .volt = 105625, .dyn_pwr = 710747,
		.lkg_pwr = {352095, 352095, 352095, 352095, 352095, 352095} },
		{.cap = 1006, .volt = 108125, .dyn_pwr = 760794,
		.lkg_pwr = {366858, 366858, 366858, 366858, 366858, 366858} },
		{.cap = 1024, .volt = 110000, .dyn_pwr = 803493,
		.lkg_pwr = {377930, 377930, 377930, 377930, 377930, 377930} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6785 = {
	.row = {
		{.cap = 459, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 495, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 537, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 582, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 624, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 663, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 731, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 786, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 832, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 888, .volt = 95625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 914, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 939, .volt = 101250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 966, .volt = 103750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 986, .volt = 105625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1006, .volt = 108125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 110000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6785 = {
	.row = {
		{.cap = 0, .volt = 60000, .dyn_pwr = 7309,
			.lkg_pwr = {48325, 48325, 48325, 48325, 48325, 48325} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 10360,
			.lkg_pwr = {56298, 56298, 56298, 56298, 56298, 56298} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 11960,
			.lkg_pwr = {58292, 58292, 58292, 58292, 58292, 58292} },
		{.cap = 0, .volt = 71250, .dyn_pwr = 14058,
			.lkg_pwr = {60417, 60417, 60417, 60417, 60417, 60417} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 16328,
			.lkg_pwr = {62609, 62609, 62609, 62609, 62609, 62609} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 19089,
			.lkg_pwr = {65531, 65531, 65531, 65531, 65531, 65531} },
		{.cap = 0, .volt = 76875, .dyn_pwr = 20997,
			.lkg_pwr = {66992, 66992, 66992, 66992, 66992, 66992} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 23343,
			.lkg_pwr = {69184, 69184, 69184, 69184, 69184, 69184} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 25467,
			.lkg_pwr = {70645, 70645, 70645, 70645, 70645, 70645} },
		{.cap = 0, .volt = 81875, .dyn_pwr = 28581,
			.lkg_pwr = {73243, 73243, 73243, 73243, 73243, 73243} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 31898,
			.lkg_pwr = {75841, 75841, 75841, 75841, 75841, 75841} },
		{.cap = 0, .volt = 86250, .dyn_pwr = 34888,
			.lkg_pwr = {79304, 79304, 79304, 79304, 79304, 79304} },
		{.cap = 0, .volt = 88125, .dyn_pwr = 37525,
			.lkg_pwr = {81902, 81902, 81902, 81902, 81902, 81902} },
		{.cap = 0, .volt = 91875, .dyn_pwr = 43186,
			.lkg_pwr = {87485, 87485, 87485, 87485, 87485, 87485} },
		{.cap = 0, .volt = 96250, .dyn_pwr = 50030,
			.lkg_pwr = {94449, 94449, 94449, 94449, 94449, 94449} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 56847,
		.lkg_pwr = {100418, 100418, 100418, 100418, 100418, 100418} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
	},
};

struct upower_tbl upower_tbl_l_6785T = {
	.row = {
		{.cap = 124, .volt = 60000, .dyn_pwr = 14733,
			.lkg_pwr = {34199, 34199, 34199, 34199, 34199, 34199} },
		{.cap = 192, .volt = 67500, .dyn_pwr = 28865,
			.lkg_pwr = {39842, 39842, 39842, 39842, 39842, 39842} },
		{.cap = 211, .volt = 70000, .dyn_pwr = 35094,
			.lkg_pwr = {41723, 41723, 41723, 41723, 41723, 41723} },
		{.cap = 228, .volt = 72500, .dyn_pwr = 41948,
			.lkg_pwr = {43791, 43791, 43791, 43791, 43791, 43791} },
		{.cap = 250, .volt = 75000, .dyn_pwr = 49495,
			.lkg_pwr = {45859, 45859, 45859, 45859, 45859, 45859} },
		{.cap = 271, .volt = 77500, .dyn_pwr = 57766,
			.lkg_pwr = {47927, 47927, 47927, 47927, 47927, 47927} },
		{.cap = 291, .volt = 80000, .dyn_pwr = 66791,
			.lkg_pwr = {49995, 49995, 49995, 49995, 49995, 49995} },
		{.cap = 311, .volt = 82500, .dyn_pwr = 76602,
			.lkg_pwr = {52446, 52446, 52446, 52446, 52446, 52446} },
		{.cap = 335, .volt = 85625, .dyn_pwr = 90016,
			.lkg_pwr = {55510, 55510, 55510, 55510, 55510, 55510} },
		{.cap = 358, .volt = 87500, .dyn_pwr = 101397,
			.lkg_pwr = {57349, 57349, 57349, 57349, 57349, 57349} },
		{.cap = 368, .volt = 90000, .dyn_pwr = 110456,
			.lkg_pwr = {59800, 59800, 59800, 59800, 59800, 59800} },
		{.cap = 380, .volt = 92500, .dyn_pwr = 121370,
			.lkg_pwr = {62616, 62616, 62616, 62616, 62616, 62616} },
		{.cap = 390, .volt = 95000, .dyn_pwr = 132968,
			.lkg_pwr = {65432, 65432, 65432, 65432, 65432, 65432} },
		{.cap = 404, .volt = 98125, .dyn_pwr = 147062,
			.lkg_pwr = {68953, 68953, 68953, 68953, 68953, 68953} },
		{.cap = 416, .volt = 100625, .dyn_pwr = 160204,
			.lkg_pwr = {72082, 72082, 72082, 72082, 72082, 72082} },
		{.cap = 427, .volt = 103125, .dyn_pwr = 174095,
			.lkg_pwr = {76151, 76151, 76151, 76151, 76151, 76151} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6785T = {
	.row = {
		{.cap = 124, .volt = 60000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 192, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 211, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 228, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 250, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 271, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 291, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 311, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 335, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 358, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 368, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 380, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 390, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 404, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 416, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 427, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6785T = {
	.row = {
		{.cap = 450, .volt = 67500, .dyn_pwr = 126720,
		.lkg_pwr = {177757, 177757, 177757, 177757, 177757, 177757} },
		{.cap = 486, .volt = 69375, .dyn_pwr = 144408,
		.lkg_pwr = {184214, 184214, 184214, 184214, 184214, 184214} },
		{.cap = 528, .volt = 71875, .dyn_pwr = 170323,
		.lkg_pwr = {193892, 193892, 193892, 193892, 193892, 193892} },
		{.cap = 571, .volt = 74375, .dyn_pwr = 197699,
		.lkg_pwr = {203926, 203926, 203926, 203926, 203926, 203926} },
		{.cap = 612, .volt = 77500, .dyn_pwr = 231091,
		.lkg_pwr = {216469, 216469, 216469, 216469, 216469, 216469} },
		{.cap = 651, .volt = 80000, .dyn_pwr = 263734,
		.lkg_pwr = {226503, 226503, 226503, 226503, 226503, 226503} },
		{.cap = 717, .volt = 84375, .dyn_pwr = 325017,
		.lkg_pwr = {245139, 245139, 245139, 245139, 245139, 245139} },
		{.cap = 771, .volt = 87500, .dyn_pwr = 376186,
		.lkg_pwr = {258451, 258451, 258451, 258451, 258451, 258451} },
		{.cap = 817, .volt = 91250, .dyn_pwr = 437591,
		.lkg_pwr = {275322, 275322, 275322, 275322, 275322, 275322} },
		{.cap = 872, .volt = 95625, .dyn_pwr = 519186,
		.lkg_pwr = {297101, 297101, 297101, 297101, 297101, 297101} },
		{.cap = 897, .volt = 98125, .dyn_pwr = 564684,
		.lkg_pwr = {309546, 309546, 309546, 309546, 309546, 309546} },
		{.cap = 922, .volt = 101250, .dyn_pwr = 620182,
		.lkg_pwr = {326261, 326261, 326261, 326261, 326261, 326261} },
		{.cap = 948, .volt = 103750, .dyn_pwr = 671188,
		.lkg_pwr = {341023, 341023, 341023, 341023, 341023, 341023} },
		{.cap = 975, .volt = 106250, .dyn_pwr = 724349,
		.lkg_pwr = {355786, 355786, 355786, 355786, 355786, 355786} },
		{.cap = 999, .volt = 109375, .dyn_pwr = 788990,
		.lkg_pwr = {374239, 374239, 374239, 374239, 374239, 374239} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 847967,
		.lkg_pwr = {392972, 392972, 392972, 392972, 392972, 392972} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6785T = {
	.row = {
		{.cap = 450, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 486, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 528, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 571, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 612, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 651, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 717, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 771, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 817, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 872, .volt = 95625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 897, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 922, .volt = 101250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 948, .volt = 103750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 975, .volt = 106250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 999, .volt = 109375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 111875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6785T = {
	.row = {
		{.cap = 0, .volt = 60000, .dyn_pwr = 7309,
			.lkg_pwr = {48325, 48325, 48325, 48325, 48325, 48325} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 10360,
			.lkg_pwr = {56298, 56298, 56298, 56298, 56298, 56298} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 11960,
			.lkg_pwr = {58292, 58292, 58292, 58292, 58292, 58292} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 14306,
			.lkg_pwr = {61148, 61148, 61148, 61148, 61148, 61148} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 16891,
			.lkg_pwr = {64070, 64070, 64070, 64070, 64070, 64070} },
		{.cap = 0, .volt = 76875, .dyn_pwr = 19725,
			.lkg_pwr = {66992, 66992, 66992, 66992, 66992, 66992} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 21685,
			.lkg_pwr = {68454, 68454, 68454, 68454, 68454, 68454} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 24090,
			.lkg_pwr = {70645, 70645, 70645, 70645, 70645, 70645} },
		{.cap = 0, .volt = 81875, .dyn_pwr = 26675,
			.lkg_pwr = {73243, 73243, 73243, 73243, 73243, 73243} },
		{.cap = 0, .volt = 84375, .dyn_pwr = 30353,
			.lkg_pwr = {76707, 76707, 76707, 76707, 76707, 76707} },
		{.cap = 0, .volt = 86250, .dyn_pwr = 33831,
			.lkg_pwr = {79304, 79304, 79304, 79304, 79304, 79304} },
		{.cap = 0, .volt = 88750, .dyn_pwr = 36940,
			.lkg_pwr = {82768, 82768, 82768, 82768, 82768, 82768} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 39685,
			.lkg_pwr = {85495, 85495, 85495, 85495, 85495, 85495} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 46174,
			.lkg_pwr = {92459, 92459, 92459, 92459, 92459, 92459} },
		{.cap = 0, .volt = 99375, .dyn_pwr = 53332,
			.lkg_pwr = {99423, 99423, 99423, 99423, 99423, 99423} },
		{.cap = 0, .volt = 103125, .dyn_pwr = 60455,
		.lkg_pwr = {107604, 107604, 107604, 107604, 107604, 107604} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
	},
};

struct upower_tbl upower_tbl_l_6783 = {
	.row = {
		{.cap = 137, .volt = 60000, .dyn_pwr = 14733,
			.lkg_pwr = {34199, 34199, 34199, 34199, 34199, 34199} },
		{.cap = 212, .volt = 67500, .dyn_pwr = 28865,
			.lkg_pwr = {39842, 39842, 39842, 39842, 39842, 39842} },
		{.cap = 234, .volt = 69375, .dyn_pwr = 34470,
			.lkg_pwr = {41253, 41253, 41253, 41253, 41253, 41253} },
		{.cap = 253, .volt = 71250, .dyn_pwr = 40514,
			.lkg_pwr = {42757, 42757, 42757, 42757, 42757, 42757} },
		{.cap = 277, .volt = 73750, .dyn_pwr = 47859,
			.lkg_pwr = {44825, 44825, 44825, 44825, 44825, 44825} },
		{.cap = 300, .volt = 75625, .dyn_pwr = 55004,
			.lkg_pwr = {46376, 46376, 46376, 46376, 46376, 46376} },
		{.cap = 323, .volt = 78125, .dyn_pwr = 63697,
			.lkg_pwr = {48444, 48444, 48444, 48444, 48444, 48444} },
		{.cap = 345, .volt = 80000, .dyn_pwr = 72030,
			.lkg_pwr = {49995, 49995, 49995, 49995, 49995, 49995} },
		{.cap = 361, .volt = 81875, .dyn_pwr = 79561,
			.lkg_pwr = {51834, 51834, 51834, 51834, 51834, 51834} },
		{.cap = 376, .volt = 83125, .dyn_pwr = 86251,
			.lkg_pwr = {53059, 53059, 53059, 53059, 53059, 53059} },
		{.cap = 397, .volt = 84375, .dyn_pwr = 94283,
			.lkg_pwr = {54285, 54285, 54285, 54285, 54285, 54285} },
		{.cap = 408, .volt = 86250, .dyn_pwr = 101443,
			.lkg_pwr = {56123, 56123, 56123, 56123, 56123, 56123} },
		{.cap = 415, .volt = 88125, .dyn_pwr = 108063,
			.lkg_pwr = {57962, 57962, 57962, 57962, 57962, 57962} },
		{.cap = 421, .volt = 89375, .dyn_pwr = 113308,
			.lkg_pwr = {59187, 59187, 59187, 59187, 59187, 59187} },
		{.cap = 427, .volt = 91250, .dyn_pwr = 120361,
			.lkg_pwr = {61208, 61208, 61208, 61208, 61208, 61208} },
		{.cap = 433, .volt = 92500, .dyn_pwr = 126062,
			.lkg_pwr = {62616, 62616, 62616, 62616, 62616, 62616} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
		{{0}, {39842} },
	},
};

struct upower_tbl upower_tbl_cluster_l_6783 = {
	.row = {
		{.cap = 137, .volt = 60000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 212, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 234, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 253, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 277, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 300, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 323, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 345, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 361, .volt = 81875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 376, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 397, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 408, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 415, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 421, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 427, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 433, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_6783 = {
	.row = {
		{.cap = 499, .volt = 67500, .dyn_pwr = 126720,
		.lkg_pwr = {177757, 177757, 177757, 177757, 177757, 177757} },
		{.cap = 539, .volt = 69375, .dyn_pwr = 144408,
		.lkg_pwr = {184214, 184214, 184214, 184214, 184214, 184214} },
		{.cap = 585, .volt = 71875, .dyn_pwr = 170323,
		.lkg_pwr = {193892, 193892, 193892, 193892, 193892, 193892} },
		{.cap = 633, .volt = 74375, .dyn_pwr = 197699,
		.lkg_pwr = {203926, 203926, 203926, 203926, 203926, 203926} },
		{.cap = 679, .volt = 77500, .dyn_pwr = 231091,
		.lkg_pwr = {216469, 216469, 216469, 216469, 216469, 216469} },
		{.cap = 722, .volt = 80000, .dyn_pwr = 263734,
		.lkg_pwr = {226503, 226503, 226503, 226503, 226503, 226503} },
		{.cap = 795, .volt = 84375, .dyn_pwr = 325017,
		.lkg_pwr = {245139, 245139, 245139, 245139, 245139, 245139} },
		{.cap = 826, .volt = 86250, .dyn_pwr = 352742,
		.lkg_pwr = {253126, 253126, 253126, 253126, 253126, 253126} },
		{.cap = 855, .volt = 87500, .dyn_pwr = 376186,
		.lkg_pwr = {258451, 258451, 258451, 258451, 258451, 258451} },
		{.cap = 881, .volt = 89375, .dyn_pwr = 406321,
		.lkg_pwr = {266438, 266438, 266438, 266438, 266438, 266438} },
		{.cap = 906, .volt = 91250, .dyn_pwr = 437591,
		.lkg_pwr = {275322, 275322, 275322, 275322, 275322, 275322} },
		{.cap = 966, .volt = 94375, .dyn_pwr = 505702,
		.lkg_pwr = {290878, 290878, 290878, 290878, 290878, 290878} },
		{.cap = 980, .volt = 96250, .dyn_pwr = 534538,
		.lkg_pwr = {300212, 300212, 300212, 300212, 300212, 300212} },
		{.cap = 995, .volt = 98125, .dyn_pwr = 564684,
		.lkg_pwr = {309546, 309546, 309546, 309546, 309546, 309546} },
		{.cap = 1009, .volt = 99375, .dyn_pwr = 588463,
		.lkg_pwr = {315768, 315768, 315768, 315768, 315768, 315768} },
		{.cap = 1024, .volt = 101250, .dyn_pwr = 621378,
		.lkg_pwr = {326261, 326261, 326261, 326261, 326261, 326261} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
		{{0}, {177757} },
	},
};

struct upower_tbl upower_tbl_cluster_b_6783 = {
	.row = {
		{.cap = 499, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 539, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 585, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 633, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 679, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 722, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 795, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 826, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 855, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 881, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 906, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 966, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 980, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 995, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1009, .volt = 99375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 101250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_6783 = {
	.row = {
		{.cap = 0, .volt = 60000, .dyn_pwr = 7309,
			.lkg_pwr = {48325, 48325, 48325, 48325, 48325, 48325} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 10360,
			.lkg_pwr = {56298, 56298, 56298, 56298, 56298, 56298} },
		{.cap = 0, .volt = 69375, .dyn_pwr = 11960,
			.lkg_pwr = {58292, 58292, 58292, 58292, 58292, 58292} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 13468,
			.lkg_pwr = {59687, 59687, 59687, 59687, 59687, 59687} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 15303,
			.lkg_pwr = {61878, 61878, 61878, 61878, 61878, 61878} },
		{.cap = 0, .volt = 73750, .dyn_pwr = 17006,
			.lkg_pwr = {63340, 63340, 63340, 63340, 63340, 63340} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 19089,
			.lkg_pwr = {65531, 65531, 65531, 65531, 65531, 65531} },
		{.cap = 0, .volt = 76875, .dyn_pwr = 20997,
			.lkg_pwr = {66992, 66992, 66992, 66992, 66992, 66992} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 23343,
			.lkg_pwr = {69184, 69184, 69184, 69184, 69184, 69184} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 25467,
			.lkg_pwr = {70645, 70645, 70645, 70645, 70645, 70645} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 27663,
			.lkg_pwr = {72377, 72377, 72377, 72377, 72377, 72377} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 30442,
			.lkg_pwr = {74975, 74975, 74975, 74975, 74975, 74975} },
		{.cap = 0, .volt = 84375, .dyn_pwr = 32376,
			.lkg_pwr = {76707, 76707, 76707, 76707, 76707, 76707} },
		{.cap = 0, .volt = 86250, .dyn_pwr = 34888,
			.lkg_pwr = {79304, 79304, 79304, 79304, 79304, 79304} },
		{.cap = 0, .volt = 88125, .dyn_pwr = 37525,
			.lkg_pwr = {81902, 81902, 81902, 81902, 81902, 81902} },
		{.cap = 0, .volt = 91875, .dyn_pwr = 43186,
			.lkg_pwr = {87485, 87485, 87485, 87485, 87485, 87485} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
		{{0}, {56298} },
	},
};

#endif /* UNIFIED_POWER_DATA_H */



