// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright (c) 2019 MediaTek Inc.
 */


/* Mouton_TO_FY_v0p5__200217 */

/**********************************************
 * unified_power_data.h
 * This header file includes:
 * 1. Macros of SRAM related address
 * 2. Raw datas of unified power tables for each bank
 **********************************************/

#ifndef UNIFIED_POWER_DATA_H
#define UNIFIED_POWER_DATA_H

/* remember to sync to sspm upower */
#define UPOWER_CSRAM_BASE 0x0012a000
#define UPOWER_CSRAM_SIZE 0x3000 /* 12K bytes */
#define UPOWER_DVFS_OFF_BOTTOM 0x8 /* ignore the last 8 bytes */
/* limit should be at 0x12CFF4 */
#define UPOWER_TBL_LIMIT \
	((UPOWER_CSRAM_BASE)+(UPOWER_CSRAM_SIZE)-(UPOWER_DVFS_OFF_BOTTOM))

struct upower_tbl upower_tbl_l_FY = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 15882,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 187, .volt = 65000, .dyn_pwr = 20646,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 198, .volt = 66250, .dyn_pwr = 23197,
			.lkg_pwr = {24952, 24952, 24952, 24952, 24952, 24952} },
		{.cap = 208, .volt = 67500, .dyn_pwr = 25896,
			.lkg_pwr = {25784, 25784, 25784, 25784, 25784, 25784} },
		{.cap = 228, .volt = 70000, .dyn_pwr = 31755,
			.lkg_pwr = {27447, 27447, 27447, 27447, 27447, 27447} },
		{.cap = 246, .volt = 72500, .dyn_pwr = 38252,
			.lkg_pwr = {29107, 29107, 29107, 29107, 29107, 29107} },
		{.cap = 256, .volt = 74375, .dyn_pwr = 43583,
			.lkg_pwr = {30353, 30353, 30353, 30353, 30353, 30353} },
		{.cap = 271, .volt = 76250, .dyn_pwr = 48809,
			.lkg_pwr = {31710, 31710, 31710, 31710, 31710, 31710} },
		{.cap = 293, .volt = 80000, .dyn_pwr = 60064,
			.lkg_pwr = {34537, 34537, 34537, 34537, 34537, 34537} },
		{.cap = 311, .volt = 82500, .dyn_pwr = 68188,
			.lkg_pwr = {36675, 36675, 36675, 36675, 36675, 36675} },
		{.cap = 319, .volt = 85000, .dyn_pwr = 76847,
			.lkg_pwr = {38812, 38812, 38812, 38812, 38812, 38812} },
		{.cap = 333, .volt = 89375, .dyn_pwr = 90570,
			.lkg_pwr = {42932, 42932, 42932, 42932, 42932, 42932} },
		{.cap = 345, .volt = 92500, .dyn_pwr = 101058,
			.lkg_pwr = {46122, 46122, 46122, 46122, 46122, 46122} },
		{.cap = 350, .volt = 94375, .dyn_pwr = 107555,
			.lkg_pwr = {48073, 48073, 48073, 48073, 48073, 48073} },
		{.cap = 361, .volt = 97500, .dyn_pwr = 118796,
			.lkg_pwr = {51703, 51703, 51703, 51703, 51703, 51703} },
		{.cap = 366, .volt = 100000, .dyn_pwr = 128148,
			.lkg_pwr = {54682, 54682, 54682, 54682, 54682, 54682} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
	},
};

struct upower_tbl upower_tbl_cluster_l_FY = {
	.row = {
		{.cap = 152, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 187, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 198, .volt = 66250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 208, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 228, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 246, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 256, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 271, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 293, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 311, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 319, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 333, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 345, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 350, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 361, .volt = 97500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 366, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_FY = {
	.row = {
		{.cap = 629, .volt = 65000, .dyn_pwr = 88354,
			.lkg_pwr = {95432, 95432, 95432, 95432, 95432, 95432} },
		{.cap = 687, .volt = 67500, .dyn_pwr = 110395,
			.lkg_pwr = {101909, 101909, 101909, 101909, 101909,
			101909} },
		{.cap = 718, .volt = 68750, .dyn_pwr = 122429,
			.lkg_pwr = {105148, 105148, 105148, 105148, 105148,
			105148} },
		{.cap = 753, .volt = 70625, .dyn_pwr = 141715,
			.lkg_pwr = {110005, 110005, 110005, 110005, 110005,
			110005} },
		{.cap = 776, .volt = 71875, .dyn_pwr = 155269,
			.lkg_pwr = {113241, 113241, 113241, 113241, 113241,
			113241} },
		{.cap = 775, .volt = 73750, .dyn_pwr = 176368,
			.lkg_pwr = {118096, 118096, 118096, 118096, 118096,
			118096} },
		{.cap = 821, .volt = 76875, .dyn_pwr = 214580,
			.lkg_pwr = {126803, 126803, 126803, 126803, 126803,
			126803} },
		{.cap = 861, .volt = 80000, .dyn_pwr = 256656,
			.lkg_pwr = {135922, 135922, 135922, 135922, 135922,
			135922} },
		{.cap = 890, .volt = 82500, .dyn_pwr = 293432,
			.lkg_pwr = {143381, 143381, 143381, 143381, 143381,
			143381} },
		{.cap = 919, .volt = 85000, .dyn_pwr = 332437,
			.lkg_pwr = {150841, 150841, 150841, 150841, 150841,
			150841} },
		{.cap = 935, .volt = 86875, .dyn_pwr = 359264,
			.lkg_pwr = {157538, 157538, 157538, 157538, 157538,
			157538} },
		{.cap = 948, .volt = 88750, .dyn_pwr = 387229,
			.lkg_pwr = {164235, 164235, 164235, 164235, 164235,
			164235} },
		{.cap = 967, .volt = 91875, .dyn_pwr = 436255,
			.lkg_pwr = {175600, 175600, 175600, 175600, 175600,
			175600} },
		{.cap = 980, .volt = 94375, .dyn_pwr = 477984,
			.lkg_pwr = {184799, 184799, 184799, 184799, 184799,
			184799} },
		{.cap = 1001, .volt = 96875, .dyn_pwr = 521566,
			.lkg_pwr = {195290, 195290, 195290, 195290, 195290,
			195290} },
		{.cap = 1024, .volt = 100000, .dyn_pwr = 579511,
			.lkg_pwr = {208942, 208942, 208942, 208942, 208942,
			208942} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
	},
};

struct upower_tbl upower_tbl_cluster_b_FY = {
	.row = {
		{.cap = 629, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 687, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 718, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 753, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 776, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 775, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 821, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 861, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 890, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 919, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 935, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 948, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 967, .volt = 91875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 980, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1001, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_FY = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 9264,
			.lkg_pwr = {30145, 30145, 30145, 30145, 30145, 30145} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 10415,
			.lkg_pwr = {31109, 31109, 31109, 31109, 31109, 31109} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 11655,
			.lkg_pwr = {32073, 32073, 32073, 32073, 32073, 32073} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 12943,
			.lkg_pwr = {33036, 33036, 33036, 33036, 33036, 33036} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 14325,
			.lkg_pwr = {34000, 34000, 34000, 34000, 34000, 34000} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 17288,
			.lkg_pwr = {35928, 35928, 35928, 35928, 35928, 35928} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 21402,
			.lkg_pwr = {38393, 38393, 38393, 38393, 38393, 38393} },
		{.cap = 0, .volt = 77500, .dyn_pwr = 24144,
			.lkg_pwr = {40008, 40008, 40008, 40008, 40008, 40008} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 28066,
			.lkg_pwr = {42160, 42160, 42160, 42160, 42160, 42160} },
		{.cap = 0, .volt = 82500, .dyn_pwr = 32335,
			.lkg_pwr = {44807, 44807, 44807, 44807, 44807, 44807} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 36965,
			.lkg_pwr = {47454, 47454, 47454, 47454, 47454, 47454} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 41335,
			.lkg_pwr = {50427, 50427, 50427, 50427, 50427, 50427} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 44838,
			.lkg_pwr = {52657, 52657, 52657, 52657, 52657, 52657} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 53703,
			.lkg_pwr = {58667, 58667, 58667, 58667, 58667, 58667} },
		{.cap = 0, .volt = 98125, .dyn_pwr = 63618,
			.lkg_pwr = {65932, 65932, 65932, 65932, 65932, 65932} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 68216,
			.lkg_pwr = {69238, 69238, 69238, 69238, 69238, 69238} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
	},
};

struct upower_tbl upower_tbl_l_MT6853 = {
	.row = {
		{.cap = 158, .volt = 65000, .dyn_pwr = 15882,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 195, .volt = 65000, .dyn_pwr = 20646,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 207, .volt = 66250, .dyn_pwr = 23197,
			.lkg_pwr = {24952, 24952, 24952, 24952, 24952, 24952} },
		{.cap = 218, .volt = 67500, .dyn_pwr = 25896,
			.lkg_pwr = {25784, 25784, 25784, 25784, 25784, 25784} },
		{.cap = 238, .volt = 70000, .dyn_pwr = 31755,
			.lkg_pwr = {27447, 27447, 27447, 27447, 27447, 27447} },
		{.cap = 257, .volt = 72500, .dyn_pwr = 38252,
			.lkg_pwr = {29107, 29107, 29107, 29107, 29107, 29107} },
		{.cap = 268, .volt = 74375, .dyn_pwr = 43583,
			.lkg_pwr = {30353, 30353, 30353, 30353, 30353, 30353} },
		{.cap = 283, .volt = 76250, .dyn_pwr = 48809,
			.lkg_pwr = {31710, 31710, 31710, 31710, 31710, 31710} },
		{.cap = 306, .volt = 80000, .dyn_pwr = 60064,
			.lkg_pwr = {34537, 34537, 34537, 34537, 34537, 34537} },
		{.cap = 324, .volt = 82500, .dyn_pwr = 68188,
			.lkg_pwr = {36675, 36675, 36675, 36675, 36675, 36675} },
		{.cap = 333, .volt = 85000, .dyn_pwr = 76847,
			.lkg_pwr = {38812, 38812, 38812, 38812, 38812, 38812} },
		{.cap = 348, .volt = 89375, .dyn_pwr = 90570,
			.lkg_pwr = {42932, 42932, 42932, 42932, 42932, 42932} },
		{.cap = 361, .volt = 92500, .dyn_pwr = 101058,
			.lkg_pwr = {46122, 46122, 46122, 46122, 46122, 46122} },
		{.cap = 366, .volt = 94375, .dyn_pwr = 107555,
			.lkg_pwr = {48073, 48073, 48073, 48073, 48073, 48073} },
		{.cap = 377, .volt = 97500, .dyn_pwr = 118796,
			.lkg_pwr = {51703, 51703, 51703, 51703, 51703, 51703} },
		{.cap = 383, .volt = 100000, .dyn_pwr = 128148,
			.lkg_pwr = {54682, 54682, 54682, 54682, 54682, 54682} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
	},
};

struct upower_tbl upower_tbl_cluster_l_MT6853 = {
	.row = {
		{.cap = 158, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 195, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 207, .volt = 66250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 218, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 238, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 257, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 268, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 283, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 306, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 324, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 333, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 348, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 361, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 366, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 377, .volt = 97500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 383, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_MT6853 = {
	.row = {
		{.cap = 657, .volt = 65000, .dyn_pwr = 88354,
			.lkg_pwr = {95432, 95432, 95432, 95432, 95432, 95432} },
		{.cap = 717, .volt = 67500, .dyn_pwr = 110395,
			.lkg_pwr = {101909, 101909, 101909, 101909, 101909,
			101909} },
		{.cap = 750, .volt = 68750, .dyn_pwr = 122429,
			.lkg_pwr = {105148, 105148, 105148, 105148, 105148,
			105148} },
		{.cap = 787, .volt = 70625, .dyn_pwr = 141715,
			.lkg_pwr = {110005, 110005, 110005, 110005, 110005,
			110005} },
		{.cap = 810, .volt = 71875, .dyn_pwr = 155269,
			.lkg_pwr = {113241, 113241, 113241, 113241, 113241,
			113241} },
		{.cap = 810, .volt = 73750, .dyn_pwr = 176368,
			.lkg_pwr = {118096, 118096, 118096, 118096, 118096,
			118096} },
		{.cap = 857, .volt = 76875, .dyn_pwr = 214580,
			.lkg_pwr = {126803, 126803, 126803, 126803, 126803,
			126803} },
		{.cap = 899, .volt = 80000, .dyn_pwr = 256656,
			.lkg_pwr = {135922, 135922, 135922, 135922, 135922,
			135922} },
		{.cap = 929, .volt = 82500, .dyn_pwr = 293432,
			.lkg_pwr = {143381, 143381, 143381, 143381, 143381,
			143381} },
		{.cap = 960, .volt = 85000, .dyn_pwr = 332437,
			.lkg_pwr = {150841, 150841, 150841, 150841, 150841,
			150841} },
		{.cap = 976, .volt = 86875, .dyn_pwr = 359264,
			.lkg_pwr = {157538, 157538, 157538, 157538, 157538,
			157538} },
		{.cap = 990, .volt = 88750, .dyn_pwr = 387229,
			.lkg_pwr = {164235, 164235, 164235, 164235, 164235,
			164235} },
		{.cap = 1001, .volt = 89375, .dyn_pwr = 400764,
			.lkg_pwr = {166468, 166468, 166468, 166468, 166468,
			166468} },
		{.cap = 1010, .volt = 91250, .dyn_pwr = 430340,
			.lkg_pwr = {173300, 173300, 173300, 173300, 173300,
			173300} },
		{.cap = 1019, .volt = 92500, .dyn_pwr = 450752,
			.lkg_pwr = {177900, 177900, 177900, 177900, 177900,
			177900} },
		{.cap = 1024, .volt = 93750, .dyn_pwr = 471674,
			.lkg_pwr = {182500, 182500, 182500, 182500, 182500,
			182500} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
	},
};

struct upower_tbl upower_tbl_cluster_b_MT6853 = {
	.row = {
		{.cap = 657, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 717, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 750, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 787, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 810, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 810, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 857, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 899, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 929, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 960, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 976, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 990, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1001, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1010, .volt = 91250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1019, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 93750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_MT6853 = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 9264,
			.lkg_pwr = {30145, 30145, 30145, 30145, 30145, 30145} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 10415,
			.lkg_pwr = {31109, 31109, 31109, 31109, 31109, 31109} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 11655,
			.lkg_pwr = {32073, 32073, 32073, 32073, 32073, 32073} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 12943,
			.lkg_pwr = {33036, 33036, 33036, 33036, 33036, 33036} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 14325,
			.lkg_pwr = {34000, 34000, 34000, 34000, 34000, 34000} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 17288,
			.lkg_pwr = {35928, 35928, 35928, 35928, 35928, 35928} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 21402,
			.lkg_pwr = {38393, 38393, 38393, 38393, 38393, 38393} },
		{.cap = 0, .volt = 77500, .dyn_pwr = 24144,
			.lkg_pwr = {40008, 40008, 40008, 40008, 40008, 40008} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 28066,
			.lkg_pwr = {42160, 42160, 42160, 42160, 42160, 42160} },
		{.cap = 0, .volt = 82500, .dyn_pwr = 32335,
			.lkg_pwr = {44807, 44807, 44807, 44807, 44807, 44807} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 36965,
			.lkg_pwr = {47454, 47454, 47454, 47454, 47454, 47454} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 41335,
			.lkg_pwr = {50427, 50427, 50427, 50427, 50427, 50427} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 44838,
			.lkg_pwr = {52657, 52657, 52657, 52657, 52657, 52657} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 53703,
			.lkg_pwr = {58667, 58667, 58667, 58667, 58667, 58667} },
		{.cap = 0, .volt = 98125, .dyn_pwr = 63618,
			.lkg_pwr = {65932, 65932, 65932, 65932, 65932, 65932} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 68216,
			.lkg_pwr = {69238, 69238, 69238, 69238, 69238, 69238} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
	},
};

struct upower_tbl upower_tbl_l_B24G = {
	.row = {
		{.cap = 147, .volt = 65000, .dyn_pwr = 15882,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 181, .volt = 65000, .dyn_pwr = 20646,
			.lkg_pwr = {24121, 24121, 24121, 24121, 24121, 24121} },
		{.cap = 193, .volt = 66250, .dyn_pwr = 23197,
			.lkg_pwr = {24952, 24952, 24952, 24952, 24952, 24952} },
		{.cap = 203, .volt = 67500, .dyn_pwr = 25896,
			.lkg_pwr = {25784, 25784, 25784, 25784, 25784, 25784} },
		{.cap = 221, .volt = 70000, .dyn_pwr = 31755,
			.lkg_pwr = {27447, 27447, 27447, 27447, 27447, 27447} },
		{.cap = 239, .volt = 72500, .dyn_pwr = 38252,
			.lkg_pwr = {29107, 29107, 29107, 29107, 29107, 29107} },
		{.cap = 249, .volt = 74375, .dyn_pwr = 43583,
			.lkg_pwr = {30353, 30353, 30353, 30353, 30353, 30353} },
		{.cap = 264, .volt = 76250, .dyn_pwr = 48809,
			.lkg_pwr = {31710, 31710, 31710, 31710, 31710, 31710} },
		{.cap = 285, .volt = 80000, .dyn_pwr = 60064,
			.lkg_pwr = {34537, 34537, 34537, 34537, 34537, 34537} },
		{.cap = 302, .volt = 82500, .dyn_pwr = 68188,
			.lkg_pwr = {36675, 36675, 36675, 36675, 36675, 36675} },
		{.cap = 310, .volt = 85000, .dyn_pwr = 76847,
			.lkg_pwr = {38812, 38812, 38812, 38812, 38812, 38812} },
		{.cap = 324, .volt = 89375, .dyn_pwr = 90570,
			.lkg_pwr = {42932, 42932, 42932, 42932, 42932, 42932} },
		{.cap = 335, .volt = 92500, .dyn_pwr = 101058,
			.lkg_pwr = {46122, 46122, 46122, 46122, 46122, 46122} },
		{.cap = 340, .volt = 94375, .dyn_pwr = 107555,
			.lkg_pwr = {48073, 48073, 48073, 48073, 48073, 48073} },
		{.cap = 350, .volt = 97500, .dyn_pwr = 118796,
			.lkg_pwr = {51703, 51703, 51703, 51703, 51703, 51703} },
		{.cap = 356, .volt = 100000, .dyn_pwr = 128148,
			.lkg_pwr = {54682, 54682, 54682, 54682, 54682, 54682} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
		{{0}, {24121} },
	},
};

struct upower_tbl upower_tbl_cluster_l_B24G = {
	.row = {
		{.cap = 147, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 181, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 193, .volt = 66250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 203, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 221, .volt = 70000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 239, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 249, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 264, .volt = 76250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 285, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 302, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 310, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 324, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 335, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 340, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 350, .volt = 97500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 356, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_B24G = {
	.row = {
		{.cap = 611, .volt = 65000, .dyn_pwr = 88354,
			.lkg_pwr = {95432, 95432, 95432, 95432, 95432, 95432} },
		{.cap = 667, .volt = 67500, .dyn_pwr = 110395,
			.lkg_pwr = {101909, 101909, 101909, 101909, 101909,
			101909} },
		{.cap = 697, .volt = 68750, .dyn_pwr = 122429,
			.lkg_pwr = {105148, 105148, 105148, 105148, 105148,
			105148} },
		{.cap = 732, .volt = 70625, .dyn_pwr = 141715,
			.lkg_pwr = {110005, 110005, 110005, 110005, 110005,
			110005} },
		{.cap = 754, .volt = 71875, .dyn_pwr = 155269,
			.lkg_pwr = {113241, 113241, 113241, 113241, 113241,
			113241} },
		{.cap = 753, .volt = 73750, .dyn_pwr = 176368,
			.lkg_pwr = {118096, 118096, 118096, 118096, 118096,
			118096} },
		{.cap = 797, .volt = 76875, .dyn_pwr = 214580,
			.lkg_pwr = {126803, 126803, 126803, 126803, 126803,
			126803} },
		{.cap = 836, .volt = 80000, .dyn_pwr = 256656,
			.lkg_pwr = {135922, 135922, 135922, 135922, 135922,
			135922} },
		{.cap = 864, .volt = 82500, .dyn_pwr = 293432,
			.lkg_pwr = {143381, 143381, 143381, 143381, 143381,
			143381} },
		{.cap = 893, .volt = 85000, .dyn_pwr = 332437,
			.lkg_pwr = {150841, 150841, 150841, 150841, 150841,
			150841} },
		{.cap = 931, .volt = 88750, .dyn_pwr = 395350,
			.lkg_pwr = {164235, 164235, 164235, 164235, 164235,
			164235} },
		{.cap = 952, .volt = 91875, .dyn_pwr = 451765,
			.lkg_pwr = {175600, 175600, 175600, 175600, 175600,
			175600} },
		{.cap = 978, .volt = 94375, .dyn_pwr = 499488,
			.lkg_pwr = {184799, 184799, 184799, 184799, 184799,
			184799} },
		{.cap = 995, .volt = 96250, .dyn_pwr = 536863,
			.lkg_pwr = {192560, 192560, 192560, 192560, 192560,
			192560} },
		{.cap = 1011, .volt = 98125, .dyn_pwr = 582221,
			.lkg_pwr = {200751, 200751, 200751, 200751, 200751,
			200751} },
		{.cap = 1024, .volt = 100000, .dyn_pwr = 629333,
			.lkg_pwr = {208942, 208942, 208942, 208942, 208942,
			208942} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
		{{0}, {95432} },
	},
};

struct upower_tbl upower_tbl_cluster_b_B24G = {
	.row = {
		{.cap = 611, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 667, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 697, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 732, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 754, .volt = 71875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 753, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 797, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 836, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 864, .volt = 82500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 893, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 931, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 952, .volt = 91875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 978, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 995, .volt = 96250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1011, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_B24G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 9264,
			.lkg_pwr = {30145, 30145, 30145, 30145, 30145, 30145} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 10415,
			.lkg_pwr = {31109, 31109, 31109, 31109, 31109, 31109} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 11655,
			.lkg_pwr = {32073, 32073, 32073, 32073, 32073, 32073} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 12943,
			.lkg_pwr = {33036, 33036, 33036, 33036, 33036, 33036} },
		{.cap = 0, .volt = 70000, .dyn_pwr = 14325,
			.lkg_pwr = {34000, 34000, 34000, 34000, 34000, 34000} },
		{.cap = 0, .volt = 72500, .dyn_pwr = 17288,
			.lkg_pwr = {35928, 35928, 35928, 35928, 35928, 35928} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 21402,
			.lkg_pwr = {38393, 38393, 38393, 38393, 38393, 38393} },
		{.cap = 0, .volt = 77500, .dyn_pwr = 24144,
			.lkg_pwr = {40008, 40008, 40008, 40008, 40008, 40008} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 28066,
			.lkg_pwr = {42160, 42160, 42160, 42160, 42160, 42160} },
		{.cap = 0, .volt = 82500, .dyn_pwr = 32335,
			.lkg_pwr = {44807, 44807, 44807, 44807, 44807, 44807} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 36965,
			.lkg_pwr = {47454, 47454, 47454, 47454, 47454, 47454} },
		{.cap = 0, .volt = 87500, .dyn_pwr = 41335,
			.lkg_pwr = {50427, 50427, 50427, 50427, 50427, 50427} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 44838,
			.lkg_pwr = {52657, 52657, 52657, 52657, 52657, 52657} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 53703,
			.lkg_pwr = {58667, 58667, 58667, 58667, 58667, 58667} },
		{.cap = 0, .volt = 98125, .dyn_pwr = 63618,
			.lkg_pwr = {65932, 65932, 65932, 65932, 65932, 65932} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 68216,
			.lkg_pwr = {69238, 69238, 69238, 69238, 69238, 69238} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
		{{0}, {30145} },
	},
};


#endif /* UNIFIED_POWER_DATA_H */

