/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019 MediaTek Inc.
*/

/* Mouton_TO_FY_v0p5__200217 */

/**********************************************
 * unified_power_data.h
 * This header file includes:
 * 1. Macros of SRAM related address
 * 2. Raw datas of unified power tables for each bank
 **********************************************/

#ifndef UNIFIED_POWER_DATA_H
#define UNIFIED_POWER_DATA_H

/* remember to sync to sspm upower */
#define UPOWER_CSRAM_BASE 0x0012a000
#define UPOWER_CSRAM_SIZE 0x3000 /* 12K bytes */
#define UPOWER_DVFS_OFF_BOTTOM 0x8 /* ignore the last 8 bytes */
/* limit should be at 0x12CFF4 */
#define UPOWER_TBL_LIMIT \
	((UPOWER_CSRAM_BASE)+(UPOWER_CSRAM_SIZE)-(UPOWER_DVFS_OFF_BOTTOM))

struct upower_tbl upower_tbl_l_FY = {
	.row = {
		{.cap = 119, .volt = 65000, .dyn_pwr = 18175,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 146, .volt = 65000, .dyn_pwr = 23628,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 160, .volt = 67500, .dyn_pwr = 28743,
			.lkg_pwr = {23553, 23553, 23553, 23553, 23553, 23553} },
		{.cap = 181, .volt = 70625, .dyn_pwr = 36858,
			.lkg_pwr = {25416, 25416, 25416, 25416, 25416, 25416} },
		{.cap = 190, .volt = 72500, .dyn_pwr = 41483,
			.lkg_pwr = {26649, 26649, 26649, 26649, 26649, 26649} },
		{.cap = 197, .volt = 73750, .dyn_pwr = 45302,
			.lkg_pwr = {27471, 27471, 27471, 27471, 27471, 27471} },
		{.cap = 205, .volt = 75625, .dyn_pwr = 50769,
			.lkg_pwr = {28758, 28758, 28758, 28758, 28758, 28758} },
		{.cap = 212, .volt = 78125, .dyn_pwr = 57700,
			.lkg_pwr = {30620, 30620, 30620, 30620, 30620, 30620} },
		{.cap = 216, .volt = 79375, .dyn_pwr = 61114,
			.lkg_pwr = {31551, 31551, 31551, 31551, 31551, 31551} },
		{.cap = 226, .volt = 81250, .dyn_pwr = 67005,
			.lkg_pwr = {33071, 33071, 33071, 33071, 33071, 33071} },
		{.cap = 234, .volt = 83125, .dyn_pwr = 72959,
			.lkg_pwr = {34653, 34653, 34653, 34653, 34653, 34653} },
		{.cap = 241, .volt = 85000, .dyn_pwr = 79007,
			.lkg_pwr = {36235, 36235, 36235, 36235, 36235, 36235} },
		{.cap = 249, .volt = 89375, .dyn_pwr = 90218,
			.lkg_pwr = {40404, 40404, 40404, 40404, 40404, 40404} },
		{.cap = 254, .volt = 92500, .dyn_pwr = 100289,
			.lkg_pwr = {43698, 43698, 43698, 43698, 43698, 43698} },
		{.cap = 260, .volt = 96875, .dyn_pwr = 114836,
			.lkg_pwr = {48686, 48686, 48686, 48686, 48686, 48686} },
		{.cap = 266, .volt = 100000, .dyn_pwr = 127065,
			.lkg_pwr = {52504, 52504, 52504, 52504, 52504, 52504} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
	},
};

struct upower_tbl upower_tbl_cluster_l_FY = {
	.row = {
		{.cap = 119, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 146, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 160, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 181, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 190, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 197, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 205, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 212, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 216, .volt = 79375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 226, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 234, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 241, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 249, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 254, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 260, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 266, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_FY = {
	.row = {
		{.cap = 430, .volt = 65000, .dyn_pwr = 164114,
			.lkg_pwr = {121191, 121191, 121191, 121191, 121191,
			121191} },
		{.cap = 477, .volt = 66875, .dyn_pwr = 194749,
			.lkg_pwr = {126991, 126991, 126991, 126991, 126991,
			126991} },
		{.cap = 557, .volt = 70625, .dyn_pwr = 258401,
			.lkg_pwr = {138839, 138839, 138839, 138839, 138839,
			138839} },
		{.cap = 612, .volt = 73125, .dyn_pwr = 308441,
			.lkg_pwr = {147566, 147566, 147566, 147566, 147566,
			147566} },
		{.cap = 652, .volt = 75000, .dyn_pwr = 348430,
			.lkg_pwr = {154111, 154111, 154111, 154111, 154111,
			154111} },
		{.cap = 715, .volt = 78125, .dyn_pwr = 416819,
			.lkg_pwr = {166409, 166409, 166409, 166409, 166409,
			166409} },
		{.cap = 762, .volt = 80625, .dyn_pwr = 474690,
			.lkg_pwr = {176561, 176561, 176561, 176561, 176561,
			176561} },
		{.cap = 798, .volt = 83125, .dyn_pwr = 530201,
			.lkg_pwr = {187654, 187654, 187654, 187654, 187654,
			187654} },
		{.cap = 834, .volt = 85000, .dyn_pwr = 581362,
			.lkg_pwr = {195974, 195974, 195974, 195974, 195974,
			195974} },
		{.cap = 870, .volt = 86875, .dyn_pwr = 637067,
			.lkg_pwr = {205359, 205359, 205359, 205359, 205359,
			205359} },
		{.cap = 894, .volt = 87500, .dyn_pwr = 668700,
			.lkg_pwr = {208487, 208487, 208487, 208487, 208487,
			208487} },
		{.cap = 916, .volt = 88750, .dyn_pwr = 710357,
			.lkg_pwr = {214743, 214743, 214743, 214743, 214743,
			214743} },
		{.cap = 950, .volt = 90000, .dyn_pwr = 762423,
			.lkg_pwr = {221000, 221000, 221000, 221000, 221000,
			221000} },
		{.cap = 975, .volt = 95000, .dyn_pwr = 876407,
			.lkg_pwr = {249216, 249216, 249216, 249216, 249216,
			249216} },
		{.cap = 997, .volt = 100000, .dyn_pwr = 998177,
			.lkg_pwr = {281041, 281041, 281041, 281041, 281041,
			281041} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1142000,
			.lkg_pwr = {316920, 316920, 316920, 316920, 316920,
			316920} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
	},
};

struct upower_tbl upower_tbl_cluster_b_FY = {
	.row = {
		{.cap = 430, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 477, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 557, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 612, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 652, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 715, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 762, .volt = 80625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 798, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 834, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 870, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 894, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 916, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 950, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 975, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 997, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_FY = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 10980,
			.lkg_pwr = {65338, 65338, 65338, 65338, 65338, 65338} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 12283,
			.lkg_pwr = {67479, 67479, 67479, 67479, 67479, 67479} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 13662,
			.lkg_pwr = {69621, 69621, 69621, 69621, 69621, 69621} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 15614,
			.lkg_pwr = {71762, 71762, 71762, 71762, 71762, 71762} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 18446,
			.lkg_pwr = {75114, 75114, 75114, 75114, 75114, 75114} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 22447,
			.lkg_pwr = {79961, 79961, 79961, 79961, 79961, 79961} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 24880,
			.lkg_pwr = {82385, 82385, 82385, 82385, 82385, 82385} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 27952,
			.lkg_pwr = {86339, 86339, 86339, 86339, 86339, 86339} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 31753,
			.lkg_pwr = {90452, 90452, 90452, 90452, 90452, 90452} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 35822,
			.lkg_pwr = {94565, 94565, 94565, 94565, 94565, 94565} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 43890,
			.lkg_pwr = {102320, 102320, 102320, 102320, 102320, 102320} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 48745,
			.lkg_pwr = {106973, 106973, 106973, 106973, 106973, 106973} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 57484,
			.lkg_pwr = {119247, 119247, 119247, 119247, 119247, 119247} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 67730,
			.lkg_pwr = {132899, 132899, 132899, 132899, 132899, 132899} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 76026,
			.lkg_pwr = {143600, 143600, 143600, 143600, 143600, 143600} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 84958,
			.lkg_pwr = {154825, 154825, 154825, 154825, 154825, 154825} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
	},
};

struct upower_tbl upower_tbl_l_B25G = {
	.row = {
		{.cap = 120, .volt = 65000, .dyn_pwr = 18175,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 148, .volt = 65000, .dyn_pwr = 23628,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 162, .volt = 67500, .dyn_pwr = 28743,
			.lkg_pwr = {23553, 23553, 23553, 23553, 23553, 23553} },
		{.cap = 184, .volt = 70625, .dyn_pwr = 36858,
			.lkg_pwr = {25416, 25416, 25416, 25416, 25416, 25416} },
		{.cap = 193, .volt = 72500, .dyn_pwr = 41483,
			.lkg_pwr = {26649, 26649, 26649, 26649, 26649, 26649} },
		{.cap = 200, .volt = 73750, .dyn_pwr = 45302,
			.lkg_pwr = {27471, 27471, 27471, 27471, 27471, 27471} },
		{.cap = 208, .volt = 75625, .dyn_pwr = 50769,
			.lkg_pwr = {28758, 28758, 28758, 28758, 28758, 28758} },
		{.cap = 215, .volt = 78125, .dyn_pwr = 57700,
			.lkg_pwr = {30620, 30620, 30620, 30620, 30620, 30620} },
		{.cap = 219, .volt = 79375, .dyn_pwr = 61114,
			.lkg_pwr = {31551, 31551, 31551, 31551, 31551, 31551} },
		{.cap = 228, .volt = 81250, .dyn_pwr = 67005,
			.lkg_pwr = {33071, 33071, 33071, 33071, 33071, 33071} },
		{.cap = 237, .volt = 83125, .dyn_pwr = 72959,
			.lkg_pwr = {34653, 34653, 34653, 34653, 34653, 34653} },
		{.cap = 245, .volt = 85000, .dyn_pwr = 79007,
			.lkg_pwr = {36235, 36235, 36235, 36235, 36235, 36235} },
		{.cap = 252, .volt = 89375, .dyn_pwr = 90218,
			.lkg_pwr = {40404, 40404, 40404, 40404, 40404, 40404} },
		{.cap = 257, .volt = 92500, .dyn_pwr = 100289,
			.lkg_pwr = {43698, 43698, 43698, 43698, 43698, 43698} },
		{.cap = 263, .volt = 96875, .dyn_pwr = 114836,
			.lkg_pwr = {48686, 48686, 48686, 48686, 48686, 48686} },
		{.cap = 269, .volt = 100000, .dyn_pwr = 127065,
			.lkg_pwr = {52504, 52504, 52504, 52504, 52504, 52504} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
	},
};

struct upower_tbl upower_tbl_cluster_l_B25G = {
	.row = {
		{.cap = 120, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 148, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 162, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 184, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 193, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 200, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 208, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 215, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 219, .volt = 79375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 228, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 237, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 245, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 252, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 257, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 263, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 269, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_B25G = {
	.row = {
		{.cap = 435, .volt = 65000, .dyn_pwr = 164114,
			.lkg_pwr = {121191, 121191, 121191, 121191, 121191,
			121191} },
		{.cap = 483, .volt = 66875, .dyn_pwr = 194749,
			.lkg_pwr = {126991, 126991, 126991, 126991, 126991,
			126991} },
		{.cap = 564, .volt = 70625, .dyn_pwr = 258401,
			.lkg_pwr = {138839, 138839, 138839, 138839, 138839,
			138839} },
		{.cap = 619, .volt = 73125, .dyn_pwr = 308441,
			.lkg_pwr = {147566, 147566, 147566, 147566, 147566,
			147566} },
		{.cap = 660, .volt = 75000, .dyn_pwr = 348430,
			.lkg_pwr = {154111, 154111, 154111, 154111, 154111,
			154111} },
		{.cap = 724, .volt = 78125, .dyn_pwr = 416819,
			.lkg_pwr = {166409, 166409, 166409, 166409, 166409,
			166409} },
		{.cap = 772, .volt = 80625, .dyn_pwr = 474690,
			.lkg_pwr = {176561, 176561, 176561, 176561, 176561,
			176561} },
		{.cap = 808, .volt = 83125, .dyn_pwr = 530201,
			.lkg_pwr = {187654, 187654, 187654, 187654, 187654,
			187654} },
		{.cap = 844, .volt = 85000, .dyn_pwr = 581362,
			.lkg_pwr = {195974, 195974, 195974, 195974, 195974,
			195974} },
		{.cap = 881, .volt = 86875, .dyn_pwr = 637067,
			.lkg_pwr = {205359, 205359, 205359, 205359, 205359,
			205359} },
		{.cap = 905, .volt = 87500, .dyn_pwr = 668700,
			.lkg_pwr = {208487, 208487, 208487, 208487, 208487,
			208487} },
		{.cap = 927, .volt = 88750, .dyn_pwr = 710357,
			.lkg_pwr = {214743, 214743, 214743, 214743, 214743,
			214743} },
		{.cap = 962, .volt = 90000, .dyn_pwr = 762423,
			.lkg_pwr = {221000, 221000, 221000, 221000, 221000,
			221000} },
		{.cap = 987, .volt = 95000, .dyn_pwr = 876407,
			.lkg_pwr = {249216, 249216, 249216, 249216, 249216,
			249216} },
		{.cap = 1009, .volt = 100000, .dyn_pwr = 998177,
			.lkg_pwr = {281041, 281041, 281041, 281041, 281041,
			281041} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1122210,
			.lkg_pwr = {316920, 316920, 316920, 316920, 316920,
			316920} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
	},
};

struct upower_tbl upower_tbl_cluster_b_B25G = {
	.row = {
		{.cap = 435, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 483, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 564, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 619, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 660, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 724, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 772, .volt = 80625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 808, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 844, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 881, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 905, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 927, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 962, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 987, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1009, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_B25G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 10980,
			.lkg_pwr = {65338, 65338, 65338, 65338, 65338, 65338} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 12283,
			.lkg_pwr = {67479, 67479, 67479, 67479, 67479, 67479} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 13662,
			.lkg_pwr = {69621, 69621, 69621, 69621, 69621, 69621} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 15614,
			.lkg_pwr = {71762, 71762, 71762, 71762, 71762, 71762} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 18446,
			.lkg_pwr = {75114, 75114, 75114, 75114, 75114, 75114} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 22447,
			.lkg_pwr = {79961, 79961, 79961, 79961, 79961, 79961} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 24880,
			.lkg_pwr = {82385, 82385, 82385, 82385, 82385, 82385} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 27952,
			.lkg_pwr = {86339, 86339, 86339, 86339, 86339, 86339} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 31753,
			.lkg_pwr = {90452, 90452, 90452, 90452, 90452, 90452} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 35822,
			.lkg_pwr = {94565, 94565, 94565, 94565, 94565, 94565} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 43890,
			.lkg_pwr = {102320, 102320, 102320, 102320, 102320, 102320} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 48745,
			.lkg_pwr = {106973, 106973, 106973, 106973, 106973, 106973} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 57484,
			.lkg_pwr = {119247, 119247, 119247, 119247, 119247, 119247} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 67730,
			.lkg_pwr = {132899, 132899, 132899, 132899, 132899, 132899} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 76026,
			.lkg_pwr = {143600, 143600, 143600, 143600, 143600, 143600} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 84958,
			.lkg_pwr = {154825, 154825, 154825, 154825, 154825, 154825} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
	},
};

struct upower_tbl upower_tbl_l_B24G = {
	.row = {
		{.cap = 122, .volt = 65000, .dyn_pwr = 18175,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 150, .volt = 65000, .dyn_pwr = 23628,
			.lkg_pwr = {22101, 22101, 22101, 22101, 22101, 22101} },
		{.cap = 164, .volt = 67500, .dyn_pwr = 28743,
			.lkg_pwr = {23553, 23553, 23553, 23553, 23553, 23553} },
		{.cap = 186, .volt = 70625, .dyn_pwr = 36858,
			.lkg_pwr = {25416, 25416, 25416, 25416, 25416, 25416} },
		{.cap = 195, .volt = 72500, .dyn_pwr = 41483,
			.lkg_pwr = {26649, 26649, 26649, 26649, 26649, 26649} },
		{.cap = 203, .volt = 73750, .dyn_pwr = 45302,
			.lkg_pwr = {27471, 27471, 27471, 27471, 27471, 27471} },
		{.cap = 211, .volt = 75625, .dyn_pwr = 50769,
			.lkg_pwr = {28758, 28758, 28758, 28758, 28758, 28758} },
		{.cap = 218, .volt = 78125, .dyn_pwr = 57700,
			.lkg_pwr = {30620, 30620, 30620, 30620, 30620, 30620} },
		{.cap = 222, .volt = 79375, .dyn_pwr = 61114,
			.lkg_pwr = {31551, 31551, 31551, 31551, 31551, 31551} },
		{.cap = 232, .volt = 81250, .dyn_pwr = 67005,
			.lkg_pwr = {33071, 33071, 33071, 33071, 33071, 33071} },
		{.cap = 240, .volt = 83125, .dyn_pwr = 72959,
			.lkg_pwr = {34653, 34653, 34653, 34653, 34653, 34653} },
		{.cap = 248, .volt = 85000, .dyn_pwr = 79007,
			.lkg_pwr = {36235, 36235, 36235, 36235, 36235, 36235} },
		{.cap = 255, .volt = 89375, .dyn_pwr = 90218,
			.lkg_pwr = {40404, 40404, 40404, 40404, 40404, 40404} },
		{.cap = 261, .volt = 92500, .dyn_pwr = 100289,
			.lkg_pwr = {43698, 43698, 43698, 43698, 43698, 43698} },
		{.cap = 267, .volt = 96875, .dyn_pwr = 114836,
			.lkg_pwr = {48686, 48686, 48686, 48686, 48686, 48686} },
		{.cap = 273, .volt = 100000, .dyn_pwr = 127065,
			.lkg_pwr = {52504, 52504, 52504, 52504, 52504, 52504} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
		{{0}, {22101} },
	},
};

struct upower_tbl upower_tbl_cluster_l_B24G = {
	.row = {
		{.cap = 122, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 150, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 164, .volt = 67500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 186, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 195, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 203, .volt = 73750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 211, .volt = 75625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 218, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 222, .volt = 79375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 232, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 240, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 248, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 255, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 261, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 267, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 273, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_B24G = {
	.row = {
		{.cap = 442, .volt = 65000, .dyn_pwr = 164114,
			.lkg_pwr = {121191, 121191, 121191, 121191, 121191,
			121191} },
		{.cap = 490, .volt = 66875, .dyn_pwr = 194749,
			.lkg_pwr = {126991, 126991, 126991, 126991, 126991,
			126991} },
		{.cap = 573, .volt = 70625, .dyn_pwr = 258401,
			.lkg_pwr = {138839, 138839, 138839, 138839, 138839,
			138839} },
		{.cap = 629, .volt = 73125, .dyn_pwr = 308441,
			.lkg_pwr = {147566, 147566, 147566, 147566, 147566,
			147566} },
		{.cap = 670, .volt = 75000, .dyn_pwr = 348430,
			.lkg_pwr = {154111, 154111, 154111, 154111, 154111,
			154111} },
		{.cap = 735, .volt = 78125, .dyn_pwr = 416819,
			.lkg_pwr = {166409, 166409, 166409, 166409, 166409,
			166409} },
		{.cap = 783, .volt = 80625, .dyn_pwr = 474690,
			.lkg_pwr = {176561, 176561, 176561, 176561, 176561,
			176561} },
		{.cap = 820, .volt = 83125, .dyn_pwr = 530201,
			.lkg_pwr = {187654, 187654, 187654, 187654, 187654,
			187654} },
		{.cap = 857, .volt = 85000, .dyn_pwr = 581362,
			.lkg_pwr = {195974, 195974, 195974, 195974, 195974,
			195974} },
		{.cap = 894, .volt = 86875, .dyn_pwr = 637067,
			.lkg_pwr = {205359, 205359, 205359, 205359, 205359,
			205359} },
		{.cap = 919, .volt = 87500, .dyn_pwr = 668700,
			.lkg_pwr = {208487, 208487, 208487, 208487, 208487,
			208487} },
		{.cap = 941, .volt = 88750, .dyn_pwr = 710357,
			.lkg_pwr = {214743, 214743, 214743, 214743, 214743,
			214743} },
		{.cap = 976, .volt = 90000, .dyn_pwr = 762423,
			.lkg_pwr = {221000, 221000, 221000, 221000, 221000,
			221000} },
		{.cap = 995, .volt = 95625, .dyn_pwr = 880592,
			.lkg_pwr = {253194, 253194, 253194, 253194, 253194,
			253194} },
		{.cap = 1010, .volt = 100625, .dyn_pwr = 993458,
			.lkg_pwr = {285525, 285525, 285525, 285525, 285525,
			285525} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1100490,
			.lkg_pwr = {316920, 316920, 316920, 316920, 316920,
			316920} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
		{{0}, {121191} },
	},
};

struct upower_tbl upower_tbl_cluster_b_B24G = {
	.row = {
		{.cap = 442, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 490, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 573, .volt = 70625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 629, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 670, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 735, .volt = 78125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 783, .volt = 80625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 820, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 857, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 894, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 919, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 941, .volt = 88750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 976, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 995, .volt = 95625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1010, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_B24G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 10980,
			.lkg_pwr = {65338, 65338, 65338, 65338, 65338, 65338} },
		{.cap = 0, .volt = 66250, .dyn_pwr = 12283,
			.lkg_pwr = {67479, 67479, 67479, 67479, 67479, 67479} },
		{.cap = 0, .volt = 67500, .dyn_pwr = 13662,
			.lkg_pwr = {69621, 69621, 69621, 69621, 69621, 69621} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 15614,
			.lkg_pwr = {71762, 71762, 71762, 71762, 71762, 71762} },
		{.cap = 0, .volt = 70625, .dyn_pwr = 18446,
			.lkg_pwr = {75114, 75114, 75114, 75114, 75114, 75114} },
		{.cap = 0, .volt = 73125, .dyn_pwr = 22447,
			.lkg_pwr = {79961, 79961, 79961, 79961, 79961, 79961} },
		{.cap = 0, .volt = 74375, .dyn_pwr = 24880,
			.lkg_pwr = {82385, 82385, 82385, 82385, 82385, 82385} },
		{.cap = 0, .volt = 76250, .dyn_pwr = 27952,
			.lkg_pwr = {86339, 86339, 86339, 86339, 86339, 86339} },
		{.cap = 0, .volt = 78125, .dyn_pwr = 31753,
			.lkg_pwr = {90452, 90452, 90452, 90452, 90452, 90452} },
		{.cap = 0, .volt = 80000, .dyn_pwr = 35822,
			.lkg_pwr = {94565, 94565, 94565, 94565, 94565, 94565} },
		{.cap = 0, .volt = 83125, .dyn_pwr = 43890,
			.lkg_pwr = {102320, 102320, 102320, 102320, 102320, 102320} },
		{.cap = 0, .volt = 85000, .dyn_pwr = 48745,
			.lkg_pwr = {106973, 106973, 106973, 106973, 106973, 106973} },
		{.cap = 0, .volt = 89375, .dyn_pwr = 57484,
			.lkg_pwr = {119247, 119247, 119247, 119247, 119247, 119247} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 67730,
			.lkg_pwr = {132899, 132899, 132899, 132899, 132899, 132899} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 76026,
			.lkg_pwr = {143600, 143600, 143600, 143600, 143600, 143600} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 84958,
			.lkg_pwr = {154825, 154825, 154825, 154825, 154825, 154825} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
		{{0}, {65338} },
	},
};


#endif /* UNIFIED_POWER_DATA_H */

