/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2021 MediaTek Inc.
 */

/* Petrus_v1p3_2k6_game_MP_190815 */

/**********************************************
 * unified_power_data.h
 * This header file includes:
 * 1. Macros of SRAM related address
 * 2. Raw datas of unified power tables for each bank
 **********************************************/

#ifndef UNIFIED_POWER_DATA_H
#define UNIFIED_POWER_DATA_H

/* remember to sync to sspm upower */
#define UPOWER_CSRAM_BASE 0x0012a000
#define UPOWER_CSRAM_SIZE 0x3000 /* 12K bytes */
#define UPOWER_DVFS_OFF_BOTTOM 0x8 /* ignore the last 8 bytes */
/* limit should be at 0x12CFF4 */
#define UPOWER_TBL_LIMIT \
	((UPOWER_CSRAM_BASE)+(UPOWER_CSRAM_SIZE)-(UPOWER_DVFS_OFF_BOTTOM))

struct upower_tbl upower_tbl_l_26G26G = {
	.row = {
		{.cap = 108, .volt = 65000, .dyn_pwr = 14987,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 124, .volt = 65000, .dyn_pwr = 17984,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 135, .volt = 66875, .dyn_pwr = 21416,
			.lkg_pwr = {24276, 24276, 24276, 24276, 24276, 24276} },
		{.cap = 144, .volt = 68750, .dyn_pwr = 25149,
			.lkg_pwr = {25421, 25421, 25421, 25421, 25421, 25421} },
		{.cap = 156, .volt = 71250, .dyn_pwr = 30613,
			.lkg_pwr = {27048, 27048, 27048, 27048, 27048, 27048} },
		{.cap = 163, .volt = 73125, .dyn_pwr = 35090,
			.lkg_pwr = {28344, 28344, 28344, 28344, 28344, 28344} },
		{.cap = 170, .volt = 75000, .dyn_pwr = 39906,
			.lkg_pwr = {29641, 29641, 29641, 29641, 29641, 29641} },
		{.cap = 178, .volt = 76875, .dyn_pwr = 44917,
			.lkg_pwr = {31108, 31108, 31108, 31108, 31108, 31108} },
		{.cap = 194, .volt = 81250, .dyn_pwr = 57055,
			.lkg_pwr = {34661, 34661, 34661, 34661, 34661, 34661} },
		{.cap = 202, .volt = 83750, .dyn_pwr = 64626,
			.lkg_pwr = {36875, 36875, 36875, 36875, 36875, 36875} },
		{.cap = 208, .volt = 86250, .dyn_pwr = 72657,
			.lkg_pwr = {39237, 39237, 39237, 39237, 39237, 39237} },
		{.cap = 213, .volt = 88125, .dyn_pwr = 78864,
			.lkg_pwr = {41118, 41118, 41118, 41118, 41118, 41118} },
		{.cap = 220, .volt = 90625, .dyn_pwr = 87149,
			.lkg_pwr = {43709, 43709, 43709, 43709, 43709, 43709} },
		{.cap = 227, .volt = 93125, .dyn_pwr = 95757,
			.lkg_pwr = {46544, 46544, 46544, 46544, 46544, 46544} },
		{.cap = 231, .volt = 95000, .dyn_pwr = 102413,
			.lkg_pwr = {48670, 48670, 48670, 48670, 48670, 48670} },
		{.cap = 241, .volt = 100000, .dyn_pwr = 120926,
			.lkg_pwr = {55099, 55099, 55099, 55099, 55099, 55099} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
	},
};

struct upower_tbl upower_tbl_cluster_l_26G26G = {
	.row = {
		{.cap = 108, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 124, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 135, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 144, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 156, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 163, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 170, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 178, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 194, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 202, .volt = 83750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 208, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 213, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 220, .volt = 90625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 227, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 231, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 241, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_bl_26G26G = {
	.row = {
		{.cap = 324, .volt = 65000, .dyn_pwr = 55814,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586,
				120586} },
		{.cap = 477, .volt = 65000, .dyn_pwr = 89404,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586,
				120586} },
		{.cap = 558, .volt = 69375, .dyn_pwr = 131234,
			.lkg_pwr = {134072, 134072, 134072, 134072, 134072,
				134072} },
		{.cap = 606, .volt = 72500, .dyn_pwr = 166204,
			.lkg_pwr = {144686, 144686, 144686, 144686, 144686,
				144686} },
		{.cap = 645, .volt = 75000, .dyn_pwr = 196399,
			.lkg_pwr = {153374, 153374, 153374, 153374, 153374,
				153374} },
		{.cap = 693, .volt = 78750, .dyn_pwr = 246482,
			.lkg_pwr = {168070, 168070, 168070, 168070, 168070,
				168070} },
		{.cap = 719, .volt = 81250, .dyn_pwr = 283405,
			.lkg_pwr = {178495, 178495, 178495, 178495, 178495,
				178495} },
		{.cap = 738, .volt = 83125, .dyn_pwr = 312760,
			.lkg_pwr = {186783, 186783, 186783, 186783, 186783,
				186783} },
		{.cap = 760, .volt = 84375, .dyn_pwr = 338836,
			.lkg_pwr = {192309, 192309, 192309, 192309, 192309,
				192309} },
		{.cap = 785, .volt = 86875, .dyn_pwr = 380067,
			.lkg_pwr = {204420, 204420, 204420, 204420, 204420,
				204420} },
		{.cap = 807, .volt = 89375, .dyn_pwr = 423468,
			.lkg_pwr = {216884, 216884, 216884, 216884, 216884,
				216884} },
		{.cap = 827, .volt = 91875, .dyn_pwr = 471982,
			.lkg_pwr = {230541, 230541, 230541, 230541, 230541,
				230541} },
		{.cap = 865, .volt = 96875, .dyn_pwr = 567591,
			.lkg_pwr = {260000, 260000, 260000, 260000, 260000,
				260000} },
		{.cap = 886, .volt = 100000, .dyn_pwr = 646914,
			.lkg_pwr = {279817, 279817, 279817, 279817, 279817,
				279817} },
		{.cap = 903, .volt = 103125, .dyn_pwr = 732694,
			.lkg_pwr = {302162, 302162, 302162, 302162, 302162,
				302162} },
		{.cap = 917, .volt = 105000, .dyn_pwr = 787757,
			.lkg_pwr = {315569, 315569, 315569, 315569, 315569,
				315569} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
	},
};

struct upower_tbl upower_tbl_cluster_bl_26G26G = {
	.row = {
		{.cap = 324, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 477, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 558, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 606, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 645, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 693, .volt = 78750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 719, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 738, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 760, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 785, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 807, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 827, .volt = 91875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 865, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 886, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 903, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 917, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_26G26G = {
	.row = {
		{.cap = 511, .volt = 65000, .dyn_pwr = 112299,
			.lkg_pwr = {171962, 171962, 171962, 171962, 171962,
				171962} },
		{.cap = 631, .volt = 69375, .dyn_pwr = 178784,
			.lkg_pwr = {189678, 189678, 189678, 189678, 189678,
				189678} },
		{.cap = 701, .volt = 72500, .dyn_pwr = 234154,
			.lkg_pwr = {203523, 203523, 203523, 203523, 203523,
				203523} },
		{.cap = 753, .volt = 75000, .dyn_pwr = 282245,
			.lkg_pwr = {214837, 214837, 214837, 214837, 214837,
				214837} },
		{.cap = 784, .volt = 77500, .dyn_pwr = 326244,
			.lkg_pwr = {227484, 227484, 227484, 227484, 227484,
				227484} },
		{.cap = 810, .volt = 80000, .dyn_pwr = 373789,
			.lkg_pwr = {240131, 240131, 240131, 240131, 240131,
				240131} },
		{.cap = 851, .volt = 83125, .dyn_pwr = 440571,
			.lkg_pwr = {257796, 257796, 257796, 257796, 257796,
				257796} },
		{.cap = 879, .volt = 85625, .dyn_pwr = 493756,
			.lkg_pwr = {272345, 272345, 272345, 272345, 272345,
				272345} },
		{.cap = 895, .volt = 87500, .dyn_pwr = 533381,
			.lkg_pwr = {284197, 284197, 284197, 284197, 284197,
				284197} },
		{.cap = 917, .volt = 90000, .dyn_pwr = 593088,
			.lkg_pwr = {300000, 300000, 300000, 300000, 300000,
				300000} },
		{.cap = 934, .volt = 92500, .dyn_pwr = 650006,
			.lkg_pwr = {317660, 317660, 317660, 317660, 317660,
				317660} },
		{.cap = 953, .volt = 94375, .dyn_pwr = 700686,
			.lkg_pwr = {330905, 330905, 330905, 330905, 330905,
				330905} },
		{.cap = 981, .volt = 98125, .dyn_pwr = 809180,
			.lkg_pwr = {359991, 359991, 359991, 359991, 359991,
				359991} },
		{.cap = 998, .volt = 100625, .dyn_pwr = 894747,
			.lkg_pwr = {380309, 380309, 380309, 380309, 380309,
				380309} },
		{.cap = 1013, .volt = 103125, .dyn_pwr = 986162,
			.lkg_pwr = {402369, 402369, 402369, 402369, 402369,
				402369} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1051050,
			.lkg_pwr = {418914, 418914, 418914, 418914, 418914,
				418914} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
	},
};

struct upower_tbl upower_tbl_cluster_b_26G26G = {
	.row = {
		{.cap = 511, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 631, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 701, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 753, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 784, .volt = 77500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 810, .volt = 80000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 851, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 879, .volt = 85625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 895, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 917, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 934, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 953, .volt = 94375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 981, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 998, .volt = 100625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1013, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_26G26G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 33904,
			.lkg_pwr = {55244, 55244, 55244, 55244, 55244, 55244} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 45734,
			.lkg_pwr = {60466, 60466, 60466, 60466, 60466, 60466} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 57081,
			.lkg_pwr = {65146, 65146, 65146, 65146, 65146, 65146} },
		{.cap = 0, .volt = 73750, .dyn_pwr = 64631,
			.lkg_pwr = {68086, 68086, 68086, 68086, 68086, 68086} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 72725,
			.lkg_pwr = {71149, 71149, 71149, 71149, 71149, 71149} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 85654,
			.lkg_pwr = {76665, 76665, 76665, 76665, 76665, 76665} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 96680,
			.lkg_pwr = {81355, 81355, 81355, 81355, 81355, 81355} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 110515,
			.lkg_pwr = {86322, 86322, 86322, 86322, 86322, 86322} },
		{.cap = 0, .volt = 86875, .dyn_pwr = 127185,
			.lkg_pwr = {93004, 93004, 93004, 93004, 93004, 93004} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 149809,
			.lkg_pwr = {101576, 101576, 101576, 101576, 101576,
				101576} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 170220,
			.lkg_pwr = {109453, 109453, 109453, 109453, 109453,
				109453} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 179386,
			.lkg_pwr = {112605, 112605, 112605, 112605, 112605,
				112605} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 191461,
			.lkg_pwr = {117924, 117924, 117924, 117924, 117924,
				117924} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 216049,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 237654,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 262346,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
	},
};

struct upower_tbl upower_tbl_l_30G26G = {
	.row = {
		{.cap = 102, .volt = 65000, .dyn_pwr = 14987,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 117, .volt = 65000, .dyn_pwr = 17984,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 128, .volt = 66875, .dyn_pwr = 21416,
			.lkg_pwr = {24276, 24276, 24276, 24276, 24276, 24276} },
		{.cap = 138, .volt = 68750, .dyn_pwr = 25149,
			.lkg_pwr = {25421, 25421, 25421, 25421, 25421, 25421} },
		{.cap = 150, .volt = 71250, .dyn_pwr = 30613,
			.lkg_pwr = {27048, 27048, 27048, 27048, 27048, 27048} },
		{.cap = 158, .volt = 73125, .dyn_pwr = 35090,
			.lkg_pwr = {28344, 28344, 28344, 28344, 28344, 28344} },
		{.cap = 164, .volt = 75000, .dyn_pwr = 39906,
			.lkg_pwr = {29641, 29641, 29641, 29641, 29641, 29641} },
		{.cap = 171, .volt = 76875, .dyn_pwr = 44917,
			.lkg_pwr = {31108, 31108, 31108, 31108, 31108, 31108} },
		{.cap = 181, .volt = 81250, .dyn_pwr = 57055,
			.lkg_pwr = {34661, 34661, 34661, 34661, 34661, 34661} },
		{.cap = 188, .volt = 83750, .dyn_pwr = 64626,
			.lkg_pwr = {36875, 36875, 36875, 36875, 36875, 36875} },
		{.cap = 196, .volt = 86250, .dyn_pwr = 72657,
			.lkg_pwr = {39237, 39237, 39237, 39237, 39237, 39237} },
		{.cap = 202, .volt = 88125, .dyn_pwr = 78864,
			.lkg_pwr = {41118, 41118, 41118, 41118, 41118, 41118} },
		{.cap = 208, .volt = 90625, .dyn_pwr = 87149,
			.lkg_pwr = {43709, 43709, 43709, 43709, 43709, 43709} },
		{.cap = 214, .volt = 93125, .dyn_pwr = 95757,
			.lkg_pwr = {46544, 46544, 46544, 46544, 46544, 46544} },
		{.cap = 217, .volt = 95000, .dyn_pwr = 102413,
			.lkg_pwr = {48670, 48670, 48670, 48670, 48670, 48670} },
		{.cap = 227, .volt = 100000, .dyn_pwr = 120926,
			.lkg_pwr = {55099, 55099, 55099, 55099, 55099, 55099} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
	},
};

struct upower_tbl upower_tbl_cluster_l_30G26G = {
	.row = {
		{.cap = 102, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 117, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 128, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 138, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 150, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 158, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 164, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 171, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 181, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 188, .volt = 83750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 196, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 202, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 208, .volt = 90625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 214, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 217, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 227, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_bl_30G26G = {
	.row = {
		{.cap = 267, .volt = 65000, .dyn_pwr = 85412,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586, 120586} },
		{.cap = 390, .volt = 65000, .dyn_pwr = 136815,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586, 120586} },
		{.cap = 474, .volt = 69375, .dyn_pwr = 191558,
			.lkg_pwr = {134072, 134072, 134072, 134072, 134072, 134072} },
		{.cap = 525, .volt = 72500, .dyn_pwr = 234235,
			.lkg_pwr = {144686, 144686, 144686, 144686, 144686, 144686} },
		{.cap = 565, .volt = 75000, .dyn_pwr = 270452,
			.lkg_pwr = {153374, 153374, 153374, 153374, 153374, 153374} },
		{.cap = 623, .volt = 78750, .dyn_pwr = 327427,
			.lkg_pwr = {168070, 168070, 168070, 168070, 168070, 168070} },
		{.cap = 658, .volt = 81250, .dyn_pwr = 367087,
			.lkg_pwr = {178495, 178495, 178495, 178495, 178495, 178495} },
		{.cap = 682, .volt = 83125, .dyn_pwr = 397344,
			.lkg_pwr = {186783, 186783, 186783, 186783, 186783, 186783} },
		{.cap = 704, .volt = 84375, .dyn_pwr = 421925,
			.lkg_pwr = {192309, 192309, 192309, 192309, 192309, 192309} },
		{.cap = 732, .volt = 86875, .dyn_pwr = 463147,
			.lkg_pwr = {204420, 204420, 204420, 204420, 204420, 204420} },
		{.cap = 756, .volt = 89375, .dyn_pwr = 515323,
			.lkg_pwr = {216884, 216884, 216884, 216884, 216884, 216884} },
		{.cap = 781, .volt = 91875, .dyn_pwr = 573438,
			.lkg_pwr = {230541, 230541, 230541, 230541, 230541, 230541} },
		{.cap = 822, .volt = 96875, .dyn_pwr = 687693,
			.lkg_pwr = {260000, 260000, 260000, 260000, 260000, 260000} },
		{.cap = 846, .volt = 100000, .dyn_pwr = 768624,
			.lkg_pwr = {279817, 279817, 279817, 279817, 279817, 279817} },
		{.cap = 866, .volt = 103125, .dyn_pwr = 853162,
			.lkg_pwr = {302162, 302162, 302162, 302162, 302162, 302162} },
		{.cap = 876, .volt = 105000, .dyn_pwr = 905920,
			.lkg_pwr = {315569, 315569, 315569, 315569, 315569, 315569} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
	},
};

struct upower_tbl upower_tbl_cluster_bl_30G26G = {
	.row = {
		{.cap = 267, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 390, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 474, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 525, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 565, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 623, .volt = 78750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 658, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 682, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 704, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 732, .volt = 86875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 756, .volt = 89375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 781, .volt = 91875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 822, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 846, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 866, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 876, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_30G26G = {
	.row = {
		{.cap = 415, .volt = 65000, .dyn_pwr = 178658,
			.lkg_pwr = {171962, 171962, 171962, 171962, 171962, 171962} },
		{.cap = 537, .volt = 69375, .dyn_pwr = 270370,
			.lkg_pwr = {189678, 189678, 189678, 189678, 189678, 189678} },
		{.cap = 610, .volt = 72500, .dyn_pwr = 342044,
			.lkg_pwr = {203523, 203523, 203523, 203523, 203523, 203523} },
		{.cap = 668, .volt = 75000, .dyn_pwr = 402744,
			.lkg_pwr = {214837, 214837, 214837, 214837, 214837, 214837} },
		{.cap = 708, .volt = 76875, .dyn_pwr = 449827,
			.lkg_pwr = {224323, 224323, 224323, 224323, 224323, 224323} },
		{.cap = 745, .volt = 78750, .dyn_pwr = 498165,
			.lkg_pwr = {233808, 233808, 233808, 233808, 233808, 233808} },
		{.cap = 789, .volt = 81250, .dyn_pwr = 564406,
			.lkg_pwr = {247197, 247197, 247197, 247197, 247197, 247197} },
		{.cap = 836, .volt = 84375, .dyn_pwr = 649281,
			.lkg_pwr = {264862, 264862, 264862, 264862, 264862, 264862} },
		{.cap = 875, .volt = 87500, .dyn_pwr = 740959,
			.lkg_pwr = {284197, 284197, 284197, 284197, 284197, 284197} },
		{.cap = 906, .volt = 90000, .dyn_pwr = 816694,
			.lkg_pwr = {300000, 300000, 300000, 300000, 300000, 300000} },
		{.cap = 933, .volt = 92500, .dyn_pwr = 894043,
			.lkg_pwr = {317660, 317660, 317660, 317660, 317660, 317660} },
		{.cap = 961, .volt = 95625, .dyn_pwr = 992451,
			.lkg_pwr = {340254, 340254, 340254, 340254, 340254, 340254} },
		{.cap = 978, .volt = 98125, .dyn_pwr = 1070900,
			.lkg_pwr = {359991, 359991, 359991, 359991, 359991, 359991} },
		{.cap = 994, .volt = 100000, .dyn_pwr = 1127780,
			.lkg_pwr = {374794, 374794, 374794, 374794, 374794, 374794} },
		{.cap = 1014, .volt = 103125, .dyn_pwr = 1219620,
			.lkg_pwr = {402369, 402369, 402369, 402369, 402369, 402369} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1273390,
			.lkg_pwr = {418914, 418914, 418914, 418914, 418914, 418914} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
	},
};

struct upower_tbl upower_tbl_cluster_b_30G26G = {
	.row = {
		{.cap = 415, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 537, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 610, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 668, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 708, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 745, .volt = 78750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 789, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 836, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 875, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 906, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 933, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 961, .volt = 95625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 978, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 994, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1014, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_30G26G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 33904,
			.lkg_pwr = {55244, 55244, 55244, 55244, 55244, 55244} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 45734,
			.lkg_pwr = {60466, 60466, 60466, 60466, 60466, 60466} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 57081,
			.lkg_pwr = {65146, 65146, 65146, 65146, 65146, 65146} },
		{.cap = 0, .volt = 73750, .dyn_pwr = 64631,
			.lkg_pwr = {68086, 68086, 68086, 68086, 68086, 68086} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 72725,
			.lkg_pwr = {71149, 71149, 71149, 71149, 71149, 71149} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 85654,
			.lkg_pwr = {76665, 76665, 76665, 76665, 76665, 76665} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 96680,
			.lkg_pwr = {81355, 81355, 81355, 81355, 81355, 81355} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 110515,
			.lkg_pwr = {86322, 86322, 86322, 86322, 86322, 86322} },
		{.cap = 0, .volt = 86875, .dyn_pwr = 127185,
			.lkg_pwr = {93004, 93004, 93004, 93004, 93004, 93004} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 149809,
			.lkg_pwr = {101576, 101576, 101576, 101576, 101576, 101576} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 170220,
			.lkg_pwr = {109453, 109453, 109453, 109453, 109453, 109453} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 179386,
			.lkg_pwr = {112605, 112605, 112605, 112605, 112605, 112605} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 191461,
			.lkg_pwr = {117924, 117924, 117924, 117924, 117924, 117924} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 216049,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791, 126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 237654,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791, 126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 262346,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791, 126791} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
	},
};

struct upower_tbl upower_tbl_l_30G28G = {
	.row = {
		{.cap = 103, .volt = 65000, .dyn_pwr = 14987,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 118, .volt = 65000, .dyn_pwr = 17984,
			.lkg_pwr = {23131, 23131, 23131, 23131, 23131, 23131} },
		{.cap = 129, .volt = 66875, .dyn_pwr = 21416,
			.lkg_pwr = {24276, 24276, 24276, 24276, 24276, 24276} },
		{.cap = 138, .volt = 68750, .dyn_pwr = 25149,
			.lkg_pwr = {25421, 25421, 25421, 25421, 25421, 25421} },
		{.cap = 149, .volt = 71250, .dyn_pwr = 30613,
			.lkg_pwr = {27048, 27048, 27048, 27048, 27048, 27048} },
		{.cap = 156, .volt = 73125, .dyn_pwr = 35090,
			.lkg_pwr = {28344, 28344, 28344, 28344, 28344, 28344} },
		{.cap = 162, .volt = 75000, .dyn_pwr = 39906,
			.lkg_pwr = {29641, 29641, 29641, 29641, 29641, 29641} },
		{.cap = 170, .volt = 76875, .dyn_pwr = 44917,
			.lkg_pwr = {31108, 31108, 31108, 31108, 31108, 31108} },
		{.cap = 186, .volt = 81250, .dyn_pwr = 57055,
			.lkg_pwr = {34661, 34661, 34661, 34661, 34661, 34661} },
		{.cap = 193, .volt = 83750, .dyn_pwr = 64626,
			.lkg_pwr = {36875, 36875, 36875, 36875, 36875, 36875} },
		{.cap = 199, .volt = 86250, .dyn_pwr = 72657,
			.lkg_pwr = {39237, 39237, 39237, 39237, 39237, 39237} },
		{.cap = 204, .volt = 88125, .dyn_pwr = 78864,
			.lkg_pwr = {41118, 41118, 41118, 41118, 41118, 41118} },
		{.cap = 211, .volt = 90625, .dyn_pwr = 87149,
			.lkg_pwr = {43709, 43709, 43709, 43709, 43709, 43709} },
		{.cap = 217, .volt = 93125, .dyn_pwr = 95757,
			.lkg_pwr = {46544, 46544, 46544, 46544, 46544, 46544} },
		{.cap = 221, .volt = 95000, .dyn_pwr = 102413,
			.lkg_pwr = {48670, 48670, 48670, 48670, 48670, 48670} },
		{.cap = 230, .volt = 100000, .dyn_pwr = 120926,
			.lkg_pwr = {55099, 55099, 55099, 55099, 55099, 55099} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
		{{0}, {23131} },
	},
};

struct upower_tbl upower_tbl_cluster_l_30G28G = {
	.row = {
		{.cap = 103, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 118, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 129, .volt = 66875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 138, .volt = 68750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 149, .volt = 71250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 156, .volt = 73125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 162, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 170, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 186, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 193, .volt = 83750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 199, .volt = 86250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 204, .volt = 88125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 211, .volt = 90625, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 217, .volt = 93125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 221, .volt = 95000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 230, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_bl_30G28G = {
	.row = {
		{.cap = 310, .volt = 65000, .dyn_pwr = 55814,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586,
				120586} },
		{.cap = 456, .volt = 65000, .dyn_pwr = 89404,
			.lkg_pwr = {120586, 120586, 120586, 120586, 120586,
				120586} },
		{.cap = 542, .volt = 69375, .dyn_pwr = 134871,
			.lkg_pwr = {134072, 134072, 134072, 134072, 134072,
				134072} },
		{.cap = 595, .volt = 72500, .dyn_pwr = 172811,
			.lkg_pwr = {144686, 144686, 144686, 144686, 144686,
				144686} },
		{.cap = 625, .volt = 74375, .dyn_pwr = 197033,
			.lkg_pwr = {151202, 151202, 151202, 151202, 151202,
				151202} },
		{.cap = 681, .volt = 78750, .dyn_pwr = 260132,
			.lkg_pwr = {168070, 168070, 168070, 168070, 168070,
				168070} },
		{.cap = 708, .volt = 81250, .dyn_pwr = 300235,
			.lkg_pwr = {178495, 178495, 178495, 178495, 178495,
				178495} },
		{.cap = 731, .volt = 83125, .dyn_pwr = 331872,
			.lkg_pwr = {186783, 186783, 186783, 186783, 186783,
				186783} },
		{.cap = 751, .volt = 85000, .dyn_pwr = 363838,
			.lkg_pwr = {195072, 195072, 195072, 195072, 195072,
				195072} },
		{.cap = 775, .volt = 87500, .dyn_pwr = 408492,
			.lkg_pwr = {207536, 207536, 207536, 207536, 207536,
				207536} },
		{.cap = 794, .volt = 90000, .dyn_pwr = 455570,
			.lkg_pwr = {220000, 220000, 220000, 220000, 220000,
				220000} },
		{.cap = 818, .volt = 92500, .dyn_pwr = 506296,
			.lkg_pwr = {234055, 234055, 234055, 234055, 234055,
				234055} },
		{.cap = 850, .volt = 96875, .dyn_pwr = 610982,
			.lkg_pwr = {260000, 260000, 260000, 260000, 260000,
				260000} },
		{.cap = 877, .volt = 101250, .dyn_pwr = 732493,
			.lkg_pwr = {288755, 288755, 288755, 288755, 288755,
				288755} },
		{.cap = 889, .volt = 103125, .dyn_pwr = 789100,
			.lkg_pwr = {302162, 302162, 302162, 302162, 302162,
				302162} },
		{.cap = 900, .volt = 105000, .dyn_pwr = 848353,
			.lkg_pwr = {315569, 315569, 315569, 315569, 315569,
				315569} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
		{{0}, {120586} },
	},
};

struct upower_tbl upower_tbl_cluster_bl_30G28G = {
	.row = {
		{.cap = 310, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 456, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 542, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 595, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 625, .volt = 74375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 681, .volt = 78750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 708, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 731, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 751, .volt = 85000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 775, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 794, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 818, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 850, .volt = 96875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 877, .volt = 101250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 889, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 900, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_b_30G28G = {
	.row = {
		{.cap = 489, .volt = 65000, .dyn_pwr = 112299,
			.lkg_pwr = {171962, 171962, 171962, 171962, 171962,
				171962} },
		{.cap = 604, .volt = 69375, .dyn_pwr = 178784,
			.lkg_pwr = {189678, 189678, 189678, 189678, 189678,
				189678} },
		{.cap = 671, .volt = 72500, .dyn_pwr = 234154,
			.lkg_pwr = {203523, 203523, 203523, 203523, 203523,
				203523} },
		{.cap = 720, .volt = 75000, .dyn_pwr = 282245,
			.lkg_pwr = {214837, 214837, 214837, 214837, 214837,
				214837} },
		{.cap = 750, .volt = 76875, .dyn_pwr = 321003,
			.lkg_pwr = {224323, 224323, 224323, 224323, 224323,
				224323} },
		{.cap = 775, .volt = 78750, .dyn_pwr = 362199,
			.lkg_pwr = {233808, 233808, 233808, 233808, 233808,
				233808} },
		{.cap = 814, .volt = 81250, .dyn_pwr = 420920,
			.lkg_pwr = {247197, 247197, 247197, 247197, 247197,
				247197} },
		{.cap = 841, .volt = 83125, .dyn_pwr = 465344,
			.lkg_pwr = {257796, 257796, 257796, 257796, 257796,
				257796} },
		{.cap = 856, .volt = 84375, .dyn_pwr = 495963,
			.lkg_pwr = {264862, 264862, 264862, 264862, 264862,
				264862} },
		{.cap = 886, .volt = 87500, .dyn_pwr = 574091,
			.lkg_pwr = {284197, 284197, 284197, 284197, 284197,
				284197} },
		{.cap = 914, .volt = 90000, .dyn_pwr = 640401,
			.lkg_pwr = {300000, 300000, 300000, 300000, 300000,
				300000} },
		{.cap = 938, .volt = 92500, .dyn_pwr = 716557,
			.lkg_pwr = {317660, 317660, 317660, 317660, 317660,
				317660} },
		{.cap = 980, .volt = 98125, .dyn_pwr = 917918,
			.lkg_pwr = {359991, 359991, 359991, 359991, 359991,
				359991} },
		{.cap = 995, .volt = 100000, .dyn_pwr = 994767,
			.lkg_pwr = {374794, 374794, 374794, 374794, 374794,
				374794} },
		{.cap = 1015, .volt = 103125, .dyn_pwr = 1127710,
			.lkg_pwr = {402369, 402369, 402369, 402369, 402369,
				402369} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 1212750,
			.lkg_pwr = {418914, 418914, 418914, 418914, 418914,
				418914} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
		{{0}, {171962} },
	},
};

struct upower_tbl upower_tbl_cluster_b_30G28G = {
	.row = {
		{.cap = 489, .volt = 65000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 604, .volt = 69375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 671, .volt = 72500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 720, .volt = 75000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 750, .volt = 76875, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 775, .volt = 78750, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 814, .volt = 81250, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 841, .volt = 83125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 856, .volt = 84375, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 886, .volt = 87500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 914, .volt = 90000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 938, .volt = 92500, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 980, .volt = 98125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 995, .volt = 100000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1015, .volt = 103125, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
		{.cap = 1024, .volt = 105000, .dyn_pwr = 0,
			.lkg_pwr = {0, 0, 0, 0, 0, 0} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
		{{0}, {0} },
	},
};

struct upower_tbl upower_tbl_cci_30G28G = {
	.row = {
		{.cap = 0, .volt = 65000, .dyn_pwr = 33904,
			.lkg_pwr = {55244, 55244, 55244, 55244, 55244, 55244} },
		{.cap = 0, .volt = 68750, .dyn_pwr = 45734,
			.lkg_pwr = {60466, 60466, 60466, 60466, 60466, 60466} },
		{.cap = 0, .volt = 71875, .dyn_pwr = 57081,
			.lkg_pwr = {65146, 65146, 65146, 65146, 65146, 65146} },
		{.cap = 0, .volt = 73750, .dyn_pwr = 64631,
			.lkg_pwr = {68086, 68086, 68086, 68086, 68086, 68086} },
		{.cap = 0, .volt = 75625, .dyn_pwr = 72725,
			.lkg_pwr = {71149, 71149, 71149, 71149, 71149, 71149} },
		{.cap = 0, .volt = 78750, .dyn_pwr = 85654,
			.lkg_pwr = {76665, 76665, 76665, 76665, 76665, 76665} },
		{.cap = 0, .volt = 81250, .dyn_pwr = 96680,
			.lkg_pwr = {81355, 81355, 81355, 81355, 81355, 81355} },
		{.cap = 0, .volt = 83750, .dyn_pwr = 110515,
			.lkg_pwr = {86322, 86322, 86322, 86322, 86322, 86322} },
		{.cap = 0, .volt = 86875, .dyn_pwr = 127185,
			.lkg_pwr = {93004, 93004, 93004, 93004, 93004, 93004} },
		{.cap = 0, .volt = 90625, .dyn_pwr = 149809,
			.lkg_pwr = {101576, 101576, 101576, 101576, 101576,
				101576} },
		{.cap = 0, .volt = 93750, .dyn_pwr = 170220,
			.lkg_pwr = {109453, 109453, 109453, 109453, 109453,
				109453} },
		{.cap = 0, .volt = 95000, .dyn_pwr = 179386,
			.lkg_pwr = {112605, 112605, 112605, 112605, 112605,
				112605} },
		{.cap = 0, .volt = 96875, .dyn_pwr = 191461,
			.lkg_pwr = {117924, 117924, 117924, 117924, 117924,
				117924} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 216049,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 237654,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
		{.cap = 0, .volt = 100000, .dyn_pwr = 262346,
			.lkg_pwr = {126791, 126791, 126791, 126791, 126791,
				126791} },
	},
	.lkg_idx = DEFAULT_LKG_IDX,
	.row_num = UPOWER_OPP_NUM,
	.nr_idle_states = NR_UPOWER_CSTATES,
	.idle_states = {
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
		{{0}, {55244} },
	},
};

#endif /* UNIFIED_POWER_DATA_H */
