// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (c) 2019 MediaTek Inc.
 */

#include <dt-bindings/gce/mt6833-gce.h>
#include "../cmdq-util.h"

const char *cmdq_thread_module_dispatch(phys_addr_t gce_pa, s32 thread)
{
	switch (thread) {
	case 0 ... 6:
	case 8 ... 9:
		return "DISP";
	case 7:
		return "VDEC";
	case 10:
	case 19 ... 22:
		return "MDP";
	case 11:
	case 13 ... 14:
	case 16 ... 18:
		return "ISP";
	case 12:
		return "VENC";
	case 15:
		return "CMDQ";
	case 23:
		return "SMI";
	default:
		return "CMDQ";
	}
}

const char *cmdq_event_module_dispatch(phys_addr_t gce_pa, const u16 event,
	s32 thread)
{
	switch (event) {
	case CMDQ_EVENT_ISP_FRAME_DONE_A:
	case CMDQ_EVENT_ISP_FRAME_DONE_B:
		return "ISP";
	case CMDQ_EVENT_CAMSV1_PASS1_DONE:
	case CMDQ_EVENT_CAMSV2_PASS1_DONE:
	case CMDQ_EVENT_CAMSV3_PASS1_DONE:
	case CMDQ_EVENT_MRAW_0_PASS1_DONE:
	case CMDQ_EVENT_SENINF_CAM0_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM1_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM2_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM3_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM4_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM5_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM6_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM7_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM8_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM9_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM10_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM11_FIFO_FULL:
	case CMDQ_EVENT_SENINF_CAM12_FIFO_FULL:
		return "CAM";
	case CMDQ_EVENT_TG_OVRUN_A_INT:
	case CMDQ_EVENT_DMA_R1_ERROR_A_INT:
	case CMDQ_EVENT_TG_OVRUN_B_INT:
	case CMDQ_EVENT_DMA_R1_ERROR_B_INT:
	case CMDQ_EVENT_TG_OVRUN_M0_INT:
	case CMDQ_EVENT_DMA_R1_ERROR_M0_INT:
	case CMDQ_EVENT_TG_GRABERR_M0_INT:
	case CMDQ_EVENT_TG_GRABERR_A_INT:
	case CMDQ_EVENT_CQ_VR_SNAP_A_INT:
	case CMDQ_EVENT_TG_GRABERR_B_INT:
	case CMDQ_EVENT_CQ_VR_SNAP_B_INT:
		return "IMG";
	case CMDQ_EVENT_VENC_CMDQ_FRAME_DONE:
	case CMDQ_EVENT_VENC_CMDQ_PAUSE_DONE:
	case CMDQ_EVENT_JPGENC_CMDQ_DONE:
	case CMDQ_EVENT_VENC_CMDQ_MB_DONE:
	case CMDQ_EVENT_VENC_CMDQ_128BYTE_CNT_DONE:
	case CMDQ_EVENT_VENC_CMDQ_PPS_DONE:
	case CMDQ_EVENT_VENC_CMDQ_SPS_DONE:
	case CMDQ_EVENT_VENC_CMDQ_VPS_DONE:
		return "VENC";
	case CMDQ_EVENT_LINE_COUNT_THRESHOLD_INTERRUPT:
	case CMDQ_EVENT_VDEC_INT:
	case CMDQ_EVENT_VDEC_PAUSE:
	case CMDQ_EVENT_VDEC_DEC_ERROR:
	case CMDQ_EVENT_MDEC_TIMEOUT:
	case CMDQ_EVENT_DRAM_ACCESS_DONE:
	case CMDQ_EVENT_INI_FETCH_RDY:
	case CMDQ_EVENT_PROCESS_FLAG:
	case CMDQ_EVENT_SEARCH_START_CODE_DONE:
	case CMDQ_EVENT_REF_REORDER_DONE:
	case CMDQ_EVENT_WP_TBLE_DONE:
	case CMDQ_EVENT_COUNT_SRAM_CLR_DONE:
	case CMDQ_EVENT_GCE_CNT_OP_THRESHOLD:
		return "VDEC";
	case CMDQ_EVENT_DVP_DONE_ASYNC_SHOT:
	case CMDQ_EVENT_DVS_DONE_ASYNC_SHOT:
		return "IMG";
	case CMDQ_EVENT_RSC_DONE:
		return "RSC";
	case CMDQ_EVENT_FE_DONE:
	case CMDQ_EVENT_FDVT_DONE:
		return "FDVT";
	case CMDQ_EVENT_GCE_IMG2_EVENT0:
	case CMDQ_EVENT_GCE_IMG2_EVENT1:
	case CMDQ_EVENT_GCE_IMG2_EVENT2:
	case CMDQ_EVENT_GCE_IMG2_EVENT3:
	case CMDQ_EVENT_GCE_IMG2_EVENT4:
	case CMDQ_EVENT_GCE_IMG2_EVENT5:
	case CMDQ_EVENT_GCE_IMG2_EVENT6:
	case CMDQ_EVENT_GCE_IMG2_EVENT7:
	case CMDQ_EVENT_GCE_IMG2_EVENT8:
	case CMDQ_EVENT_GCE_IMG2_EVENT9:
	case CMDQ_EVENT_GCE_IMG2_EVENT10:
	case CMDQ_EVENT_GCE_IMG2_EVENT11:
	case CMDQ_EVENT_GCE_IMG2_EVENT12:
	case CMDQ_EVENT_GCE_IMG2_EVENT13:
	case CMDQ_EVENT_GCE_IMG2_EVENT14:
	case CMDQ_EVENT_GCE_IMG2_EVENT15:
	case CMDQ_EVENT_GCE_IMG2_EVENT16:
	case CMDQ_EVENT_GCE_IMG2_EVENT17:
	case CMDQ_EVENT_GCE_IMG2_EVENT18:
	case CMDQ_EVENT_GCE_IMG2_EVENT19:
	case CMDQ_EVENT_GCE_IMG2_EVENT20:
	case CMDQ_EVENT_GCE_IMG2_EVENT21:
	case CMDQ_EVENT_GCE_IMG2_EVENT22:
	case CMDQ_EVENT_GCE_IMG2_EVENT23:
	case CMDQ_EVENT_GCE_IMG1_EVENT0:
	case CMDQ_EVENT_GCE_IMG1_EVENT1:
	case CMDQ_EVENT_GCE_IMG1_EVENT2:
	case CMDQ_EVENT_GCE_IMG1_EVENT3:
	case CMDQ_EVENT_GCE_IMG1_EVENT4:
	case CMDQ_EVENT_GCE_IMG1_EVENT5:
	case CMDQ_EVENT_GCE_IMG1_EVENT6:
	case CMDQ_EVENT_GCE_IMG1_EVENT7:
	case CMDQ_EVENT_GCE_IMG1_EVENT8:
	case CMDQ_EVENT_GCE_IMG1_EVENT9:
	case CMDQ_EVENT_GCE_IMG1_EVENT10:
	case CMDQ_EVENT_GCE_IMG1_EVENT11:
	case CMDQ_EVENT_GCE_IMG1_EVENT12:
	case CMDQ_EVENT_GCE_IMG1_EVENT13:
	case CMDQ_EVENT_GCE_IMG1_EVENT14:
	case CMDQ_EVENT_GCE_IMG1_EVENT15:
	case CMDQ_EVENT_GCE_IMG1_EVENT16:
	case CMDQ_EVENT_GCE_IMG1_EVENT17:
	case CMDQ_EVENT_GCE_IMG1_EVENT18:
	case CMDQ_EVENT_GCE_IMG1_EVENT19:
	case CMDQ_EVENT_GCE_IMG1_EVENT20:
	case CMDQ_EVENT_GCE_IMG1_EVENT21:
	case CMDQ_EVENT_GCE_IMG1_EVENT22:
	case CMDQ_EVENT_GCE_IMG1_EVENT23:
		return "IMG";
	case CMDQ_EVENT_MDP_RDMA0_SOF:
	case CMDQ_EVENT_MDP_RDMA1_SOF:
	case CMDQ_EVENT_MDP_AAL0_SOF:
	case CMDQ_EVENT_MDP_AAL1_SOF:
	case CMDQ_EVENT_MDP_HDR0_SOF:
	case CMDQ_EVENT_MDP_RSZ0_SOF:
	case CMDQ_EVENT_MDP_RSZ1_SOF:
	case CMDQ_EVENT_MDP_WROT0_SOF:
	case CMDQ_EVENT_MDP_WROT1_SOF:
	case CMDQ_EVENT_MDP_TDSHP0_SOF:
	case CMDQ_EVENT_MDP_TDSHP1_SOF:
		return "MDP";
	case CMDQ_EVENT_IMG_DL_RELAY0_SOF:
	case CMDQ_EVENT_IMG_DL_RELAY1_SOF:
		return "IMG";
	case CMDQ_EVENT_MDP_COLOR0_SOF:
	case CMDQ_EVENT_MDP_WROT3_FRAME_DONE:
	case CMDQ_EVENT_MDP_WROT2_FRAME_DONE:
	case CMDQ_EVENT_MDP_WROT1_FRAME_DONE:
	case CMDQ_EVENT_MDP_WROT0_FRAME_DONE:
	case CMDQ_EVENT_MDP_TDSHP3_FRAME_DONE:
	case CMDQ_EVENT_MDP_TDSHP2_FRAME_DONE:
	case CMDQ_EVENT_MDP_TDSHP1_FRAME_DONE:
	case CMDQ_EVENT_MDP_TDSHP0_FRAME_DONE:
	case CMDQ_EVENT_MDP_RSZ3_FRAME_DONE:
	case CMDQ_EVENT_MDP_RSZ2_FRAME_DONE:
	case CMDQ_EVENT_MDP_RSZ1_FRAME_DONE:
	case CMDQ_EVENT_MDP_RSZ0_FRAME_DONE:
	case CMDQ_EVENT_MDP_RDMA3_FRAME_DONE:
	case CMDQ_EVENT_MDP_RDMA2_FRAME_DONE:
	case CMDQ_EVENT_MDP_RDMA1_FRAME_DONE:
	case CMDQ_EVENT_MDP_RDMA0_FRAME_DONE:
	case CMDQ_EVENT_MDP_HDR1_FRAME_DONE:
	case CMDQ_EVENT_MDP_HDR0_FRAME_DONE:
	case CMDQ_EVENT_MDP_COLOR0_FRAME_DONE:
	case CMDQ_EVENT_MDP_AAL3_FRAME_DONE:
	case CMDQ_EVENT_MDP_AAL2_FRAME_DONE:
	case CMDQ_EVENT_MDP_AAL1_FRAME_DONE:
	case CMDQ_EVENT_MDP_AAL0_FRAME_DONE:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_0:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_1:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_2:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_3:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_4:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_5:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_6:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_7:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_8:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_9:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_10:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_11:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_12:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_13:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_14:
	case CMDQ_EVENT_MDP_STREAM_DONE_ENG_EVENT_15:
	case CMDQ_EVENT_MDP_WROT3_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_WROT2_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_WROT1_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_WROT0_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_RDMA3_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_RDMA2_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_RDMA1_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_MDP_RDMA0_SW_RST_DONE_ENG_EVENT:
		return "MDP";
	case CMDQ_EVENT_DISP_OVL0_SOF:
	case CMDQ_EVENT_DISP_OVL0_2L_SOF:
	case CMDQ_EVENT_DISP_RDMA0_SOF:
	case CMDQ_EVENT_DISP_RSZ0_SOF:
	case CMDQ_EVENT_DISP_COLOR0_SOF:
	case CMDQ_EVENT_DISP_CCORR0_SOF:
	case CMDQ_EVENT_DISP_CCORR1_SOF:
	case CMDQ_EVENT_DISP_AAL0_SOF:
	case CMDQ_EVENT_DISP_GAMMA0_SOF:
	case CMDQ_EVENT_DISP_POSTMASK0_SOF:
	case CMDQ_EVENT_DISP_DITHER0_SOF:
	case CMDQ_EVENT_DISP_CM0_SOF:
	case CMDQ_EVENT_DISP_SPR0_SOF:
	case CMDQ_EVENT_DISP_DSC_WRAP0_SOF:
	case CMDQ_EVENT_DSI0_SOF:
	case CMDQ_EVENT_DISP_WDMA0_SOF:
	case CMDQ_EVENT_DISP_PWM0_SOF:
	case CMDQ_EVENT_DSI0_FRAME_DONE:
	case CMDQ_EVENT_DISP_WDMA0_FRAME_DONE:
	case CMDQ_EVENT_DISP_SPR0_FRAME_DONE:
	case CMDQ_EVENT_DISP_RSZ0_FRAME_DONE:
	case CMDQ_EVENT_DISP_RDMA0_FRAME_DONE:
	case CMDQ_EVENT_DISP_POSTMASK0_FRAME_DONE:
	case CMDQ_EVENT_DISP_OVL0_FRAME_DONE:
	case CMDQ_EVENT_DISP_OVL0_2L_FRAME_DONE:
	case CMDQ_EVENT_DISP_GAMMA0_FRAME_DONE:
	case CMDQ_EVENT_DISP_DSC_WRAP0_CORE1_FRAME_DONE:
	case CMDQ_EVENT_DISP_DSC_WRAP0_CORE0_FRAME_DONE:
	case CMDQ_EVENT_DISP_DITHER0_FRAME_DONE:
	case CMDQ_EVENT_DISP_COLOR0_FRAME_DONE:
	case CMDQ_EVENT_DISP_CM0_FRAME_DONE:
	case CMDQ_EVENT_DISP_CCORR1_FRAME_DONE:
	case CMDQ_EVENT_DISP_CCORR0_FRAME_DONE:
	case CMDQ_EVENT_DISP_AAL0_FRAME_DONE:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_0:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_1:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_2:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_3:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_4:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_5:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_6:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_7:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_8:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_9:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_10:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_11:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_12:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_13:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_14:
	case CMDQ_EVENT_DISP_STREAM_DONE_ENG_EVENT_15:
	case CMDQ_EVENT_DSI0_TE_ENG_EVENT:
	case CMDQ_EVENT_DSI0_IRQ_ENG_EVENT:
	case CMDQ_EVENT_DSI0_DONE_ENG_EVENT:
	case CMDQ_EVENT_DISP_WDMA0_SW_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_DISP_SMIASSERT_ENG_EVENT:
	case CMDQ_EVENT_DISP_POSTMASK0_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_DISP_OVL0_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_DISP_OVL0_2L_RST_DONE_ENG_EVENT:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_0:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_1:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_2:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_3:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_4:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_5:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_6:
	case CMDQ_EVENT_BUF_UNDERRUN_ENG_EVENT_7:
		return "DISP";
	default:
		return cmdq_thread_module_dispatch(gce_pa, thread);
	}
}
EXPORT_SYMBOL(cmdq_event_module_dispatch);

u32 cmdq_util_hw_id(u32 pa)
{
	return 0;
}

u32 cmdq_test_get_subsys_list(u32 **regs_out)
{
	static u32 regs[] = {
		0x14000100,	/* mmsys MMSYS_CG_CON0 */
		0x112300a0,	/* msdc0 SW_DBG_SEL: LSB 16-bit only */
		0x1121004c,	/* To-do: audio AFE_I2S_CON3_OFFSET */
		0x110020bc,	/* uart0:LSB 1-bit only */
	};

	*regs_out = regs;
	return ARRAY_SIZE(regs);
}

const char *cmdq_util_hw_name(void *chan)
{
	return "GCE";
}

bool cmdq_thread_ddr_user_check(const s32 thread)
{
	switch (thread) {
	case 0 ... 6:
	case 8 ... 9:
	case 15:
		return false;
	default:
		return true;
	}
}
EXPORT_SYMBOL(cmdq_thread_ddr_user_check);
