/*
 * Copyright (C) 2015 MediaTek Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */

#define pr_fmt(fmt) KBUILD_MODNAME ": %s: " fmt, __func__

#include <linux/types.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/hrtimer.h>
#include <linux/ktime.h>
#include <linux/workqueue.h>
#include <linux/mutex.h>
#include <linux/of.h>
#include <linux/list.h>
#include <linux/delay.h>
#include <linux/pinctrl/consumer.h>

#include "flashlight-core.h"
#include "flashlight-dt.h"



/* define device tree */
#ifndef AW3641_V690_DTNAME
#define AW3641_V690_DTNAME "mediatek,flashlights_aw3641_v690"
#endif

#define AW3641_V690_NAME "flashlights_aw3641_v690"

/* define registers */

/* define mutex and work queue */
static DEFINE_MUTEX(aw3641_v690_mutex);
static struct work_struct aw3641_v690_work;

/* define pinctrl */
#define AW3641_V690_PINCTRL_PIN_HWEN 0
#define AW3641_V690_PINCTRL_PIN_HWMODE 1
#define AW3641_V690_PINCTRL_PINSTATE_LOW 0
#define AW3641_V690_PINCTRL_PINSTATE_HIGH 1
#define AW3641_V690_PINCTRL_STATE_HWEN_HIGH "hwen_high"
#define AW3641_V690_PINCTRL_STATE_HWEN_LOW  "hwen_low"
#define AW3641_V690_PINCTRL_STATE_HWMODE_HIGH "hwmode_high"
#define AW3641_V690_PINCTRL_STATE_HWMODE_LOW  "hwmode_low"
#define AW3641_V690_LEVEL_NUM 9
#define AW3641_V690_LEVEL_TORCH 2

static struct pinctrl *aw3641_v690_pinctrl=NULL;
static struct pinctrl_state *aw3641_v690_hwen_high=NULL;
static struct pinctrl_state *aw3641_v690_hwen_low=NULL;
static struct pinctrl_state *aw3641_v690_hwmode_high=NULL;
static struct pinctrl_state *aw3641_v690_hwmode_low=NULL;


/* define usage count */
static int use_count;

static int g_flash_duty = -1;

/* platform data */
struct aw3641_v690_platform_data {
	int channel_num;
	struct flashlight_device_id *dev_id;
};


/******************************************************************************
 * Pinctrl configuration
 *****************************************************************************/
static int aw3641_v690_pinctrl_init(struct platform_device *pdev)
{
	int ret = 0;
	printk("Flashlight_use_gpio_probe enter \n");
	/* get pinctrl */
	aw3641_v690_pinctrl = devm_pinctrl_get(&pdev->dev);
	if (IS_ERR(aw3641_v690_pinctrl)) {
		 pr_err("Failed to get flashlight pinctrl.\n");
		ret = PTR_ERR(aw3641_v690_pinctrl);
	}

	/*  Flashlight pin initialization */
	aw3641_v690_hwen_high = pinctrl_lookup_state(aw3641_v690_pinctrl, "flashlightpin_en1");
	if (IS_ERR(aw3641_v690_hwen_high)) {
		printk("Failed to init en (%s)\n", AW3641_V690_PINCTRL_STATE_HWEN_HIGH);
		ret = PTR_ERR(aw3641_v690_hwen_high);
	}
	aw3641_v690_hwen_low = pinctrl_lookup_state(aw3641_v690_pinctrl, "flashlightpin_en0");
	if (IS_ERR(aw3641_v690_hwen_low)) {
		printk("Failed to init en(%s)\n", AW3641_V690_PINCTRL_STATE_HWEN_LOW);
		ret = PTR_ERR(aw3641_v690_hwen_low);
	}

	aw3641_v690_hwmode_high = pinctrl_lookup_state(aw3641_v690_pinctrl, "flashlightpin_cfg1");
	if (IS_ERR(aw3641_v690_hwen_high)) {
		printk("Failed to init mode high(%s)\n", AW3641_V690_PINCTRL_STATE_HWMODE_HIGH);
		ret = PTR_ERR(aw3641_v690_hwen_high);
	}
	aw3641_v690_hwmode_low = pinctrl_lookup_state(aw3641_v690_pinctrl, "flashlightpin_cfg0");
	if (IS_ERR(aw3641_v690_hwen_low)) {
		printk("Failed to init mode low(%s)\n", AW3641_V690_PINCTRL_STATE_HWMODE_LOW);
		ret = PTR_ERR(aw3641_v690_hwen_low);
	}
    	printk("Flashlight_use_gpio_probe exit\n");
	return ret;
}

static int aw3641_v690_pinctrl_set(int pin,int state)
{
	int ret = 0;

	if (IS_ERR(aw3641_v690_pinctrl)) {
		printk("pinctrl is not available\n");
		return -1;
	}

	switch (pin) {
	case AW3641_V690_PINCTRL_PIN_HWEN:
		if (state == AW3641_V690_PINCTRL_PINSTATE_LOW && !IS_ERR(aw3641_v690_hwen_low)){
			ret = pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwen_low);
			}
		else if (state == AW3641_V690_PINCTRL_PINSTATE_HIGH && !IS_ERR(aw3641_v690_hwen_high)){
			ret = pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwen_high);
			}
		else
			printk("set err, pin(%d) state(%d)\n", pin, state);
		break;
	case AW3641_V690_PINCTRL_PIN_HWMODE:
		if (state == AW3641_V690_PINCTRL_PINSTATE_LOW && !IS_ERR(aw3641_v690_hwmode_low)){
			ret = pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwmode_low);
			printk("flashmode is low");
			}
		else if (state == AW3641_V690_PINCTRL_PINSTATE_HIGH && !IS_ERR(aw3641_v690_hwmode_high)){
			ret = pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwmode_high);
			printk("flashmode is high");
			}
		else
			printk("set err, pin(%d) state(%d)\n", pin, state);
		break;
	default:
		printk("set err, pin(%d) state(%d)\n", pin, state);
		break;
	}
	pr_debug("pin(%d) state(%d), ret:%d\n", pin, state, ret);

	return ret;
}

static int aw3641_v690_is_torch(int level)
{
	if(level >= AW3641_V690_LEVEL_TORCH)
		return -1;
	return 0;
}

static int aw3641_v690_verify_level(int level)
{
	if (level < 0)
		level = 0;
	else if (level >= AW3641_V690_LEVEL_NUM)
		level = AW3641_V690_LEVEL_NUM;

	return level;
}
#if 0
static int aw3641_v690_flash_cur_adjust(int num)
{
	int number,i;
	if (num>9)
		num=9;

	number=AW3641_V690_LEVEL_NUM-num;
	for(i=0;i<number;i++)
	{
		aw3641_v690_pinctrl_set(0,0);
		//printk("en_low");
		udelay(1);
		aw3641_v690_pinctrl_set(0,1);		
	}
	return 	number;	
}
#endif
/******************************************************************************
 * aw3641_v690 operations
 *****************************************************************************/
/* flashlight enable function */
static int aw3641_v690_enable(void)
{
	int pin = 0 ,state=1,i;

	if (aw3641_v690_is_torch(g_flash_duty)== 0) {
		aw3641_v690_pinctrl_set(pin, state);
		aw3641_v690_pinctrl_set(pin+1, state-1);
	} else if(aw3641_v690_is_torch(g_flash_duty)==-1){
		pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwmode_high);
		pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwen_high);
		for(i=0;i<AW3641_V690_LEVEL_NUM-g_flash_duty;i++)
			{
			pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwen_low);
			pinctrl_select_state(aw3641_v690_pinctrl, aw3641_v690_hwen_high);
			}
	}
	return 0;
}

/* flashlight disable function */
static int aw3641_v690_disable(void)
{
	int pin = 0, state = 0;
	aw3641_v690_pinctrl_set(pin+1, state);
	return aw3641_v690_pinctrl_set(pin, state);
}

/* set flashlight level */
static int aw3641_v690_set_level(int level)
{
 	level= aw3641_v690_verify_level(level);
	g_flash_duty = level;
	return 0;
}

/* flashlight init */
static int aw3641_v690_init(void)
{
	int pin = 0, state = 0;
	aw3641_v690_pinctrl_set(pin+1, state);
	return aw3641_v690_pinctrl_set(pin, state);
}

/* flashlight uninit */
static int aw3641_v690_uninit(void)
{
	int pin = 0, state = 0;
	aw3641_v690_pinctrl_set(pin+1, state);
	return aw3641_v690_pinctrl_set(pin, state);
}

/******************************************************************************
 * Timer and work queue
 *****************************************************************************/
static struct hrtimer aw3641_v690_timer;
static unsigned int aw3641_v690_timeout_ms;

static void aw3641_v690_work_disable(struct work_struct *data)
{
	pr_debug("work queue callback\n");
	aw3641_v690_disable();
}

static enum hrtimer_restart aw3641_v690_timer_func(struct hrtimer *timer)
{
	schedule_work(&aw3641_v690_work);
	return HRTIMER_NORESTART;
}


/******************************************************************************
 * Flashlight operations
 *****************************************************************************/
static int aw3641_v690_ioctl(unsigned int cmd, unsigned long arg)
{
	struct flashlight_dev_arg *fl_arg;
	int channel;
	ktime_t ktime;

	fl_arg = (struct flashlight_dev_arg *)arg;
	channel = fl_arg->channel;

	switch (cmd) {
	case FLASH_IOC_SET_TIME_OUT_TIME_MS:
		printk("aw3641_v690 FLASH_IOC_SET_TIME_OUT_TIME_MS(%d): %d\n",
				channel, (int)fl_arg->arg);
		aw3641_v690_timeout_ms = fl_arg->arg;
		break;

	case FLASH_IOC_SET_DUTY:
		printk("aw3641_v690_FLASH_IOC_SET_DUTY(%d): %d\n",
				channel, (int)fl_arg->arg);
		aw3641_v690_set_level(fl_arg->arg);
		break;

	case FLASH_IOC_SET_ONOFF:
		printk("aw3641_v690 FLASH_IOC_SET_ONOFF(%d): %d\n",
				channel, (int)fl_arg->arg);
		if (fl_arg->arg == 1) {
			if (aw3641_v690_timeout_ms) {
				ktime = ktime_set(aw3641_v690_timeout_ms / 1000,
						(aw3641_v690_timeout_ms % 1000) * 1000000);
				hrtimer_start(&aw3641_v690_timer, ktime, HRTIMER_MODE_REL);
			}
			aw3641_v690_enable();
		} else {
			aw3641_v690_disable();
			hrtimer_cancel(&aw3641_v690_timer);
		}
		break;
	default:
		pr_info("aw3641_v690 No such command and arg(%d): (%d, %d)\n",
				channel, _IOC_NR(cmd), (int)fl_arg->arg);
		return -ENOTTY;
	}

	return 0;
}

static int aw3641_v690_open(void)
{
	/* Move to set driver for saving power */
	return 0;
}

static int aw3641_v690_release(void)
{
	/* Move to set driver for saving power */
	return 0;
}

static int aw3641_v690_set_driver(int set)
{
	int ret = 0;

	/* set chip and usage count */
	mutex_lock(&aw3641_v690_mutex);
	if (set) {
		if (!use_count)
			ret = aw3641_v690_init();
		use_count++;
		pr_debug("Set driver: %d\n", use_count);
	} else {
		use_count--;
		if (!use_count)
			ret = aw3641_v690_uninit();
		if (use_count < 0)
			use_count = 0;
		pr_debug("Unset driver: %d\n", use_count);
	}
	mutex_unlock(&aw3641_v690_mutex);

	return ret;
}

static ssize_t aw3641_v690_strobe_store(struct flashlight_arg arg)
{
	aw3641_v690_set_driver(1);
	aw3641_v690_set_level(arg.level);
	aw3641_v690_timeout_ms = 0;
	aw3641_v690_enable();
	msleep(arg.dur);
	aw3641_v690_disable();
	aw3641_v690_set_driver(0);

	return 0;
}

static struct flashlight_operations aw3641_v690_ops = {
	aw3641_v690_open,
	aw3641_v690_release,
	aw3641_v690_ioctl,
	aw3641_v690_strobe_store,
	aw3641_v690_set_driver
};


/******************************************************************************
 * Platform device and driver
 *****************************************************************************/
static int aw3641_v690_chip_init(void)
{
	/* NOTE: Chip initialication move to "set driver" operation for power saving issue.
	 * aw3641_v690_init();
	 */

	return 0;
}

static int aw3641_v690_parse_dt(struct device *dev,
		struct aw3641_v690_platform_data *pdata)
{
	struct device_node *np, *cnp;
	u32 decouple = 0;
	int i = 0;

	if (!dev || !dev->of_node || !pdata)
		return -ENODEV;

	np = dev->of_node;

	pdata->channel_num = of_get_child_count(np);
	if (!pdata->channel_num) {
		pr_info("Parse no dt, node.\n");
		return 0;
	}
	pr_info("Channel number(%d).\n", pdata->channel_num);

	if (of_property_read_u32(np, "decouple", &decouple))
		pr_info("Parse no dt, decouple.\n");

	pdata->dev_id = devm_kzalloc(dev,
			pdata->channel_num * sizeof(struct flashlight_device_id),
			GFP_KERNEL);
	if (!pdata->dev_id)
		return -ENOMEM;

	for_each_child_of_node(np, cnp) {
		if (of_property_read_u32(cnp, "type", &pdata->dev_id[i].type))
			goto err_node_put;
		if (of_property_read_u32(cnp, "ct", &pdata->dev_id[i].ct))
			goto err_node_put;
		if (of_property_read_u32(cnp, "part", &pdata->dev_id[i].part))
			goto err_node_put;
		snprintf(pdata->dev_id[i].name, FLASHLIGHT_NAME_SIZE, AW3641_V690_NAME);
		pdata->dev_id[i].channel = i;
		pdata->dev_id[i].decouple = decouple;

		pr_info("Parse dt (type,ct,part,name,channel,decouple)=(%d,%d,%d,%s,%d,%d).\n",
				pdata->dev_id[i].type, pdata->dev_id[i].ct,
				pdata->dev_id[i].part, pdata->dev_id[i].name,
				pdata->dev_id[i].channel, pdata->dev_id[i].decouple);
		i++;
	}

	return 0;

err_node_put:
	of_node_put(cnp);
	return -EINVAL;
}

static int aw3641_v690_probe(struct platform_device *pdev)
{
	struct aw3641_v690_platform_data *pdata = dev_get_platdata(&pdev->dev);
	int err;
	int i;

	pr_debug("Probe start.\n");

	/* init pinctrl */
	if (aw3641_v690_pinctrl_init(pdev)) {
		pr_debug("Failed to init pinctrl.\n");
		err = -EFAULT;
		goto err;
	}

	/* init platform data */
	if (!pdata) {
		pdata = devm_kzalloc(&pdev->dev, sizeof(*pdata), GFP_KERNEL);
		if (!pdata) {
			err = -ENOMEM;
			goto err;
		}
		pdev->dev.platform_data = pdata;
		err = aw3641_v690_parse_dt(&pdev->dev, pdata);
		if (err)
			goto err;
	}

	/* init work queue */
	INIT_WORK(&aw3641_v690_work, aw3641_v690_work_disable);

	/* init timer */
	hrtimer_init(&aw3641_v690_timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
	aw3641_v690_timer.function = aw3641_v690_timer_func;
	aw3641_v690_timeout_ms = 100;

	/* init chip hw */
	aw3641_v690_chip_init();

	/* clear usage count */
	use_count = 0;

	/* register flashlight device */
	if (pdata->channel_num) {
		for (i = 0; i < pdata->channel_num; i++)
			if (flashlight_dev_register_by_device_id(&pdata->dev_id[i], &aw3641_v690_ops)) {
				err = -EFAULT;
				goto err;
			}
	} else {
		if (flashlight_dev_register(AW3641_V690_NAME, &aw3641_v690_ops)) {
			err = -EFAULT;
			goto err;
		}
	}

	pr_debug("Probe done.\n");

	return 0;
err:
	return err;
}

static int aw3641_v690_remove(struct platform_device *pdev)
{
	struct aw3641_v690_platform_data *pdata = dev_get_platdata(&pdev->dev);
	int i;

	pr_debug("Remove start.\n");

	pdev->dev.platform_data = NULL;

	/* unregister flashlight device */
	if (pdata && pdata->channel_num)
		for (i = 0; i < pdata->channel_num; i++)
			flashlight_dev_unregister_by_device_id(&pdata->dev_id[i]);
	else
		flashlight_dev_unregister(AW3641_V690_NAME);

	/* flush work queue */
	flush_work(&aw3641_v690_work);

	pr_debug("Remove done.\n");

	return 0;
}

#ifdef CONFIG_OF
static const struct of_device_id aw3641_v690_gpio_of_match[] = {
	{.compatible = AW3641_V690_DTNAME},
	{},
};
MODULE_DEVICE_TABLE(of, aw3641_v690_gpio_of_match);
#else
static struct platform_device aw3641_v690_gpio_platform_device[] = {
	{
		.name = AW3641_V690_NAME,
		.id = 0,
		.dev = {}
	},
	{}
};
MODULE_DEVICE_TABLE(platform, aw3641_v690_gpio_platform_device);
#endif

static struct platform_driver aw3641_v690_platform_driver = {
	.probe = aw3641_v690_probe,
	.remove = aw3641_v690_remove,
	.driver = {
		.name = AW3641_V690_NAME,
		.owner = THIS_MODULE,
#ifdef CONFIG_OF
		.of_match_table = aw3641_v690_gpio_of_match,
#endif
	},
};

static int __init flashlight_aw3641_v690_init(void)
{
	int ret;

	pr_debug("Init start.\n");

#ifndef CONFIG_OF
	ret = platform_device_register(&aw3641_v690_gpio_platform_device);
	if (ret) {
		printk("Failed to register platform device\n");
		return ret;
	}
#endif

	ret = platform_driver_register(&aw3641_v690_platform_driver);
	if (ret) {
		printk("Failed to register platform driver\n");
		return ret;
	}

	pr_debug("Init done.\n");

	return 0;
}

static void __exit flashlight_aw3641_v690_exit(void)
{
	pr_debug("Exit start.\n");

	platform_driver_unregister(&aw3641_v690_platform_driver);

	pr_debug("Exit done.\n");
}

module_init(flashlight_aw3641_v690_init);
module_exit(flashlight_aw3641_v690_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Simon Wang <Simon-TCH.Wang@mediatek.com>");
MODULE_DESCRIPTION("MTK Flashlight AW3641_V690 GPIO Driver");

