/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019 MediaTek Inc.
 */

/*****************************************************************************
 *
 * Filename:
 * ---------
 *	 OV5693mipi_Sensor.h
 *
 * Project:
 * --------
 *	 ALPS
 *
 * Description:
 * ------------
 *	 CMOS sensor header file
 *
 * Author:
 * -------
 * Dream Yeh (MTK08783)
 *
 *
 ****************************************************************************/
#ifndef _S5K4E6MIPI_SENSOR_H
#define _S5K4E6MIPI_SENSOR_H


enum IMGSENSOR_MODE {
	IMGSENSOR_MODE_INIT,
	IMGSENSOR_MODE_PREVIEW,
	IMGSENSOR_MODE_CAPTURE,
	IMGSENSOR_MODE_VIDEO,
	IMGSENSOR_MODE_HIGH_SPEED_VIDEO,
	IMGSENSOR_MODE_SLIM_VIDEO,
};

struct imgsensor_mode_struct {
	kal_uint32 pclk;	/* record different mode's pclk */
	kal_uint32 linelength;	/* record different mode's linelength */
	kal_uint32 framelength;	/* record different mode's framelength */

	/* record different mode's startx of grabwindow */
	kal_uint8 startx;

	/* record different mode's starty of grabwindow */
	kal_uint8 starty;

	/* record different mode's width of grabwindow */
	kal_uint16 grabwindow_width;

	/* record different mode's height of grabwindow */
	kal_uint16 grabwindow_height;

	/*       following for MIPIDataLowPwr2HighSpeedSettleDelayCount by
	 *	 different scenario
	 */
	kal_uint8 mipi_data_lp2hs_settle_dc;

	/*       following for GetDefaultFramerateByScenario()  */
	kal_uint16 max_framerate;

};

/* SENSOR PRIVATE STRUCT FOR VARIABLES*/
struct imgsensor_struct {
	kal_uint8 mirror;	/* mirrorflip information */

	kal_uint8 sensor_mode;	/* record IMGSENSOR_MODE enum value */

	kal_uint32 shutter;	/* current shutter */
	kal_uint16 gain;	/* current gain */

	kal_uint32 pclk;	/* current pclk */

	kal_uint32 frame_length;	/* current framelength */
	kal_uint32 line_length;	/* current linelength */

	/* current min	framelength to max framerate */
	kal_uint32 min_frame_length;

	kal_uint16 dummy_pixel;	/* current dummypixel */
	kal_uint16 dummy_line;	/* current dummline */

	kal_uint16 current_fps;	/* current max fps */

	/* record autoflicker enable or disable */
	kal_bool autoflicker_en;

	kal_bool test_pattern;	/* record test pattern mode or not */

	/* current scenario id */
	enum MSDK_SCENARIO_ID_ENUM current_scenario_id;

	kal_uint8 ihdr_en;	/* ihdr enable or disable */

	/* HDR mODE : 0: disable HDR, 1:IHDR, 2:HDR, 9:ZHDR */
	kal_uint8 hdr_mode;

	kal_uint8 i2c_write_id;	/* record current sensor's i2c write id */

	/* Update sensor awb from otp or not */
	kal_uint8 update_sensor_otp_awb;

	/* Update sensor lsc from otp or not */
	kal_uint8 update_sensor_otp_lsc;
};

/* SENSOR PRIVATE STRUCT FOR CONSTANT*/
struct imgsensor_info_struct {

	/* record sensor id defined in Kd_imgsensor.h */
	kal_uint32 sensor_id;

	/* checksum value for Camera Auto Test */
	kal_uint32 checksum_value;

	/* preview scenario relative information */
	struct imgsensor_mode_struct pre;

	/* capture scenario relative information */
	struct imgsensor_mode_struct cap;

	/* capture for PIP 15fps relative information*/
	struct imgsensor_mode_struct cap1;

	/* capture for PIP 24fps relative information*/
	struct imgsensor_mode_struct cap2;

	/* normal video  scenario relative information */
	struct imgsensor_mode_struct normal_video;

	/* high speed video scenario relative information */
	struct imgsensor_mode_struct hs_video;

	/* slim video for VT scenario relative information */
	struct imgsensor_mode_struct slim_video;

	/* shutter delay frame for AE cycle */
	kal_uint8 ae_shut_delay_frame;

	/* sensor gain delay frame for AE cycle */
	kal_uint8 ae_sensor_gain_delay_frame;

	/* isp gain delay frame for AE cycle */
	kal_uint8 ae_ispGain_delay_frame;

	kal_uint8 ihdr_support;		/* 1, support; 0,not support */
	kal_uint8 ihdr_le_firstline;	/* 1,le first ; 0, se first */
	kal_uint8 sensor_mode_num;	/* support sensor mode num */

	/* enter capture delay frame num */
	kal_uint8 cap_delay_frame;

	/* enter preview delay frame num */
	kal_uint8 pre_delay_frame;

	/* enter video delay frame num */
	kal_uint8 video_delay_frame;

	/* enter high speed video  delay frame num */
	kal_uint8 hs_video_delay_frame;

	/* enter slim video delay frame num */
	kal_uint8 slim_video_delay_frame;

	/* sensor framelength & shutter margin */
	kal_uint8 margin;

	/* min shutter */
	kal_uint32 min_shutter;

	/* max framelength by sensor register's limitation */
	kal_uint32 max_frame_length;


	kal_uint8 isp_driving_current;		/* mclk driving current */
	kal_uint8 sensor_interface_type;	/* sensor_interface_type */
	kal_uint8 mipi_sensor_type;
	kal_uint8 mipi_settle_delay_mode;	/* always manual */

	/* sensor output first pixel color */
	kal_uint8 sensor_output_dataformat;

	/* mclk value, suggest 24 or 26 for 24Mhz or 26Mhz */
	kal_uint8 mclk;

	kal_uint8 mipi_lane_num;	/* mipi lane num */
	kal_uint16 i2c_speed;

	/* record sensor support all write id addr  */
	/* only supprt 4must end with 0xff          */
	kal_uint8 i2c_addr_table[5];
};

extern int iReadRegI2CTiming
	(u8 *a_pSendData, u16 a_sizeSendData,
	 u8 *a_pRecvData, u16 a_sizeRecvData,
	 u16 i2cId,	  u16 timing);

extern int iWriteRegI2CTiming
	(u8 *a_pSendData, u16 a_sizeSendData,
	 u16 i2cId,	  u16 timing);

extern int iBurstWriteReg_multi
	(u8 *pData, u32 bytes, u16 i2cId,
	 u16 transfer_length,  u16 timing);

extern int iReadRegI2C(u8 *a_pSendData, u16 a_sizeSendData,
	u8 *a_pRecvData, u16 a_sizeRecvData, u16 i2cId);

extern int iWriteRegI2C(u8 *a_pSendData, u16 a_sizeSendData, u16 i2cId);

#endif
