/*****************************************************************************
 *
 * Filename:
 * ---------
 *     sp5508_dd_v590_mipi_raw_Sensor.c
 *
 * Project:
 * --------
 *     ALPS
 *
 * Description:
 * ------------
 *     Source code of Sensor driver
 *
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by CC/CQ. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/

#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/fs.h>
#include <linux/atomic.h>
#include <linux/types.h>

#include "kd_camera_typedef.h"
#include "kd_imgsensor.h"
#include "kd_imgsensor_define.h"
#include "kd_imgsensor_errcode.h"

#ifdef  CONFIG_T_PRODUCT_INFO
#include <dev_info.h>
#endif

#include "sp5508_dd_v590_mipi_raw_Sensor.h"
/****************************Modify Following Strings for Debug****************************/
#define PFX "SP5508_camera_sensor"
#define LOG_1 LOG_INF("SP5508,MIPI 2LANE\n")
/****************************   Modify end    *******************************************/

#define LOG_INF(format, args...)    //xlog_printk(ANDROID_LOG_INFO   , PFX, "[%s] " format, __FUNCTION__, ##args)

static DEFINE_SPINLOCK(imgsensor_drv_lock);
//static kal_uint32 shutter_time = 10;

static struct imgsensor_info_struct imgsensor_info = {
    .sensor_id = SP5508_DD_V590_SENSOR_ID,

    .checksum_value = 0x8e8fb430,        //checksum value for Camera Auto Test

    .pre = {
        .pclk = 93600000,              //record different mode's pclk
		.linelength = 1562,             //record different mode's linelength
		.framelength = 1994,            //record different mode's framelength
        .startx = 0,                    //record different mode's startx of grabwindow
        .starty = 0,                    //record different mode's starty of grabwindow
        .grabwindow_width = 2592,        //record different mode's width of grabwindow
        .grabwindow_height = 1944,        //record different mode's height of grabwindow
        /*     following for MIPIDataLowPwr2HighSpeedSettleDelayCount by different scenario    */
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .cap = {
        .pclk = 93600000,
		.linelength = 1562,
		.framelength = 1994,
        .startx = 0,
        .starty = 0,
        .grabwindow_width = 2592,
        .grabwindow_height = 1944,
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .cap1 = {
        .pclk = 93600000,
		.linelength = 1562,
		.framelength = 3988,
        .startx = 0,
        .starty = 0,
        .grabwindow_width = 2592,
        .grabwindow_height = 1944,
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .normal_video = {
        .pclk = 93600000,
		.linelength = 1562,
		.framelength = 1994,
        .startx = 0,
        .starty = 0,
        .grabwindow_width = 2592,
        .grabwindow_height = 1944,
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .hs_video = {
        .pclk = 93600000,
		.linelength = 1562,
		.framelength = 1994,
        .startx = 0,
        .starty = 0,
        .grabwindow_width = 2592,
        .grabwindow_height = 1944,
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .slim_video = {
        .pclk = 93600000,
		.linelength = 1562,
		.framelength = 1994,
        .startx = 0,
        .starty = 0,
        .grabwindow_width = 2592,
        .grabwindow_height = 1944,
        .mipi_data_lp2hs_settle_dc = 23,//unit , ns
		.mipi_pixel_rate = 187200000,
        .max_framerate = 300,
    },
    .margin = 12,            //sensor framelength & shutter margin
    .min_shutter = 4,        //min shutter
    .max_frame_length = 0x7fff,//max framelength by sensor register's limitation
    .ae_shut_delay_frame = 0,    //shutter delay frame for AE cycle, 2 frame with ispGain_delay-shut_delay=2-0=2
    .ae_sensor_gain_delay_frame = 0,//sensor gain delay frame for AE cycle,2 frame with ispGain_delay-sensor_gain_delay=2-0=2
    .ae_ispGain_delay_frame = 2,//isp gain delay frame for AE cycle
    .ihdr_support = 0,      //1, support; 0,not support
    .ihdr_le_firstline = 0,  //1,le first ; 0, se first
    .sensor_mode_num = 5,      //support sensor mode num

    .cap_delay_frame = 2,
    .pre_delay_frame = 2,
    .video_delay_frame = 2,
    .hs_video_delay_frame = 2,
    .slim_video_delay_frame = 2,

    .isp_driving_current = ISP_DRIVING_8MA, //mclk driving current
    .sensor_interface_type = SENSOR_INTERFACE_TYPE_MIPI,//sensor_interface_type
    .mipi_sensor_type = MIPI_OPHY_NCSI2, //0,MIPI_OPHY_NCSI2;  1,MIPI_OPHY_CSI2
    .mipi_settle_delay_mode = MIPI_SETTLEDELAY_AUTO,//0,MIPI_SETTLEDELAY_AUTO; 1,MIPI_SETTLEDELAY_MANNUAL
    .sensor_output_dataformat = SENSOR_OUTPUT_FORMAT_RAW_Gr,//sensor output first pixel color//B
    .mclk = 24,//mclk value, suggest 24 or 26 for 24Mhz or 26Mhz
    .mipi_lane_num = SENSOR_MIPI_2_LANE,//mipi lane num
    .i2c_addr_table = {0x78, 0xff}, //5409H---0x78 /5409---0x7a  zhanghua
    .i2c_speed = 200,
};


static struct imgsensor_struct imgsensor = {
    .mirror = IMAGE_NORMAL,                //mirrorflip information
    .sensor_mode = IMGSENSOR_MODE_INIT, //IMGSENSOR_MODE enum value,record current sensor mode,such as: INIT, Preview, Capture, Video,High Speed Video, Slim Video
    .shutter = 0x3D0,                    //current shutter
    .gain = 0x100,                        //current gain
    .dummy_pixel = 0,                    //current dummypixel
    .dummy_line = 0,                    //current dummyline
    .current_fps = 300,  //full size current fps : 24fps for PIP, 30fps for Normal or ZSD
    .autoflicker_en = KAL_FALSE,  //auto flicker enable: KAL_FALSE for disable auto flicker, KAL_TRUE for enable auto flicker
    .test_pattern = KAL_FALSE,        //test pattern mode or not. KAL_FALSE for in test pattern mode, KAL_TRUE for normal output
    .current_scenario_id = MSDK_SCENARIO_ID_CAMERA_PREVIEW,//current scenario id
    .ihdr_en = 0, //sensor need support LE, SE with HDR feature
    .i2c_write_id = 0x78,//record current sensor's i2c write id //5409H---0x78 /5409/5409V---0x7a   zhanghua
};


/* Sensor output window information */
static struct SENSOR_WINSIZE_INFO_STRUCT imgsensor_winsize_info[5] =
{{ 2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0000, 0000, 2592, 1944}, // Preview 2112*1558
	{ 2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0000, 0000, 2592, 1944}, // capture 4206*3128
	{ 2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0000, 0000, 2592, 1944}, // video
	{ 2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0000, 0000, 2592, 1944}, //hight speed video
	{ 2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0000, 0000, 2592, 1944},// slim video
};


static kal_uint16 read_cmos_sensor(kal_uint32 addr)
{
    kal_uint16 get_byte=0;
	char pu_send_cmd[1] = { (char)(addr & 0xFF) };

	kdSetI2CSpeed(imgsensor_info.i2c_speed); // Add this func to set i2c speed by each sensor

    iReadRegI2C(pu_send_cmd, 1, (u8*)&get_byte, 1, imgsensor.i2c_write_id);

    return get_byte;
}

static void write_cmos_sensor(kal_uint32 addr, kal_uint32 para)
{
    char pu_send_cmd[2] = { (char)(addr & 0xFF), (char)(para & 0xFF)};

	kdSetI2CSpeed(imgsensor_info.i2c_speed); // Add this func to set i2c speed by each sensor

    iWriteRegI2C(pu_send_cmd, 2, imgsensor.i2c_write_id);
}
//#define DEBUG_SENSOR
#ifdef DEBUG_SENSOR
	#define SP5508_DD_V590_MIPI_RAW_OP_CODE_INI		0x00		/* Initial value. */
	#define SP5508_DD_V590_MIPI_RAW_OP_CODE_REG		0x01		/* Register */
	#define SP5508_DD_V590_MIPI_RAW_OP_CODE_DLY		0x02		/* Delay */
	#define SP5508_DD_V590_MIPI_RAW_OP_CODE_END		0x03		/* End of initial setting. */


		typedef struct
	{
		u16 init_reg;
		u16 init_val;	/* Save the register value and delay tick */
		u8 op_code;		/* 0 - Initial value, 1 - Register, 2 - Delay, 3 - End of setting. */
	} SP5508_DD_V590_MIPI_RAW_initial_set_struct;

	SP5508_DD_V590_MIPI_RAW_initial_set_struct SP5508_DD_V590_MIPI_RAW_Init_Reg[1000];
	static UINT32 fromsd;//gpwdebug

 static u32 strtol(const char *nptr, u8 base)
{
	u8 ret;
	if(!nptr || (base!=16 && base!=10 && base!=8))
	{
		printk("%s(): NULL pointer input\n", __FUNCTION__);
		return -1;
	}
	for(ret=0; *nptr; nptr++)
	{
		if((base==16 && *nptr>='A' && *nptr<='F') ||
			(base==16 && *nptr>='a' && *nptr<='f') ||
			(base>=10 && *nptr>='0' && *nptr<='9') ||
			(base>=8 && *nptr>='0' && *nptr<='7') )
		{
			ret *= base;
			if(base==16 && *nptr>='A' && *nptr<='F')
				ret += *nptr-'A'+10;
			else if(base==16 && *nptr>='a' && *nptr<='f')
				ret += *nptr-'a'+10;
			else if(base>=10 && *nptr>='0' && *nptr<='9')
				ret += *nptr-'0';
			else if(base>=8 && *nptr>='0' && *nptr<='7')
				ret += *nptr-'0';
		}
		else
			return ret;
	}
	return ret;
}

 u8 SP5508_DD_V590_MIPI_RAW_Initialize_from_T_Flash()
{
	//FS_HANDLE fp = -1;				/* Default, no file opened. */
	//u8 *data_buff = NULL;
	u8 *curr_ptr = NULL;
	u32 file_size = 0;
	//u32 bytes_read = 0;
	u32 i = 0, j = 0;
	u8 func_ind[4] = {0};	/* REG or DLY */


	struct file *fp;
	mm_segment_t fs;
	loff_t pos = 0;
	static u8 data_buff[10*1024] ;

	fp = filp_open("/mnt/sdcard/sp5508_sd", O_RDONLY , 0);
	if (IS_ERR(fp))
	{
		printk("create file error\n");
		return -1;
	}
	fs = get_fs();
	set_fs(KERNEL_DS);

	file_size = vfs_llseek(fp, 0, SEEK_END);
	vfs_read(fp, data_buff, file_size, &pos);
	//printk("%s %d %d\n", buf,iFileLen,pos);
	filp_close(fp, NULL);
	set_fs(fs);

	/* Start parse the setting witch read from t-flash. */
	curr_ptr = data_buff;
	while (curr_ptr < (data_buff + file_size))
	{
		while ((*curr_ptr == ' ') || (*curr_ptr == '\t'))/* Skip the Space & TAB */
			curr_ptr++;

		if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '*'))
		{
			while (!(((*curr_ptr) == '*') && ((*(curr_ptr + 1)) == '/')))
			{
				curr_ptr++;		/* Skip block comment code. */
			}

			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}

		if (((*curr_ptr) == '/') || ((*curr_ptr) == '{') || ((*curr_ptr) == '}'))		/* Comment line, skip it. */
		{
			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}
		/* This just content one enter line. */
		if (((*curr_ptr) == 0x0D) && ((*(curr_ptr + 1)) == 0x0A))
		{
			curr_ptr += 2;
			continue ;
		}
		//printk(" curr_ptr1 = %s\n",curr_ptr);
		memcpy(func_ind, curr_ptr, 3);


		if (strcmp((const char *)func_ind, "REG") == 0)		/* REG */
		{
			curr_ptr += 6;				/* Skip "REG(0x" or "DLY(" */
			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].op_code = SP5508_DD_V590_MIPI_RAW_OP_CODE_REG;

			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_reg = strtol((const char *)curr_ptr, 16);
			curr_ptr += 5;	/* Skip "00, 0x" */

			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_val = strtol((const char *)curr_ptr, 16);
			curr_ptr += 4;	/* Skip "00);" */

		}
		else 									/* DLY */
		{
			/* Need add delay for this setting. */
			curr_ptr += 4;
			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].op_code = SP5508_DD_V590_MIPI_RAW_OP_CODE_DLY;

			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_reg = 0xFF;
			SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_val = strtol((const char *)curr_ptr,  10);	/* Get the delay ticks, the delay should less then 50 */
		}
		i++;


		/* Skip to next line directly. */
		while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
		{
			curr_ptr++;
		}
		curr_ptr += 2;
	}

	/* (0xFFFF, 0xFFFF) means the end of initial setting. */
	SP5508_DD_V590_MIPI_RAW_Init_Reg[i].op_code = SP5508_DD_V590_MIPI_RAW_OP_CODE_END;
	SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_reg = 0xFF;
	SP5508_DD_V590_MIPI_RAW_Init_Reg[i].init_val = 0xFF;
	i++;
	//for (j=0; j<i; j++)
		//printk(" %x  ==  %x\n",SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_reg, SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_val);

	/* Start apply the initial setting to sensor. */
	#if 1
	for (j=0; j<i; j++)
	{
		if (SP5508_DD_V590_MIPI_RAW_Init_Reg[j].op_code == SP5508_DD_V590_MIPI_RAW_OP_CODE_END)	/* End of the setting. */
		{
			break ;
		}
		else if (SP5508_DD_V590_MIPI_RAW_Init_Reg[j].op_code == SP5508_DD_V590_MIPI_RAW_OP_CODE_DLY)
		{
			msleep(SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_val);		/* Delay */
		}
		else if (SP5508_DD_V590_MIPI_RAW_Init_Reg[j].op_code == SP5508_DD_V590_MIPI_RAW_OP_CODE_REG)
		{

			write_cmos_sensor(SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_reg, SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_val);
			printk("%x = %x\n",SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_reg,SP5508_DD_V590_MIPI_RAW_Init_Reg[j].init_val);
		}
		else
		{
			printk("REG ERROR!\n");
		}
	}
#endif

	return 1;
}

#endif
static void set_dummy(void)
{
   LOG_INF("dummyline = %d, dummypixels = %d \n", imgsensor.dummy_line, imgsensor.dummy_pixel);
    /* you can set dummy by imgsensor.dummy_line and imgsensor.dummy_pixel, or you can set dummy by imgsensor.frame_length and imgsensor.line_length */

	write_cmos_sensor(0xfd, 0x01);
	write_cmos_sensor(0x05, (imgsensor.dummy_line >> 8) & 0xFF);
	write_cmos_sensor(0x06, imgsensor.dummy_line & 0xFF);
	write_cmos_sensor(0x01, 0x01);

}    /*    set_dummy  */

static kal_uint32 return_sensor_id(void)
{
    write_cmos_sensor(0xfd, 0x00);
    return ((read_cmos_sensor(0x02) << 8) | read_cmos_sensor(0x03));
}
static void set_max_framerate(UINT16 framerate,kal_bool min_framelength_en)
{
    kal_uint32 frame_length = imgsensor.frame_length;

   // LOG_INF("framerate = %d, min framelength should enable? \n", framerate,min_framelength_en);

    frame_length = imgsensor.pclk / framerate * 10 / imgsensor.line_length;

    spin_lock(&imgsensor_drv_lock);
    imgsensor.frame_length = (frame_length > imgsensor.min_frame_length) ? frame_length : imgsensor.min_frame_length;
    imgsensor.dummy_line = imgsensor.frame_length - imgsensor.min_frame_length;
    //dummy_line = frame_length - imgsensor.min_frame_length;
    //if (dummy_line < 0)
        //imgsensor.dummy_line = 0;
    //else
        //imgsensor.dummy_line = dummy_line;
    //imgsensor.frame_length = frame_length + imgsensor.dummy_line;
    if (imgsensor.frame_length > imgsensor_info.max_frame_length)
    {
        imgsensor.frame_length = imgsensor_info.max_frame_length;
        imgsensor.dummy_line = imgsensor.frame_length - imgsensor.min_frame_length;
    }
    if (min_framelength_en)
        imgsensor.min_frame_length = imgsensor.frame_length;
    spin_unlock(&imgsensor_drv_lock);

    set_dummy();
}    /*    set_max_framerate  */

static void write_shutter(kal_uint16 shutter)
{
	LOG_INF("huazai_shutter = %d == %x\n",shutter,shutter);
	// Update Shutter
    shutter = (shutter >> 1) << 1;	//Set shutter as even num
	write_cmos_sensor(0xfd, 0x01);
	write_cmos_sensor(0x03, (shutter >> 8) & 0xFF);
	write_cmos_sensor(0x04, shutter  & 0xFF);
	write_cmos_sensor(0x01, 0x01);


	LOG_INF("huazai_read 0x03 = 0x%x,0x04 == 0x%x\n",read_cmos_sensor(0x03),read_cmos_sensor(0x04));
    //LOG_INF("shutter =%d, framelength =%d\n", shutter, imgsensor.frame_length);

}/*	write_shutter  */


/*************************************************************************
* FUNCTION
*    set_shutter
*
* DESCRIPTION
*    This function set e-shutter of sensor to change exposure time.
*
* PARAMETERS
*    iShutter : exposured lines
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static void set_shutter(kal_uint16 shutter)
{
	unsigned long flags;

	if (shutter < 4)
		shutter = 4;
	else if(shutter > 0x7ff3)
		shutter = 0x7ff3;

    spin_lock_irqsave(&imgsensor_drv_lock, flags);
    imgsensor.shutter = shutter;
    spin_unlock_irqrestore(&imgsensor_drv_lock, flags);

    write_shutter(shutter);
}


/*************************************************************************
* FUNCTION
*    set_gain
*
* DESCRIPTION
*    This function is to set global gain to sensor.
*
* PARAMETERS
*    iGain : sensor global gain(base: 0x40)
*
* RETURNS
*    the actually gain set to sensor.
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint16 set_gain(kal_uint16 gain)
{
    kal_uint16 reg_gain = 0x0000;

    if(gain >= BASEGAIN && gain <= 16*BASEGAIN)
   	 {
   	    	reg_gain = 0x10 * gain/BASEGAIN ;        //change mtk gain base to aptina gain base

   	    	 if(reg_gain<=0x10)
   	    	 {
   	    	    	write_cmos_sensor(0xfd, 0x01);
   	    	    	write_cmos_sensor(0x24, 0x10);//0x23
   	    	    	write_cmos_sensor(0x01, 0x01);
   	    	   	  LOG_INF("SP5508_DD_V590_MIPI_RAW reg_gain =%d, SP5508_DD_V590_MIPI_RAW Gain = %d", reg_gain, gain);
   	    	 }
   	    	 else if(reg_gain>= 0xf8)
   	    	 {
   	    	      write_cmos_sensor(0xfd, 0x01);
   	    	    	write_cmos_sensor(0x24, 0xf8);
   	    	    	write_cmos_sensor(0x01, 0x01);
   	    	      LOG_INF("SP5508_DD_V590_MIPI_RAW reg_gain =%d, SP5508_DD_V590_MIPI_RAW Gain = %d",  reg_gain, gain);
	        }
   	    	 else
   	    	{
   	    	    	 write_cmos_sensor(0xfd, 0x01);
   	    	    	 write_cmos_sensor(0x24, (kal_uint8)reg_gain);
   	    	    	 write_cmos_sensor(0x01, 0x01);
   	    	    	 LOG_INF("SP5508_DD_V590_MIPI_RAW reg_gain =%d, SP5508_DD_V590_MIPI_RAW Gain = %d",  reg_gain, gain);
	        }
   	 }
   	 else
   	    	 LOG_INF("error gain setting");

	  return gain;
}    /*    set_gain  */

static void ihdr_write_shutter_gain(kal_uint16 le, kal_uint16 se, kal_uint16 gain)
{
    //not support HDR
    //LOG_INF("le:0x%x, se:0x%x, gain:0x%x\n",le,se,gain);
}


#if 0
static void set_mirror_flip(kal_uint8 image_mirror)
{
  //  LOG_INF("image_mirror = %d\n", image_mirror);

    /********************************************************
       *

       *   ISP and Sensor flip or mirror register bit should be the same!!
       *
       ********************************************************/
    switch (image_mirror) {
        case IMAGE_NORMAL:

            break;
        case IMAGE_H_MIRROR:

            break;
        case IMAGE_V_MIRROR:

            break;
        case IMAGE_HV_MIRROR:

            break;
        default:
  //          LOG_INF("Error image_mirror setting\n");
    }
}
#endif

/*************************************************************************
* FUNCTION
*    night_mode
*
* DESCRIPTION
*    This function night mode of sensor.
*
* PARAMETERS
*    bEnable: KAL_TRUE -> enable night mode, otherwise, disable night mode
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static void night_mode(kal_bool enable)
{
/*No Need to implement this function*/
}    /*    night_mode    */

static void sensor_init(void)
    /*    MIPI_sensor_Init  */
{
    #ifdef DEBUG_SENSOR
	if(fromsd == 1)//\CAǷ\F1\B4\D3SD\B6\C1ȡ//gepeiwei   120903
		{
		LOG_INF("preview_setting1\n");
		SP5508_DD_V590_MIPI_RAW_Initialize_from_T_Flash();//\B4\D3SD\BF\A8\B6\C1ȡ\B5\C4\D6\F7Ҫ\BA\AF\CA\FD

		}
       else
    #endif

	{
		write_cmos_sensor(0xfd,0x00);
		write_cmos_sensor(0x2e,0x24);
		write_cmos_sensor(0x2f,0x01);
		write_cmos_sensor(0xfd,0x01);
		write_cmos_sensor(0x3f,0x02);   //mirror&flip
		write_cmos_sensor(0xa0,0x00);   //;mipi buf en
		write_cmos_sensor(0x03,0x07);   //;30ms
		write_cmos_sensor(0x04,0x02);
		write_cmos_sensor(0x06,0x01);
		write_cmos_sensor(0x24,0xff);
		write_cmos_sensor(0x39,0x0b);
		write_cmos_sensor(0x31,0x00);
		write_cmos_sensor(0x01,0x01);
		write_cmos_sensor(0x11,0x60);
		write_cmos_sensor(0x33,0xb0);
		write_cmos_sensor(0x12,0x03);
		write_cmos_sensor(0x13,0xd0);
		write_cmos_sensor(0x45,0x1e);
		write_cmos_sensor(0x16,0xea);
		write_cmos_sensor(0x19,0xf7);
		write_cmos_sensor(0x1a,0x5f);
		write_cmos_sensor(0x1c,0x0c);
		write_cmos_sensor(0x1d,0x06);
		write_cmos_sensor(0x1e,0x09);
		write_cmos_sensor(0x20,0x07);
		write_cmos_sensor(0x2a,0x0f);
		write_cmos_sensor(0x2c,0x10);
		write_cmos_sensor(0x25,0x0d);
		write_cmos_sensor(0x26,0x0d);
		write_cmos_sensor(0x27,0x08);
		write_cmos_sensor(0x29,0x01);
		write_cmos_sensor(0x55,0x14);
		write_cmos_sensor(0x56,0x00);
		write_cmos_sensor(0x57,0x17);
		write_cmos_sensor(0x59,0x00);
		write_cmos_sensor(0x5a,0x04);
		write_cmos_sensor(0x50,0x10);
		write_cmos_sensor(0x53,0x0e);
		write_cmos_sensor(0x6b,0x10);
		write_cmos_sensor(0x5c,0x20);
		write_cmos_sensor(0x5d,0x00);
		write_cmos_sensor(0x5e,0x06);
		write_cmos_sensor(0x66,0x38);
		write_cmos_sensor(0x68,0x30);
		write_cmos_sensor(0x71,0x3f);
		write_cmos_sensor(0x72,0x05);
		write_cmos_sensor(0x73,0x3e);
		write_cmos_sensor(0x81,0x22);
		write_cmos_sensor(0x8a,0x66);
		write_cmos_sensor(0x8b,0x66);
		write_cmos_sensor(0xc0,0x02);
		write_cmos_sensor(0xc1,0x02);
		write_cmos_sensor(0xc2,0x02);
		write_cmos_sensor(0xc3,0x02);
		write_cmos_sensor(0xc4,0x82);
		write_cmos_sensor(0xc5,0x82);
		write_cmos_sensor(0xc6,0x82);
		write_cmos_sensor(0xc7,0x82);
		write_cmos_sensor(0xfb,0x43);
		//write_cmos_sensor(0xf0,0x2e);
		//write_cmos_sensor(0xf1,0x2e);
		//write_cmos_sensor(0xf2,0x2e);
		//write_cmos_sensor(0xf3,0x2e);
		write_cmos_sensor(0xb1,0xad);
		write_cmos_sensor(0xb6,0x42);
		write_cmos_sensor(0xa1,0x04);
		//write_cmos_sensor(0xa0,0x01);
		write_cmos_sensor(0xfd,0x02);
		write_cmos_sensor(0x34,0xc8);
		write_cmos_sensor(0x60,0x99);
		write_cmos_sensor(0x93,0x03);
		write_cmos_sensor(0x18,0xe0);
		write_cmos_sensor(0x19,0xa0);
		write_cmos_sensor(0xfd,0x04);
		write_cmos_sensor(0x31,0x4b);
		write_cmos_sensor(0x32,0x4b);
		//write_cmos_sensor(0xfd,0x03);
		//write_cmos_sensor(0xc0,0x01);
		write_cmos_sensor(0xfd,0x01);
		write_cmos_sensor(0x8e,0x0a);
		write_cmos_sensor(0x8f,0x30);
		write_cmos_sensor(0x90,0x07);
		write_cmos_sensor(0x91,0xa8);
		write_cmos_sensor(0xfd,0x02);
		write_cmos_sensor(0xd0,0x40);   //;offset
        write_cmos_sensor(0xd1,0x40);
        write_cmos_sensor(0xd2,0x40);
        write_cmos_sensor(0xd3,0x40);
		write_cmos_sensor(0xfd,0x01);
		//write_cmos_sensor(0xa0,0x00);
		//write_cmos_sensor(0xa1,0x08);
		//write_cmos_sensor(0xa2,0x07);
		//write_cmos_sensor(0xa3,0x98);
		//write_cmos_sensor(0xa4,0x00);
		//write_cmos_sensor(0xa5,0x08);
		//write_cmos_sensor(0xa6,0x0a);
		//write_cmos_sensor(0xa7,0x20);
		//write_cmos_sensor(0xfd,0x01);

	}
}    /*    MIPI_sensor_Init  */

static void preview_setting(void)
{
	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0xa0,0x01);
}    /*    preview_setting  */


static void capture_setting(kal_uint16 currefps)
{
	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0xa0,0x01);
}    /*    capture_setting  */

static void normal_video_setting(kal_uint16 currefps)
{
	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0xa0,0x01);
}    /*    preview_setting  */


static void video_1080p_setting(void)
{
	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0xa0,0x01);
}    /*    preview_setting  */

static void video_720p_setting(void)
{
	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0xa0,0x01);
}    /*    preview_setting  */


static void hs_video_setting(void)
{
  //  LOG_INF("E\n");

    video_1080p_setting();
}

static void slim_video_setting(void)
{
  //  LOG_INF("E\n");

    video_720p_setting();
}

kal_uint16 sp5508_otp_info (void) {
	//kal_uint16 group1_flag = 0;
	//kal_uint16 group2_flag = 0;
	kal_uint16 module_id = 0;
	kal_uint16 otp_status = 0;
	kal_uint16 rg_ration = 0;
	//kal_uint16 rgh_bgh_ration = 0;
	kal_uint16 bg_ration = 0;
	kal_uint16 awb_checksum = 0;
	kal_uint16 r_gain = 0x80;
	kal_uint16 b_gain = 0x80;
	kal_uint16 g_gain = 0x80;
	//kal_uint16 awb_checksum_cal = 0;
	kal_uint16 golden_rg_gain = 83;
	kal_uint16 golden_bg_gain = 86;
	kal_uint16 temp_grp_flag = 0;
	kal_uint16 temp_awb_flag = 0;
	//kal_uint16 awb_grp1_flag = 0;
	//kal_uint16 awb_grp2_flag = 0;

	write_cmos_sensor(0xfd,0x03);
	otp_status = read_cmos_sensor(0xec);

	LOG_INF("sp5508 otp_status=0x%x\n", otp_status);
	if (otp_status != 0) {
		module_id = 0x100;
		return module_id;
	}

	write_cmos_sensor(0xfd,0x05);
	temp_grp_flag = read_cmos_sensor(0x80);
	//group1_flag = (temp_grp_flag&0x0c);
	//group2_flag = (temp_grp_flag&0x03);
	LOG_INF("sp5508 otp flag of base info=%x\n",
					temp_grp_flag);

	temp_awb_flag = read_cmos_sensor(0xa5);
	//awb_grp1_flag = (temp_awb_flag&0x0c);
	//awb_grp2_flag = (temp_awb_flag&0x03);
	LOG_INF("sp5508 otp awb_flag=%x\n",
					temp_awb_flag);

	if (temp_grp_flag == 0x01 ) {
		module_id = read_cmos_sensor(0x81);
		//rgl_ration = read_cmos_sensor(0x8e);
		//rgh_bgh_ration = read_cmos_sensor(0x8f);
		//bgl_ration = read_cmos_sensor(0x90);
		//awb_checksum = read_cmos_sensor(0x91);
	}
	if (temp_grp_flag == 0x13) {
		module_id = read_cmos_sensor(0x8d);
		//rgl_ration = read_cmos_sensor(0x8e);
		//rgh_bgh_ration = read_cmos_sensor(0x8f);
		//bgl_ration = read_cmos_sensor(0x90);
		//awb_checksum = read_cmos_sensor(0x91);
	}
	if (temp_grp_flag == 0x37) {
		module_id = read_cmos_sensor(0x99);
		//rgl_ration = read_cmos_sensor(0x8e);
		//rgh_bgh_ration = read_cmos_sensor(0x8f);
		//bgl_ration = read_cmos_sensor(0x90);
		//awb_checksum = read_cmos_sensor(0x91);
	}

	if (temp_awb_flag == 0x01) {
		rg_ration = read_cmos_sensor(0xa6);
		bg_ration = read_cmos_sensor(0xa7);
		awb_checksum = read_cmos_sensor(0xaf);
	}
	if (temp_awb_flag == 0x13) {
		rg_ration = read_cmos_sensor(0xb0);
		bg_ration = read_cmos_sensor(0xb1);
		awb_checksum = read_cmos_sensor(0xb9);
	}
	if (temp_awb_flag == 0x37) {
		rg_ration = read_cmos_sensor(0xba);
		bg_ration = read_cmos_sensor(0xbb);
		awb_checksum = read_cmos_sensor(0xc3);
	}

	printk("sp5508 otp mid=0x%x\n", module_id);
	printk("sp5508 otp rg:%x,  bg:%x, awb_crc:%x\n",
								rg_ration,
								bg_ration,
								awb_checksum);

	//awb_checksum_cal = (rg_ration + rgh_bgh_ration + bg_ration) % 255 + 1;
	//LOG_INF("sp5508 otp awb_checksum_cal=0x%x, awb_checksum=0x%x\n",awb_checksum_cal, awb_checksum);

	r_gain = 0x80 * golden_rg_gain/rg_ration ;
	b_gain = 0x80 * golden_bg_gain/bg_ration ;
	LOG_INF("sp5508 otp r_gain=0x%x, b_gain=0x%x\n", r_gain, b_gain);
	if (r_gain < b_gain) {
		if (r_gain < 0x80) {
			g_gain = 0x80 * g_gain/r_gain;
			b_gain = 0x80 * b_gain/r_gain;
			r_gain = 0x80;
		}
	} else {
		if (b_gain < 0x80) {
			r_gain = 0x80 * r_gain/b_gain;
			g_gain = 0x80 * g_gain/b_gain;
			b_gain = 0x80;
		}
	}

	printk("sp5508 otp r_gain:%x,g_gain:%x, b_gain:%x\n",
								r_gain, g_gain, b_gain);

	write_cmos_sensor(0xfd,0x01);
	write_cmos_sensor(0x40,r_gain);
	write_cmos_sensor(0x41,g_gain);
	write_cmos_sensor(0x42,g_gain);
	write_cmos_sensor(0x43,b_gain);

	return module_id;
}

/*************************************************************************
* FUNCTION
*    get_imgsensor_id
*
* DESCRIPTION
*    This function get the sensor ID
*
* PARAMETERS
*    *sensorID : return the sensor ID
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/

static kal_uint32 get_imgsensor_id(UINT32 *sensor_id)
{
    kal_uint8 i = 0;
    kal_uint8 retry = 2;

    //sensor have two i2c address 0x78 & 0x7a, we should detect the module used i2c address
    while (imgsensor_info.i2c_addr_table[i] != 0xff) {
        spin_lock(&imgsensor_drv_lock);
        imgsensor.i2c_write_id = imgsensor_info.i2c_addr_table[i];
        spin_unlock(&imgsensor_drv_lock);
        do {
            *sensor_id = return_sensor_id();
            if (*sensor_id == imgsensor_info.sensor_id) {
		#ifdef CONFIG_T_PRODUCT_INFO
   		FULL_PRODUCT_DEVICE_INFO_CAMERA(SP5508_DD_V590_SENSOR_ID, 1, "sp5508_dd_v590_mipi_sensor", 2592,1944);
		#endif
                printk("SP5508 superpix i2c write id: 0x%x, sensor id: 0x%x\n", imgsensor.i2c_write_id,*sensor_id);
                return ERROR_NONE;
            }
            printk("SP5508 superpix Read sensor id fail, id: 0x%x, sensor id: 0x%x\n", imgsensor.i2c_write_id,*sensor_id);
            retry--;
        } while(retry > 0);
        i++;
        retry = 2;
    }
    if (*sensor_id != imgsensor_info.sensor_id) {
        // if Sensor ID is not correct, Must set *sensor_id to 0xFFFFFFFF
        *sensor_id = 0xFFFFFFFF;
        return ERROR_SENSOR_CONNECT_FAIL;
    }
    return ERROR_NONE;
}


/*************************************************************************
* FUNCTION
*    open
*
* DESCRIPTION
*    This function initialize the registers of CMOS sensor
*
* PARAMETERS
*    None
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 open(void)
{
    kal_uint8 i = 0;
    kal_uint8 retry = 2;
    kal_uint32 sensor_id = 0;
	kal_uint8 moduleID = 0;
    //sensor have two i2c address 0x6c 0x6d & 0x21 0x20, we should detect the module used i2c address
    while (imgsensor_info.i2c_addr_table[i] != 0xff) {
        spin_lock(&imgsensor_drv_lock);
        imgsensor.i2c_write_id = imgsensor_info.i2c_addr_table[i];
        spin_unlock(&imgsensor_drv_lock);
        do {
            sensor_id = return_sensor_id();
            if (sensor_id == imgsensor_info.sensor_id) {
                LOG_INF("2015-2-6 superpix i2c write id: 0x%x, sensor id: 0x%x\n", imgsensor.i2c_write_id,sensor_id);
                break;
            }
            LOG_INF("2015-2-6 superpix Read sensor id fail, id: 0x%x\n", imgsensor.i2c_write_id,sensor_id);
            retry--;
        } while(retry > 0);
        i++;
        if (sensor_id == imgsensor_info.sensor_id)
            break;
        retry = 2;
    }
    if (imgsensor_info.sensor_id != sensor_id)
        return ERROR_SENSOR_CONNECT_FAIL;
	#ifdef DEBUG_SENSOR  //
      struct file *fp;
         mm_segment_t fs;
         loff_t pos = 0;
	     static char buf[10*1024] ;

          fp = filp_open("/mnt/sdcard/sp5508_sd", O_RDONLY , 0);
          if (IS_ERR(fp)) {
		     fromsd = 0;
		     printk("gpww open file error\n");
		     //return 0;
           }

	    else
	    {
		    fromsd = 1;
	          //SP5508_DD_V590_MIPI_RAW_Initialize_from_T_Flash();
	          printk("gpww read ok!\n");

	           filp_close(fp, NULL);
                  set_fs(fs);
		}
          #endif

    /* initail sequence write in  */
    sensor_init();
	moduleID = sp5508_otp_info();
    spin_lock(&imgsensor_drv_lock);

    imgsensor.autoflicker_en= KAL_FALSE;
    imgsensor.sensor_mode = IMGSENSOR_MODE_INIT;
    imgsensor.pclk = imgsensor_info.pre.pclk;
    imgsensor.frame_length = imgsensor_info.pre.framelength;
    imgsensor.line_length = imgsensor_info.pre.linelength;
    imgsensor.min_frame_length = imgsensor_info.pre.framelength;
    imgsensor.dummy_pixel = 0;
    imgsensor.dummy_line = 0;
    imgsensor.ihdr_en = 0;
    imgsensor.test_pattern = KAL_FALSE;
    imgsensor.current_fps = imgsensor_info.pre.max_framerate;
    spin_unlock(&imgsensor_drv_lock);

    return ERROR_NONE;
}    /*    open  */



/*************************************************************************
* FUNCTION
*    close
*
* DESCRIPTION
*
*
* PARAMETERS
*    None
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 close(void)
{
    LOG_INF("E\n");

    /*No Need to implement this function*/

    return ERROR_NONE;
}    /*    close  */


/*************************************************************************
* FUNCTION
* preview
*
* DESCRIPTION
*    This function start the sensor preview.
*
* PARAMETERS
*    *image_window : address pointer of pixel numbers in one period of HSYNC
*  *sensor_config_data : address pointer of line numbers in one period of VSYNC
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 preview(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_PREVIEW;
    imgsensor.pclk = imgsensor_info.pre.pclk;
    //imgsensor.video_mode = KAL_FALSE;
    imgsensor.line_length = imgsensor_info.pre.linelength;
    imgsensor.frame_length = imgsensor_info.pre.framelength;
    imgsensor.min_frame_length = imgsensor_info.pre.framelength;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    preview_setting();
    //set_mirror_flip(sensor_config_data->SensorImageMirror);
    return ERROR_NONE;
}    /*    preview   */

/*************************************************************************
* FUNCTION
*    capture
*
* DESCRIPTION
*    This function setup the CMOS sensor in capture MY_OUTPUT mode
*
* PARAMETERS
*
* RETURNS
*    None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 capture(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                          MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_CAPTURE;
    if (imgsensor.current_fps == imgsensor_info.cap1.max_framerate) {//PIP capture: 24fps for less than 13M, 20fps for 16M,15fps for 20M
        imgsensor.pclk = imgsensor_info.cap1.pclk;
        imgsensor.line_length = imgsensor_info.cap1.linelength;
        imgsensor.frame_length = imgsensor_info.cap1.framelength;
        imgsensor.min_frame_length = imgsensor_info.cap1.framelength;
        imgsensor.autoflicker_en = KAL_FALSE;
    } else {
        if (imgsensor.current_fps != imgsensor_info.cap.max_framerate)
            LOG_INF("Warning: current_fps %d fps is not support, so use cap1's setting: %d fps!\n",imgsensor.current_fps,imgsensor_info.cap1.max_framerate/10);
        imgsensor.pclk = imgsensor_info.cap.pclk;
        imgsensor.line_length = imgsensor_info.cap.linelength;
        imgsensor.frame_length = imgsensor_info.cap.framelength;
        imgsensor.min_frame_length = imgsensor_info.cap.framelength;
        imgsensor.autoflicker_en = KAL_FALSE;
    }
    spin_unlock(&imgsensor_drv_lock);
    capture_setting(imgsensor.current_fps);
    //set_mirror_flip(sensor_config_data->SensorImageMirror);
    return ERROR_NONE;
}    /* capture() */
static kal_uint32 normal_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_VIDEO;
    imgsensor.pclk = imgsensor_info.normal_video.pclk;
    imgsensor.line_length = imgsensor_info.normal_video.linelength;
    imgsensor.frame_length = imgsensor_info.normal_video.framelength;
    imgsensor.min_frame_length = imgsensor_info.normal_video.framelength;
    //imgsensor.current_fps = 300;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    normal_video_setting(imgsensor.current_fps);
    //set_mirror_flip(sensor_config_data->SensorImageMirror);
    return ERROR_NONE;
}    /*    normal_video   */

static kal_uint32 hs_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_HIGH_SPEED_VIDEO;
    imgsensor.pclk = imgsensor_info.hs_video.pclk;
    //imgsensor.video_mode = KAL_TRUE;
    imgsensor.line_length = imgsensor_info.hs_video.linelength;
    imgsensor.frame_length = imgsensor_info.hs_video.framelength;
    imgsensor.min_frame_length = imgsensor_info.hs_video.framelength;
    imgsensor.dummy_line = 0;
    imgsensor.dummy_pixel = 0;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    hs_video_setting();
    //set_mirror_flip(sensor_config_data->SensorImageMirror);
    return ERROR_NONE;
}    /*    hs_video   */

static kal_uint32 slim_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_SLIM_VIDEO;
    imgsensor.pclk = imgsensor_info.slim_video.pclk;
    imgsensor.line_length = imgsensor_info.slim_video.linelength;
    imgsensor.frame_length = imgsensor_info.slim_video.framelength;
    imgsensor.min_frame_length = imgsensor_info.slim_video.framelength;
    imgsensor.dummy_line = 0;
    imgsensor.dummy_pixel = 0;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    slim_video_setting();
    //set_mirror_flip(sensor_config_data->SensorImageMirror);

    return ERROR_NONE;
}    /*    slim_video     */



static kal_uint32 get_resolution(MSDK_SENSOR_RESOLUTION_INFO_STRUCT *sensor_resolution)
{
    LOG_INF("E\n");
    sensor_resolution->SensorFullWidth = imgsensor_info.cap.grabwindow_width;
    sensor_resolution->SensorFullHeight = imgsensor_info.cap.grabwindow_height;

    sensor_resolution->SensorPreviewWidth = imgsensor_info.pre.grabwindow_width;
    sensor_resolution->SensorPreviewHeight = imgsensor_info.pre.grabwindow_height;

    sensor_resolution->SensorVideoWidth = imgsensor_info.normal_video.grabwindow_width;
    sensor_resolution->SensorVideoHeight = imgsensor_info.normal_video.grabwindow_height;


    sensor_resolution->SensorHighSpeedVideoWidth     = imgsensor_info.hs_video.grabwindow_width;
    sensor_resolution->SensorHighSpeedVideoHeight     = imgsensor_info.hs_video.grabwindow_height;

    sensor_resolution->SensorSlimVideoWidth     = imgsensor_info.slim_video.grabwindow_width;
    sensor_resolution->SensorSlimVideoHeight     = imgsensor_info.slim_video.grabwindow_height;
    return ERROR_NONE;
}    /*    get_resolution    */

static kal_uint32 get_info(enum MSDK_SCENARIO_ID_ENUM scenario_id,
                      MSDK_SENSOR_INFO_STRUCT *sensor_info,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("scenario_id = %d\n", scenario_id);


    //sensor_info->SensorVideoFrameRate = imgsensor_info.normal_video.max_framerate/10; /* not use */
    //sensor_info->SensorStillCaptureFrameRate= imgsensor_info.cap.max_framerate/10; /* not use */
    //imgsensor_info->SensorWebCamCaptureFrameRate= imgsensor_info.v.max_framerate; /* not use */

    sensor_info->SensorClockPolarity = SENSOR_CLOCK_POLARITY_LOW;
    sensor_info->SensorClockFallingPolarity = SENSOR_CLOCK_POLARITY_LOW; /* not use */
    sensor_info->SensorHsyncPolarity = SENSOR_CLOCK_POLARITY_LOW; // inverse with datasheet
    sensor_info->SensorVsyncPolarity = SENSOR_CLOCK_POLARITY_LOW;
    sensor_info->SensorInterruptDelayLines = 4; /* not use */
    sensor_info->SensorResetActiveHigh = FALSE; /* not use */
    sensor_info->SensorResetDelayCount = 5; /* not use */

    sensor_info->SensroInterfaceType = imgsensor_info.sensor_interface_type;
    sensor_info->MIPIsensorType = imgsensor_info.mipi_sensor_type;
    sensor_info->SettleDelayMode = imgsensor_info.mipi_settle_delay_mode;
    sensor_info->SensorOutputDataFormat = imgsensor_info.sensor_output_dataformat;

    sensor_info->CaptureDelayFrame = imgsensor_info.cap_delay_frame;
    sensor_info->PreviewDelayFrame = imgsensor_info.pre_delay_frame;
    sensor_info->VideoDelayFrame = imgsensor_info.video_delay_frame;
    sensor_info->HighSpeedVideoDelayFrame = imgsensor_info.hs_video_delay_frame;
    sensor_info->SlimVideoDelayFrame = imgsensor_info.slim_video_delay_frame;

    sensor_info->SensorMasterClockSwitch = 0; /* not use */
    sensor_info->SensorDrivingCurrent = imgsensor_info.isp_driving_current;

    sensor_info->AEShutDelayFrame = imgsensor_info.ae_shut_delay_frame;          /* The frame of setting shutter default 0 for TG int */
    sensor_info->AESensorGainDelayFrame = imgsensor_info.ae_sensor_gain_delay_frame;    /* The frame of setting sensor gain */
    sensor_info->AEISPGainDelayFrame = imgsensor_info.ae_ispGain_delay_frame;
    sensor_info->IHDR_Support = imgsensor_info.ihdr_support;
    sensor_info->IHDR_LE_FirstLine = imgsensor_info.ihdr_le_firstline;
    sensor_info->SensorModeNum = imgsensor_info.sensor_mode_num;

    sensor_info->SensorMIPILaneNumber = imgsensor_info.mipi_lane_num;
    sensor_info->SensorClockFreq = imgsensor_info.mclk;
    sensor_info->SensorClockDividCount = 3; /* not use */
    sensor_info->SensorClockRisingCount = 0;
    sensor_info->SensorClockFallingCount = 2; /* not use */
    sensor_info->SensorPixelClockCount = 3; /* not use */
    sensor_info->SensorDataLatchCount = 2; /* not use */

    sensor_info->MIPIDataLowPwr2HighSpeedTermDelayCount = 0;
    sensor_info->MIPICLKLowPwr2HighSpeedTermDelayCount = 0;
    sensor_info->SensorWidthSampling = 0;  // 0 is default 1x
    sensor_info->SensorHightSampling = 0;    // 0 is default 1x
    sensor_info->SensorPacketECCOrder = 1;

    switch (scenario_id) {
        case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
            sensor_info->SensorGrabStartX = imgsensor_info.pre.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.pre.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.pre.mipi_data_lp2hs_settle_dc;

            break;
        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
            sensor_info->SensorGrabStartX = imgsensor_info.cap.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.cap.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.cap.mipi_data_lp2hs_settle_dc;

            break;
        case MSDK_SCENARIO_ID_VIDEO_PREVIEW:

            sensor_info->SensorGrabStartX = imgsensor_info.normal_video.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.normal_video.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.normal_video.mipi_data_lp2hs_settle_dc;

            break;
        case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
            sensor_info->SensorGrabStartX = imgsensor_info.hs_video.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.hs_video.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.hs_video.mipi_data_lp2hs_settle_dc;

            break;
        case MSDK_SCENARIO_ID_SLIM_VIDEO:
            sensor_info->SensorGrabStartX = imgsensor_info.slim_video.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.slim_video.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.slim_video.mipi_data_lp2hs_settle_dc;

            break;
        default:
            sensor_info->SensorGrabStartX = imgsensor_info.pre.startx;
            sensor_info->SensorGrabStartY = imgsensor_info.pre.starty;

            sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.pre.mipi_data_lp2hs_settle_dc;
            break;
    }

    return ERROR_NONE;
}    /*    get_info  */


static kal_uint32 control(enum MSDK_SCENARIO_ID_ENUM scenario_id, MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
    LOG_INF("scenario_id = %d\n", scenario_id);
    spin_lock(&imgsensor_drv_lock);
    imgsensor.current_scenario_id = scenario_id;
    spin_unlock(&imgsensor_drv_lock);
    switch (scenario_id) {
        case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
            preview(image_window, sensor_config_data);
            break;
        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
            capture(image_window, sensor_config_data);
            break;
        case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
            normal_video(image_window, sensor_config_data);
            break;
        case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
            hs_video(image_window, sensor_config_data);
            break;
        case MSDK_SCENARIO_ID_SLIM_VIDEO:
            slim_video(image_window, sensor_config_data);
            break;
        default:
            LOG_INF("Error ScenarioId setting");
            preview(image_window, sensor_config_data);
            return ERROR_INVALID_SCENARIO_ID;
    }
    return ERROR_NONE;
}    /* control() */



static kal_uint32 set_video_mode(UINT16 framerate)
{//This Function not used after ROME
    LOG_INF("framerate = %d\n ", framerate);
    // SetVideoMode Function should fix framerate
    if (framerate == 0)
        // Dynamic frame rate
        return ERROR_NONE;
    spin_lock(&imgsensor_drv_lock);
    if ((framerate == 300) && (imgsensor.autoflicker_en == KAL_TRUE))
        imgsensor.current_fps = 296;
    else if ((framerate == 150) && (imgsensor.autoflicker_en == KAL_TRUE))
        imgsensor.current_fps = 146;
    else
        imgsensor.current_fps = framerate;
    spin_unlock(&imgsensor_drv_lock);
    set_max_framerate(imgsensor.current_fps,1);

    return ERROR_NONE;
}

static kal_uint32 set_auto_flicker_mode(kal_bool enable, UINT16 framerate)
{
    LOG_INF("enable = %d, framerate = %d \n", enable, framerate);
    spin_lock(&imgsensor_drv_lock);
    if (enable) //enable auto flicker
        imgsensor.autoflicker_en = KAL_TRUE;
    else //Cancel Auto flick
        imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    return ERROR_NONE;
}


static kal_uint32 set_max_framerate_by_scenario(enum MSDK_SCENARIO_ID_ENUM scenario_id, MUINT32 framerate)
{
    kal_uint32 frame_length;

    LOG_INF("scenario_id = %d, framerate = %d\n", scenario_id, framerate);

    switch (scenario_id) {
        case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
            frame_length = imgsensor_info.pre.pclk / framerate * 10 / imgsensor_info.pre.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.pre.framelength) ? (frame_length - imgsensor_info.pre.framelength) : 0;
            imgsensor.frame_length = imgsensor_info.pre.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            break;
        case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
            if(framerate == 0)
                return ERROR_NONE;
            frame_length = imgsensor_info.normal_video.pclk / framerate * 10 / imgsensor_info.normal_video.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.normal_video.framelength) ? (frame_length - imgsensor_info.normal_video.framelength) : 0;
            imgsensor.frame_length = imgsensor_info.normal_video.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            break;
        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
            frame_length = imgsensor_info.cap.pclk / framerate * 10 / imgsensor_info.cap.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.cap.framelength) ? (frame_length - imgsensor_info.cap.framelength) : 0;
            imgsensor.frame_length = imgsensor_info.cap.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            break;
        case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
            frame_length = imgsensor_info.hs_video.pclk / framerate * 10 / imgsensor_info.hs_video.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.hs_video.framelength) ? (frame_length - imgsensor_info.hs_video.framelength) : 0;
            imgsensor.frame_length = imgsensor_info.hs_video.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            break;
        case MSDK_SCENARIO_ID_SLIM_VIDEO:
            frame_length = imgsensor_info.slim_video.pclk / framerate * 10 / imgsensor_info.slim_video.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.slim_video.framelength) ? (frame_length - imgsensor_info.slim_video.framelength): 0;
            imgsensor.frame_length = imgsensor_info.slim_video.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            break;
        default:  //coding with  preview scenario by default
            frame_length = imgsensor_info.pre.pclk / framerate * 10 / imgsensor_info.pre.linelength;
            spin_lock(&imgsensor_drv_lock);
            imgsensor.dummy_line = (frame_length > imgsensor_info.pre.framelength) ? (frame_length - imgsensor_info.pre.framelength) : 0;
            imgsensor.frame_length = imgsensor_info.pre.framelength + imgsensor.dummy_line;
            imgsensor.min_frame_length = imgsensor.frame_length;
            spin_unlock(&imgsensor_drv_lock);
            set_dummy();
            LOG_INF("error scenario_id = %d, we use preview scenario \n", scenario_id);
            break;
    }
    return ERROR_NONE;
}


static kal_uint32 get_default_framerate_by_scenario(enum MSDK_SCENARIO_ID_ENUM scenario_id, MUINT32 *framerate)
{
    LOG_INF("scenario_id = %d\n", scenario_id);

    switch (scenario_id) {
        case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
            *framerate = imgsensor_info.pre.max_framerate;
            break;
        case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
            *framerate = imgsensor_info.normal_video.max_framerate;
            break;
        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
            *framerate = imgsensor_info.cap.max_framerate;
            break;
        case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
            *framerate = imgsensor_info.hs_video.max_framerate;
            break;
        case MSDK_SCENARIO_ID_SLIM_VIDEO:
            *framerate = imgsensor_info.slim_video.max_framerate;
            break;
        default:
            break;
    }

    return ERROR_NONE;
}



static kal_uint32 set_test_pattern_mode(kal_bool enable)
{
    LOG_INF("enable: %d\n", enable);

	if (enable) {
		write_cmos_sensor(0xfd,0x01);
		write_cmos_sensor(0x39,0x00);
		write_cmos_sensor(0x01,0x01);
		write_cmos_sensor(0xfd,0x02);
		write_cmos_sensor(0xd0,0x00);
		write_cmos_sensor(0xd1,0x00);
		write_cmos_sensor(0xd2,0x00);
		write_cmos_sensor(0xd3,0x00);
		LOG_INF("sp5508 test_pattern black\n");
	} else {
		write_cmos_sensor(0xfd,0x01);
		write_cmos_sensor(0x39,0x0b);
		write_cmos_sensor(0x01,0x01);
		write_cmos_sensor(0xfd,0x02);
		write_cmos_sensor(0xd0,0x40);
		write_cmos_sensor(0xd1,0x40);
		write_cmos_sensor(0xd2,0x40);
		write_cmos_sensor(0xd3,0x40);
		}

        spin_lock(&imgsensor_drv_lock);
        imgsensor.test_pattern = enable;
        spin_unlock(&imgsensor_drv_lock);
        return ERROR_NONE;
}


static kal_uint32 feature_control(MSDK_SENSOR_FEATURE_ENUM feature_id,
                             UINT8 *feature_para,UINT32 *feature_para_len)
{
    UINT16 *feature_return_para_16=(UINT16 *) feature_para;
    UINT16 *feature_data_16=(UINT16 *) feature_para;
    UINT32 *feature_return_para_32=(UINT32 *) feature_para;
    UINT32 *feature_data_32=(UINT32 *) feature_para;
    unsigned long long *feature_data=(unsigned long long *) feature_para;
    //unsigned long long *feature_return_para=(unsigned long long *) feature_para;

	struct SENSOR_WINSIZE_INFO_STRUCT *wininfo;
    MSDK_SENSOR_REG_INFO_STRUCT *sensor_reg_data=(MSDK_SENSOR_REG_INFO_STRUCT *) feature_para;

    //printk("feature_id = %d\n", feature_id);
    switch (feature_id) {
        case SENSOR_FEATURE_GET_PERIOD:
            *feature_return_para_16++ = imgsensor.line_length;
            *feature_return_para_16 = imgsensor.frame_length;
            *feature_para_len=4;
            break;
        case SENSOR_FEATURE_GET_PIXEL_CLOCK_FREQ:
            LOG_INF("feature_Control imgsensor.pclk = %d,imgsensor.current_fps = %d\n", imgsensor.pclk,imgsensor.current_fps);
            *feature_return_para_32 = imgsensor.pclk;
            *feature_para_len=4;
            break;
        case SENSOR_FEATURE_SET_ESHUTTER:
            set_shutter(*feature_data);
            break;
        case SENSOR_FEATURE_SET_NIGHTMODE:
            night_mode((BOOL) *feature_data);
            break;
        case SENSOR_FEATURE_SET_GAIN:
            set_gain((UINT16) *feature_data);
            break;
        case SENSOR_FEATURE_SET_FLASHLIGHT:
            break;
        case SENSOR_FEATURE_SET_ISP_MASTER_CLOCK_FREQ:
            break;
        case SENSOR_FEATURE_SET_REGISTER:
            write_cmos_sensor(sensor_reg_data->RegAddr, sensor_reg_data->RegData);
            break;
        case SENSOR_FEATURE_GET_REGISTER:
            sensor_reg_data->RegData = read_cmos_sensor(sensor_reg_data->RegAddr);
            break;
        case SENSOR_FEATURE_GET_LENS_DRIVER_ID:
            // get the lens driver ID from EEPROM or just return LENS_DRIVER_ID_DO_NOT_CARE
            // if EEPROM does not exist in camera module.
            *feature_return_para_32=LENS_DRIVER_ID_DO_NOT_CARE;
            *feature_para_len=4;
            break;
        case SENSOR_FEATURE_SET_VIDEO_MODE:
            set_video_mode(*feature_data);
            break;
        case SENSOR_FEATURE_CHECK_SENSOR_ID:
            get_imgsensor_id(feature_return_para_32);
            break;
        case SENSOR_FEATURE_SET_AUTO_FLICKER_MODE:
            set_auto_flicker_mode((BOOL)*feature_data_16,*(feature_data_16+1));
            break;
        case SENSOR_FEATURE_SET_MAX_FRAME_RATE_BY_SCENARIO:
		set_max_framerate_by_scenario((enum MSDK_SCENARIO_ID_ENUM)*feature_data, *(feature_data + 1));
            break;
        case SENSOR_FEATURE_GET_DEFAULT_FRAME_RATE_BY_SCENARIO:
		get_default_framerate_by_scenario((enum MSDK_SCENARIO_ID_ENUM)*(feature_data),
			(MUINT32 *)(uintptr_t)(*(feature_data + 1)));
            break;
        case SENSOR_FEATURE_SET_TEST_PATTERN:
            set_test_pattern_mode((BOOL)*feature_data);
            break;
        case SENSOR_FEATURE_GET_TEST_PATTERN_CHECKSUM_VALUE: //for factory mode auto testing
            *feature_return_para_32 = imgsensor_info.checksum_value;
            *feature_para_len=4;
            break;
        case SENSOR_FEATURE_SET_FRAMERATE:
            LOG_INF("current fps :%d\n", (UINT32)*feature_data);
            spin_lock(&imgsensor_drv_lock);
            imgsensor.current_fps = *feature_data;
            spin_unlock(&imgsensor_drv_lock);
            break;
        case SENSOR_FEATURE_SET_HDR:
            LOG_INF("ihdr enable :%d\n", (BOOL)*feature_data);
            spin_lock(&imgsensor_drv_lock);
            imgsensor.ihdr_en = (BOOL)*feature_data;
            spin_unlock(&imgsensor_drv_lock);
            break;
	case SENSOR_FEATURE_GET_PIXEL_RATE:
		switch (*feature_data) {
			case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			*(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
			(imgsensor_info.cap.pclk /
			(imgsensor_info.cap.linelength - 80))*
			imgsensor_info.cap.grabwindow_width;
			break;
			case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			*(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
			(imgsensor_info.normal_video.pclk /
			(imgsensor_info.normal_video.linelength - 80))*
			imgsensor_info.normal_video.grabwindow_width;
			break;
			case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			*(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
			(imgsensor_info.hs_video.pclk /
			(imgsensor_info.hs_video.linelength - 80))*
			imgsensor_info.hs_video.grabwindow_width;
			break;
			case MSDK_SCENARIO_ID_SLIM_VIDEO:
			*(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
			(imgsensor_info.slim_video.pclk /
			(imgsensor_info.slim_video.linelength - 80))*
			imgsensor_info.slim_video.grabwindow_width;
			break;
			case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			default:
			*(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
			(imgsensor_info.pre.pclk /
			(imgsensor_info.pre.linelength - 80))*
			imgsensor_info.pre.grabwindow_width;
			break;
			}
			break;
	case SENSOR_FEATURE_GET_MIPI_PIXEL_RATE:
	        switch (*feature_data) {
	        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
	        *(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
	        imgsensor_info.cap.mipi_pixel_rate;
	        break;
	        case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
	        *(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
                imgsensor_info.normal_video.mipi_pixel_rate;
                break;
                case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
                *(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
                imgsensor_info.hs_video.mipi_pixel_rate;
                break;
                case MSDK_SCENARIO_ID_SLIM_VIDEO:
                *(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
                imgsensor_info.slim_video.mipi_pixel_rate;
                break;
                case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
                default:
                *(MUINT32 *)(uintptr_t)(*(feature_data + 1)) =
                imgsensor_info.pre.mipi_pixel_rate;
                break;
            }
            break;
        case SENSOR_FEATURE_GET_CROP_INFO:
            LOG_INF("SENSOR_FEATURE_GET_CROP_INFO scenarioId:%d\n", (UINT32)*feature_data);

		wininfo = (struct SENSOR_WINSIZE_INFO_STRUCT *)(uintptr_t)(*(feature_data + 1));

            switch (*feature_data_32) {
                case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			memcpy((void *)wininfo, (void *)&imgsensor_winsize_info[1], sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
			break;
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			memcpy((void *)wininfo, (void *)&imgsensor_winsize_info[2], sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
			break;
		case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			memcpy((void *)wininfo, (void *)&imgsensor_winsize_info[3], sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
			break;
		case MSDK_SCENARIO_ID_SLIM_VIDEO:
			memcpy((void *)wininfo, (void *)&imgsensor_winsize_info[4], sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
			break;
                case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
                default:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[0],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
                    break;
            }
        case SENSOR_FEATURE_SET_IHDR_SHUTTER_GAIN:
            LOG_INF("SENSOR_SET_SENSOR_IHDR LE=%d, SE=%d, Gain=%d\n",(UINT16)*feature_data,(UINT16)*(feature_data+1),(UINT16)*(feature_data+2));
            ihdr_write_shutter_gain((UINT16)*feature_data,(UINT16)*(feature_data+1),(UINT16)*(feature_data+2));
            break;
        default:
            break;
    }

    return ERROR_NONE;
}    /*    feature_control()  */

static struct SENSOR_FUNCTION_STRUCT sensor_func = {
    open,
    get_info,
    get_resolution,
    feature_control,
    control,
    close
};

UINT32 SP5508_DD_V590_MIPI_RAW_SensorInit(struct SENSOR_FUNCTION_STRUCT **pfFunc)
{
    /* To Do : Check Sensor status here */
    if (pfFunc!=NULL)
        *pfFunc=&sensor_func;
    return ERROR_NONE;
}    /*    SP5508_DD_V590_MIPI_RAWSensorInit    */
