/*****************************************************************************
 *
 * Filename:
 * ---------
 *	 IMX214mipi_Sensor.c
 *
 * Project:
 * --------
 *	 ALPS
 *
 * Description:
 * ------------
 *	 Source code of Sensor driver
 *
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by CC/CQ. DO NOT MODIFY!!
 *============================================================================
 ****************************************************************************/

#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/fs.h>
#include <linux/atomic.h>
#include <linux/types.h>

//#include "kd_camera_hw.h"
#include "kd_imgsensor.h"
#include "kd_imgsensor_define.h"
#include "kd_imgsensor_errcode.h"
#include "kd_camera_typedef.h"
#include "s5k5e9yx_dd_u5020_mipi_raw.h"


#define PFX "s5k5e9yx_dd_u5020_mipi_raw"
#define LOG_INF(format, args...)	pr_err(PFX "[%s] " format, __func__, ##args)

static DEFINE_SPINLOCK(imgsensor_drv_lock);
//Tinno Begin Ramiel add camera info ++++
#ifdef  CONFIG_T_PRODUCT_INFO
#include <dev_info.h>
#endif
//Tinno Begin Ramiel add camera info ----

static imgsensor_info_struct imgsensor_info = {
	.sensor_id = S5K5E9YX_DD_U5020_SENSOR_ID,

	.checksum_value = 0xB1F1B3CC,

	.pre = {
    .pclk = 190000000,              //record different mode's pclk
		.linelength = 3112,	/* record different mode's linelength */
		.framelength = 2030,         //record different mode's framelength
		.startx = 0,					//record different mode's startx of grabwindow
		.starty = 0,					//record different mode's starty of grabwindow
		.grabwindow_width = 1296,	/* record different mode's width of grabwindow */
		.grabwindow_height = 972,	/* record different mode's height of grabwindow */
		/*	 following for MIPIDataLowPwr2HighSpeedSettleDelayCount by different scenario	*/
		.mipi_data_lp2hs_settle_dc = 85,
		/*	 following for GetDefaultFramerateByScenario()	*/
		.max_framerate = 300,
	},
	.cap = {
		.pclk =190000000,
		.linelength = 3112,
		.framelength = 2036, //2035: 30.0019
		.startx =0,
		.starty = 0,
		.grabwindow_width = 2592,
		.grabwindow_height = 1944,
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 300,
	},
	#if 0
	.cap1 = {
		.pclk =560000000,
		.linelength = 6400,
		.framelength = 3643,
		.startx = 0,
		.starty = 0,
		.grabwindow_width = 4640,
		.grabwindow_height = 3488,
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 240,
	},
	.cap2 = {
		.pclk =560000000,
		.linelength = 10240,
		.framelength = 3643,
		.startx = 0,
		.starty =0,
		.grabwindow_width = 4640,
		.grabwindow_height = 3488,
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 150,
	},
	#endif
	.normal_video = {
		.pclk =190000000,
		.linelength = 3112,
		.framelength = 2030,
		.startx =0,
		.starty = 0,
		.grabwindow_width = 2592,
		.grabwindow_height = 1460,
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 300,
	},
	.hs_video = {
		.pclk = 190000000,
		.linelength = 3112,
		.framelength =2030,
		.startx = 0,
		.starty = 0,
		.grabwindow_width = 2592,
		.grabwindow_height = 1460,
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 300, //1200
	},
	#if 0
	.slim_video = {
		.pclk = 560000000,              //record different mode's pclk
		.linelength = 5120,	/* record different mode's linelength */
		.framelength = 3644,	/* record different mode's framelength */
		.startx = 0,					//record different mode's startx of grabwindow
		.starty = 0,					//record different mode's starty of grabwindow
		.grabwindow_width = 1920,	/* record different mode's width of grabwindow */
		.grabwindow_height = 1080,	/* record different mode's height of grabwindow */
		.mipi_data_lp2hs_settle_dc = 85,
		.max_framerate = 300,
	},
	#endif

	.custom1 = {
	 .pclk = 190000000,
	 .linelength = 3112,
	   .framelength = 2544,
	   .startx = 0,
	   .starty = 0,
	   .grabwindow_width = 2592,
	   .grabwindow_height = 1944,
	   .mipi_data_lp2hs_settle_dc = 85,
	   .max_framerate = 240,
	},
	.custom2 = {
	   .pclk = 190000000,
	   .linelength = 3112,
	   .framelength = 2544,
	   .startx = 0,
	   .starty = 0,
	   .grabwindow_width = 1296,
	   .grabwindow_height = 972,
	   .mipi_data_lp2hs_settle_dc = 85,
	   .max_framerate = 240,
	},

	.margin = 4,
	.min_shutter = 4,
	.max_frame_length = 0xffff,
	.ae_shut_delay_frame = 0,
	.ae_sensor_gain_delay_frame =0,
	.ae_ispGain_delay_frame = 2,
	.frame_time_delay_frame = 1,
	.ihdr_support = 0,	  //1, support; 0,not support
	.ihdr_le_firstline = 0,  //1,le first ; 0, se first
	.sensor_mode_num = 6,	  //support sensor mode num

	.cap_delay_frame = 3,
	.pre_delay_frame = 3,
	.video_delay_frame = 3,
	.hs_video_delay_frame = 3,
	.slim_video_delay_frame = 3,
	.custom1_delay_frame = 3,
	.custom2_delay_frame = 3,

	.isp_driving_current = ISP_DRIVING_2MA,
	.sensor_interface_type = SENSOR_INTERFACE_TYPE_MIPI,
	.mipi_sensor_type = MIPI_OPHY_NCSI2, //0,MIPI_OPHY_NCSI2;  1,MIPI_OPHY_CSI2
	.mipi_settle_delay_mode = 0, //0,MIPI_SETTLEDELAY_AUTO; 1,MIPI_SETTLEDELAY_MANNUAL
	.sensor_output_dataformat = SENSOR_OUTPUT_FORMAT_RAW_Gb,
	.mclk = 24,
	.mipi_lane_num = SENSOR_MIPI_2_LANE,
	.i2c_addr_table = {0x20,0xff},
};

static imgsensor_struct imgsensor = {
	.mirror = IMAGE_HV_MIRROR,				//mirrorflip information
	.sensor_mode = IMGSENSOR_MODE_INIT, //IMGSENSOR_MODE enum value,record current sensor mode,such as: INIT, Preview, Capture, Video,High Speed Video, Slim Video
	.shutter = 0x3D0,					//current shutter
	.gain = 0x100,						//current gain
	.dummy_pixel = 0,					//current dummypixel
	.dummy_line = 0,					//current dummyline
	.current_fps = 0,  //full size current fps : 24fps for PIP, 30fps for Normal or ZSD
	.autoflicker_en = KAL_FALSE,  //auto flicker enable: KAL_FALSE for disable auto flicker, KAL_TRUE for enable auto flicker
	.test_pattern = KAL_FALSE,		//test pattern mode or not. KAL_FALSE for in test pattern mode, KAL_TRUE for normal output
	.current_scenario_id = MSDK_SCENARIO_ID_CAMERA_PREVIEW,//current scenario id
	.ihdr_mode = 0, //sensor need support LE, SE with HDR feature
	.i2c_write_id = 0x20,

};


/* Sensor output window information */
static struct SENSOR_WINSIZE_INFO_STRUCT imgsensor_winsize_info[7] =
{	{2592, 1944, 0, 0, 2592, 1944, 1296, 972, 0000, 0000, 1296, 972, 0, 0, 1296, 972},	/* Preview */
	{2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944, 0, 0, 2592, 1944},	/* capture */
	{2592, 1944, 0, 242, 2592, 1460, 2592, 1460, 0000, 0000, 2592, 1460, 0, 0, 2592, 1460},	/* video */
	{2592, 1944, 0, 242, 2592, 1460, 2592, 1460, 0000, 0000, 2592, 1460, 0, 0, 2592, 1460},	/* hight speed video */
	{2592, 1944, 0, 0, 2592, 1944, 2592, 1944, 0000, 0000, 2592, 1944,   0, 0, 2592, 1944}, /*custom1*/
	{2592, 1944, 0, 0, 2592, 1944, 1296,  972, 0000, 0000, 1296,  972,   0, 0, 1296,  972},  /*custom2*/
};

extern int iReadReg(u16 a_u2Addr , u8 * a_puBuff , u16 i2cId);
extern int iWriteReg(u16 a_u2Addr , u32 a_u4Data , u32 a_u4Bytes , u16 i2cId);

static kal_uint16 read_cmos_sensor_8(kal_uint16 addr)
{
    kal_uint16 get_byte=0;
    char pusendcmd[2] = {(char)(addr >> 8) , (char)(addr & 0xFF) };
	//imgsensor.i2c_write_id = 0x20;
    iReadRegI2C(pusendcmd , 2, (u8*)&get_byte,1,imgsensor.i2c_write_id);
    return get_byte;
}

static void write_cmos_sensor_8(kal_uint16 addr, kal_uint8 para)
{
    char pusendcmd[4] = {(char)(addr >> 8) , (char)(addr & 0xFF) ,(char)(para & 0xFF)};
	//imgsensor.i2c_write_id = 0x20;
    iWriteRegI2C(pusendcmd , 3, imgsensor.i2c_write_id);
}

static void write_cmos_sensor_16(kal_uint16 addr, kal_uint16 para)
{
    char pusendcmd[5] = {(char)(addr >> 8) , (char)(addr & 0xFF) ,(char)(para >> 8), (char)(para & 0xFF)};
	//imgsensor.i2c_write_id = 0x20;
    iWriteRegI2C(pusendcmd , 4, imgsensor.i2c_write_id);
}


static void set_dummy(void)
{
	LOG_INF("dummyline = %d, dummypixels = %d \n", imgsensor.dummy_line, imgsensor.dummy_pixel);
	//return; //for test
	write_cmos_sensor_8(0x0340, imgsensor.frame_length >> 8);
	write_cmos_sensor_8(0x0341, imgsensor.frame_length & 0xFF);
	write_cmos_sensor_8(0x0342, imgsensor.line_length >> 8);
	write_cmos_sensor_8(0x0343, imgsensor.line_length & 0xFF);
}	/*	set_dummy  */


static void set_max_framerate(UINT16 framerate,kal_bool min_framelength_en)
{
	//kal_int16 dummy_line;
	kal_uint32 frame_length = imgsensor.frame_length;

	LOG_INF("framerate = %d, min framelength should enable %d\n", framerate,
		min_framelength_en);

	frame_length = imgsensor.pclk / framerate * 10 / imgsensor.line_length;
	spin_lock(&imgsensor_drv_lock);
	if (frame_length >= imgsensor.min_frame_length)
		imgsensor.frame_length = frame_length;
	else
		imgsensor.frame_length = imgsensor.min_frame_length;
	imgsensor.dummy_line = imgsensor.frame_length - imgsensor.min_frame_length;

	if (imgsensor.frame_length > imgsensor_info.max_frame_length)
	{
		imgsensor.frame_length = imgsensor_info.max_frame_length;
		imgsensor.dummy_line = imgsensor.frame_length - imgsensor.min_frame_length;
	}
	if (min_framelength_en)
		imgsensor.min_frame_length = imgsensor.frame_length;
	spin_unlock(&imgsensor_drv_lock);
	set_dummy();
}	/*	set_max_framerate  */

static void write_shutter(kal_uint16 shutter)
{
	kal_uint16 realtime_fps = 0;
	// kal_uint32 frame_length = 0;


	// if shutter bigger than frame_length, should extend frame length first
	spin_lock(&imgsensor_drv_lock);
	if (shutter > imgsensor.min_frame_length - imgsensor_info.margin)
		imgsensor.frame_length = shutter + imgsensor_info.margin;
	else
		imgsensor.frame_length = imgsensor.min_frame_length;
	if (imgsensor.frame_length > imgsensor_info.max_frame_length)
		imgsensor.frame_length = imgsensor_info.max_frame_length;
	spin_unlock(&imgsensor_drv_lock);
	shutter = (shutter < imgsensor_info.min_shutter) ? imgsensor_info.min_shutter : shutter;
	shutter = (shutter > (imgsensor_info.max_frame_length - imgsensor_info.margin)) ? (imgsensor_info.max_frame_length - imgsensor_info.margin) : shutter;

	if (imgsensor.autoflicker_en) {
		realtime_fps = imgsensor.pclk / imgsensor.line_length * 10 / imgsensor.frame_length;
		if(realtime_fps >= 297 && realtime_fps <= 305)
			set_max_framerate(296,0);
		else if(realtime_fps >= 147 && realtime_fps <= 150)
			set_max_framerate(146,0);
		else {
		// Extend frame length
		write_cmos_sensor_8(0x0340, imgsensor.frame_length >> 8);
		write_cmos_sensor_8(0x0341, imgsensor.frame_length & 0xFF);
		}
	} else {
		// Extend frame length
		write_cmos_sensor_8(0x0340, imgsensor.frame_length >> 8);
		write_cmos_sensor_8(0x0341, imgsensor.frame_length & 0xFF);
	}
	// Update Shutter
	//write_cmos_sensor(0x0104, 0x01);   //group hold
	write_cmos_sensor_8(0x0202, shutter >> 8);
	write_cmos_sensor_8(0x0203, shutter & 0xFF);
	//write_cmos_sensor(0x0104, 0x00);   //group hold

	LOG_INF("shutter =%d, framelength =%d\n", shutter,imgsensor.frame_length);
	//LOG_INF("frame_length = %d ", frame_length);
} /* write_shutter  */

static void set_shutter_frame_length(kal_uint16 shutter, kal_uint16 frame_length)
{
	kal_uint16 realtime_fps = 0;
	kal_int32 dummy_line = 0;

	//LOG_INF("shutter =%d, frame_time =%d\n", shutter, frame_time);
	/* 0x3500, 0x3501, 0x3502 will increase VBLANK to get exposure larger than frame exposure */
	/* AE doesn't update sensor gain at capture mode, thus extra exposure lines must be updated here. */

	// OV Recommend Solution
	// if shutter bigger than frame_length, should extend frame length first
	spin_lock(&imgsensor_drv_lock);
	/*Change frame time*/
	if(frame_length > 1)
		dummy_line = frame_length - imgsensor.frame_length;
	imgsensor.frame_length = imgsensor.frame_length + dummy_line;

    //
	if (shutter > imgsensor.frame_length - imgsensor_info.margin){
		imgsensor.frame_length = shutter + imgsensor_info.margin;
	}

	if (imgsensor.frame_length > imgsensor_info.max_frame_length){
		imgsensor.frame_length = imgsensor_info.max_frame_length;
	}

	spin_unlock(&imgsensor_drv_lock);
	shutter = (shutter < imgsensor_info.min_shutter) ? imgsensor_info.min_shutter : shutter;
	shutter = (shutter > (imgsensor_info.max_frame_length - imgsensor_info.margin)) ? (imgsensor_info.max_frame_length - imgsensor_info.margin) : shutter;

	if (imgsensor.autoflicker_en) {
		realtime_fps = imgsensor.pclk / imgsensor.line_length * 10 / imgsensor.frame_length;
		if(realtime_fps >= 297 && realtime_fps <= 305)
			set_max_framerate(296,0);
		else if(realtime_fps >= 147 && realtime_fps <= 150)
			set_max_framerate(146,0);
		else {
			// Extend frame length
			write_cmos_sensor_8(0x0340, imgsensor.frame_length >> 8);
			write_cmos_sensor_8(0x0341, imgsensor.frame_length & 0xFF);
		}
	} else {
		// Extend frame length
		write_cmos_sensor_8(0x0340, imgsensor.frame_length >> 8);
		write_cmos_sensor_8(0x0341, imgsensor.frame_length & 0xFF);
	}

	// Update Shutter
	//write_cmos_sensor(0x0104, 0x01);   //group hold
	write_cmos_sensor_8(0x0202, shutter >> 8);
	write_cmos_sensor_8(0x0203, shutter & 0xFF);
	//write_cmos_sensor(0x0104, 0x00);   //group hold

	LOG_INF("shutter =%d, framelength =%d/%d, dummy_line=%d\n", shutter,imgsensor.frame_length, frame_length, dummy_line);

	//LOG_INF("frame_length = %d ", frame_length);

}	/*	write_shutter  */



/*************************************************************************
* FUNCTION
*	set_shutter
*
* DESCRIPTION
*	This function set e-shutter of sensor to change exposure time.
*
* PARAMETERS
*	iShutter : exposured lines
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static void set_shutter(kal_uint16 shutter)
{
	unsigned long flags;
	spin_lock_irqsave(&imgsensor_drv_lock, flags);
	imgsensor.shutter = shutter;
	spin_unlock_irqrestore(&imgsensor_drv_lock, flags);

	write_shutter(shutter);
}	/*	set_shutter */



static kal_uint16 gain2reg(const kal_uint16 gain)
{
	 kal_uint16 reg_gain = 0x0;

    reg_gain = gain/2;
    return (kal_uint16)reg_gain;
}

/*************************************************************************
* FUNCTION
*	set_gain
*
* DESCRIPTION
*	This function is to set global gain to sensor.
*
* PARAMETERS
*	iGain : sensor global gain(base: 0x40)
*
* RETURNS
*	the actually gain set to sensor.
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint16 set_gain(kal_uint16 gain)
{
    kal_uint16 reg_gain;

    if (gain < BASEGAIN || gain > 32 * BASEGAIN) {
        LOG_INF("Error gain setting");

        if (gain < BASEGAIN)
            gain = BASEGAIN;
        else if (gain > 32 * BASEGAIN)
            gain = 32 * BASEGAIN;
    }

    reg_gain = gain2reg(gain);
    spin_lock(&imgsensor_drv_lock);
    imgsensor.gain = reg_gain;
    spin_unlock(&imgsensor_drv_lock);
    LOG_INF("gain = %d , reg_gain = 0x%x\n ", gain, reg_gain);

    //write_cmos_sensor(0x0204,reg_gain);
    write_cmos_sensor_8(0x0204,(reg_gain>>8));
    write_cmos_sensor_8(0x0205,(reg_gain&0xff));

    return gain;
}	/*	set_gain  */
#if 1
static void set_mirror_flip(kal_uint8 image_mirror)
{
	kal_uint8 itemp;
	LOG_INF("image_mirror = %d\n", image_mirror);
	itemp=read_cmos_sensor_8(0x0101);
	itemp &= ~0x03;

	switch(image_mirror)
		{

		   case IMAGE_NORMAL:
		   	     write_cmos_sensor_8(0x0101, itemp);
			      break;

		   case IMAGE_V_MIRROR:
			     write_cmos_sensor_8(0x0101, itemp | 0x02);
			     break;

		   case IMAGE_H_MIRROR:
			     write_cmos_sensor_8(0x0101, itemp | 0x01);
			     break;

		   case IMAGE_HV_MIRROR:
			     write_cmos_sensor_8(0x0101, itemp | 0x03);
			     break;
		}
}
#endif
/*************************************************************************
* FUNCTION
*	night_mode
*
* DESCRIPTION
*	This function night mode of sensor.
*
* PARAMETERS
*	bEnable: KAL_TRUE -> enable night mode, otherwise, disable night mode
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
#if 0
static void night_mode(kal_bool enable)
{
/*No Need to implement this function*/
}	/*	night_mode	*/
#endif

static kal_uint32 streaming_control(kal_bool enable)
{
	int timeout = 100;
	int i = 0;
	int framecnt = 0;

	LOG_INF("streaming_enable(0=Sw Standby,1=streaming): %d\n", enable);
	if (enable) {
		//write_cmos_sensor_8(0x6028,0x4000);
		write_cmos_sensor_8(0x0100, 0x01);
		mDELAY(10);
	} else {
		//write_cmos_sensor_8(0x6028,0x4000);
		write_cmos_sensor_8(0x0100, 0x00);
		for (i = 0; i < timeout; i++) {
			mDELAY(10);
			framecnt = read_cmos_sensor_8(0x0005);
			if ( framecnt == 0xFF) {
				LOG_INF(" Stream Off OK at i=%d.\n", i);
				return ERROR_NONE;
			}
		}
		LOG_INF("Stream Off Fail! framecnt=%d.\n", framecnt);
	}
	return ERROR_NONE;
}
static void sensor_init(void)
{
LOG_INF("E\n");
write_cmos_sensor_8(0x0100,	0x00);  //gobal
write_cmos_sensor_8(0x0000,	0x04);
write_cmos_sensor_8(0x0000,	0x55);
write_cmos_sensor_8(0x0A02,	0x3F);
write_cmos_sensor_8(0x3B45, 0x01);
write_cmos_sensor_8(0x0B05, 0x01);
write_cmos_sensor_8(0x392F, 0x01);
write_cmos_sensor_8(0x3930, 0x00);
write_cmos_sensor_8(0x3924, 0x7F);
write_cmos_sensor_8(0x3925, 0xFD);
write_cmos_sensor_8(0x3C08, 0xFF);
write_cmos_sensor_8(0x3C09, 0xFF);
write_cmos_sensor_8(0x3C31, 0xFF);
write_cmos_sensor_8(0x3C32, 0xFF);
write_cmos_sensor_8(0x3290, 0x10);
write_cmos_sensor_8(0x3200, 0x01);
write_cmos_sensor_8(0x3074, 0x06);
write_cmos_sensor_8(0x3075, 0x2F);
write_cmos_sensor_8(0x308A, 0x20);
write_cmos_sensor_8(0x308B, 0x08);
write_cmos_sensor_8(0x308C, 0x0B);
write_cmos_sensor_8(0x3081, 0x07);
write_cmos_sensor_8(0x307B, 0x85);
write_cmos_sensor_8(0x307A, 0x0A);
write_cmos_sensor_8(0x3079, 0x0A);
write_cmos_sensor_8(0x306E, 0x71);
write_cmos_sensor_8(0x306F, 0x28);
write_cmos_sensor_8(0x301F, 0x20);
write_cmos_sensor_8(0x3012, 0x4E);
write_cmos_sensor_8(0x306B, 0x9A);
write_cmos_sensor_8(0x3091, 0x16);
write_cmos_sensor_8(0x30C4, 0x06);
write_cmos_sensor_8(0x306A, 0x79);
write_cmos_sensor_8(0x30B0, 0xFF);
write_cmos_sensor_8(0x306D, 0x08);
write_cmos_sensor_8(0x3084, 0x16);
write_cmos_sensor_8(0x3070, 0x0F);
write_cmos_sensor_8(0x30C2, 0x05);
write_cmos_sensor_8(0x3069, 0x87);
write_cmos_sensor_8(0x3C0F, 0x00);
write_cmos_sensor_8(0x3083, 0x14);
write_cmos_sensor_8(0x3080, 0x08);
write_cmos_sensor_8(0x3C34, 0xEA);
write_cmos_sensor_8(0x3C35, 0x5C);

//add for hw sync
//write_cmos_sensor_8(0x30C2,	0x01);
//write_cmos_sensor_8(0x3C05,	0x1D);
//write_cmos_sensor_8(0x3500,	0x03);

}	/*      sensor_init  */

static void preview_setting(void)
{
//Preview 2320*1744 30fps 24M MCLK 4lane 732Mbps/lane
// preview 30.01fps
LOG_INF("E\n");
write_cmos_sensor_8(0x0136,	0x18);
write_cmos_sensor_8(0x0137,	0x00);
write_cmos_sensor_8(0x0305,	0x04);
write_cmos_sensor_8(0x0306,	0x00);
write_cmos_sensor_8(0x0307,	0x5F);
write_cmos_sensor_8(0x030D,	0x04);
write_cmos_sensor_8(0x030E,	0x00);
//write_cmos_sensor_8(0x030F,	0x96);
write_cmos_sensor_8(0x3C1F,	0x00);
write_cmos_sensor_8(0x030F,	0x90);//20221107,PM,samsung FAE support to recoer radio noise mipi rate 864MHz
write_cmos_sensor_8(0x3C17,	0x00);
write_cmos_sensor_8(0x0112,	0x0A);
write_cmos_sensor_8(0x0113,	0x0A);
write_cmos_sensor_8(0x0114,	0x01);
write_cmos_sensor_8(0x0820,	0x03);
//write_cmos_sensor_8(0x0821,	0x6C);
write_cmos_sensor_8(0x0821,	0x60);////20221107,PM,samsung FAE support to recoer radio noise mipi rate 864MHz
write_cmos_sensor_8(0x0822,	0x00);
write_cmos_sensor_8(0x0823,	0x00);
write_cmos_sensor_8(0x3929,	0x0F);
write_cmos_sensor_8(0x0344,	0x00);
write_cmos_sensor_8(0x0345,	0x08);
write_cmos_sensor_8(0x0346,	0x00);
write_cmos_sensor_8(0x0347,	0x08);
write_cmos_sensor_8(0x0348,	0x0A);
write_cmos_sensor_8(0x0349,	0x27);
write_cmos_sensor_8(0x034A,	0x07);
write_cmos_sensor_8(0x034B,	0x9F);
write_cmos_sensor_8(0x034C,	0x05);
write_cmos_sensor_8(0x034D,	0x10);
write_cmos_sensor_8(0x034E,	0x03);
write_cmos_sensor_8(0x034F,	0xCC);
write_cmos_sensor_8(0x0900,	0x01);
write_cmos_sensor_8(0x0901,	0x22);
write_cmos_sensor_8(0x0381,	0x01);
write_cmos_sensor_8(0x0383,	0x01);
write_cmos_sensor_8(0x0385,	0x01);
write_cmos_sensor_8(0x0387,	0x03);
write_cmos_sensor_8(0x0101,	0x00);
write_cmos_sensor_8(0x0340,	0x07);
write_cmos_sensor_8(0x0341,	0xEE);
write_cmos_sensor_8(0x0342,	0x0C);
write_cmos_sensor_8(0x0343,	0x28);
write_cmos_sensor_8(0x0200,	0x0B);
write_cmos_sensor_8(0x0201,	0x9C);
write_cmos_sensor_8(0x0202,	0x00);
write_cmos_sensor_8(0x0203,	0x02);
write_cmos_sensor_8(0x30B8,	0x2A);
write_cmos_sensor_8(0x30BA,	0x2E);
}	/*	preview_setting  */

static void custom1_setting(void)
{
//24fps fullsize
LOG_INF("E\n");
write_cmos_sensor_8(0x0136, 0x18);
write_cmos_sensor_8(0x0137, 0x00);
write_cmos_sensor_8(0x0305, 0x04);
write_cmos_sensor_8(0x0306, 0x00);
write_cmos_sensor_8(0x0307, 0x5F);
write_cmos_sensor_8(0x030D, 0x04);
write_cmos_sensor_8(0x030E, 0x00);
write_cmos_sensor_8(0x030F, 0x96);
write_cmos_sensor_8(0x3C1F, 0x00);
write_cmos_sensor_8(0x3C17, 0x00);
write_cmos_sensor_8(0x0112, 0x0A);
write_cmos_sensor_8(0x0113, 0x0A);
write_cmos_sensor_8(0x0114, 0x01);
write_cmos_sensor_8(0x0820, 0x03);
write_cmos_sensor_8(0x0821, 0x6C);
write_cmos_sensor_8(0x0822, 0x00);
write_cmos_sensor_8(0x0823, 0x00);
write_cmos_sensor_8(0x3929, 0x0F);
write_cmos_sensor_8(0x0344, 0x00);
write_cmos_sensor_8(0x0345, 0x08);
write_cmos_sensor_8(0x0346, 0x00);
write_cmos_sensor_8(0x0347, 0x08);
write_cmos_sensor_8(0x0348, 0x0A);
write_cmos_sensor_8(0x0349, 0x27);
write_cmos_sensor_8(0x034A, 0x07);
write_cmos_sensor_8(0x034B, 0x9f);
write_cmos_sensor_8(0x034C, 0x0A);
write_cmos_sensor_8(0x034D, 0x20);
write_cmos_sensor_8(0x034E, 0x07);
write_cmos_sensor_8(0x034F, 0x98);
write_cmos_sensor_8(0x0900, 0x00);
write_cmos_sensor_8(0x0901, 0x00);
write_cmos_sensor_8(0x0381, 0x01);
write_cmos_sensor_8(0x0383, 0x01);
write_cmos_sensor_8(0x0385, 0x01);
write_cmos_sensor_8(0x0387, 0x01);
write_cmos_sensor_8(0x0101, 0x00);
write_cmos_sensor_8(0x0340, 0x09);
write_cmos_sensor_8(0x0341, 0xF0);
write_cmos_sensor_8(0x0342, 0x0C);
write_cmos_sensor_8(0x0343, 0x28);
write_cmos_sensor_8(0x0200, 0x0B);
write_cmos_sensor_8(0x0201, 0x9C);
write_cmos_sensor_8(0x0202, 0x00);
write_cmos_sensor_8(0x0203, 0x02);
write_cmos_sensor_8(0x30B8, 0x2E);
write_cmos_sensor_8(0x30BA, 0x36);
}

static void custom2_setting(void)
{
//binning,24fps
LOG_INF("E\n");
write_cmos_sensor_8(0x0136, 0x18);
write_cmos_sensor_8(0x0137, 0x00);
write_cmos_sensor_8(0x0305, 0x04);
write_cmos_sensor_8(0x0306, 0x00);
write_cmos_sensor_8(0x0307, 0x5F);
write_cmos_sensor_8(0x030D, 0x04);
write_cmos_sensor_8(0x030E, 0x00);
write_cmos_sensor_8(0x030F, 0x96);
write_cmos_sensor_8(0x3C1F, 0x00);
write_cmos_sensor_8(0x3C17, 0x00);
write_cmos_sensor_8(0x0112, 0x0A);
write_cmos_sensor_8(0x0113, 0x0A);
write_cmos_sensor_8(0x0114, 0x01);
write_cmos_sensor_8(0x0820, 0x03);
write_cmos_sensor_8(0x0821, 0x6C);
write_cmos_sensor_8(0x0822, 0x00);
write_cmos_sensor_8(0x0823, 0x00);
write_cmos_sensor_8(0x3929, 0x0F);
write_cmos_sensor_8(0x0344, 0x00);
write_cmos_sensor_8(0x0345, 0x08);
write_cmos_sensor_8(0x0346, 0x00);
write_cmos_sensor_8(0x0347, 0x08);
write_cmos_sensor_8(0x0348, 0x0A);
write_cmos_sensor_8(0x0349, 0x27);
write_cmos_sensor_8(0x034A, 0x07);
write_cmos_sensor_8(0x034B, 0x9F);
write_cmos_sensor_8(0x034C, 0x05);
write_cmos_sensor_8(0x034D, 0x10);
write_cmos_sensor_8(0x034E, 0x03);
write_cmos_sensor_8(0x034F, 0xCC);
write_cmos_sensor_8(0x0900, 0x01);
write_cmos_sensor_8(0x0901, 0x22);
write_cmos_sensor_8(0x0381, 0x01);
write_cmos_sensor_8(0x0383, 0x01);
write_cmos_sensor_8(0x0385, 0x01);
write_cmos_sensor_8(0x0387, 0x03);
write_cmos_sensor_8(0x0101, 0x00);
write_cmos_sensor_8(0x0340, 0x09);
write_cmos_sensor_8(0x0341, 0xF0);
write_cmos_sensor_8(0x0342, 0x0C);
write_cmos_sensor_8(0x0343, 0x28);
write_cmos_sensor_8(0x0200, 0x0B);
write_cmos_sensor_8(0x0201, 0x9C);
write_cmos_sensor_8(0x0202, 0x00);
write_cmos_sensor_8(0x0203, 0x02);
write_cmos_sensor_8(0x30B8, 0x2A);
write_cmos_sensor_8(0x30BA, 0x2E);
}
static void capture_setting(kal_uint16 currefps)
{
LOG_INF("capture_setting() E! currefps:%d\n", currefps);

write_cmos_sensor_8(0x0136,	0x18);
write_cmos_sensor_8(0x0137,	0x00);
write_cmos_sensor_8(0x0305,	0x04);
write_cmos_sensor_8(0x0306,	0x00);
write_cmos_sensor_8(0x0307,	0x5F);
write_cmos_sensor_8(0x030D,	0x04);
write_cmos_sensor_8(0x030E,	0x00);
write_cmos_sensor_8(0x030F,	0x96);
write_cmos_sensor_8(0x3C1F,	0x00);
write_cmos_sensor_8(0x3C17,	0x00);
write_cmos_sensor_8(0x0112,	0x0A);
write_cmos_sensor_8(0x0113,	0x0A);
write_cmos_sensor_8(0x0114,	0x01);
write_cmos_sensor_8(0x0820,	0x03);
write_cmos_sensor_8(0x0821,	0x6C);
write_cmos_sensor_8(0x0822,	0x00);
write_cmos_sensor_8(0x0823,	0x00);
write_cmos_sensor_8(0x3929,	0x0F);
write_cmos_sensor_8(0x0344,	0x00);
write_cmos_sensor_8(0x0345,	0x08);
write_cmos_sensor_8(0x0346,	0x00);
write_cmos_sensor_8(0x0347,	0x08);
write_cmos_sensor_8(0x0348,	0x0A);
write_cmos_sensor_8(0x0349,	0x27);
write_cmos_sensor_8(0x034A,	0x07);
write_cmos_sensor_8(0x034B,	0x9f);
write_cmos_sensor_8(0x034C,	0x0A);
write_cmos_sensor_8(0x034D,	0x20);
write_cmos_sensor_8(0x034E,	0x07);
write_cmos_sensor_8(0x034F,	0x98);
write_cmos_sensor_8(0x0900,	0x00);
write_cmos_sensor_8(0x0901,	0x00);
write_cmos_sensor_8(0x0381,	0x01);
write_cmos_sensor_8(0x0383,	0x01);
write_cmos_sensor_8(0x0385,	0x01);
write_cmos_sensor_8(0x0387,	0x01);
write_cmos_sensor_8(0x0101,	0x00);
write_cmos_sensor_8(0x0340,	0x07);
write_cmos_sensor_8(0x0341,	0xF4);
write_cmos_sensor_8(0x0342,	0x0C);
write_cmos_sensor_8(0x0343,	0x28);
write_cmos_sensor_8(0x0200,	0x0B);
write_cmos_sensor_8(0x0201,	0x9C);
write_cmos_sensor_8(0x0202,	0x00);
write_cmos_sensor_8(0x0203,	0x02);
write_cmos_sensor_8(0x30B8,	0x2E);
write_cmos_sensor_8(0x30BA,	0x36);
}

static void normal_video_setting(kal_uint16 currefps)
{
LOG_INF("E! currefps:%d\n",currefps);
// full size 30fps
write_cmos_sensor_8(0x0136,	0x18);
write_cmos_sensor_8(0x0137,	0x00);
write_cmos_sensor_8(0x0305,	0x04);
write_cmos_sensor_8(0x0306,	0x00);
write_cmos_sensor_8(0x0307,	0x5F);
write_cmos_sensor_8(0x030D,	0x04);
write_cmos_sensor_8(0x030E,	0x00);
write_cmos_sensor_8(0x030F,	0x96);
write_cmos_sensor_8(0x3C1F,	0x00);
write_cmos_sensor_8(0x3C17,	0x00);
write_cmos_sensor_8(0x0112,	0x0A);
write_cmos_sensor_8(0x0113,	0x0A);
write_cmos_sensor_8(0x0114,	0x01);
write_cmos_sensor_8(0x0820,	0x03);
write_cmos_sensor_8(0x0821,	0x6C);
write_cmos_sensor_8(0x0822,	0x00);
write_cmos_sensor_8(0x0823,	0x00);
write_cmos_sensor_8(0x3929,	0x0F);
write_cmos_sensor_8(0x0344,	0x00);
write_cmos_sensor_8(0x0345,	0x08);
write_cmos_sensor_8(0x0346,	0x00);
write_cmos_sensor_8(0x0347,	0xFA);
write_cmos_sensor_8(0x0348,	0x0A);
write_cmos_sensor_8(0x0349,	0x27);
write_cmos_sensor_8(0x034A,	0x06);
write_cmos_sensor_8(0x034B,	0xAD);
write_cmos_sensor_8(0x034C,	0x0A);
write_cmos_sensor_8(0x034D,	0x20);
write_cmos_sensor_8(0x034E,	0x05);
write_cmos_sensor_8(0x034F,	0xB4);
write_cmos_sensor_8(0x0900,	0x00);
write_cmos_sensor_8(0x0901,	0x00);
write_cmos_sensor_8(0x0381,	0x01);
write_cmos_sensor_8(0x0383,	0x01);
write_cmos_sensor_8(0x0385,	0x01);
write_cmos_sensor_8(0x0387,	0x01);
write_cmos_sensor_8(0x0101,	0x00);
write_cmos_sensor_8(0x0340,	0x07);
write_cmos_sensor_8(0x0341,	0xEE);
write_cmos_sensor_8(0x0342,	0x0C);
write_cmos_sensor_8(0x0343,	0x28);
write_cmos_sensor_8(0x0200,	0x0B);
write_cmos_sensor_8(0x0201,	0x9C);
write_cmos_sensor_8(0x0202,	0x00);
write_cmos_sensor_8(0x0203,	0x02);
write_cmos_sensor_8(0x30B8,	0x2E);
write_cmos_sensor_8(0x30BA,	0x36);
}
static void hs_video_setting(void)
{
LOG_INF("E\n");
//720p 120fps
write_cmos_sensor_8(0x0136,	0x18);
write_cmos_sensor_8(0x0137,	0x00);
write_cmos_sensor_8(0x0305,	0x04);
write_cmos_sensor_8(0x0306,	0x00);
write_cmos_sensor_8(0x0307,	0x5F);
write_cmos_sensor_8(0x030D,	0x04);
write_cmos_sensor_8(0x030E,	0x00);
write_cmos_sensor_8(0x030F,	0x96);
write_cmos_sensor_8(0x3C1F,	0x00);
write_cmos_sensor_8(0x3C17,	0x00);
write_cmos_sensor_8(0x0112,	0x0A);
write_cmos_sensor_8(0x0113,	0x0A);
write_cmos_sensor_8(0x0114,	0x01);
write_cmos_sensor_8(0x0820,	0x03);
write_cmos_sensor_8(0x0821,	0x6C);
write_cmos_sensor_8(0x0822,	0x00);
write_cmos_sensor_8(0x0823,	0x00);
write_cmos_sensor_8(0x3929,	0x0F);
write_cmos_sensor_8(0x0344,	0x00);
write_cmos_sensor_8(0x0345,	0x08);
write_cmos_sensor_8(0x0346,	0x00);
write_cmos_sensor_8(0x0347,	0xFA);
write_cmos_sensor_8(0x0348,	0x0A);
write_cmos_sensor_8(0x0349,	0x27);
write_cmos_sensor_8(0x034A,	0x06);
write_cmos_sensor_8(0x034B,	0xAD);
write_cmos_sensor_8(0x034C,	0x0A);
write_cmos_sensor_8(0x034D,	0x20);
write_cmos_sensor_8(0x034E,	0x05);
write_cmos_sensor_8(0x034F,	0xB4);
write_cmos_sensor_8(0x0900,	0x00);
write_cmos_sensor_8(0x0901,	0x00);
write_cmos_sensor_8(0x0381,	0x01);
write_cmos_sensor_8(0x0383,	0x01);
write_cmos_sensor_8(0x0385,	0x01);
write_cmos_sensor_8(0x0387,	0x01);
write_cmos_sensor_8(0x0101,	0x00);
write_cmos_sensor_8(0x0340,	0x07);
write_cmos_sensor_8(0x0341,	0xEE);
write_cmos_sensor_8(0x0342,	0x0C);
write_cmos_sensor_8(0x0343,	0x28);
write_cmos_sensor_8(0x0200,	0x0B);
write_cmos_sensor_8(0x0201,	0x9C);
write_cmos_sensor_8(0x0202,	0x00);
write_cmos_sensor_8(0x0203,	0x02);
write_cmos_sensor_8(0x30B8,	0x2E);
write_cmos_sensor_8(0x30BA,	0x36);
}

static void slim_video_setting(void)
{
	LOG_INF("E\n");
	preview_setting();
}

typedef struct s5k5e9_otp_info
{
        kal_uint16 module_id;
        kal_uint16 lens_id;
        kal_uint16 year;
        kal_uint16 month;
        kal_uint16 day;
        kal_uint16 rg_gain;
        kal_uint16 bg_gain;
	kal_uint16 awb_flag;
}s5k5e9_otp_info;

static s5k5e9_otp_info otp_info;

/*Yijun.Tan@Camera.Driver 20180116 add for OTP*/
void start_read_otp(void)
{
	int i = 0, j = 0;
	kal_uint32 sum = 0;
	int flag = 0;
	kal_uint16 addr = 0;
	kal_uint16 data[30] = {0};

	otp_info.awb_flag = 0;
	write_cmos_sensor_8(0x0100,0x01);
	mdelay(50);
	write_cmos_sensor_8(0x0A02,17);   //set ready page: page 17
	write_cmos_sensor_8(0x0A00,0x01);   //Enter read mode by writing 01h
	for(i = 0; i < 30; i++) {
		flag = read_cmos_sensor_8(0X0A01);
		if(flag == 1)
			break;
		mdelay(1);
	}
	if(i == 30) {
		pr_err("read otp no ack!");
		return;
	}

	//Moudle Infor Flag
	flag = read_cmos_sensor_8(0X0A04);
	if((flag & 0x03) == 0x01) {
		addr = 0x0A05;
	} else if((flag & 0x0c) == 0x04){
		addr = 0x0A0F;
	} else {
		pr_err("err! module info flag = %x\n", flag);
		return ;
	}
	for(i = addr, j = 0; j < 10; i++, j++)
	{
		data[j] = read_cmos_sensor_8(i);
		mdelay(2);

		if(j < 9)
			sum += data[j];
		LOG_INF("data[%d] = %d sum = %d", j, data[j], sum);
	}

        if (((sum % 255) + 1 ) != data[9])
        {
		LOG_INF("module info checksum err\n");
		return;
        }

	otp_info.module_id = data[0];
	otp_info.year = data[1];
	otp_info.month = data[2];
	otp_info.day = data[3];
	otp_info.lens_id = data[4];

	//awb flag
	flag = read_cmos_sensor_8(0X0A26);
	if((flag & 0x03) == 0x01) {
		addr = 0x0A27;
	} else if((flag & 0x0c) == 0x04){
		addr = 0x0A34;
	} else {
		pr_err("err! awb flag = %x\n", flag);
		return ;
	}

	sum = 0;
	for(j = 0, i = addr; j < 13; j++, i++)
	{
		data[j] = read_cmos_sensor_8(i);
		mdelay(2);
		if(j < 12)
			sum += data[j];
		LOG_INF("data[%d] = %d sum = %d", j, data[j], sum);
	}
	 if (((sum % 255) + 1 ) != data[12])
        {
		LOG_INF("awb checksum err\n");
		return;
        }

	otp_info.rg_gain = data[0] << 8 | data[1];
        otp_info.bg_gain = data[2] << 8 | data[3];
	otp_info.awb_flag = 1;
	LOG_INF("rg_gain = %d, bg_gain = %d\n", otp_info.rg_gain, otp_info.bg_gain);
}
/*************************************************************************************************
* Function    :  stop_read_otp
* Description :  after read otp , stop and reset otp block setting
**************************************************************************************************/
void stop_read_otp(void)
{
   write_cmos_sensor_8(0x0A00, 0x04);
   write_cmos_sensor_8(0x0A00, 0x00);
}
void module_info_otp_read(void)
{
	LOG_INF("module_info_otp_read");
	start_read_otp();
	stop_read_otp();
}

#define RG_TYPICAL 548
#define BG_TYPICAL 750

static void awb_lsc_update_otp(void)
{
	int R_gain,G_gain,B_gain,Base_gain;
	int rg, bg, RG_Ratio_Typical, BG_Ratio_Typical;

	rg = otp_info.rg_gain;
	bg = otp_info.bg_gain;

	RG_Ratio_Typical = RG_TYPICAL;
	BG_Ratio_Typical = BG_TYPICAL;

	//calibration G gain
	R_gain = (RG_Ratio_Typical*1000)/rg;
	B_gain = (BG_Ratio_Typical*1000)/bg;
	G_gain = 1000;
	if(R_gain < 1000 || B_gain < 1000)
	{
		if(R_gain < B_gain)
			Base_gain = R_gain;
		else
			Base_gain = B_gain;
	}
	else
	{
		Base_gain = G_gain;
	}
	R_gain = 0x100*R_gain/(Base_gain);
	B_gain = 0x100*B_gain/(Base_gain);
	G_gain = 0x100*G_gain/(Base_gain);

	LOG_INF("OTP_UPDATE_AWB:r_gain = %d , g_gain = %d , b_gain = %d \n",R_gain,G_gain,B_gain);

	//update sensor wb gainkin
	if(R_gain > 0x100)
	{
		write_cmos_sensor_16(0x0210, R_gain);
	}
	if (G_gain>0x100)
	{

		write_cmos_sensor_16(0x020E, G_gain);
		write_cmos_sensor_16(0x0214, G_gain);
	}
	if (B_gain>0x100)
	{
		write_cmos_sensor_16(0x0212,B_gain);
	}
#if 0
	LOG_INF("r_gain = %d , g_gain = %d , b_gain = %d \n",
	read_cmos_sensor_8(0x210) << 8 | read_cmos_sensor_8(0x211),
	read_cmos_sensor_8(0x20E) << 8 | read_cmos_sensor_8(0x20F),
	read_cmos_sensor_8(0x212) << 8 | read_cmos_sensor_8(0x213)
	);
#endif
	//lsc enable
	//write_cmos_sensor_8(0x3400, 0x00);
	//write_cmos_sensor_8(0x0b00, 0x01);
}

/*************************************************************************
* FUNCTION
*	get_imgsensor_id
*
* DESCRIPTION
*	This function get the sensor ID
*
* PARAMETERS
*	*sensorID : return the sensor ID
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 get_imgsensor_id(UINT32 *sensor_id)
{
	kal_uint8 i = 0;
	kal_uint8 retry = 2;
  kal_uint8 module_id = 0;

	//sensor have two i2c address 0x6c 0x6d & 0x21 0x20, we should detect the module used i2c address
	while (imgsensor_info.i2c_addr_table[i] != 0xff) {
		spin_lock(&imgsensor_drv_lock);
		imgsensor.i2c_write_id = imgsensor_info.i2c_addr_table[i];
		spin_unlock(&imgsensor_drv_lock);
		do {
			*sensor_id = ((read_cmos_sensor_8(0x0000) << 8) | read_cmos_sensor_8(0x0001))+1;
			LOG_INF("*sensor_id = 0x%x, read_0x0000=0x%x, 0x0001=0x%x\n",
					*sensor_id,
					read_cmos_sensor_8(0x0000),read_cmos_sensor_8(0x0001));

			if (*sensor_id == imgsensor_info.sensor_id) {
				LOG_INF("i2c write id: 0x%x, sensor id: 0x%x\n", imgsensor.i2c_write_id,*sensor_id);
				sensor_init();
				preview_setting();

				module_info_otp_read();
				module_id = otp_info.module_id;
       if(module_id == 0x47)
       {
				//add for TinnoProductInfo
				#ifdef CONFIG_T_PRODUCT_INFO
				LOG_INF(" s5k5e9yx_dd_u5020_mipi_raw FULL_PRODUCT_DEVICE_INFO_CAMERA\n");
 				FULL_PRODUCT_DEVICE_INFO_CAMERA( S5K5E9YX_DD_U5020_SENSOR_ID, 1, "s5k5e9yx_dd_u5020_mipi_raw", imgsensor_info.cap.grabwindow_width,imgsensor_info.cap.grabwindow_height);
				#endif
				return ERROR_NONE;
       }
       else
       {
          *sensor_id = 0xFFFFFFFF;
		      return ERROR_SENSOR_CONNECT_FAIL;
       }
			}
			LOG_INF("Read sensor id fail, id: 0x%x ensor id: 0x%x\n", imgsensor.i2c_write_id,*sensor_id);
			retry--;
		} while(retry > 0);
		i++;
		retry = 2;
	}
	if (*sensor_id != imgsensor_info.sensor_id) {
		// if Sensor ID is not correct, Must set *sensor_id to 0xFFFFFFFF
		*sensor_id = 0xFFFFFFFF;
		return ERROR_SENSOR_CONNECT_FAIL;
	}

	return ERROR_NONE;
}


/*************************************************************************
* FUNCTION
*	open
*
* DESCRIPTION
*	This function initialize the registers of CMOS sensor
*
* PARAMETERS
*	None
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 open(void)
{
	kal_uint8 i = 0;
	kal_uint8 retry = 2;
	kal_uint16 sensor_id = 0;
	LOG_INF("tanyijun debug PLATFORM:MT6595,MIPI 2LANE\n");
	LOG_INF("tanyijun debug preview 1280*960@30fps,864Mbps/lane; video 1280*960@30fps,864Mbps/lane; capture 5M@30fps,864Mbps/lane\n");

	//sensor have two i2c address 0x6c 0x6d & 0x21 0x20, we should detect the module used i2c address
	while (imgsensor_info.i2c_addr_table[i] != 0xff) {
		spin_lock(&imgsensor_drv_lock);
		imgsensor.i2c_write_id = imgsensor_info.i2c_addr_table[i];
		spin_unlock(&imgsensor_drv_lock);
		do {
                     sensor_id = ((read_cmos_sensor_8(0x0000) << 8) | read_cmos_sensor_8(0x0001))+1;
			//sensor_id = 0x559B;
			if (sensor_id == imgsensor_info.sensor_id) {
				LOG_INF("i2c write id: 0x%x, sensor id: 0x%x\n", imgsensor.i2c_write_id,sensor_id);
				break;
			}
			LOG_INF("Read sensor id fail, id: 0x%x\n", imgsensor.i2c_write_id);
			retry--;
		} while(retry > 0);
		i++;
		if (sensor_id == imgsensor_info.sensor_id)
			break;
		retry = 2;
	}
	if (imgsensor_info.sensor_id != sensor_id)
		return ERROR_SENSOR_CONNECT_FAIL;

	/* initail sequence write in  */
	sensor_init();

	if(otp_info.awb_flag == 1)
		awb_lsc_update_otp();

	spin_lock(&imgsensor_drv_lock);

	imgsensor.autoflicker_en= KAL_FALSE;
	imgsensor.sensor_mode = IMGSENSOR_MODE_INIT;
	imgsensor.shutter = 0x3D0;
	imgsensor.gain = 0x100;
	imgsensor.pclk = imgsensor_info.pre.pclk;
	imgsensor.frame_length = imgsensor_info.pre.framelength;
	imgsensor.line_length = imgsensor_info.pre.linelength;
	imgsensor.min_frame_length = imgsensor_info.pre.framelength;
	imgsensor.dummy_pixel = 0;
	imgsensor.dummy_line = 0;
	imgsensor.ihdr_mode = 0;
	imgsensor.test_pattern = KAL_FALSE;
	imgsensor.current_fps = imgsensor_info.pre.max_framerate;
	spin_unlock(&imgsensor_drv_lock);

	return ERROR_NONE;
}	/*	open  */



/*************************************************************************
* FUNCTION
*	close
*
* DESCRIPTION
*
*
* PARAMETERS
*	None
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 close(void)
{
	LOG_INF("E\n");

	/*No Need to implement this function*/
	streaming_control(0);

	return ERROR_NONE;
}	/*	close  */


/*************************************************************************
* FUNCTION
* preview
*
* DESCRIPTION
*	This function start the sensor preview.
*
* PARAMETERS
*	*image_window : address pointer of pixel numbers in one period of HSYNC
*  *sensor_config_data : address pointer of line numbers in one period of VSYNC
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 preview(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("E\n");

	spin_lock(&imgsensor_drv_lock);
	imgsensor.sensor_mode = IMGSENSOR_MODE_PREVIEW;
	imgsensor.pclk = imgsensor_info.pre.pclk;
	imgsensor.line_length = imgsensor_info.pre.linelength;
	imgsensor.frame_length = imgsensor_info.pre.framelength;
	imgsensor.min_frame_length = imgsensor_info.pre.framelength;
	imgsensor.autoflicker_en = KAL_FALSE;
	spin_unlock(&imgsensor_drv_lock);

	preview_setting();
	set_mirror_flip(imgsensor.mirror);

	return ERROR_NONE;
}	/*	preview   */

/*************************************************************************
* FUNCTION
*	capture
*
* DESCRIPTION
*	This function setup the CMOS sensor in capture MY_OUTPUT mode
*
* PARAMETERS
*
* RETURNS
*	None
*
* GLOBALS AFFECTED
*
*************************************************************************/
static kal_uint32 capture(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
						  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("E\n");
	spin_lock(&imgsensor_drv_lock);
	imgsensor.sensor_mode = IMGSENSOR_MODE_CAPTURE;
	if (imgsensor.current_fps == imgsensor_info.cap1.max_framerate) {//PIP capture: 24fps for less than 13M, 20fps for 16M,15fps for 20M
		imgsensor.pclk = imgsensor_info.cap1.pclk;
		imgsensor.line_length = imgsensor_info.cap1.linelength;
		imgsensor.frame_length = imgsensor_info.cap1.framelength;
		imgsensor.min_frame_length = imgsensor_info.cap1.framelength;
		imgsensor.autoflicker_en = KAL_FALSE;
	}  else if(imgsensor.current_fps == imgsensor_info.cap2.max_framerate){
		if (imgsensor.current_fps != imgsensor_info.cap.max_framerate)
			LOG_INF("Warning: current_fps %d fps is not support, so use cap1's setting: %d fps!\n",imgsensor.current_fps,imgsensor_info.cap1.max_framerate/10);
		imgsensor.pclk = imgsensor_info.cap2.pclk;
		imgsensor.line_length = imgsensor_info.cap2.linelength;
		imgsensor.frame_length = imgsensor_info.cap2.framelength;
		imgsensor.min_frame_length = imgsensor_info.cap2.framelength;
		imgsensor.autoflicker_en = KAL_FALSE;
	}else {
		if (imgsensor.current_fps != imgsensor_info.cap.max_framerate)
			LOG_INF("Warning: current_fps %d fps is not support, so use cap's setting: %d fps!\n",
				imgsensor.current_fps, imgsensor_info.cap.max_framerate / 10);
		imgsensor.pclk = imgsensor_info.cap.pclk;
		imgsensor.line_length = imgsensor_info.cap.linelength;
		imgsensor.frame_length = imgsensor_info.cap.framelength;
		imgsensor.min_frame_length = imgsensor_info.cap.framelength;
		imgsensor.autoflicker_en = KAL_FALSE;
	}
	spin_unlock(&imgsensor_drv_lock);

	capture_setting(imgsensor.current_fps);
	set_mirror_flip(imgsensor.mirror);

	return ERROR_NONE;
}	/* capture() */
static kal_uint32 normal_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("E\n");

	spin_lock(&imgsensor_drv_lock);
	imgsensor.sensor_mode = IMGSENSOR_MODE_VIDEO;
	imgsensor.pclk = imgsensor_info.normal_video.pclk;
	imgsensor.line_length = imgsensor_info.normal_video.linelength;
	imgsensor.frame_length = imgsensor_info.normal_video.framelength;
	imgsensor.min_frame_length = imgsensor_info.normal_video.framelength;
	imgsensor.autoflicker_en = KAL_FALSE;
	spin_unlock(&imgsensor_drv_lock);
    normal_video_setting(imgsensor.current_fps);
	set_mirror_flip(imgsensor.mirror);

	return ERROR_NONE;
}	/*	normal_video   */

static kal_uint32 hs_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("E\n");

	spin_lock(&imgsensor_drv_lock);
	imgsensor.sensor_mode = IMGSENSOR_MODE_HIGH_SPEED_VIDEO;
	imgsensor.pclk = imgsensor_info.hs_video.pclk;
	//imgsensor.video_mode = KAL_TRUE;
	imgsensor.line_length = imgsensor_info.hs_video.linelength;
	imgsensor.frame_length = imgsensor_info.hs_video.framelength;
	imgsensor.min_frame_length = imgsensor_info.hs_video.framelength;
	imgsensor.dummy_line = 0;
	imgsensor.dummy_pixel = 0;
	//imgsensor.current_fps = 300;
	imgsensor.autoflicker_en = KAL_FALSE;
	spin_unlock(&imgsensor_drv_lock);
	hs_video_setting();
	set_mirror_flip(imgsensor.mirror);

	return ERROR_NONE;
}	/*	hs_video   */

static kal_uint32 slim_video(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("E\n");

	spin_lock(&imgsensor_drv_lock);
	imgsensor.sensor_mode = IMGSENSOR_MODE_SLIM_VIDEO;
	imgsensor.pclk = imgsensor_info.slim_video.pclk;
	//imgsensor.video_mode = KAL_TRUE;
	imgsensor.line_length = imgsensor_info.slim_video.linelength;
	imgsensor.frame_length = imgsensor_info.slim_video.framelength;
	imgsensor.min_frame_length = imgsensor_info.slim_video.framelength;
	imgsensor.dummy_line = 0;
	imgsensor.dummy_pixel = 0;
	//imgsensor.current_fps = 300;
	imgsensor.autoflicker_en = KAL_FALSE;
	spin_unlock(&imgsensor_drv_lock);
	slim_video_setting();
	set_mirror_flip(imgsensor.mirror);

	return ERROR_NONE;
}	/*	slim_video	 */


static kal_uint32 Custom1(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
  //  printk("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_CUSTOM1;
    imgsensor.pclk = imgsensor_info.custom1.pclk;
    //imgsensor.video_mode = KAL_FALSE;
    imgsensor.line_length = imgsensor_info.custom1.linelength;
    imgsensor.frame_length = imgsensor_info.custom1.framelength;
    imgsensor.min_frame_length = imgsensor_info.custom1.framelength;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    custom1_setting();
    return ERROR_NONE;
}   /*  Custom1   */

static kal_uint32 Custom2(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
                      MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
  // printk("E\n");

    spin_lock(&imgsensor_drv_lock);
    imgsensor.sensor_mode = IMGSENSOR_MODE_CUSTOM2;
    imgsensor.pclk = imgsensor_info.custom2.pclk;
    //imgsensor.video_mode = KAL_FALSE;
    imgsensor.line_length = imgsensor_info.custom2.linelength;
    imgsensor.frame_length = imgsensor_info.custom2.framelength;
    imgsensor.min_frame_length = imgsensor_info.custom2.framelength;
    imgsensor.autoflicker_en = KAL_FALSE;
    spin_unlock(&imgsensor_drv_lock);
    custom2_setting();
    return ERROR_NONE;
}   /*  Custom2   */


static kal_uint32 get_resolution(MSDK_SENSOR_RESOLUTION_INFO_STRUCT *sensor_resolution)
{
	LOG_INF("E\n");
	sensor_resolution->SensorFullWidth = imgsensor_info.cap.grabwindow_width;
	sensor_resolution->SensorFullHeight = imgsensor_info.cap.grabwindow_height;

	sensor_resolution->SensorPreviewWidth = imgsensor_info.pre.grabwindow_width;
	sensor_resolution->SensorPreviewHeight = imgsensor_info.pre.grabwindow_height;

	sensor_resolution->SensorVideoWidth = imgsensor_info.normal_video.grabwindow_width;
	sensor_resolution->SensorVideoHeight = imgsensor_info.normal_video.grabwindow_height;


	sensor_resolution->SensorHighSpeedVideoWidth	 = imgsensor_info.hs_video.grabwindow_width;
	sensor_resolution->SensorHighSpeedVideoHeight	 = imgsensor_info.hs_video.grabwindow_height;

	sensor_resolution->SensorSlimVideoWidth	 = imgsensor_info.slim_video.grabwindow_width;
	sensor_resolution->SensorSlimVideoHeight	 = imgsensor_info.slim_video.grabwindow_height;

	sensor_resolution->SensorCustom1Width  = imgsensor_info.custom1.grabwindow_width;
	sensor_resolution->SensorCustom1Height     = imgsensor_info.custom1.grabwindow_height;

	sensor_resolution->SensorCustom2Width  = imgsensor_info.custom2.grabwindow_width;
	sensor_resolution->SensorCustom2Height     = imgsensor_info.custom2.grabwindow_height;

	return ERROR_NONE;
}	/*	get_resolution	*/

static kal_uint32 get_info(enum MSDK_SCENARIO_ID_ENUM scenario_id,
					  MSDK_SENSOR_INFO_STRUCT *sensor_info,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("scenario_id = %d\n", scenario_id);

	sensor_info->SensorClockPolarity = SENSOR_CLOCK_POLARITY_LOW;
	sensor_info->SensorClockFallingPolarity = SENSOR_CLOCK_POLARITY_LOW; /* not use */
	sensor_info->SensorHsyncPolarity = SENSOR_CLOCK_POLARITY_LOW; // inverse with datasheet
	sensor_info->SensorVsyncPolarity = SENSOR_CLOCK_POLARITY_LOW;
	sensor_info->SensorInterruptDelayLines = 4; /* not use */
	sensor_info->SensorResetActiveHigh = FALSE; /* not use */
	sensor_info->SensorResetDelayCount = 5; /* not use */

	sensor_info->SensroInterfaceType = imgsensor_info.sensor_interface_type;
	sensor_info->MIPIsensorType = imgsensor_info.mipi_sensor_type;
	sensor_info->SettleDelayMode = imgsensor_info.mipi_settle_delay_mode;
	sensor_info->SensorOutputDataFormat = imgsensor_info.sensor_output_dataformat;

	sensor_info->CaptureDelayFrame = imgsensor_info.cap_delay_frame;
	sensor_info->PreviewDelayFrame = imgsensor_info.pre_delay_frame;
	sensor_info->VideoDelayFrame = imgsensor_info.video_delay_frame;
	sensor_info->HighSpeedVideoDelayFrame = imgsensor_info.hs_video_delay_frame;
	sensor_info->SlimVideoDelayFrame = imgsensor_info.slim_video_delay_frame;
	sensor_info->Custom1DelayFrame = imgsensor_info.custom1_delay_frame;
	sensor_info->Custom2DelayFrame = imgsensor_info.custom2_delay_frame;

	sensor_info->SensorMasterClockSwitch = 0; /* not use */
	sensor_info->SensorDrivingCurrent = imgsensor_info.isp_driving_current;

	sensor_info->AEShutDelayFrame = imgsensor_info.ae_shut_delay_frame; 		 /* The frame of setting shutter default 0 for TG int */
	sensor_info->AESensorGainDelayFrame = imgsensor_info.ae_sensor_gain_delay_frame;	/* The frame of setting sensor gain */
	sensor_info->AEISPGainDelayFrame = imgsensor_info.ae_ispGain_delay_frame;
	sensor_info->FrameTimeDelayFrame = imgsensor_info.frame_time_delay_frame; /* The delay frame of setting frame length  */
	sensor_info->IHDR_Support = imgsensor_info.ihdr_support;
	sensor_info->IHDR_LE_FirstLine = imgsensor_info.ihdr_le_firstline;
	sensor_info->SensorModeNum = imgsensor_info.sensor_mode_num;
	sensor_info->SensorMIPILaneNumber = imgsensor_info.mipi_lane_num;
	sensor_info->SensorClockFreq = imgsensor_info.mclk;
	sensor_info->SensorClockDividCount = 3; /* not use */
	sensor_info->SensorClockRisingCount = 0;
	sensor_info->SensorClockFallingCount = 2; /* not use */
	sensor_info->SensorPixelClockCount = 3; /* not use */
	sensor_info->SensorDataLatchCount = 2; /* not use */

	sensor_info->MIPIDataLowPwr2HighSpeedTermDelayCount = 0;
	sensor_info->MIPICLKLowPwr2HighSpeedTermDelayCount = 0;
	sensor_info->SensorWidthSampling = 0;  // 0 is default 1x
	sensor_info->SensorHightSampling = 0;	// 0 is default 1x
	sensor_info->SensorPacketECCOrder = 1;

	switch (scenario_id) {
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			sensor_info->SensorGrabStartX = imgsensor_info.pre.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.pre.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.pre.mipi_data_lp2hs_settle_dc;

			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			sensor_info->SensorGrabStartX = imgsensor_info.cap.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.cap.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.cap.mipi_data_lp2hs_settle_dc;

			break;
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:

			sensor_info->SensorGrabStartX = imgsensor_info.normal_video.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.normal_video.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.normal_video.mipi_data_lp2hs_settle_dc;

			break;
		case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			sensor_info->SensorGrabStartX = imgsensor_info.hs_video.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.hs_video.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.hs_video.mipi_data_lp2hs_settle_dc;

			break;
		case MSDK_SCENARIO_ID_SLIM_VIDEO:
			sensor_info->SensorGrabStartX = imgsensor_info.slim_video.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.slim_video.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.slim_video.mipi_data_lp2hs_settle_dc;

			break;
		case MSDK_SCENARIO_ID_CUSTOM1:
			sensor_info->SensorGrabStartX = imgsensor_info.custom1.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.custom1.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.custom1.mipi_data_lp2hs_settle_dc;
			break;
		case MSDK_SCENARIO_ID_CUSTOM2:
			sensor_info->SensorGrabStartX = imgsensor_info.custom2.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.custom2.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.custom1.mipi_data_lp2hs_settle_dc;
			break;
		default:
			sensor_info->SensorGrabStartX = imgsensor_info.pre.startx;
			sensor_info->SensorGrabStartY = imgsensor_info.pre.starty;

			sensor_info->MIPIDataLowPwr2HighSpeedSettleDelayCount = imgsensor_info.pre.mipi_data_lp2hs_settle_dc;
			break;
	}

	return ERROR_NONE;
}	/*	get_info  */


static kal_uint32 control(enum MSDK_SCENARIO_ID_ENUM scenario_id, MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
					  MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	LOG_INF("scenario_id = %d\n", scenario_id);
	spin_lock(&imgsensor_drv_lock);
	imgsensor.current_scenario_id = scenario_id;
	spin_unlock(&imgsensor_drv_lock);
	switch (scenario_id) {
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			preview(image_window, sensor_config_data);
			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			capture(image_window, sensor_config_data);
			break;
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			normal_video(image_window, sensor_config_data);
			break;
		case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			hs_video(image_window, sensor_config_data);
			break;
		case MSDK_SCENARIO_ID_SLIM_VIDEO:
			slim_video(image_window, sensor_config_data);
			break;
		case MSDK_SCENARIO_ID_CUSTOM1:
			Custom1(image_window, sensor_config_data);  // Custom1
			break;
		case MSDK_SCENARIO_ID_CUSTOM2:
			Custom2(image_window, sensor_config_data);  // Custom2
			break;
		default:
			LOG_INF("Error ScenarioId setting");
			preview(image_window, sensor_config_data);
			return ERROR_INVALID_SCENARIO_ID;
	}
	return ERROR_NONE;
}	/* control() */



static kal_uint32 set_video_mode(UINT16 framerate)
{
	LOG_INF("framerate = %d\n ", framerate);
	// SetVideoMode Function should fix framerate
	if (framerate == 0)
		// Dynamic frame rate
		return ERROR_NONE;
	spin_lock(&imgsensor_drv_lock);
	if ((framerate == 300) && (imgsensor.autoflicker_en == KAL_TRUE))
		imgsensor.current_fps = 296;
	else if ((framerate == 150) && (imgsensor.autoflicker_en == KAL_TRUE))
		imgsensor.current_fps = 146;
	else
		imgsensor.current_fps = framerate;
	spin_unlock(&imgsensor_drv_lock);
	set_max_framerate(imgsensor.current_fps,1);

	return ERROR_NONE;
}

static kal_uint32 set_auto_flicker_mode(kal_bool enable, UINT16 framerate)
{
	LOG_INF("enable = %d, framerate = %d \n", enable, framerate);
	spin_lock(&imgsensor_drv_lock);
	if (enable) //enable auto flicker
		imgsensor.autoflicker_en = KAL_TRUE;
	else //Cancel Auto flick
		imgsensor.autoflicker_en = KAL_FALSE;
	spin_unlock(&imgsensor_drv_lock);
	return ERROR_NONE;
}


static kal_uint32 set_max_framerate_by_scenario(enum MSDK_SCENARIO_ID_ENUM scenario_id, MUINT32 framerate)
{
	kal_uint32 frame_length;

	LOG_INF("scenario_id = %d, framerate = %d\n", scenario_id, framerate);

	switch (scenario_id) {
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			frame_length = imgsensor_info.pre.pclk / framerate * 10 / imgsensor_info.pre.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.pre.framelength) ? (frame_length - imgsensor_info.pre.framelength) : 0;
			imgsensor.frame_length = imgsensor_info.pre.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			if(framerate == 0)
				return ERROR_NONE;
			frame_length = imgsensor_info.normal_video.pclk / framerate * 10 / imgsensor_info.normal_video.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.normal_video.framelength) ? (frame_length - imgsensor_info.normal_video.framelength) : 0;
			imgsensor.frame_length = imgsensor_info.normal_video.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
        case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
        	  if (imgsensor.current_fps == imgsensor_info.cap1.max_framerate) {
                frame_length = imgsensor_info.cap1.pclk / framerate * 10 / imgsensor_info.cap1.linelength;
                spin_lock(&imgsensor_drv_lock);
		            imgsensor.dummy_line = (frame_length > imgsensor_info.cap1.framelength) ? (frame_length - imgsensor_info.cap1.framelength) : 0;
		            imgsensor.frame_length = imgsensor_info.cap1.framelength + imgsensor.dummy_line;
		            imgsensor.min_frame_length = imgsensor.frame_length;
		            spin_unlock(&imgsensor_drv_lock);
            } else if (imgsensor.current_fps == imgsensor_info.cap2.max_framerate) {
                frame_length = imgsensor_info.cap2.pclk / framerate * 10 / imgsensor_info.cap2.linelength;
                spin_lock(&imgsensor_drv_lock);
		            imgsensor.dummy_line = (frame_length > imgsensor_info.cap2.framelength) ? (frame_length - imgsensor_info.cap2.framelength) : 0;
		            imgsensor.frame_length = imgsensor_info.cap2.framelength + imgsensor.dummy_line;
		            imgsensor.min_frame_length = imgsensor.frame_length;
		            spin_unlock(&imgsensor_drv_lock);
            } else {
        		    if (imgsensor.current_fps != imgsensor_info.cap.max_framerate)
                    LOG_INF("Warning: current_fps %d fps is not support, so use cap's setting: %d fps!\n",framerate,imgsensor_info.cap.max_framerate/10);
                frame_length = imgsensor_info.cap.pclk / framerate * 10 / imgsensor_info.cap.linelength;
                spin_lock(&imgsensor_drv_lock);
		            imgsensor.dummy_line = (frame_length > imgsensor_info.cap.framelength) ? (frame_length - imgsensor_info.cap.framelength) : 0;
		            imgsensor.frame_length = imgsensor_info.cap.framelength + imgsensor.dummy_line;
		            imgsensor.min_frame_length = imgsensor.frame_length;
		            spin_unlock(&imgsensor_drv_lock);
            }
		if (imgsensor.frame_length > imgsensor.shutter){
			set_dummy();
            }
            break;
		case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			frame_length = imgsensor_info.hs_video.pclk / framerate * 10 / imgsensor_info.hs_video.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.hs_video.framelength) ? (frame_length - imgsensor_info.hs_video.framelength) : 0;
			imgsensor.frame_length = imgsensor_info.hs_video.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
		case MSDK_SCENARIO_ID_SLIM_VIDEO:
			frame_length = imgsensor_info.slim_video.pclk / framerate * 10 / imgsensor_info.slim_video.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.slim_video.framelength) ? (frame_length - imgsensor_info.slim_video.framelength): 0;
			imgsensor.frame_length = imgsensor_info.slim_video.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
		case MSDK_SCENARIO_ID_CUSTOM1:
			frame_length = imgsensor_info.custom1.pclk / framerate * 10 / imgsensor_info.custom1.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.custom1.framelength) ? (frame_length - imgsensor_info.custom1.framelength) : 0;
			if (imgsensor.dummy_line < 0) {
				imgsensor.dummy_line = 0;
			}
			imgsensor.frame_length = imgsensor_info.custom1.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
		case MSDK_SCENARIO_ID_CUSTOM2:
			frame_length = imgsensor_info.custom2.pclk / framerate * 10 / imgsensor_info.custom2.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.custom2.framelength) ? (frame_length - imgsensor_info.custom2.framelength) : 0;
			if (imgsensor.dummy_line < 0) {
				imgsensor.dummy_line = 0;
			}
			imgsensor.frame_length = imgsensor_info.custom2.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			break;
		default:  //coding with  preview scenario by default
			frame_length = imgsensor_info.pre.pclk / framerate * 10 / imgsensor_info.pre.linelength;
			spin_lock(&imgsensor_drv_lock);
			imgsensor.dummy_line = (frame_length > imgsensor_info.pre.framelength) ? (frame_length - imgsensor_info.pre.framelength) : 0;
			imgsensor.frame_length = imgsensor_info.pre.framelength + imgsensor.dummy_line;
			imgsensor.min_frame_length = imgsensor.frame_length;
			spin_unlock(&imgsensor_drv_lock);
			if (imgsensor.frame_length > imgsensor.shutter){
				set_dummy();
			}
			LOG_INF("error scenario_id = %d, we use preview scenario \n", scenario_id);
			break;
	}
	return ERROR_NONE;
}


static kal_uint32 get_default_framerate_by_scenario(enum MSDK_SCENARIO_ID_ENUM scenario_id, MUINT32 *framerate)
{
	LOG_INF("scenario_id = %d\n", scenario_id);

	switch (scenario_id) {
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			*framerate = imgsensor_info.pre.max_framerate;
			break;
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			*framerate = imgsensor_info.normal_video.max_framerate;
			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			*framerate = imgsensor_info.cap.max_framerate;
			break;
		case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
			*framerate = imgsensor_info.hs_video.max_framerate;
			break;
		case MSDK_SCENARIO_ID_SLIM_VIDEO:
			*framerate = imgsensor_info.slim_video.max_framerate;
			break;
		case MSDK_SCENARIO_ID_CUSTOM1:
			*framerate = imgsensor_info.custom1.max_framerate;
			break;
		case MSDK_SCENARIO_ID_CUSTOM2:
			*framerate = imgsensor_info.custom2.max_framerate;
			break;
		default:
			break;
	}

	return ERROR_NONE;
}

static kal_uint32 set_test_pattern_mode(kal_bool enable)
{
	LOG_INF("enable: %d\n", enable);

	if (enable) {
		// 0x5E00[8]: 1 enable,  0 disable
		// 0x5E00[1:0]; 00 Color bar, 01 Random Data, 10 Square, 11 BLACK
    write_cmos_sensor_8(0x0602,0x00);
    write_cmos_sensor_8(0x0603,0x00);
    write_cmos_sensor_8(0x0604,0x00);
    write_cmos_sensor_8(0x0605,0x00);
    write_cmos_sensor_8(0x0606,0x00);
    write_cmos_sensor_8(0x0607,0x00);
    write_cmos_sensor_8(0x0608,0x00);
    write_cmos_sensor_8(0x0609,0x00);
		write_cmos_sensor_8(0x0601,0x01);
	} else {
		// 0x5E00[8]: 1 enable,  0 disable
		// 0x5E00[1:0]; 00 Color bar, 01 Random Data, 10 Square, 11 BLACK
		write_cmos_sensor_8(0x0600,0x0000);
	}
	spin_lock(&imgsensor_drv_lock);
	imgsensor.test_pattern = enable;
	spin_unlock(&imgsensor_drv_lock);
	return ERROR_NONE;
}
static kal_uint32 feature_control(MSDK_SENSOR_FEATURE_ENUM feature_id,
							 UINT8 *feature_para,UINT32 *feature_para_len)
{
	UINT16 *feature_return_para_16=(UINT16 *) feature_para;
	UINT16 *feature_data_16=(UINT16 *) feature_para;
	UINT32 *feature_return_para_32=(UINT32 *) feature_para;
	UINT32 *feature_data_32=(UINT32 *) feature_para;
	unsigned long long *feature_data=(unsigned long long *) feature_para;
	struct SENSOR_WINSIZE_INFO_STRUCT *wininfo;
	MSDK_SENSOR_REG_INFO_STRUCT *sensor_reg_data=(MSDK_SENSOR_REG_INFO_STRUCT *) feature_para;

	/*LOG_INF("feature_id = %d\n", feature_id);*/
	switch (feature_id) {
		case SENSOR_FEATURE_GET_PERIOD:
			*feature_return_para_16++ = imgsensor.line_length;
			*feature_return_para_16 = imgsensor.frame_length;
			*feature_para_len=4;
			break;
		case SENSOR_FEATURE_GET_PIXEL_CLOCK_FREQ:
			*feature_return_para_32 = imgsensor.pclk;
			*feature_para_len=4;
			break;
		case SENSOR_FEATURE_SET_ESHUTTER:
                     set_shutter(*feature_data);
			break;
		case SENSOR_FEATURE_SET_NIGHTMODE:
                     //night_mode((BOOL) *feature_data);
			break;
#if 0
		#ifdef VENDOR_EDIT
		/*zhengjiang.zhu@Camera.driver, 2017/10/17	add  for  module id*/
		case SENSOR_FEATURE_CHECK_MODULE_ID:
			*feature_return_para_32 = imgsensor_info.module_id;
			break;
		#endif
#endif
		case SENSOR_FEATURE_SET_GAIN:
                     set_gain((UINT16) *feature_data);
			break;
		case SENSOR_FEATURE_SET_FLASHLIGHT:
			break;
		case SENSOR_FEATURE_SET_ISP_MASTER_CLOCK_FREQ:
			break;
		case SENSOR_FEATURE_SET_REGISTER:
			write_cmos_sensor_8(sensor_reg_data->RegAddr, sensor_reg_data->RegData);
			break;
		case SENSOR_FEATURE_GET_REGISTER:
			sensor_reg_data->RegData = read_cmos_sensor_8(sensor_reg_data->RegAddr);
			break;
		case SENSOR_FEATURE_GET_LENS_DRIVER_ID:
			// get the lens driver ID from EEPROM or just return LENS_DRIVER_ID_DO_NOT_CARE
			// if EEPROM does not exist in camera module.
			*feature_return_para_32=LENS_DRIVER_ID_DO_NOT_CARE;
			*feature_para_len=4;
			break;
		case SENSOR_FEATURE_SET_VIDEO_MODE:
                    set_video_mode(*feature_data);
			break;
		case SENSOR_FEATURE_CHECK_SENSOR_ID:
			get_imgsensor_id(feature_return_para_32);
			break;
		case SENSOR_FEATURE_SET_AUTO_FLICKER_MODE:
			set_auto_flicker_mode((BOOL)*feature_data_16,*(feature_data_16+1));
			break;
		case SENSOR_FEATURE_SET_MAX_FRAME_RATE_BY_SCENARIO:
                     set_max_framerate_by_scenario((enum MSDK_SCENARIO_ID_ENUM)*feature_data, *(feature_data+1));
			break;
		case SENSOR_FEATURE_GET_DEFAULT_FRAME_RATE_BY_SCENARIO:
                     get_default_framerate_by_scenario((enum MSDK_SCENARIO_ID_ENUM)*(feature_data), (MUINT32 *)(uintptr_t)(*(feature_data+1)));
			break;
		case SENSOR_FEATURE_SET_TEST_PATTERN:
                       set_test_pattern_mode((BOOL)*feature_data);
			break;
		case SENSOR_FEATURE_GET_TEST_PATTERN_CHECKSUM_VALUE: //for factory mode auto testing
			*feature_return_para_32 = imgsensor_info.checksum_value;
			*feature_para_len=4;
			break;
		case SENSOR_FEATURE_SET_FRAMERATE:
            LOG_INF("current fps :%d\n", (UINT32)*feature_data_32);
			spin_lock(&imgsensor_drv_lock);
                     imgsensor.current_fps = *feature_data_32;
			spin_unlock(&imgsensor_drv_lock);
			break;
		case SENSOR_FEATURE_SET_HDR:
            LOG_INF("ihdr enable :%d\n", (BOOL)*feature_data_32);
			spin_lock(&imgsensor_drv_lock);
			imgsensor.ihdr_mode = *feature_data_32;
			spin_unlock(&imgsensor_drv_lock);
			break;
		case SENSOR_FEATURE_GET_CROP_INFO:
            LOG_INF("SENSOR_FEATURE_GET_CROP_INFO scenarioId:%d\n", (UINT32)*feature_data);
            wininfo = (struct SENSOR_WINSIZE_INFO_STRUCT *)(uintptr_t)(*(feature_data+1));

			switch (*feature_data_32) {
				case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[1],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[2],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[3],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_SLIM_VIDEO:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[4],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_CUSTOM1:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[4],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_CUSTOM2:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[5],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
				case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
				default:
					memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[0],sizeof(struct SENSOR_WINSIZE_INFO_STRUCT));
					break;
			}
			break;
		case SENSOR_FEATURE_SET_IHDR_SHUTTER_GAIN:
			LOG_INF("SENSOR_SET_SENSOR_IHDR LE=%d, SE=%d, Gain=%d\n",(UINT16)*feature_data,(UINT16)*(feature_data+1),(UINT16)*(feature_data+2));
			break;
		case SENSOR_FEATURE_SET_AWB_GAIN:
			break;
		case SENSOR_FEATURE_SET_SHUTTER_FRAME_TIME:
			set_shutter_frame_length((UINT16)*feature_data,(UINT16)*(feature_data+1));
			break;
		case SENSOR_FEATURE_SET_HDR_SHUTTER:
			LOG_INF("SENSOR_FEATURE_SET_HDR_SHUTTER LE=%d, SE=%d\n",(UINT16)*feature_data,(UINT16)*(feature_data+1));
			//ihdr_write_shutter((UINT16)*feature_data,(UINT16)*(feature_data+1));
			break;
		case SENSOR_FEATURE_SET_STREAMING_SUSPEND:
			LOG_INF("SENSOR_FEATURE_SET_STREAMING_SUSPEND\n");
			streaming_control(KAL_FALSE);
			break;
		case SENSOR_FEATURE_SET_STREAMING_RESUME:
			LOG_INF("SENSOR_FEATURE_SET_STREAMING_RESUME, shutter:%llu\n", *feature_data);
			if (*feature_data != 0)
				set_shutter(*feature_data);
			streaming_control(KAL_TRUE);
			break;
		case SENSOR_FEATURE_GET_VC_INFO:
			LOG_INF("SENSOR_FEATURE_GET_VC_INFO %d\n", (UINT16)*feature_data);
			switch (*feature_data_32) {
			case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
				break;
			case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
				break;
			case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
			case MSDK_SCENARIO_ID_CUSTOM1:
			case MSDK_SCENARIO_ID_CUSTOM2:
			default:
				break;
			}
		default:
			break;
	}

	return ERROR_NONE;
}	/*	feature_control()  */

static struct SENSOR_FUNCTION_STRUCT sensor_func = {
	open,
	get_info,
	get_resolution,
	feature_control,
	control,
	close
};

UINT32 S5K5E9YX_DD_U5020_MIPI_RAW_SensorInit(struct SENSOR_FUNCTION_STRUCT **pfFunc)
{
	/* To Do : Check Sensor status here */
	if (pfFunc!=NULL)
		*pfFunc=&sensor_func;
	return ERROR_NONE;
}	/*	OV5693_MIPI_RAW_SensorInit	*/
