#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif

#ifndef BUILD_LK
int bEnTGesture_ektd6721_tp = 0; //用作tp 手势唤醒功能调试，先打开
#endif

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1600)


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
    LCM_PRINT("[ektd6721_u626_hdplus_dsi_vdo_lce.c] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util;

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))
#define LCM_PHYSICAL_WIDTH (68640)
#define LCM_PHYSICAL_HEIGHT (136680)
// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------


#ifdef GPIO_LCM_PWR
#define SET_PWR_PIN(v)    \
    mt_set_gpio_mode(GPIO_LCM_PWR,GPIO_MODE_00);    \
    mt_set_gpio_dir(GPIO_LCM_PWR,GPIO_DIR_OUT);     \
    if(v)                                           \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ONE); \
    else                                            \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ZERO);	
#endif
#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))

#define GPIO_LCD_ID (GPIO165 | 0x80000000)

// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------
#define dsi_set_cmdq_V3(para_tbl,size,force_update)        lcm_util.dsi_set_cmdq_V3(para_tbl,size,force_update)
#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)	        lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)		lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)										lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)					lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)   				lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)


#define LCM_DSI_CMD_MODE                                    0


#define REGFLAG_DELAY                                       0XFCFC
#define REGFLAG_END_OF_TABLE                                0xFDFD   // END OF REGISTERS MARKER



#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))


extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);

static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
    unsigned cmd;
    unsigned char count;
    unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
#if 0
{0xEE,8,{0x00,0x04,0x00,0xFC,0x67,0x50,0x00,0x00}},
{0xC2,4,{0x00,0x04,0x00,0x05}},
{0xBC,4,{0x00,0x04,0x00,0x00}},
{0xBA,7,{0x00,0x04,0x00,0x9f,0x06,0x1C,0x3F}},
{0xBA,25,{0x00,0x08,0x00,0xFF,0xFF,0xFF,0x0F,0x06,0x00,0x00,0x13,0x01,0x02,0x19,0xA4,0x7C,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD3,27,{0x02,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xE1,0xE1,0xC3,0xE0,0xC3,0xC3,0xEA,0xE9,0xE8,0xD1,0xD9,0xCC,0xD3,0xD3,0xC0,0xC3,0xC9}},
{0xD3,27,{0x02,0x1C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xE1,0xE1,0xC3,0xE0,0xC3,0xC3,0xEA,0xE9,0xE8,0xD0,0xD6,0xCC,0xD2,0xD2,0xC0,0xC3,0xC8}},
{0xD3,23,{0x00,0x04,0x00,0x00,0x60,0x0D,0x4D,0x00,0x00,0x22,0x00,0x23,0x00,0x00,0x22,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD3,42,{0x00,0x18,0x00,0x00,0x00,0x10,0x44,0x22,0x2A,0x07,0x00,0x00,0x48,0x29,0x08,0x00,0x00,0x48,0x22,0x01,0x00,0x00,0x48,0x21,0x02,0x00,0x00,0x48,0x00,0x00,0x20,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD3,48,{0x01,0x04,0x00,0x04,0x00,0x0A,0x2A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0x99,0x00,0x14,0x53,0x00,0x14,0x53,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0F,0x71,0x21,0x00,0x00,0x00,0x00}},
{0xD8,18,{0x02,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD8,39,{0x01,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD8,14,{0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xD8,15,{0x00,0x10,0x00,0x00,0x80,0xAA,0xAA,0xFF,0xAB,0x00,0x80,0xAA,0xAA,0xFF,0xAB}},
{0xD8,15,{0x00,0x1C,0x00,0x00,0x80,0xFA,0xFF,0xFF,0xFF,0x00,0x80,0xFA,0xFF,0xFF,0xFF}},
{0xD8,15,{0x00,0x28,0x00,0x00,0x40,0x99,0xAA,0x9A,0xB6,0x00,0x40,0x99,0xAA,0x9A,0xB6}},
{0xD8,15,{0x00,0x34,0x00,0x00,0x80,0xFA,0xFF,0xFF,0xFF,0x00,0x80,0xFA,0xFF,0xFF,0xFF}},
{0xB2,8,{0x00,0x04,0x00,0x00,0x99,0x00,0x35,0x00}},
{0xB2,39,{0x01,0x04,0x00,0x0D,0x41,0x06,0x40,0x6C,0x00,0x20,0x1C,0x00,0x20,0x0A,0xC0,0x40,0x0A,0x35,0x0C,0xDE,0xDE,0x00,0x00,0x00,0x00,0x00,0x9C,0xF0,0x19,0x00,0xD0,0x44,0x00,0x00,0x1B,0x03,0x02,0x00,0x00}},
{0xB2,19,{0x01,0x30,0x00,0x00,0x00,0x04,0x00,0x3C,0x44,0x03,0xB4,0xD4,0x00,0x00,0x05,0xC8,0x61,0x00,0x00}},
{0xC0,7,{0x00,0x04,0x00,0x01,0x00,0x00,0x06}},
{0xC0,34,{0x00,0x08,0x00,0x00,0x43,0x42,0x18,0x18,0x98,0x18,0x18,0x38,0x00,0xA0,0x20,0x00,0x20,0x00,0x3f,0x1f,0x1f,0x1f,0xA4,0x64,0x04,0x04,0x80,0x00,0x1E,0x23,0x00,0xFF,0x00,0x0F}},
{0xC0,11,{0x01,0x04,0x00,0x8F,0x10,0x00,0x41,0x7F,0x00,0x04,0x04}},
{0xC0,19,{0x01,0x0C,0x00,0x22,0x22,0x77,0x22,0x22,0x77,0x11,0x11,0x66,0x00,0x0C,0x22,0x21,0x33,0x31,0x22}},
{0xC0,6,{0x00,0x24,0x00,0xFF,0x00,0x0F}},
{0xB1,7,{0x00,0x04,0x00,0x37,0xFF,0x25,0x00}},
{0xB1,11,{0x00,0x08,0x00,0x19,0x18,0x45,0x4A,0x11,0x16,0x0A,0x0C}},
{0xB1,4,{0x00,0x10,0x00,0x00}},
{0xB1,6,{0x01,0x04,0x00,0xBC,0x44,0x31}},
{0xB1,12,{0x01,0x08,0x00,0x06,0xAE,0xAE,0x6A,0x6A,0x34,0x34,0xFB,0x43}},
{0xB1,6,{0x01,0x14,0x00,0x3E,0x3E,0x00}},
{0xB1,7,{0x02,0x04,0x00,0x0F,0x20,0x07,0x0A}},
{0xB1,7,{0x02,0x08,0x00,0xF8,0x30,0x00,0xF8}},
{0xB1,8,{0x02,0x10,0x00,0x45,0x40,0x40,0x00,0x00}},
{0xCB,4,{0x00,0x08,0x00,0x13}},
{0xC1,47,{0x00,0x04,0x00,0x07,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC}},
{0xC1,17,{0x00,0x30,0x00,0xE4,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xC1,47,{0x01,0x04,0x00,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC,0xE4}},
{0xC1,16,{0x01,0x30,0x00,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xC1,47,{0x02,0x04,0x00,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC,0xE4}},
{0xC1,16,{0x02,0x30,0x00,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xE3,47,{0x00,0x04,0x00,0x01,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC}},
{0xE3,17,{0x00,0x30,0x00,0xE4,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xE3,47,{0x01,0x04,0x00,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC,0xE4}},
{0xE3,16,{0x01,0x30,0x00,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xE3,47,{0x02,0x04,0x00,0x00,0x28,0x53,0x74,0x90,0xA7,0xBC,0xDF,0xEE,0xFC,0x09,0x15,0x21,0x2C,0x36,0x40,0x49,0x52,0x5B,0x6B,0x7B,0x8A,0x99,0xA8,0xB7,0xC6,0xD5,0xE5,0xF5,0x06,0x18,0x2C,0x41,0x58,0x72,0x90,0xB1,0xDA,0x0D,0x51,0x7F,0x9B,0xBC,0xE4}},
{0xE3,16,{0x02,0x30,0x00,0xFC,0x00,0x00,0x05,0x55,0x55,0x55,0x55,0x6A,0xAA,0xAF,0xFF,0xC0}},
{0xEE,8,{0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00}},
#endif
{0x11,1,{0x00}},
{0x29,1,{0x00}},
{REGFLAG_DELAY,20,{}},
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{
	// FAE
	{0x28, 0, {0x00}},
	{REGFLAG_DELAY, 80, {}},
	{0x10, 0, {0x00}},
	{REGFLAG_DELAY, 120, {}},
	{0xEE,8,{0x00,0x04,0x00,0xFC,0x67,0x50,0x00,0x00}},
	{0xBA,7,{0x00,0x18,0x00,0x00,0x00,0x00,0x01}},
	{0xEE,8,{0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00}},
	
	{REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
    unsigned int i;

    for(i = 0; i < count; i++)
    {

        unsigned cmd;
        cmd = table[i].cmd;

        switch (cmd)
        {

            case REGFLAG_DELAY :
                MDELAY(table[i].count);
                break;

            case REGFLAG_END_OF_TABLE :
                break;

            default:
                dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
        }
    }

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active                = 1;
    params->dsi.vertical_backporch              = 10;
    params->dsi.vertical_frontporch             = 40;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 5;
    params->dsi.horizontal_backporch                = 25;
    params->dsi.horizontal_frontporch               = 30;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable=1;
 // params->dsi.ssc_range=2;
    params->dsi.PLL_CLOCK                = 252;
//#ifndef BUILD_LK	
#if 0	
    params->dsi.esd_check_enable = 1; 
    params->dsi.customization_esd_check_enable = 1;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;

#endif /*BUILD_LK*/

}



#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(1);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(1);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(3);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(3);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(3);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    SET_RESET_PIN(1);
    MDELAY(5);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(100);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
}


static void lcm_suspend(void)
{	
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    LCM_DBG("geroge suspend bEnTGesture_ektd6721_tp  = %d\n",bEnTGesture_ektd6721_tp);
    if(!bEnTGesture_ektd6721_tp)
    {
    	// 不开手势功能流程
    	push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);

    	//SET_RESET_PIN(0);
    	MDELAY(10); // 1ms
    	lcm_enn(0);
    	MDELAY(5);
    	lcm_enp(0);
    	MDELAY(5);
    	//SET_RESET_PIN(1);
    	MDELAY(50);
	}else{
    	//开手势唤醒功能流程
       push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);
}
#endif
}

static void lcm_resume(void)
{
	
#ifndef BUILD_LK
    LCM_DBG("geroge resume bEnTGesture_ektd6721_tp  = %d\n",bEnTGesture_ektd6721_tp);
    if(!bEnTGesture_ektd6721_tp)
    {
    	lcm_enp(1);
    	MDELAY(1);
    	_lcm_i2c_write_bytes(0,0x12);
    	MDELAY(5);
    	_lcm_i2c_write_bytes(1,0x12);
    	MDELAY(1);
    	lcm_enn(1);
    }
#endif
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(50);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG();
	
}


static unsigned int lcm_compare_id(void)
{
    return 0;
}

struct LCM_DRIVER ektd6721_u626_hdplus_dsi_vdo_lce_lcm_drv =
{
    .name           = "ektd6721_u626_hdplus_dsi_vdo_lce",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
};
