#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1560)


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
    LCM_PRINT("[ft8006s_u5020f_hdplus_dsi_vdo_tcl.c] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util;

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))
#define LCM_PHYSICAL_WIDTH (65000)
#define LCM_PHYSICAL_HEIGHT (140000)
// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------


#ifdef GPIO_LCM_PWR
#define SET_PWR_PIN(v)    \
    mt_set_gpio_mode(GPIO_LCM_PWR,GPIO_MODE_00);    \
    mt_set_gpio_dir(GPIO_LCM_PWR,GPIO_DIR_OUT);     \
    if(v)                                           \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ONE); \
    else                                            \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ZERO);	
#endif
#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))

#define GPIO_LCD_ID (GPIO165 | 0x80000000)

// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------
#define dsi_set_cmdq_V3(para_tbl,size,force_update)        lcm_util.dsi_set_cmdq_V3(para_tbl,size,force_update)
#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)	        lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)		lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)										lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)					lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)   				lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)


#define LCM_DSI_CMD_MODE                                    0


#define REGFLAG_DELAY                                       0XFCFC
#define REGFLAG_END_OF_TABLE                                0xFDFD   // END OF REGISTERS MARKER

#define LCM_ID       (0x9882)


#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))


extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);
extern int get_lcm_id_status(void);


static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
    unsigned cmd;
    unsigned char count;
    unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
{0x41,1,{0x5A}},
{0x41,2,{0x5A,0x24}},
{0x90,1,{0x5A}},
{0x41,2,{0x5A,0x08}},
{0x80,4,{0xC8,0x2C,0x11,0x00}},
{0x41,2,{0x5A,0x09}},
{0x80,16,{0x00,0x00,0x00,0x00,0x60,0xC5,0x4A,0x01,0x40,0xA0,0x0C,0x82,0x20,0x08,0xF0,0x03}},
{0x90,16,{0x99,0x24,0x42,0x0A,0xE3,0x91,0x80,0xF0,0xD6,0xC3,0x70,0xD8,0x2A,0x21,0x00,0x00}},
{0xA0,16,{0x01,0x55,0x55,0x00,0xA0,0x4D,0x06,0x11,0x0D,0x60,0x5A,0xFF,0xFF,0x03,0xA5,0xE6}},
{0xB0,16,{0x08,0xC9,0x16,0x64,0x0B,0x00,0x00,0x11,0x01,0x60,0x00,0xFF,0xFF,0x03,0xFF,0x34}},
{0xC0,8,{0x0C,0x3F,0x1F,0x9F,0x0F,0x00,0x08,0x00}},
{0x41,2,{0x5A,0x0A}},
{0x80,16,{0xFF,0x72,0x6B,0x03,0x00,0x11,0x1F,0x30,0x42,0x54,0x5B,0x8E,0x71,0xAD,0x87,0x50}},
{0x90,16,{0x82,0x55,0x51,0x41,0x31,0x21,0x12,0x09,0x00,0x11,0x1F,0x30,0x42,0x54,0x5B,0x8E}},
{0xA0,13,{0x71,0xAD,0x87,0x50,0x82,0x55,0x51,0x41,0x31,0x21,0x12,0x09,0x00}},
{0x41,2,{0x5A,0x0B}},
{0x80,16,{0x00,0x00,0x00,0x00,0x00,0x00,0xA0,0x47,0x00,0x00,0x00,0x00,0x00,0x00,0xB0,0x23}},
{0x90,2,{0x15,0x00}},
{0x41,2,{0x5A,0x0C}},
{0x80,16,{0xBA,0x68,0x68,0xC1,0xB0,0x75,0xB0,0x04,0x00,0x60,0x15,0x00,0x50,0x15,0x56,0x51}},
{0x90,16,{0x15,0x55,0x61,0x15,0x00,0x60,0x15,0x00,0x50,0x15,0x56,0x51,0x15,0x55,0x61,0x95}},
{0xA0,16,{0xAB,0x18,0x00,0x05,0x00,0x05,0x00,0x05,0x00,0x49,0x29,0x84,0x52,0x01,0x09,0x00}},
{0xB0,2,{0x00,0x00}},
{0x41,2,{0x5A,0x0D}},
{0x80,15,{0xF0,0xB1,0x71,0xEF,0x4B,0xC0,0x80,0x5A,0x00,0x0E,0x00,0xC0,0x12,0x00,0xAF}},
{0x41,2,{0x5A,0x0E}},
{0x80,9,{0xFF,0x00,0x55,0x55,0x23,0x98,0x76,0x07,0x1C}},
{0x41,2,{0x5A,0x0F}},
{0x80,16,{0xD1,0x07,0x70,0xC0,0x12,0x20,0x64,0x08,0x52,0x51,0x58,0x49,0x03,0x52,0x4C,0x4C}},
{0x90,16,{0x68,0x68,0x68,0x4C,0x4C,0x7C,0x14,0x00,0x20,0x06,0xC2,0x00,0x04,0x06,0x0C,0x40}},
{0xA0,4,{0x00,0x92,0x00,0x04}},
{0x41,2,{0x5A,0x10}},
{0x80,16,{0x00,0x00,0x03,0xE7,0x1F,0x17,0x00,0x48,0x70,0x42,0xD0,0x18,0x30,0x88,0x41,0x8A}},
{0x90,15,{0x39,0x28,0xA9,0xC5,0x9A,0x7B,0xF0,0x07,0x7E,0xE0,0x07,0x7E,0x20,0x10,0x00}},
{0x41,2,{0x5A,0x11}},
{0x80,16,{0x49,0x77,0x03,0x00,0xCA,0xF3,0xFF,0x83,0x30,0x08,0xC4,0x06,0xA1,0xD8,0x24,0x18}},
{0x90,16,{0x30,0xC6,0x66,0xC1,0x80,0x31,0x15,0xCB,0xE5,0xD2,0x68,0x6C,0x36,0x1D,0x04,0x08}},
{0xA0,16,{0x5A,0xD8,0x88,0x60,0xB0,0x81,0x40,0x1A,0x1B,0x48,0x63,0x03,0xB9,0x00,0x1C,0x80}},
{0xB0,16,{0x50,0x30,0x00,0xE0,0xE1,0x01,0x00,0x28,0x0E,0x06,0x43,0x55,0x55,0x55,0x55,0x55}},
{0xC0,16,{0x95,0x88,0x88,0x88,0x88,0x88,0x88,0x08,0x86,0xC6,0xE3,0x81,0x00,0x20,0x00,0x21}},
{0xD0,16,{0x42,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x31,0x04,0x41,0x06,0x00,0x00,0x00}},
{0xE0,16,{0x00,0x92,0x04,0x00,0x92,0x04,0x00,0x00,0x00,0x00,0x92,0x04,0x00,0x85,0x11,0x0C}},
{0xF0,15,{0x00,0x00,0x40,0x00,0x00,0x00,0x00,0x5E,0x4A,0x55,0x79,0x00,0x08,0x31,0x08}},
{0x41,2,{0x5A,0x12}},
{0x80,16,{0x00,0x00,0x00,0x00,0x00,0x02,0x03,0x00,0x00,0x00,0x00,0x02,0x03,0x00,0x41,0x37}},
{0x90,16,{0xF1,0xE7,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x2D,0x23,0x05}},
{0xA0,7,{0xFB,0x08,0x2D,0x23,0x05,0xFB,0x0C}},
{0x41,2,{0x5A,0x13}},
{0x80,16,{0xFD,0x0F,0x00,0x0C,0x00,0x00,0x00,0xA0,0x01,0x08,0x01,0x1C,0x44,0x0C,0xCE,0xE7}},
{0x90,16,{0x62,0x0E,0x24,0x98,0xAC,0x21,0x01,0x00,0xD0,0x93,0x24,0x49,0x06,0x18,0x88,0x13}},
{0xA0,16,{0x88,0x13,0x88,0x13,0x9F,0xB8,0x8B,0x58,0x02,0x88,0x13,0x88,0x13,0x58,0x02,0x00}},
{0xB0,11,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1B,0x10}},
{0x41,2,{0x5A,0x14}},
{0x80,16,{0x01,0x02,0x41,0x36,0xE9,0xEF,0xF7,0xFB,0xFD,0x7E,0x00,0x00,0x00,0x86,0xC5,0x87}},
{0x90,16,{0xC7,0x43,0x20,0x36,0x3C,0x1E,0x02,0xDB,0x90,0x4E,0x13,0xD8,0x88,0x74,0x9A,0x00}},
{0xA0,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xB0,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xDC,0xB0,0x0D,0x2B,0x00,0x40,0xA1}},
{0xC0,16,{0x50,0x78,0x17,0x9E,0xF4,0x90,0x51,0x1C,0x6F,0xC2,0x93,0x1E,0x32,0x8A,0xE3,0x3D}},
{0xD0,16,{0x78,0xD2,0x43,0x46,0x71,0xBC,0x05,0x4F,0x7A,0xC8,0x28,0x8E,0x77,0xE0,0x49,0x0F}},
{0xE0,16,{0x19,0xC5,0xF1,0x06,0x3C,0xE9,0x21,0xA3,0x38,0x5E,0x80,0x27,0x3D,0x64,0x14,0xC7}},
{0xF0,16,{0x2B,0xF0,0xA4,0x87,0x8C,0xE2,0x18,0x01,0xBF,0x5F,0x08,0x00,0x00,0x00,0x00,0x00}},
{0x41,2,{0x5A,0x15}},
{0x80,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x52,0xD9,0xE0,0xF7}},
{0x90,16,{0xA3,0x00,0x0C,0xCB,0x03,0xEA,0x49,0x03,0x00,0x00,0xF6,0x80,0x7A,0xD2,0x00,0x00}},
{0xA0,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0xB0,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xD3,0x34,0xCD}},
{0xC0,16,{0x30,0xA4,0x40,0x13,0x04,0x08,0x84,0x81,0xDA,0xB6,0x6D,0xDB,0xB6,0x35,0x4D,0xD3}},
{0xD0,16,{0x0C,0x43,0xC9,0x04,0x41,0x47,0x31,0x04,0xAB,0x6D,0xDB,0xB6,0x6D,0xDB,0xB6,0x35}},
{0xE0,16,{0xCD,0x30,0xB4,0x40,0x13,0x14,0x0C,0xC5,0x91,0xDA,0xD6,0x6C,0xDB,0xB6,0x6D,0x5B}},
{0xF0,16,{0xD3,0x0C,0x43,0xC8,0x04,0x41,0x06,0x21,0x00,0xAA,0x6D,0xDB,0xB6,0x6D,0x5B,0x00}},
{0x41,2,{0x5A,0x16}},
{0x80,16,{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0x90,6,{0x00,0x00,0x00,0x00,0xF0,0x20}},
{0x41,2,{0x5A,0x18}},
{0x80,16,{0x04,0xFF,0x00,0x80,0x80,0x00,0x04,0x20,0x00,0x01,0x08,0x40,0x00,0x02,0x10,0x80}},
{0x90,2,{0x00,0x04}},
{0x41,2,{0x5A,0x19}},
{0x80,16,{0xC0,0xAF,0xA3,0x9B,0x92,0x8D,0x8A,0x86,0x84,0x83,0x82,0x81,0x00,0x50,0xF6,0xCF}},
{0x90,16,{0xFC,0x2F,0xF3,0xEF,0xCF,0xBF,0x0F,0xFF,0xAF,0xB5,0x71,0x0E,0x6C,0x4A,0x69,0x08}},
{0xA0,4,{0x00,0x00,0x46,0x00}},
{0x41,2,{0x5A,0x1A}},
{0x80,16,{0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38,0x00,0x40}},
{0x90,16,{0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00,0xA0,0xA8}},
{0xA0,16,{0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC,0xFE,0xFF}},
{0xB0,16,{0x00,0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38,0x00}},
{0xC0,16,{0x40,0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00,0xA0}},
{0xD0,16,{0xA8,0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC,0xFE}},
{0xE0,16,{0xFF,0x00,0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38}},
{0xF0,16,{0x00,0x40,0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00}},
{0x41,2,{0x5A,0x1B}},
{0x80,16,{0xA0,0xA8,0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC}},
{0x90,4,{0xFE,0xFF,0x00,0x00}},
{0x41,2,{0x5A,0x20}},
{0x80,7,{0x84,0x00,0x02,0x14,0x00,0x00,0x00}},
{0x41,2,{0x5A,0x22}},
{0x80,13,{0x2D,0xD3,0x00,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x9F,0x00}},
{0x41,2,{0x5A,0x23}},
{0x80,16,{0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0x90,12,{0xFF,0x0F,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0xFF,0x07,0x35}},
{0x41,2,{0x5A,0x24}},
{0x80,16,{0x00,0x03,0x00,0xFF,0xFF,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x2B}},
{0x90,16,{0x5A,0x5A,0x5A,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
{0x42,1,{0x24}},
{0x90,1,{0x00}},
{0x41,2,{0x5A,0x2F}},
{0x19,1,{0x01}},
{0x4C,1,{0x03}},
{0x35,1,{0x00}},
{0x11,0,{}},
{REGFLAG_DELAY,120, {}},
{0x29,0,{}},
{REGFLAG_DELAY,10, {}},
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{
    // Sleep Mode On
    // Display off sequence
    {0x28, 1, {0x00}},
    {REGFLAG_DELAY, 120, {}},//change 50 to 120,FAE,Tinno
    // Sleep Mode On
    {0x10, 1, {0x00}},
    {REGFLAG_DELAY, 50, {}},//change 20 to 50,FAE,2021-10-26
    {0x17,1,{0x5A}},
    {0x18,1,{0x5A}},
    {REGFLAG_DELAY, 50, {}},
    {REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
    unsigned int i;

    for(i = 0; i < count; i++)
    {

        unsigned cmd;
        cmd = table[i].cmd;

        switch (cmd)
        {

            case REGFLAG_DELAY :
                MDELAY(table[i].count);
                break;

            case REGFLAG_END_OF_TABLE :
                break;

            default:
                dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
        }
    }

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active                = 10;
    params->dsi.vertical_backporch              = 150;
    params->dsi.vertical_frontporch             = 240;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 16;
    params->dsi.horizontal_backporch                = 40;
    params->dsi.horizontal_frontporch               = 40;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable=1;
 // params->dsi.ssc_range=2;
    params->dsi.PLL_CLOCK                = 305;

    params->dsi.cont_clock=0;
    params->dsi.clk_lp_per_line_enable=0;
#ifndef BUILD_LK	
    params->dsi.esd_check_enable = 1; 
    params->dsi.customization_esd_check_enable = 1;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;
#endif /*BUILD_LK*/

}



#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(1);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(1);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(3);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(3);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(3);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    SET_RESET_PIN(1);
    MDELAY(5);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(100);
    LCM_DBG("debug,lcm reset end \n");
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG("debug,lcm init end \n");
}


static void lcm_suspend(void)
{	
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    	// 不开手势功能流程
    push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);

    //SET_RESET_PIN(0);
    MDELAY(10); // 1ms
    lcm_enn(0);
    MDELAY(5);
    lcm_enp(0);
    MDELAY(5);
    	//SET_RESET_PIN(1);
    MDELAY(50);
#endif
}

static void lcm_resume(void)
{
	
#ifndef BUILD_LK
    LCM_DBG();
    lcm_enp(1);
    MDELAY(1);
    _lcm_i2c_write_bytes(0,0x12);
    MDELAY(5);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(1);
    lcm_enn(1);
#endif
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(50);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);

}


static unsigned int lcm_compare_id(void)
{
    return 0;
}

struct LCM_DRIVER ft8006s_u5020f_hdplus_dsi_vdo_tcl_lcm_drv =
{
    .name           = "ft8006s_u5020f_hdplus_dsi_vdo_tcl",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
};
