#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif


// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define REGFLAG_DELAY                                       0XFFFC
#define REGFLAG_END_OF_TABLE                                0xFFFD    // END OF REGISTERS MARKER
#define LCM_DSI_CMD_MODE                                    0


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
	LCM_PRINT("[gc9702_u590_hdplus_dsi_vdo_lce] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1440)
#define LCM_PHYSICAL_WIDTH									(62000)
#define LCM_PHYSICAL_HEIGHT									(124000)
// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util = {0};

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))

#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))

#if !defined BUILD_LK
extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);
#endif

static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
	unsigned cmd;
	unsigned char count;
	unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
	{0xD5, 3, {0x61,0x74,0x97}},
	{0xD0, 1, {0x10}},
	{0x60, 1, {0x07}},
	{0x61, 1, {0x3f}},
	{0x62, 1, {0xC3}},
	{0x63, 1, {0x17}},
	{0x64, 1, {0x02}},
	{0x66, 1, {0x40}},
	{0x67, 1, {0xff}},
	{0x68, 1, {0xfB}},
	{0x69, 1, {0x02}},
	{0x6A, 1, {0x0C}},
	{0x6B, 1, {0x7E}},
	{0xba, 1, {0x01}},//4lane en
	{0xb6, 1, {0x22}},
	{0xa5, 1, {0x1B}}, 
	{0xa1, 1, {0xff}},                              
	{0xa4, 1, {0x55}},                                     
	{0xa6, 1, {0x9C}},
	{0xa7, 1, {0x30}},    
	{0xa0, 1, {0xaa}},                       
	{0xF1, 1, {0x54}}, 
	{0xF2, 1, {0x03}},   
	{0xE8, 1, {0x0F}},                       
	{0xE9, 1, {0xFF}}, 
	{0xEA, 1, {0x4F}},  
    {0x9D,3,{0x9D,0x9D,0x9D}},
	{0xC1, 2, {0x01,0x5F}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xD0, 1, {0x11}},
	{0x60, 1, {0x06}},
	{0x62, 1, {0xC8}},
	{0x63, 1, {0x60}},
	{0x64, 1, {0x04}},
	{0x66, 1, {0x89}},
	{0x67, 1, {0x13}},
	{0x68, 1, {0x89}},
	{0x69, 1, {0xC3}},
	{0xB8,2,{0x05,0x50}},
	{0xB9, 1, {0xC3}},
	{0xB7, 1, {0x07}},
	{0xBC,2,{0x00,0x1B}},
	{0xB3, 44,	{0x35,0x36,0x07,0x01,0x24,0x26,0x18,0x1A,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x34,0x34,0x34,0x33,0x38,0x35,0x36,0x06,0x00,0x14,0x16,0x08,0x0A,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x34,0x34,0x34,0x33,0x38}},
	{0xA0, 8, {0x30,0x13,0x15,0x6D,0x30,0x14,0x15,0x6D}},
	{0xA3, 8, {0x30,0x15,0x15,0x6D,0x30,0x16,0x15,0x6D}},
	{0xA7, 5, {0x33,0x15,0x6d,0x15,0x6d}},
	{0xA8, 10, {0x00,0x15,0x05,0xBD,0x03,0x00,0x16,0x05,0xBE,0x03}},
	{0xA9, 10, {0x00,0x17,0x05,0xBF,0x03,0x00,0x18,0x05,0xC0,0x03}},
	{0xAE, 10, {0x00,0x19,0x05,0xC1,0x03,0x00,0x1A,0x05,0xC2,0x03}},
	{0xAF, 10, {0x00,0x1B,0x05,0xC3,0x03,0x00,0x1C,0x05,0xC4,0x03}},
	{0xB0,7,{0x01,0x22,0x01,0x22,0x22,0x22,0x44}},
	{0xE7, 3, {0x01,0x01,0x6f}},
	{0xED, 1, {0x07}},
	{0xEE, 1, {0x75}},
	{0xE6, 3, {0xe7,0x1a,0x70}},

	{0XF1, 19, {0x7f,0x6a,0x5b,0x4e,0x4a,0x3b,0x3f,0x27,0x41,0x3f,0x3d,0x58,0x47,0x4b,0x3f,0x3a,0x2d,0x1f,0x00}},
	{0XF2, 19, {0x7f,0x6a,0x5b,0x4e,0x4a,0x3b,0x3f,0x27,0x41,0x3f,0x3d,0x58,0x47,0x4b,0x3f,0x3a,0x2d,0x1f,0x00}},

	{0xC2, 1, {0x00}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xff, 1, {0xff}},
	{0xD0, 1, {0x00}},
	{0xff, 1, {0xff}},
	{0x35, 1, {0x00}},
	{0x11, 1, {0x00}},
	{0xff, 1, {0xff}},
	{REGFLAG_DELAY, 120, {}},
	{0x29, 1, {0x00}},
	{0xff, 1, {0xff}},
	{REGFLAG_DELAY, 20, {}},
	{REGFLAG_END_OF_TABLE, 0x00, {}}
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{

    {0x28, 1, {0x00}},
    {REGFLAG_DELAY, 50, {}},

    // Sleep Mode On
    {0x10, 1, {0x00}},
    {REGFLAG_DELAY, 20, {}},

    {REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
	unsigned int i;

	for(i = 0; i < count; i++)
	{

		unsigned cmd;
		cmd = table[i].cmd;

		switch (cmd)
		{

			case REGFLAG_DELAY :
				MDELAY(table[i].count);
				break;

			case REGFLAG_END_OF_TABLE :
				break;

			default:
				dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
		}
	}

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active            = 8;
    params->dsi.vertical_backporch              = 16;
    params->dsi.vertical_frontporch             = 16;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 8;
    params->dsi.horizontal_backporch                = 40;
    params->dsi.horizontal_frontporch               = 40;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable= 1;
    params->dsi.ssc_range= 2;
    params->dsi.cont_clock=0;
    params->dsi.clk_lp_per_line_enable=0;
    params->dsi.PLL_CLOCK                = 224;	
#ifndef BUILD_LK
    params->dsi.esd_check_enable = 1;
    params->dsi.customization_esd_check_enable = 1;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;
#endif /*BUILD_LK*/
}

#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
    SET_RESET_PIN(1);
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(20);
    LCM_DBG("debug,lcm reset end \n");
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(1);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(1);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(3);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(3);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(3);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    MDELAY(30);
    LCM_DBG("debug,lcm reset end \n");
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG("debug,lcm init end \n");
}

static void lcm_suspend(void)
{
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);
    SET_RESET_PIN(0);
    MDELAY(10); // 1ms
	
    lcm_enn(0);
    MDELAY(5);
    lcm_enp(0);
    MDELAY(5);
        //SET_RESET_PIN(1);
    MDELAY(50);
#endif
}

static void lcm_resume(void)
{
    LCM_DBG("lcm_resume enter");
    SET_RESET_PIN(1);
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(20);
#ifndef BUILD_LK
    lcm_enp(1);
    MDELAY(10);
    _lcm_i2c_write_bytes(0,0x12);
    MDELAY(10);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(10);
    lcm_enn(1);
#endif
    MDELAY(30);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG();
}

static unsigned int lcm_compare_id(void)
{
    return 1;
}

struct LCM_DRIVER gc9702_u590_hdplus_dsi_vdo_lce_lcm_drv =
{
    .name           = "gc9702_v590_hdplus_dsi_vdo_lce",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
#if (LCM_DSI_CMD_MODE)
    .set_backlight  = lcm_setbacklight,
    .update         = lcm_update,
#endif
};


