#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif

#ifndef BUILD_LK
int bEnTGesture_ili9881_tp = 0; //用作tp 手势唤醒功能调试，先打开
#endif

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1600)


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
    LCM_PRINT("[ili9881_u616_hdplus_dsi_vdo_hcj.c] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util;

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))
#define LCM_PHYSICAL_WIDTH (68640)
#define LCM_PHYSICAL_HEIGHT (136680)
// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------


#ifdef GPIO_LCM_PWR
#define SET_PWR_PIN(v)    \
    mt_set_gpio_mode(GPIO_LCM_PWR,GPIO_MODE_00);    \
    mt_set_gpio_dir(GPIO_LCM_PWR,GPIO_DIR_OUT);     \
    if(v)                                           \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ONE); \
    else                                            \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ZERO);	
#endif
#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))

#define GPIO_LCD_ID (GPIO165 | 0x80000000)

// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------
#define dsi_set_cmdq_V3(para_tbl,size,force_update)        lcm_util.dsi_set_cmdq_V3(para_tbl,size,force_update)
#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)	        lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)		lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)										lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)					lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)   				lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)


#define LCM_DSI_CMD_MODE                                    0


#define REGFLAG_DELAY                                       0XFCFC
#define REGFLAG_END_OF_TABLE                                0xFDFD   // END OF REGISTERS MARKER



#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))


extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);

static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
    unsigned cmd;
    unsigned char count;
    unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
	{0xFF,0x03,{0x98,0x81,0x01}},
	{0x00,0x01,{0x46}},
	{0x01,0x01,{0x16}},
	{0x02,0x01,{0x00}},
	{0x03,0x01,{0x00}},		  
	{0x08,0x01,{0x80}},
	{0x09,0x01,{0x0A}},
	{0x0a,0x01,{0x71}},
	{0x0b,0x01,{0x00}},
	{0x0c,0x01,{0x00}},
	{0x0d,0x01,{0x00}},
	{0x0e,0x01,{0x01}},
	{0x0f,0x01,{0x01}},

	{0x10,0x01,{0x01}},
	{0x11,0x01,{0x01}},
	{0x12,0x01,{0x01}},

	{0x24,0x01,{0x01}},
	{0x25,0x01,{0x0B}},
	{0x26,0x01,{0x00}},	   
	{0x27,0x01,{0x00}},

	{0x31,0x01,{0x21}},
	{0x32,0x01,{0x07}},
	{0x33,0x01,{0x01}},
	{0x34,0x01,{0x00}},
	{0x35,0x01,{0x02}},
	{0x36,0x01,{0x07}},
	{0x37,0x01,{0x07}},
	{0x38,0x01,{0x07}}, 
	{0x39,0x01,{0x07}},  
	{0x3a,0x01,{0x17}},
	{0x3b,0x01,{0x15}},
	{0x3c,0x01,{0x07}},  
	{0x3d,0x01,{0x07}},  
	{0x3e,0x01,{0x13}},
	{0x3f,0x01,{0x11}},

	{0x40,0x01,{0x09}},
	{0x41,0x01,{0x07}},
	{0x42,0x01,{0x07}},
	{0x43,0x01,{0x07}},
	{0x44,0x01,{0x07}},
	{0x45,0x01,{0x07}},
	{0x46,0x01,{0x07}},
	{0x47,0x01,{0x20}},
	{0x48,0x01,{0x07}},
	{0x49,0x01,{0x01}},
	{0x4a,0x01,{0x00}},
	{0x4b,0x01,{0x02}},
	{0x4c,0x01,{0x07}},
	{0x4d,0x01,{0x07}},
	{0x4e,0x01,{0x07}},
	{0x4f,0x01,{0x07}},

	{0x50,0x01,{0x16}},
	{0x51,0x01,{0x14}},
	{0x52,0x01,{0x07}},
	{0x53,0x01,{0x07}},  
	{0x54,0x01,{0x12}},
	{0x55,0x01,{0x10}},
	{0x56,0x01,{0x08}},
	{0x57,0x01,{0x07}},
	{0x58,0x01,{0x07}},  
	{0x59,0x01,{0x07}},  
	{0x5a,0x01,{0x07}},
	{0x5b,0x01,{0x07}},
	{0x5c,0x01,{0x07}},

	{0x61,0x01,{0x08}},  
	{0x62,0x01,{0x07}},
	{0x63,0x01,{0x01}},
	{0x64,0x01,{0x00}},  
	{0x65,0x01,{0x02}},
	{0x66,0x01,{0x07}},
	{0x67,0x01,{0x07}},
	{0x68,0x01,{0x07}},
	{0x69,0x01,{0x07}},
	{0x6a,0x01,{0x10}},
	{0x6b,0x01,{0x12}},
	{0x6c,0x01,{0x07}},
	{0x6d,0x01,{0x07}},
	{0x6e,0x01,{0x14}},
	{0x6f,0x01,{0x16}},

	{0x70,0x01,{0x20}},
	{0x71,0x01,{0x07}},
	{0x72,0x01,{0x07}},
	{0x73,0x01,{0x07}},
	{0x74,0x01,{0x07}},
	{0x75,0x01,{0x07}},
	{0x76,0x01,{0x07}},
	{0x77,0x01,{0x09}},
	{0x78,0x01,{0x07}},  
	{0x79,0x01,{0x01}},
	{0x7a,0x01,{0x00}},
	{0x7b,0x01,{0x02}},
	{0x7c,0x01,{0x07}},
	{0x7d,0x01,{0x07}},
	{0x7e,0x01,{0x07}},
	{0x7f,0x01,{0x07}},

	{0x80,0x01,{0x11}},
	{0x81,0x01,{0x13}},
	{0x82,0x01,{0x07}},
	{0x83,0x01,{0x07}},
	{0x84,0x01,{0x15}},
	{0x85,0x01,{0x17}},
	{0x86,0x01,{0x21}},
	{0x87,0x01,{0x07}},
	{0x88,0x01,{0x07}},
	{0x89,0x01,{0x07}},
	{0x8a,0x01,{0x07}},
	{0x8b,0x01,{0x07}},
	{0x8c,0x01,{0x07}},

	{0xB0,0x01,{0x12}},
	{0xB1,0x01,{0x33}},
	{0xB2,0x01,{0x22}},
	{0xB9,0x01,{0x40}},

	{0xd0,0x01,{0x01}},
	{0xd1,0x01,{0x00}},
	{0xdc,0x01,{0x35}},
	{0xdd,0x01,{0x42}},
	{0xe2,0x01,{0x00}},
	{0xe6,0x01,{0x22}},
	{0xe7,0x01,{0x54}},

	{0xFF,0x03,{0x98,0x81,0x02}},
	{0x40,0x01,{0x4A}},
	{0x4B,0x01,{0x5A}},
	{0x4D,0x01,{0x4E}},
	{0x4E,0x01,{0x00}},
	{0x1A,0x01,{0x48}},

	// GVDDP GVDDN VCOM VGH VGHO VGL VGLO setup
	{0xFF,0x03,{0x98,0x81,0x05}},
	{0x03,0x01,{0x00}}, //VCOM
	{0x04,0x01,{0xcc}}, //VCOM
	{0x58,0x01,{0x62}}, //VGL x2 
	{0x63,0x01,{0x7E}}, //GVDDN  5.0V
	{0x64,0x01,{0x7E}}, //GVDDP -5.0V
	{0x68,0x01,{0xa1}}, //VGHO   15V
	{0x69,0x01,{0xa7}}, //VGH    16V
	{0x6A,0x01,{0xa1}}, //VGLO  79  -10V    -12V 
	{0x6B,0x01,{0x93}}, //VGL   6B -11V   -13V  -12V   //0724

	// Resolution 720RGB*1600
	{0xFF,0x03,{0x98,0x81,0x06}},
	{0x2E,0x01,{0x01}}, //NL enable
	{0xC0,0x01,{0x1F}},
	{0xC1,0x01,{0x03}},

	{0xFF,0x03,{0x98,0x81,0x06}},
	{0x11,0x01,{0x03}},
	{0x13,0x01,{0x44}},		 
	{0x14,0x01,{0x41}},
	{0x15,0x01,{0xF1}},
	{0x16,0x01,{0x40}},
	{0xC2,0x01,{0x04}},
	{0x27,0x01,{0x20}},
	{0x28,0x01,{0x20}},
	{0x48,0x01,{0x0F}}, 
	{0x4D,0x01,{0x80}}, 
	{0x4E,0x01,{0x40}}, 
	{0x7F,0x01,{0x78}},
	{0xD6,0x01,{0x87}},
	{0xDD,0x01,{0x18}},//4LANES
	///////////////////GAMMA/////////////

	{0xFF,0x03,{0x98,0x81,0x08}},							
        {0xE0,0x27,{0x00,0x24,0x6C,0x9E,0xDC,0x55,0x0E,0x35,0x62,0x86,0xA5,0xBE,0xEA,0x12,0x38,0xAA,0x60,0x94,0xB5,0xE0,0xFF,0x05,0x33,0x6B,0x99,0x03,0xEC}},		
        {0xE1,0x27,{0x00,0x24,0x6C,0x9E,0xDC,0x55,0x0E,0x35,0x62,0x86,0xA5,0xBE,0xEA,0x12,0x38,0xAA,0x60,0x94,0xB5,0xE0,0xFF,0x05,0x33,0x6B,0x99,0x03,0xEC}},


	{0xFF,0x03,{0x98,0x81,0x0E}},
	{0x00,0x01,{0xA0}},
	{0x01,0x01,{0x26}},
	{0x11,0x01,{0x90}},
	{0x13,0x01,{0x10}},
               
	{0xFF,0x3,{0x98,0x81,0x03}},
	{0xBC,0x1,{0x06}},
	{0x83,0x1,{0xE0}},
	{0x84,0x1,{0x07}},
	
	{0xFF,0x03,{0x98,0x81,0x00}},
	{0x51,0x02,{0x00,0x00}},
	{0x68,0x02,{0x04,0x00}},
	{0x55,0x01,{0x00}},
	{0x35,0x01,{0x00}},            
	{0x11,01,{0x00}},                   
	{REGFLAG_DELAY,100,{}},      
	{0x29,01,{0x00}},            
	{REGFLAG_DELAY,20,{}},
	{REGFLAG_END_OF_TABLE, 0x00, {}}
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{
	// FAE
	{0x28, 0, {0x00}},
	{REGFLAG_DELAY, 80, {}},
	{0x10, 0, {0x00}},
	{REGFLAG_DELAY, 120, {}},
	{REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
    unsigned int i;

    for(i = 0; i < count; i++)
    {

        unsigned cmd;
        cmd = table[i].cmd;

        switch (cmd)
        {

            case REGFLAG_DELAY :
                MDELAY(table[i].count);
                break;

            case REGFLAG_END_OF_TABLE :
                break;

            default:
                dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
        }
    }

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active            = 8;
    params->dsi.vertical_backporch              = 60;
    params->dsi.vertical_frontporch             = 230;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 16;
    params->dsi.horizontal_backporch                = 60;
    params->dsi.horizontal_frontporch               = 60;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable=1;
    params->dsi.ssc_range=2;
    params->dsi.PLL_CLOCK                = 305;//257
#ifndef BUILD_LK
    params->dsi.esd_check_enable = 1; 
    params->dsi.customization_esd_check_enable = 1;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;
    params->dsi.lcm_esd_check_table[1].cmd = 0x0D;
    params->dsi.lcm_esd_check_table[1].count = 1;
    params->dsi.lcm_esd_check_table[1].para_list[0] = 0x00;
    params->dsi.lcm_esd_check_table[1].cmd = 0x0B;
    params->dsi.lcm_esd_check_table[1].count = 1;
    params->dsi.lcm_esd_check_table[1].para_list[0] = 0x00;

#endif /*BUILD_LK*/

}



#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(1);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(1);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(3);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(3);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(3);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    SET_RESET_PIN(1);
    MDELAY(5);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(100);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
}


static void lcm_suspend(void)
{	
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    LCM_DBG("geroge suspend bEnTGesture_ili9881_tp  = %d\n",bEnTGesture_ili9881_tp);
    if(!bEnTGesture_ili9881_tp)
    {
    	// 不开手势功能流程
    	push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);

    	//SET_RESET_PIN(0);
    	MDELAY(10); // 1ms
    	lcm_enn(0);
    	MDELAY(5);
    	lcm_enp(0);
    	MDELAY(5);
    	//SET_RESET_PIN(1);
    	MDELAY(50);
	}else{
    	//开手势唤醒功能流程
       push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);
}
#endif
}

static void lcm_resume(void)
{
	
#ifndef BUILD_LK
    LCM_DBG("geroge resume bEnTGesture_ili9881_tp  = %d\n",bEnTGesture_ili9881_tp);
    if(!bEnTGesture_ili9881_tp)
    {
    	lcm_enp(1);
    	MDELAY(1);
    	_lcm_i2c_write_bytes(0,0x12);
    	MDELAY(5);
    	_lcm_i2c_write_bytes(1,0x12);
    	MDELAY(1);
    	lcm_enn(1);
    }
#endif
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(50);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG();
	
}


static unsigned int lcm_compare_id(void)
{
    return 1;
}

struct LCM_DRIVER ili9881_u626_hdplus_dsi_vdo_hcj_lcm_drv =
{
    .name           = "ili9881_u616_hdplus_dsi_vdo_hcj",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
};
