#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif


// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define REGFLAG_DELAY                                       0XFFFC
#define REGFLAG_END_OF_TABLE                                0xFFFD    // END OF REGISTERS MARKER
#define LCM_DSI_CMD_MODE                                    0


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
	LCM_PRINT("[nv3051f_u590_hdplus_dsi_vdo_hlt] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1440)
#define LCM_PHYSICAL_WIDTH									(62000)
#define LCM_PHYSICAL_HEIGHT									(124000)
// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util = {0};

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))

#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))

#if !defined BUILD_LK
extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);
#endif

static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
	unsigned cmd;
	unsigned char count;
	unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
 {0xFF, 1,{0x30}},
 {0xFF, 1,{0x52}},
 {0xFF, 1,{0x01}},
 {0xE3, 1,{0x04}},
 {0x08, 1,{0x0E}},
 {0x25, 1,{0x14}},
 {0x28, 1,{0x5F}},
 {0x29, 1,{0xc5}},
 {0x2a, 1,{0x9f}},
 {0x37, 1,{0xAC}},
 {0x38, 1,{0xB7}},
 {0x39, 1,{0x2C}},
 {0x44, 1,{0x00}},
 {0x49, 1,{0x00}},
 {0x59, 1,{0xfe}},
 {0x5C, 1,{0x00}},
 //{0x6D, 1,{0x00}},
// {0x6E, 1,{0x00}},
 {0x91, 1,{0x67}},
 {0x92, 1,{0x67}},
 {0xA0, 1,{0x55}},
 {0xA1, 1,{0x50}},
 {0xA3, 1,{0x58}},
 {0xA4, 1,{0x9C}},
 {0xA7, 1,{0x02}},
 {0xA8, 1,{0x01}},
 {0xA9, 1,{0x21}},
 {0xAA, 1,{0xFC}},
 {0xAB, 1,{0x28}},
 {0xAC, 1,{0x06}},
 {0xAD, 1,{0x06}},
 {0xAE, 1,{0x06}},
 {0xAF, 1,{0x03}},
 {0xB0, 1,{0x08}},
 {0xB1, 1,{0x26}},
 {0xB2, 1,{0x28}},
 {0xB3, 1,{0x28}},
 {0xB4, 1,{0x03}},
 {0xB5, 1,{0x08}},
 {0xB6, 1,{0x26}},
 {0xB7, 1,{0x08}},
 {0xB8, 1,{0x26}},
 {0xF0, 1,{0x00}},
 {0xF6, 1,{0xC0}},
 {0xC0, 1,{0x00}},
 {0xC1, 1,{0x00}},
 {0xC3, 1,{0x0F}},
 {0x2C, 1,{0x22}},
 {0xFF, 1,{0x30}},
 {0xFF, 1,{0x52}},
 {0xFF, 1,{0x02}},
 {0x28, 1,{0x0A}},
 {0x29, 1,{0x01}},
 {0x2A, 1,{0x1f}},
 {0xB1, 1,{0x14}},
 {0xD1, 1,{0x0A}},
 {0xB4, 1,{0x33}},
 {0xD4, 1,{0x2D}},
 {0xB2, 1,{0x15}},
 {0xD2, 1,{0x0B}},
 {0xB3, 1,{0x35}},
 {0xD3, 1,{0x2F}},
 {0xB6, 1,{0x26}},
 {0xD6, 1,{0x20}},
 {0xB7, 1,{0x42}},
 {0xD7, 1,{0x3C}},
 {0xC1, 1,{0x09}},
 {0xE1, 1,{0x09}},
 {0xB8, 1,{0x0D}},
 {0xD8, 1,{0x0D}},
 {0xB9, 1,{0x03}},
 {0xD9, 1,{0x05}},
 {0xBD, 1,{0x12}},
 {0xDD, 1,{0x14}},
 {0xBC, 1,{0x10}},
 {0xDC, 1,{0x12}},
 {0xBB, 1,{0x0F}},
 {0xDB, 1,{0x11}},
 {0xBA, 1,{0x11}},
 {0xDA, 1,{0x11}},
 {0xBE, 1,{0x1A}},
 {0xDE, 1,{0x1A}},
 {0xBF, 1,{0x11}},
 {0xDF, 1,{0x11}},
 {0xC0, 1,{0x18}},
 {0xE0, 1,{0x18}},
 {0xB5, 1,{0x37}},
 {0xD5, 1,{0x32}},
 {0xB0, 1,{0x02}},
 {0xD0, 1,{0x05}},
 {0xFF, 1,{0x30}},
 {0xFF, 1,{0x52}},
 {0xFF, 1,{0x03}},
 {0x04, 1,{0x51}},
 {0x05, 1,{0xc0}},
 {0x06, 1,{0xc0}},
 {0x08, 1,{0x87}},
 {0x09, 1,{0x86}},
 {0x0a, 1,{0x85}},
 {0x0b, 1,{0x84}},
 {0x24, 1,{0x51}},
 {0x25, 1,{0xc0}},
 {0x26, 1,{0xc0}},
 {0x2a, 1,{0x9f}},
 {0x2b, 1,{0xa0}},
 {0x2c, 1,{0xa1}},
 {0x2d, 1,{0xa2}},
 {0x34, 1,{0xa1}},
 {0x35, 1,{0x53}},
 {0x36, 1,{0x53}},
 {0x37, 1,{0x13}},
 {0x40, 1,{0x85}},
 {0x41, 1,{0x84}},
 {0x42, 1,{0x83}},
 {0x43, 1,{0x82}},
 {0x45, 1,{0xa0}},
 {0x46, 1,{0xa1}},
 {0x48, 1,{0xa2}},
 {0x49, 1,{0xa3}},
 {0x50, 1,{0x81}},
 {0x51, 1,{0x80}},
 {0x52, 1,{0x01}},
 {0x53, 1,{0x02}},
 {0x55, 1,{0xa4}},
 {0x56, 1,{0xa5}},
 {0x58, 1,{0xa6}},
 {0x59, 1,{0xa7}},
 {0x80, 1,{0x03}},
 {0x81, 1,{0x02}},
 {0x82, 1,{0x07}},
 {0x83, 1,{0x06}},
 {0x84, 1,{0x05}},
 {0x85, 1,{0x04}},
 {0x92, 1,{0x00}},
 {0x93, 1,{0x01}},
 {0x96, 1,{0x03}},
 {0x97, 1,{0x02}},
 {0x98, 1,{0x07}},
 {0x99, 1,{0x06}},
 {0x9a, 1,{0x05}},
 {0x9b, 1,{0x04}},
 {0xa8, 1,{0x00}},
 {0xa9, 1,{0x01}},
 {0xFF, 1,{0x30}},
 {0xFF, 1,{0x52}},
 {0xFF, 1,{0x00}},
 {0x36, 1,{0x02}},
 {0x35, 1,{0x00}},
 {0x11, 1, {0x00}},
 {REGFLAG_DELAY, 120, {}},
 {0x29, 1, {0x00}},
 {REGFLAG_DELAY, 20, {}},
 {REGFLAG_END_OF_TABLE, 0x00, {}}
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{

    {0x28, 1, {0x00}},
    {REGFLAG_DELAY, 50, {}},

    // Sleep Mode On
    {0x10, 1, {0x00}},
    {REGFLAG_DELAY, 20, {}},

    {REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
	unsigned int i;

	for(i = 0; i < count; i++)
	{

		unsigned cmd;
		cmd = table[i].cmd;

		switch (cmd)
		{

			case REGFLAG_DELAY :
				MDELAY(table[i].count);
				break;

			case REGFLAG_END_OF_TABLE :
				break;

			default:
				dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
		}
	}

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active            = 8;
    params->dsi.vertical_backporch              = 12;//10;
    params->dsi.vertical_frontporch             = 16;//14;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 8;
    params->dsi.horizontal_backporch                = 46;//30;
    params->dsi.horizontal_frontporch               = 46;//24;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable= 1;
    params->dsi.ssc_range= 2;
    params->dsi.cont_clock=0;
    params->dsi.clk_lp_per_line_enable=0;
    params->dsi.PLL_CLOCK                = 224;
#ifndef BUILD_LK
    params->dsi.esd_check_enable = 1;
    params->dsi.customization_esd_check_enable = 0;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;
#endif
}

#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(5);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(20);
    SET_RESET_PIN(1);
    MDELAY(10);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(5);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(5);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(20);
    SET_RESET_PIN(1);
    MDELAY(10);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    MDELAY(20);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG("debug,lcm init end \n");
}

static void lcm_suspend(void)
{
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);
//    SET_RESET_PIN(0);
    MDELAY(10); // 1ms	
    lcm_enn(0);
    MDELAY(5);
    lcm_enp(0);
    MDELAY(5);
    MDELAY(50);
#endif
}

static void lcm_resume(void)
{
    LCM_DBG("lcm_resume enter");
#ifndef BUILD_LK
    lcm_enp(1);
    MDELAY(10);
    _lcm_i2c_write_bytes(0,0x12);
    MDELAY(10);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(20);
    SET_RESET_PIN(1);
    MDELAY(10);
    lcm_enn(1);
#endif
    MDELAY(30);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG();
}

static unsigned int lcm_compare_id(void)
{
    return 1;
}

struct LCM_DRIVER nv3051f_u590_hdplus_dsi_vdo_hlt_lcm_drv =
{
    .name           = "nv3051f_v590_hdplus_dsi_vdo_hlt",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
#if (LCM_DSI_CMD_MODE)
    .set_backlight  = lcm_setbacklight,
    .update         = lcm_update,
#endif
};


