#ifndef BUILD_LK
#include <linux/string.h>
#include <linux/kernel.h>
#endif
#include "lcm_drv.h"

#ifdef BUILD_LK
#include <platform/mt_gpio.h>
#elif defined(BUILD_UBOOT)
#include <asm/arch/mt_gpio.h>
#else
#endif

#ifndef BUILD_LK
int bEnTGesture_ft8006s_tp = 0; //用作tp 手势唤醒功能调试，先打开
#endif

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------

#define FRAME_WIDTH  (720)
#define FRAME_HEIGHT (1560)


#ifndef TRUE
#define   TRUE     1
#endif

#ifndef FALSE
#define   FALSE    0
#endif

#ifdef BUILD_LK
#define LCM_PRINT printf
#else
#if defined(BUILD_UBOOT)
#define LCM_PRINT printf
#else
#define LCM_PRINT printk
#endif
#endif

#define LCM_DBG(fmt, arg...) \
    LCM_PRINT("[st7123_u5020f_hdplus_dsi_vdo_hlt.c] %s (line:%d) :" fmt "\r\n", __func__, __LINE__, ## arg)

// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static struct LCM_UTIL_FUNCS lcm_util;

#define SET_RESET_PIN(v)                                    (lcm_util.set_reset_pin((v)))
#define LCM_PHYSICAL_WIDTH (65000)
#define LCM_PHYSICAL_HEIGHT (140000)
// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------


#ifdef GPIO_LCM_PWR
#define SET_PWR_PIN(v)    \
    mt_set_gpio_mode(GPIO_LCM_PWR,GPIO_MODE_00);    \
    mt_set_gpio_dir(GPIO_LCM_PWR,GPIO_DIR_OUT);     \
    if(v)                                           \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ONE); \
    else                                            \
        mt_set_gpio_out(GPIO_LCM_PWR,GPIO_OUT_ZERO);	
#endif
#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))

#define GPIO_LCD_ID (GPIO165 | 0x80000000)

// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------
#define dsi_set_cmdq_V3(para_tbl,size,force_update)        lcm_util.dsi_set_cmdq_V3(para_tbl,size,force_update)
#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)	        lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)		lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)										lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)					lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)   				lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)


#define LCM_DSI_CMD_MODE                                    0


#define REGFLAG_DELAY                                       0XFCFC
#define REGFLAG_END_OF_TABLE                                0xFDFD   // END OF REGISTERS MARKER

#define LCM_ID       (0x9882)


#define UDELAY(n)                                           (lcm_util.udelay(n))
#define MDELAY(n)                                           (lcm_util.mdelay(n))


extern void lcm_enn(int onoff);
extern void lcm_enp(int onoff);
extern  int _lcm_i2c_write_bytes(unsigned char addr, unsigned char value);
extern int get_lcm_id_status(void);


static unsigned int lcm_compare_id(void);
// ---------------------------------------------------------------------------
//  Local Functions
// ---------------------------------------------------------------------------

#define dsi_set_cmdq_V2(cmd, count, ppara, force_update)    lcm_util.dsi_set_cmdq_V2(cmd, count, ppara, force_update)
#define dsi_set_cmdq(pdata, queue_size, force_update)       lcm_util.dsi_set_cmdq(pdata, queue_size, force_update)
#define wrtie_cmd(cmd)                                      lcm_util.dsi_write_cmd(cmd)
#define write_regs(addr, pdata, byte_nums)                  lcm_util.dsi_write_regs(addr, pdata, byte_nums)
#define read_reg(cmd)                                       lcm_util.dsi_read_reg(cmd)
#define read_reg_v2(cmd, buffer, buffer_size)               lcm_util.dsi_dcs_read_lcm_reg_v2(cmd, buffer, buffer_size)

struct LCM_setting_table
{
    unsigned cmd;
    unsigned char count;
    unsigned char para_list[64];
};


static struct LCM_setting_table lcm_initialization_setting[] =
{
{0x10,0,{}},
{0x28,0,{}},
{0x60,3,{0x71, 0x23, 0xa2}},
{0x60,3,{0x71, 0x23, 0xa3}},
{0x60,3,{0x71, 0x23, 0xa4}},
{0xCF,1,{0x18}},
{0x85,6,{0x48, 0x87, 0xE7, 0x04, 0x87, 0x46}},
{0x86,12,{0x85, 0x19, 0x40, 0x15, 0xF5, 0x01, 0x00, 0xA5, 0xD9, 0x55, 0xF5, 0x0D}},
{0xd7,6,{0x00, 0x11, 0xD6, 0xBB, 0x0c, 0x0c}},
{0xB2,15,{0x15, 0x19, 0x05, 0x23, 0x41, 0xFF, 0x03, 0x52, 0x62, 0xD2, 0xFF, 0x10, 0x20, 0xE9, 0x10}},
{0xE8,5,{0x20, 0x6A, 0x04, 0x7B, 0x7B}},
{0xb7,6,{0x00, 0x00, 0x64, 0x64, 0x44, 0x1A}},
{0xB0,7,{0x22, 0x57, 0x11, 0x75, 0x25, 0x2F, 0x2F}},
{0xA3,39,{0x88, 0x01, 0x8C, 0x30, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46, 0x00, 0x00, 0x1E, 0x5C, 0x1E, 0x80, 0x00, 0x6F, 0x45, 0x00, 0x00, 0x00, 0x00, 0x00, 0x46, 0x00, 0x00, 0x1E, 0x5C, 0x1E, 0x80, 0x00, 0x20, 0x58, 0x00,  0x00, 0x00, 0xFF}},
{0xA6,50,{0x5F, 0x10, 0x24, 0x55, 0x36, 0x00, 0x00, 0x03, 0x43, 0x43, 0x91, 0xFF, 0x00, 0x24, 0x55, 0x37, 0x00, 0x00, 0x02, 0x43, 0x43, 0x91, 0xFF, 0x50, 0xE4, 0x55, 0x07, 0x07, 0x07, 0x09, 0x43, 0x43, 0x91, 0xFF, 0x0F, 0xDB, 0x55, 0x07, 0x07, 0x07, 0x09, 0x43, 0x43, 0xFF, 0xFF, 0x00, 0x08, 0x80, 0x08, 0x80}},
{0xA7,60,{0x19, 0x19, 0x81, 0x64, 0x40, 0x07, 0x16, 0x40, 0x00, 0x44, 0x03, 0x43, 0x43, 0x91, 0xFF, 0x00, 0x80, 0x64, 0x40, 0x25, 0x34, 0x40, 0x00, 0x02, 0x01, 0x43, 0x43, 0x91, 0xFF, 0x00, 0x80, 0x64, 0x40, 0x4B, 0xC3, 0x46, 0x0E, 0x0D, 0x05, 0xFF, 0x90, 0x91, 0xFF, 0x00, 0x80, 0x64, 0x40, 0x69, 0xE1, 0x08, 0x10, 0x2B, 0x03, 0xFF, 0x90, 0x91, 0xFF, 0xF0, 0x80, 0x44}},
{0xAC,40,{0x00, 0x00, 0x1C, 0x1C, 0x04, 0x04, 0x01, 0x18, 0x19, 0x06, 0x09, 0x0B, 0x12, 0x10, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1C, 0x1C, 0x1C, 0x1C, 0x04, 0x04, 0x03, 0x18, 0x19, 0x06, 0x08, 0x0A, 0x13, 0x11, 0x00, 0x00, 0x00, 0x00}},
{0xAD,25,{0xF0, 0x00, 0x46, 0x07, 0x07, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x01, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff}},
{0xAE,7,{0xFC, 0x7F, 0x00, 0xFF, 0x7F, 0x00, 0x00}},
{0xA9,10,{0x00, 0x00, 0x73, 0xFF, 0x00, 0x02, 0x00, 0x00, 0x02, 0x00}},
{0xE7,36,{0x3B, 0x00, 0x00, 0x7C, 0xA1, 0x8C, 0x20, 0x1A, 0xF0, 0xB1, 0x50, 0x00, 0x50, 0xB1, 0x50, 0xB1, 0x50, 0xD8, 0x00, 0x55, 0x00, 0xB1, 0x00, 0x45, 0xC9, 0x6A, 0xFF, 0x5A, 0xD8, 0x38, 0x88, 0x15, 0xB1, 0x01, 0x01, 0x71}},
{0xEA,10,{0x16, 0x00, 0xFF, 0x01, 0x01, 0x86, 0x01, 0x1E, 0x80, 0x40}},
{0x75,1,{0x29}},
{0x89,2,{0x00, 0x00}},
{0xBF,1,{0x8d}},
{0xC8,37,{0x00, 0x00, 0x07, 0x0D, 0x18, 0x00, 0x2B, 0x3C, 0x55, 0x04, 0x10, 0xB0, 0x07, 0x29, 0x03, 0x11, 0x83, 0x00, 0xCE, 0x00, 0x22, 0x15, 0x0E, 0x68, 0x08, 0x32, 0xDE, 0x0A, 0x42, 0x0C, 0xF3, 0x94, 0x01, 0xC9, 0xE6, 0x03, 0xFF}},
{0xC9,37,{0x00, 0x00, 0x07, 0x0D, 0x18, 0x00, 0x2B, 0x3C, 0x55, 0x04, 0x10, 0xB0, 0x07, 0x29, 0x03, 0x11, 0x83, 0x00, 0xCE, 0x00, 0x22, 0x15, 0x0E, 0x68, 0x08, 0x32, 0xDE, 0x0A, 0x42, 0x0C, 0xF3, 0x94, 0x01, 0xC9, 0xE6, 0x03, 0xFF}},
{0x11,0,{}},
{REGFLAG_DELAY,120, {}},
{0x29,0,{}},
{0x35,1,{0x00}},
{REGFLAG_DELAY,10, {}}
};

static struct LCM_setting_table lcm_deep_sleep_mode_in_setting[] =
{
    // Sleep Mode On
    // Display off sequence
    {0x28, 1, {0x00}},
    {REGFLAG_DELAY, 120, {}},
    // Sleep Mode On
    {0x10, 1, {0x00}},
    {REGFLAG_DELAY, 50, {}},
    {0x60,3,{0x71, 0x23, 0xa2}},
    {0x60,3,{0x71, 0x23, 0xa3}},
    {0x60,3,{0x71, 0x23, 0xa4}},
    {0x4F,1,{0x01}},
    {REGFLAG_END_OF_TABLE, 0x00, {}}
};

static void push_table(struct LCM_setting_table *table, unsigned int count, unsigned char force_update)
{
    unsigned int i;

    for(i = 0; i < count; i++)
    {

        unsigned cmd;
        cmd = table[i].cmd;

        switch (cmd)
        {

            case REGFLAG_DELAY :
                MDELAY(table[i].count);
                break;

            case REGFLAG_END_OF_TABLE :
                break;

            default:
                dsi_set_cmdq_V2(cmd, table[i].count, table[i].para_list, force_update);
        }
    }

}


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const struct LCM_UTIL_FUNCS *util)
{
    memcpy(&lcm_util, util, sizeof(struct LCM_UTIL_FUNCS));
}

static void lcm_get_params(struct LCM_PARAMS *params)
{
    memset(params, 0, sizeof(struct LCM_PARAMS));

    params->type   = LCM_TYPE_DSI;

    params->width  = FRAME_WIDTH;
    params->height = FRAME_HEIGHT;

#if (LCM_DSI_CMD_MODE)
    params->dsi.mode   = CMD_MODE;
#else
    params->dsi.mode   = SYNC_PULSE_VDO_MODE;
#endif


    // DSI
    /* Command mode setting */
    params->dsi.LANE_NUM                = LCM_FOUR_LANE;
    //The following defined the fomat for data coming from LCD engine.
    params->dsi.data_format.color_order = LCM_COLOR_ORDER_RGB;
    params->dsi.data_format.trans_seq   = LCM_DSI_TRANS_SEQ_MSB_FIRST;
    params->dsi.data_format.padding     = LCM_DSI_PADDING_ON_LSB;
    params->dsi.data_format.format      = LCM_DSI_FORMAT_RGB888;
    params->physical_width              = LCM_PHYSICAL_WIDTH/1000;
    params->physical_height             = LCM_PHYSICAL_HEIGHT/1000;

    // Video mode setting
    params->dsi.PS=LCM_PACKED_PS_24BIT_RGB888;

    params->dsi.vertical_sync_active                = 2;
    params->dsi.vertical_backporch              = 15;
    params->dsi.vertical_frontporch             = 214;
    params->dsi.vertical_active_line                = FRAME_HEIGHT;

    params->dsi.horizontal_sync_active              = 16;
    params->dsi.horizontal_backporch                = 50;
    params->dsi.horizontal_frontporch               = 50;
    params->dsi.horizontal_active_pixel         = FRAME_WIDTH;
    params->dsi.compatibility_for_nvk = 0;
    params->dsi.ssc_disable=1;
 // params->dsi.ssc_range=2;
    params->dsi.PLL_CLOCK                = 283;
#ifndef BUILD_LK	
    params->dsi.esd_check_enable = 1; 
    params->dsi.customization_esd_check_enable = 1;//0:te esd check 1:read register
    params->dsi.lcm_esd_check_table[0].cmd = 0x0A;
    params->dsi.lcm_esd_check_table[0].count = 1;
    params->dsi.lcm_esd_check_table[0].para_list[0] = 0x9C;

#endif /*BUILD_LK*/

}



#ifdef BUILD_LK
static int gpio_bl_enp   = (168 | 0x80000000);
static int gpio_bl_enn   = (164 | 0x80000000);
static struct mt_i2c_t LCD_i2c;

static kal_uint32 lcd_write_byte(kal_uint8 addr, kal_uint8 value)
{
    kal_uint32 ret_code = I2C_OK;
    kal_uint8 write_data[2];
    kal_uint16 len;

    write_data[0]= addr;
    write_data[1] = value;

    LCD_i2c.id = I2C0;

    LCD_i2c.addr = (0x3e);
    LCD_i2c.mode = ST_MODE;
    LCD_i2c.speed = 100;
    len = 2;

    ret_code = i2c_write(&LCD_i2c, write_data, len);

    return ret_code;
}
#endif

static void lcm_init(void)
{
    LCM_DBG();
#ifdef BUILD_LK
    mt_set_gpio_mode(gpio_bl_enp,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enp,GPIO_DIR_OUT);
    mt_set_gpio_mode(gpio_bl_enn,GPIO_MODE_00);
    mt_set_gpio_dir(gpio_bl_enn,GPIO_DIR_OUT);
    mt_set_gpio_out(gpio_bl_enp,GPIO_OUT_ONE);
    MDELAY(1);
    lcd_write_byte(0,0x12);
    MDELAY(1);
    lcd_write_byte(1,0x12);
    MDELAY(1);
    mt_set_gpio_out(gpio_bl_enn,GPIO_OUT_ONE);
#else //Kernel driver
    lcm_enp(1);
    MDELAY(3);
    _lcm_i2c_write_bytes(0,0x12);//5.8
    MDELAY(3);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(3);
    lcm_enn(1);
#endif
#ifdef GPIO_LCM_PWR
    SET_PWR_PIN(0);
    MDELAY(20);
    SET_PWR_PIN(1);
    MDELAY(150);
#endif
    SET_RESET_PIN(1);
    MDELAY(5);
    SET_RESET_PIN(0);
    MDELAY(10);
    SET_RESET_PIN(1);
    MDELAY(100);
    LCM_DBG("debug,lcm reset end \n");
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
    LCM_DBG("debug,lcm init end \n");
}


static void lcm_suspend(void)
{	
    LCM_DBG("lcm_suspend enter");
#ifndef BUILD_LK
    // 不开手势功能流程
    push_table(lcm_deep_sleep_mode_in_setting, sizeof(lcm_deep_sleep_mode_in_setting) / sizeof(struct LCM_setting_table), 1);
    //SET_RESET_PIN(0);
    MDELAY(10); // 1ms
    lcm_enn(0);
    MDELAY(5);
    lcm_enp(0);
    MDELAY(5);
    //SET_RESET_PIN(1);
    MDELAY(50);
#endif
}

static void lcm_resume(void)
{
    LCM_DBG();	
#ifndef BUILD_LK
    lcm_enp(1);
    MDELAY(1);
    _lcm_i2c_write_bytes(0,0x12);
    MDELAY(5);
    _lcm_i2c_write_bytes(1,0x12);
    MDELAY(1);
    lcm_enn(1);
#endif
    MDELAY(10);
    SET_RESET_PIN(0);
    MDELAY(5);
    SET_RESET_PIN(1);
    MDELAY(50);
    push_table(lcm_initialization_setting, sizeof(lcm_initialization_setting) / sizeof(struct LCM_setting_table), 1);
}


static unsigned int lcm_compare_id(void)
{
    return 0;
}

struct LCM_DRIVER st7123_u5020f_hdplus_dsi_vdo_hlt_lcm_drv =
{
    .name           = "st7123_u5020f_hdplus_dsi_vdo_hlt",
    .set_util_funcs = lcm_set_util_funcs,
    .get_params     = lcm_get_params,
    .init           = lcm_init,
    .suspend        = lcm_suspend,
    .resume         = lcm_resume,
    .compare_id    = lcm_compare_id,
};
