/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2019 MediaTek Inc.
*/

#ifndef __M4U_PORT_PRIV_H__
#define __M4U_PORT_PRIV_H__

static const char *const gM4U_SMILARB[] = {
	"mediatek,smi_larb0", "mediatek,smi_larb1", "mediatek,smi_larb2",
	"mediatek,smi_larb3", "mediatek,smi_larb4"
};

#define M4U0_PORT_INIT(name, slave, larb, port)  {\
		name, 0, slave, larb, port, (((larb)<<7)|((port)<<2)), 1\
}

#define M4U_SLAVE0 (0)
#define M4U_SLAVE1 (0)
#define M4U_SLAVE2 (0)
#define M4U_SLAVE3 (0)
#define M4U_SLAVE4 (0)

#define M4U_LARB0 (0)
#define M4U_LARB1 (1)
#define M4U_LARB2 (2)
#define M4U_LARB3 (3)
#define M4U_LARB4 (4)

struct m4u_port_t gM4uPort[] = {
	/*Larb0 */
	M4U0_PORT_INIT("DISP_OVL0", M4U_SLAVE0, M4U_LARB0, 0),
	M4U0_PORT_INIT("DISP_2L_OVL0_LARB0", M4U_SLAVE0, M4U_LARB0, 1),
	M4U0_PORT_INIT("DISP_RDMA0", M4U_SLAVE0, M4U_LARB0, 2),
	M4U0_PORT_INIT("DISP_WDMA0", M4U_SLAVE0, M4U_LARB0, 3),
	M4U0_PORT_INIT("MDP_RDMA0", M4U_SLAVE0, M4U_LARB0, 4),
	M4U0_PORT_INIT("MDP_WDMA0", M4U_SLAVE0, M4U_LARB0, 5),
	M4U0_PORT_INIT("MDP_WROT0", M4U_SLAVE0, M4U_LARB0, 6),
	M4U0_PORT_INIT("DISP_FAKE0", M4U_SLAVE0, M4U_LARB0, 7),
	/*Larb1 */
	M4U0_PORT_INIT("VDEC_MC", M4U_SLAVE1, M4U_LARB1, 0),
	M4U0_PORT_INIT("VDEC_PP", M4U_SLAVE1, M4U_LARB1, 1),
	M4U0_PORT_INIT("VDEC_VLD", M4U_SLAVE1, M4U_LARB1, 2),
	M4U0_PORT_INIT("VDEC_VLD2", M4U_SLAVE1, M4U_LARB1, 3),
	M4U0_PORT_INIT("VDEC_AVC_MV", M4U_SLAVE1, M4U_LARB1, 4),
	M4U0_PORT_INIT("VDEC_PRED_RD", M4U_SLAVE1, M4U_LARB1, 5),
	M4U0_PORT_INIT("VDEC_PRED_WR", M4U_SLAVE1, M4U_LARB1, 6),
	M4U0_PORT_INIT("VDEC_PPWRAP", M4U_SLAVE1, M4U_LARB1, 7),
	M4U0_PORT_INIT("VDEC_TILE", M4U_SLAVE1, M4U_LARB1, 8),
	/*Larb2 */
	M4U0_PORT_INIT("CAM_IMGI", M4U_SLAVE2, M4U_LARB2, 0),
	M4U0_PORT_INIT("CAM_IMG2O", M4U_SLAVE2, M4U_LARB2, 1),
	M4U0_PORT_INIT("CAM_IMG3O", M4U_SLAVE2, M4U_LARB2, 2),
	M4U0_PORT_INIT("CAM_VIPI", M4U_SLAVE2, M4U_LARB2, 3),
	M4U0_PORT_INIT("CAM_LCEI", M4U_SLAVE2, M4U_LARB2, 4),
	M4U0_PORT_INIT("CAM_FD_RP", M4U_SLAVE2, M4U_LARB2, 5),
	M4U0_PORT_INIT("CAM_FD_WR", M4U_SLAVE2, M4U_LARB2, 6),
	M4U0_PORT_INIT("CAM_FD_RB", M4U_SLAVE2, M4U_LARB2, 7),
	M4U0_PORT_INIT("CAM_DPE_RDMA", M4U_SLAVE2, M4U_LARB2, 8),
	M4U0_PORT_INIT("CAM_DPE_WDMA", M4U_SLAVE2, M4U_LARB2, 9),
	M4U0_PORT_INIT("CAM_RSC_RDMA", M4U_SLAVE2, M4U_LARB2, 10),
	M4U0_PORT_INIT("CAM_RSC_WDMA", M4U_SLAVE2, M4U_LARB2, 11),
	/*Larb3 */
	M4U0_PORT_INIT("CAM_IMGO", M4U_SLAVE3, M4U_LARB3, 0),
	M4U0_PORT_INIT("CAM_RRZO", M4U_SLAVE3, M4U_LARB3, 1),
	M4U0_PORT_INIT("CAM_AAO", M4U_SLAVE3, M4U_LARB3, 2),
	M4U0_PORT_INIT("CAM_AFO", M4U_SLAVE3, M4U_LARB3, 3),
	M4U0_PORT_INIT("CAM_LSCI0", M4U_SLAVE3, M4U_LARB3, 4),
	M4U0_PORT_INIT("CAM_LSCI1", M4U_SLAVE3, M4U_LARB3, 5),
	M4U0_PORT_INIT("CAM_PDO", M4U_SLAVE3, M4U_LARB3, 6),
	M4U0_PORT_INIT("CAM_BPCI", M4U_SLAVE3, M4U_LARB3, 7),
	M4U0_PORT_INIT("CAM_LCSO", M4U_SLAVE3, M4U_LARB3, 8),
	M4U0_PORT_INIT("CAM_RSSO_A", M4U_SLAVE3, M4U_LARB3, 9),
	M4U0_PORT_INIT("CAM_RSSO_B", M4U_SLAVE3, M4U_LARB3, 10),
	M4U0_PORT_INIT("CAM_UFEO", M4U_SLAVE3, M4U_LARB3, 11),
	M4U0_PORT_INIT("CAM_SOCO", M4U_SLAVE3, M4U_LARB3, 12),
	M4U0_PORT_INIT("CAM_SOC1", M4U_SLAVE3, M4U_LARB3, 13),
	M4U0_PORT_INIT("CAM_SOC2", M4U_SLAVE3, M4U_LARB3, 14),
	M4U0_PORT_INIT("CAM_CCUI", M4U_SLAVE3, M4U_LARB3, 15),
	M4U0_PORT_INIT("CAM_CCUO", M4U_SLAVE3, M4U_LARB3, 16),
	M4U0_PORT_INIT("CAM_CACI", M4U_SLAVE3, M4U_LARB3, 17),
	M4U0_PORT_INIT("CAM_RAWI_A", M4U_SLAVE3, M4U_LARB3, 18),
	M4U0_PORT_INIT("CAM_RAWI_B", M4U_SLAVE3, M4U_LARB3, 19),
	M4U0_PORT_INIT("CAM_CCUG", M4U_SLAVE3, M4U_LARB3, 20),
	/*Larb4 */
	M4U0_PORT_INIT("VENC_RCPU", M4U_SLAVE4, M4U_LARB4, 0),
	M4U0_PORT_INIT("VENC_REC", M4U_SLAVE4, M4U_LARB4, 1),
	M4U0_PORT_INIT("VENC_BSDMA", M4U_SLAVE4, M4U_LARB4, 2),
	M4U0_PORT_INIT("VENC_SV_COMV", M4U_SLAVE4, M4U_LARB4, 3),
	M4U0_PORT_INIT("VENC_RD_COMV", M4U_SLAVE4, M4U_LARB4, 4),
	M4U0_PORT_INIT("JPGENC_RDMA", M4U_SLAVE4, M4U_LARB4, 5),
	M4U0_PORT_INIT("JPGENC_BSDMA", M4U_SLAVE4, M4U_LARB4, 6),
	M4U0_PORT_INIT("VENC_CUR_LUMA", M4U_SLAVE4, M4U_LARB4, 7),
	M4U0_PORT_INIT("VENC_CUR_CHROMA", M4U_SLAVE4, M4U_LARB4, 8),
	M4U0_PORT_INIT("VENC_REF_LUMA", M4U_SLAVE4, M4U_LARB4, 9),
	M4U0_PORT_INIT("VENC_REF_CHROMA", M4U_SLAVE4, M4U_LARB4, 10),

	M4U0_PORT_INIT("UNKNOWN", 0, 0, 0)
};
#endif
