#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/gpio.h>
#include <linux/of.h>
#include <linux/of_gpio.h>
#include <linux/of_device.h>
#include <linux/err.h>
#include <linux/i2c.h>
#include <linux/interrupt.h>
#include <linux/power_supply.h>
#include <linux/irq.h>
#include <linux/delay.h>
#include <linux/workqueue.h>
#include "inc/tcpci.h"
//#include "inc/tcpci.h"
#include "inc/tcpci_typec.h"
#include "inc/tcpci_timer.h"

#define ATTACHED_TYPE_NONE 0x00
#define ATTACHED_TYPE_SRC 0x01
#define ATTACHED_TYPE_SNK 0x02
#define ATTACHED_TYPE_ACCESSORY 0x03

#define ATTACHED_STATE_MASK  (0xC0)
#define DRP_MODE_MASK (0x3)
#define CABLE_DIR_MASK (0xDF)

static struct i2c_client *sgm7220_client;
static int cc_attached = 0;
static int cc_orientation = 0;
bool first_check_state = true;
extern int g_TypeC_Src_Detect;

struct sgm7220_chip
{
    struct i2c_client *client;
    struct workqueue_struct  *cc_wq;
    struct tcpc_device *tcpc;
    struct tcpc_desc *tcpc_desc;
    int irq_gpio;
    int irq_gpio_num;
    struct work_struct dwork;
    struct delayed_work first_check_typec_work;
    struct mutex mlock;
    struct semaphore suspend_lock;
};

static void sgm7220_work_handler(struct work_struct *work)
{
    struct sgm7220_chip *chip =
        container_of(work, struct sgm7220_chip, dwork);
    struct device *cdev = &chip->client->dev;
    int ret = 0,  attach = 0, reg_val = 0;
    int attach_type = ATTACHED_TYPE_NONE;

    if (first_check_state == true)
        return;

    ret = i2c_smbus_read_byte_data(sgm7220_client, 0x08);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "read 0x08 failed");
        return;
    }

    reg_val = i2c_smbus_read_byte_data(sgm7220_client, 0x09);
    if (reg_val < 0)
    {
        dev_err(&sgm7220_client->dev, "isr read 0x09 register failed");
        goto out;
    }

    attach_type = (reg_val & ATTACHED_STATE_MASK) >> 6;
    dev_info(&sgm7220_client->dev, "%s:ATTACHED_STATE=0x%x %d\n", __func__, reg_val, attach_type);
    if (attach_type == ATTACHED_TYPE_SRC)
    {
        attach = 1;
        chip->tcpc_desc->rp_lvl = TYPEC_CC_RP_3_0;
        if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SRC)
        {
            chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SRC;
            tcpci_source_vbus(chip->tcpc,
                              TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_5V, 3000);
            tcpci_notify_typec_state(chip->tcpc);
            chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SRC;
        }
        g_TypeC_Src_Detect = 1;
    }
    else if (attach_type == ATTACHED_TYPE_SNK)
    {
        attach = 1;
        if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SNK)
        {
            chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SNK;
            tcpci_notify_typec_state(chip->tcpc);
            chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SNK;
        }
        g_TypeC_Src_Detect = 0;
    }
    /*else if (attach_type == ATTACHED_TYPE_ACCESSORY)
    {
        attach = 1;
    }*/
    else
    {
        attach = 0;
        chip->tcpc->typec_attach_new = TYPEC_UNATTACHED;
        tcpci_notify_typec_state(chip->tcpc);
        if (chip->tcpc->typec_attach_old == TYPEC_ATTACHED_SRC)
        {
            tcpci_source_vbus(chip->tcpc,
                              TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_0V, 0);
        }
        chip->tcpc->typec_attach_old = TYPEC_UNATTACHED;
        dev_err(cdev, "%s: typec_attach_old=%d, typec_attach_new=%d\n",
                __func__, chip->tcpc->typec_attach_old, chip->tcpc->typec_attach_new);
        g_TypeC_Src_Detect = 0;
    }

    if (attach == cc_attached)
    {
        dev_info(&sgm7220_client->dev, "same state [%s]",
                 cc_attached ? "attached" : "detached");
        goto out;
    }

    cc_attached = attach;
    if (attach_type)
    {
        cc_orientation = !(reg_val & BIT(5)) ? 1 : 2;
    }
    else
    {
        cc_orientation = 0;
    }
    dev_info(&sgm7220_client->dev, "new state [%s]", cc_attached ? "attached" : "detached");
    dev_info(&sgm7220_client->dev, "cc_orientation [%d]", cc_orientation);

out:
    ret = i2c_smbus_write_byte_data(sgm7220_client, 0x09, (reg_val | 0x10));
    if (ret < 0)
        dev_err(&sgm7220_client->dev, "isr clear 0x09 bit4 failed");
}

static int sgm7220_set_drp_mode(void)
{
    int ret = 0;
    u8 value = 0;

    ret = i2c_smbus_read_byte_data(sgm7220_client, 0x0A);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "read 0x0a failed");
        return -ENODEV;
    }

    //dev_info(&sgm7220_client->dev, "0x0a [%x]", ret);
    value = (ret | (DRP_MODE_MASK << 4));
    //dev_info(&sgm7220_client->dev, "0x0a [%x]", value);

    ret = i2c_smbus_write_byte_data(sgm7220_client, 0x0A, value);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "write 0x0a failed");
        return -ENODEV;
    }

    return 0;
}
#if 0
static int sgm7220_set_cable_dir(void)
{
    int ret = 0;
    u8 value = 0;

    ret = i2c_smbus_read_byte_data(sgm7220_client, 0x09);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "read 0x09 failed");
        return -ENODEV;
    }

    //dev_info(&sgm7220_client->dev, "0x09 [%x]", ret);
    value = (ret & CABLE_DIR_MASK);
    //dev_info(&sgm7220_client->dev, "0x09 [%x]", value);

    ret = i2c_smbus_write_byte_data(sgm7220_client, 0x09, value);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "write 0x09 failed");
        return -ENODEV;
    }

    return 0;
}
#endif
static void sgm7220_first_check_typec_work(struct work_struct *work)
{
    struct sgm7220_chip *chip = container_of(work,
                                struct sgm7220_chip, first_check_typec_work.work);
    int ret = 0, reg_val = 0;
    int attach_type = ATTACHED_TYPE_NONE;

    ret = i2c_smbus_read_byte_data(sgm7220_client, 0x08);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "read 0x08 failed");
        return;
    }

    reg_val = i2c_smbus_read_byte_data(sgm7220_client, 0x09);
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "read 0x09 failed");
        return;
    }

    attach_type = (reg_val & ATTACHED_STATE_MASK) >> 6;
    dev_info(&sgm7220_client->dev, "%s:ATTACHED_STATE=%x %d\n", __func__, reg_val, attach_type);
    if (attach_type == ATTACHED_TYPE_SRC)
    {
        if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SRC)
        {
            chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SRC;
            tcpci_source_vbus(chip->tcpc,
                              TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_5V, 3000);
            tcpci_notify_typec_state(chip->tcpc);
            chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SRC;
        }
        g_TypeC_Src_Detect = 1;
    }
    else if (attach_type == ATTACHED_TYPE_SNK)
    {
        if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SNK)
        {
            chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SNK;
            tcpci_notify_typec_state(chip->tcpc);
            chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SNK;
        }
        g_TypeC_Src_Detect = 0;
    }
    /*else if (attach_type == ATTACHED_TYPE_ACCESSORY)
    {
        dev_info(&sgm7220_client->dev, "accessory attached");
    }*/
    else
    {
        chip->tcpc->typec_attach_new = TYPEC_UNATTACHED;
        tcpci_notify_typec_state(chip->tcpc);
        if (chip->tcpc->typec_attach_old == TYPEC_ATTACHED_SRC)
        {
            tcpci_source_vbus(chip->tcpc,
                              TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_0V, 0);
        }
        chip->tcpc->typec_attach_old = TYPEC_UNATTACHED;
        g_TypeC_Src_Detect = 0;
        dev_err(&sgm7220_client->dev, "%s: typec_attach_old=%d typec_attach_new=%d\n",
                __func__, chip->tcpc->typec_attach_old, chip->tcpc->typec_attach_new);
    }

    ret = i2c_smbus_write_byte_data(sgm7220_client, 0x09, (reg_val | 0x10));
    if (ret < 0)
    {
        dev_err(&sgm7220_client->dev, "isr clr write 0x09 failed");
        return;
    }

    first_check_state = false;
}

static irqreturn_t sgm7220_interrupt(int irq, void *data)
{
    struct sgm7220_chip *chip = (struct sgm7220_chip *)data;
    if (!chip)
    {
        pr_err("%s : called before init.\n", __func__);
        return IRQ_HANDLED;
    }

    queue_work(chip->cc_wq, &chip->dwork);
    return IRQ_HANDLED;
}

static int sgm7220_init_gpio(struct sgm7220_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int ret = 0;

    chip->irq_gpio_num = of_get_named_gpio(cdev->of_node,
                                           "sgm7220,int-gpio", 0);
    dev_info(cdev, "%s: int_gpio: %d\n", __func__, chip->irq_gpio_num);

    ret = devm_gpio_request(&chip->client->dev,
                            chip->irq_gpio_num, "type_c_port0");
    if (ret < 0)
    {
        dev_err(cdev, "Error: failed to request GPIO %d (ret = %d)\n",
                chip->irq_gpio_num, ret);
    }

    ret = gpio_direction_input(chip->irq_gpio_num);
    if (ret < 0)
    {
        dev_err(cdev,
                "Error: failed to set GPIO %d as input pin(ret = %d)\n",
                chip->irq_gpio_num, ret);
    }

    chip->irq_gpio = gpio_to_irq(chip->irq_gpio_num);
    if (chip->irq_gpio  <= 0)
    {
        dev_err(cdev, "gpio to irq fail, chip->irq(%d)\n",
                chip->irq_gpio);
    }

    ret = request_irq(chip->irq_gpio, sgm7220_interrupt,
                      IRQF_TRIGGER_FALLING | IRQF_ONESHOT,
                      "sgm7220_irq", chip);
    if (ret)
        dev_err(cdev, "unable to request int_gpio %d\n",
                chip->irq_gpio_num);

    dev_info(cdev, "%s: name=%s, gpio=%d, IRQ number=%d\n",
             __func__, chip->tcpc_desc->name,
             chip->irq_gpio_num, chip->irq_gpio);
    return ret;
}

static void sgm7220_free_gpio(struct sgm7220_chip *chip)
{
    if (gpio_is_valid(chip->irq_gpio_num))
        gpio_free(chip->irq_gpio_num);
}
static int sgm7220_read_device_id(void)
{
    int rc;

    rc = i2c_smbus_read_byte_data(sgm7220_client, 0x00);
    if (rc < 0)
        return rc;

    dev_info(&sgm7220_client->dev, "%s: device id=0x%02x\n", __func__, rc);
    return rc;
}

int sgm7220_alert_status_clear(struct tcpc_device *tcpc,
                               uint32_t mask)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_tcpc_init(struct tcpc_device *tcpc, bool sw_reset)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int sgm7220_fault_status_clear(struct tcpc_device *tcpc,
                               uint8_t status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int sgm7220_get_alert_mask(struct tcpc_device *tcpc, uint32_t *mask)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int sgm7220_get_alert_status(struct tcpc_device *tcpc,
                             uint32_t *alert)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_get_power_status(struct tcpc_device *tcpc,
                                    uint16_t *pwr_status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int sgm7220_get_fault_status(struct tcpc_device *tcpc,
                             uint8_t *status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_get_cc(struct tcpc_device *tcpc, int *cc1, int *cc2)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_set_cc(struct tcpc_device *tcpc, int pull)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_set_polarity(struct tcpc_device *tcpc, int polarity)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_set_low_rp_duty(struct tcpc_device *tcpc,
                                   bool low_rp)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_set_vconn(struct tcpc_device *tcpc, int enable)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int sgm7220_tcpc_deinit(struct tcpc_device *tcpc_dev)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}

static struct tcpc_ops sgm7220_tcpc_ops =
{
    .init = sgm7220_tcpc_init,
    .alert_status_clear = sgm7220_alert_status_clear,
    .fault_status_clear = sgm7220_fault_status_clear,
    .get_alert_mask = sgm7220_get_alert_mask,
    .get_alert_status = sgm7220_get_alert_status,
    .get_power_status = sgm7220_get_power_status,
    .get_fault_status = sgm7220_get_fault_status,
    .get_cc = sgm7220_get_cc,
    .set_cc = sgm7220_set_cc,
    .set_polarity = sgm7220_set_polarity,
    .set_low_rp_duty = sgm7220_set_low_rp_duty,
    .set_vconn = sgm7220_set_vconn,
    //.set_role = sgm7220_set_role,
    //.get_mode = sgm7220_get_mode,
    .deinit = sgm7220_tcpc_deinit,
};

static int sgm7220_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
    struct sgm7220_chip *chip;
    struct device *cdev = &client->dev;
    struct tcpc_desc *desc;
    int ret;

    if (!i2c_check_functionality(client->adapter,
                                 I2C_FUNC_SMBUS_BYTE_DATA |
                                 I2C_FUNC_SMBUS_WORD_DATA))
    {
        dev_err(cdev, "%s: smbus data not supported!\n", __func__);
        return -EIO;
    }

    chip = devm_kzalloc(cdev, sizeof(struct sgm7220_chip), GFP_KERNEL);
    if (!chip)
    {
        dev_err(cdev, "%s: can't alloc sgm7220_chip\n", __func__);
        return -ENOMEM;
    }

    chip->client = client;
    i2c_set_clientdata(client, chip);
    sgm7220_client = chip->client;
    first_check_state = true;

    ret = sgm7220_read_device_id();
    if (ret != 0x30)
    {
        dev_err(cdev, "%s: sgm7220 not support\n", __func__);
        goto err;
    }

    ret = sgm7220_set_drp_mode();
    if (ret)
    {
        dev_err(&sgm7220_client->dev, "sgm7220 set cc to drp mode failed");
        return -EPERM;
    }

    /*ret = sgm7220_set_cable_dir();
    if (ret)
    {
        dev_err(&sgm7220_client->dev, "sgm7220 set cc cable dir failed");
        return -EPERM;
    }*/

    chip->cc_wq = alloc_ordered_workqueue("sgm7220-wq", WQ_HIGHPRI);
    if (!chip->cc_wq)
    {
        dev_err(cdev, "%s: unable to create workqueue sgm7220-wq\n",
                __func__);
        goto err;
    }

    INIT_WORK(&chip->dwork, sgm7220_work_handler);
    INIT_DELAYED_WORK(&chip->first_check_typec_work, sgm7220_first_check_typec_work);

    desc = devm_kzalloc(cdev, sizeof(*desc), GFP_KERNEL);
    if (!desc)
        return -ENOMEM;

    desc->name = kzalloc(13, GFP_KERNEL);
    if (!desc->name)
        return -ENOMEM;

    strcpy((char *)desc->name, "type_c_port0");
    desc->role_def = TYPEC_ROLE_TRY_SRC;
    desc->rp_lvl = TYPEC_CC_RP_1_5;
    chip->tcpc_desc = desc;

    dev_info(cdev, "%s: type_c_port0, role=%d\n", __func__, desc->role_def);

    chip->tcpc = tcpc_device_register(cdev, desc, &sgm7220_tcpc_ops, chip);
    chip->tcpc->typec_attach_old = TYPEC_UNATTACHED;
    chip->tcpc->typec_attach_new = TYPEC_UNATTACHED;

    ret = sgm7220_init_gpio(chip);
    if (ret)
    {
        dev_err(cdev, "%s: fail to init gpio\n", __func__);
        goto err2;
    }

    schedule_delayed_work(&chip->first_check_typec_work,
                          msecs_to_jiffies(3000));

    enable_irq_wake(chip->irq_gpio);

    dev_err(&client->dev, "sgm7220 probe finished");

    return 0;

err2:
    destroy_workqueue(chip->cc_wq);
    sgm7220_free_gpio(chip);
err:
    i2c_set_clientdata(client, NULL);
    devm_kfree(cdev, chip);
    return ret;
}

static int sgm7220_remove(struct i2c_client *client)
{
    struct sgm7220_chip *chip = i2c_get_clientdata(client);
    struct device *cdev = &client->dev;

    dev_err(cdev, "%s: enter\n", __func__);

    if (!chip)
    {
        dev_err(cdev, "%s: chip is null\n", __func__);
        return -ENODEV;
    }
    if (chip->irq_gpio > 0)
        devm_free_irq(cdev, chip->irq_gpio, chip);

    cancel_delayed_work_sync(&chip->first_check_typec_work);

    destroy_workqueue(chip->cc_wq);
    sgm7220_free_gpio(chip);
    i2c_set_clientdata(client, NULL);
    devm_kfree(cdev, chip);
    return 0;

    return 0;
}

static const struct i2c_device_id sgm7220_i2c_ids[] =
{
    { "sgm7220", 0 }, { },
};

static const struct of_device_id sgm7220_of_match[] =
{
    { .compatible = "sgmicro,sgm7220", }, { },
};

static struct i2c_driver sgm7220_i2c_driver =
{
    .driver = {
        .name = "sgm7220",
        .of_match_table = sgm7220_of_match,
    },
    .probe = sgm7220_probe,
    .remove = sgm7220_remove,
    .id_table = sgm7220_i2c_ids,
};

static __init int sgm7220_i2c_init(void)
{
    return i2c_add_driver(&sgm7220_i2c_driver);
}
static __exit void sgm7220_i2c_exit(void)
{
    i2c_del_driver(&sgm7220_i2c_driver);
}
subsys_initcall(sgm7220_i2c_init);
module_exit(sgm7220_i2c_exit);

MODULE_AUTHOR("zhang.tao@tinno.com");
MODULE_DESCRIPTION("I2C bus driver for sgm7220 USB Type-C");
MODULE_LICENSE("GPL v2");

