/************************************************************************
 *
 *  WILLSEMI TypeC Chipset Driver for Linux & Android.
 *
 *
 * ######################################################################
 *
 *  Author: lei.huang (lhuang@sh-willsemi.com)
 *
 * Copyright (c) 2019, WillSemi Inc. All rights reserved.
 *
 ************************************************************************/

/************************************************************************
 *  Include files
 ************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/gpio.h>
#include <linux/of.h>
#include <linux/of_gpio.h>
#include <linux/of_device.h>
#include <linux/err.h>
#include <linux/i2c.h>
#include <linux/interrupt.h>
#include <linux/power_supply.h>
#include <linux/irq.h>
#include <linux/delay.h>
#include <linux/workqueue.h>
#include "inc/tcpci.h"
//#include "inc/tcpci.h"
#include "inc/tcpci_typec.h"
#include "inc/tcpci_timer.h"
/**
 * Options to enable force detection feature for DRP
 */

/*#define __WITH_POWER_BANK_MODE__*/

/*
 *Bit operations if we don't want to include #include <linux/bitops.h>
 */

#undef  __CONST_FFS
#define __CONST_FFS(_x) \
    ((_x) & 0x0F ? ((_x) & 0x03 ? ((_x) & 0x01 ? 0 : 1) :\
                    ((_x) & 0x04 ? 2 : 3)) :\
     ((_x) & 0x30 ? ((_x) & 0x10 ? 4 : 5) :\
      ((_x) & 0x40 ? 6 : 7)))

#undef  FFS
#define FFS(_x) \
    ((_x) ? __CONST_FFS(_x) : 0)

#undef  BITS
#define BITS(_end, _start) \
    ((BIT(_end) - BIT(_start)) + BIT(_end))

#undef  __BITS_GET
#define __BITS_GET(_byte, _mask, _shift) \
    (((_byte) & (_mask)) >> (_shift))

#undef  BITS_GET
#define BITS_GET(_byte, _bit) \
    __BITS_GET(_byte, _bit, FFS(_bit))

#undef  __BITS_SET
#define __BITS_SET(_byte, _mask, _shift, _val) \
    (((_byte) & ~(_mask)) | (((_val) << (_shift)) & (_mask)))

#undef  BITS_SET
#define BITS_SET(_byte, _bit, _val) \
    __BITS_SET(_byte, _bit, FFS(_bit), _val)

#undef  BITS_MATCH
#define BITS_MATCH(_byte, _bit) \
    (((_byte) & (_bit)) == (_bit))

/* Register Map */

#define WUSB3801_REG_VERSION_ID         0x01
#define WUSB3801_REG_CONTROL0           0x02
#define WUSB3801_REG_INTERRUPT          0x03
#define WUSB3801_REG_STATUS             0x04
#define WUSB3801_REG_CONTROL1           0x05
#define WUSB3801_REG_TEST0              0x06
#define WUSB3801_REG_TEST_01            0x07
#define WUSB3801_REG_TEST_02            0x08
#define WUSB3801_REG_TEST_03            0x09
#define WUSB3801_REG_TEST_04            0x0A
#define WUSB3801_REG_TEST_05            0x0B
#define WUSB3801_REG_TEST_06            0x0C
#define WUSB3801_REG_TEST_07            0x0D
#define WUSB3801_REG_TEST_08            0x0E
#define WUSB3801_REG_TEST_09            0x0F
#define WUSB3801_REG_TEST_0A            0x10
#define WUSB3801_REG_TEST_0B            0x11
#define WUSB3801_REG_TEST_0C            0x12
#define WUSB3801_REG_TEST_0D            0x13
#define WUSB3801_REG_TEST_0E            0x14
#define WUSB3801_REG_TEST_0F            0x15
#define WUSB3801_REG_TEST_10            0x16
#define WUSB3801_REG_TEST_11            0x17
#define WUSB3801_REG_TEST_12            0x18


#define WUSB3801_SLAVE_ADDR0            0xc0
#define WUSB3801_SLAVE_ADDR1            0xd0

/*Registers relevant values*/
#define WUSB3801_VENDOR_ID              0x06

/*Switch to enable/disable feature of specified Registers*/
#define BIT_REG_CTRL0_DIS_ACC           (0x01 << 7)
#define BIT_REG_CTRL0_TRY_SRC           (0x02 << 5)
#define BIT_REG_CTRL0_TRY_SNK           (0x01 << 5)
#define BIT_REG_CTRL0_CUR_DEF           (0x00 << 3)
#define BIT_REG_CTRL0_CUR_1P5           (0x01 << 3)
#define BIT_REG_CTRL0_CUR_3P0           (0x02 << 3)
#define BIT_REG_CTRL0_RLE_SNK           (0x00 << 1)
#define BIT_REG_CTRL0_RLE_SRC           (0x01 << 1)
#define BIT_REG_CTRL0_RLE_DRP           (0x02 << 1)
#define BIT_REG_CTRL0_INT_MSK           (0x01 << 0)


#define BIT_REG_STATUS_VBUS             (0x01 << 7)
#define BIT_REG_STATUS_STANDBY          (0x00 << 5)
#define BIT_REG_STATUS_CUR_DEF          (0x01 << 5)
#define BIT_REG_STATUS_CUR_MID          (0x02 << 5)
#define BIT_REG_STATUS_CUR_HIGH         (0x03 << 5)

#define BIT_REG_STATUS_ATC_STB          (0x00 << 1)
#define BIT_REG_STATUS_ATC_SNK          (0x01 << 1)
#define BIT_REG_STATUS_ATC_SRC          (0x02 << 1)
#define BIT_REG_STATUS_ATC_ACC          (0x03 << 1)
#define BIT_REG_STATUS_ATC_DACC         (0x04 << 1)

#define BIT_REG_STATUS_PLR_STB          (0x00 << 0)
#define BIT_REG_STATUS_PLR_CC1          (0x01 << 0)
#define BIT_REG_STATUS_PLR_CC2          (0x02 << 0)
#define BIT_REG_STATUS_PLR_BOTH         (0x03 << 0)

#define BIT_REG_CTRL1_SW02_DIN          (0x01 << 4)
#define BIT_REG_CTRL1_SW02_EN           (0x01 << 3)
#define BIT_REG_CTRL1_SW01_DIN          (0x01 << 2)
#define BIT_REG_CTRL1_SW01_EN           (0x01 << 1)
#define BIT_REG_CTRL1_SM_RST            (0x01 << 0)

/*Available modes*/
#define WUSB3801_DRP_ACC                (BIT_REG_CTRL0_RLE_DRP)
#define WUSB3801_DRP                    (BIT_REG_CTRL0_RLE_DRP | BIT_REG_CTRL0_DIS_ACC)
#define WUSB3801_SNK_ACC                (BIT_REG_CTRL0_RLE_SNK)
#define WUSB3801_SNK                    (BIT_REG_CTRL0_RLE_SNK | BIT_REG_CTRL0_DIS_ACC)
#define WUSB3801_SRC_ACC                (BIT_REG_CTRL0_RLE_SRC)
#define WUSB3801_SRC                    (BIT_REG_CTRL0_RLE_SRC | BIT_REG_CTRL0_DIS_ACC)
#define WUSB3801_DRP_PREFER_SRC_ACC     (WUSB3801_DRP_ACC | BIT_REG_CTRL0_TRY_SRC)
#define WUSB3801_DRP_PREFER_SRC         (WUSB3801_DRP     | BIT_REG_CTRL0_TRY_SRC)
#define WUSB3801_DRP_PREFER_SNK_ACC     (WUSB3801_DRP_ACC | BIT_REG_CTRL0_TRY_SNK)
#define WUSB3801_DRP_PREFER_SNK         (WUSB3801_DRP     | BIT_REG_CTRL0_TRY_SNK)

/*TODO: redefine your prefer role here*/
#define WUSB3801_INIT_MODE              (WUSB3801_DRP_PREFER_SNK)

#define BIT_REG_TEST02_FORCE_ERR_RCY    (0x01)

#define WUSB3801_WAIT_VBUS               0x40
/*Fixed duty cycle period. 40ms:40ms*/
#define WUSB3801_TGL_40MS                0
#define WUSB3801_HOST_DEFAULT            0
#define WUSB3801_HOST_1500MA             1
#define WUSB3801_HOST_3000MA             2
#define WUSB3801_INT_ENABLE              0x00
#define WUSB3801_INT_DISABLE             0x01
#define WUSB3801_DISABLED                0x0A
#define WUSB3801_ERR_REC                 0x01
#define WUSB3801_VBUS_OK                 0x80

#define WUSB3801_SNK_0MA                (0x00 << 5)
#define WUSB3801_SNK_DEFAULT            (0x01 << 5)
#define WUSB3801_SNK_1500MA             (0x02 << 5)
#define WUSB3801_SNK_3000MA             (0x03 << 5)
#define WUSB3801_ATTACH                  0x1C

//#define WUSB3801_TYPE_PWR_ACC           (0x00 << 2) /*Ra/Rd treated as Open*/
#define WUSB3801_TYPE_INVALID           (0x00)
#define WUSB3801_TYPE_SRC               (0x01 << 2)  ///4
#define WUSB3801_TYPE_SNK               (0x02 << 2)//8
#define WUSB3801_TYPE_AUD_ACC           (0x03 << 2)
#define WUSB3801_TYPE_DBG_ACC           (0x04 << 2)

#define WUSB3801_INT_DETACH              (0x01 << 1)
#define WUSB3801_INT_ATTACH              (0x01 << 0)

#define WUSB3801_REV20                   0x02

/* Masks for Read-Modified-Write operations*/
#define WUSB3801_HOST_CUR_MASK           0x18  /*Host current for IIC*/
#define WUSB3801_INT_MASK                0x01
#define WUSB3801_BCLVL_MASK              0x60
#define WUSB3801_TYPE_MASK               0x1C
#define WUSB3801_MODE_MASK               0xE6  /*Roles relevant bits*/
#define WUSB3801_INT_STS_MASK            0x03
#define WUSB3801_FORCE_ERR_RCY_MASK      0x80  /*Force Error recovery*/
#define WUSB3801_ROLE_MASK               0x06
#define WUSB3801_VENDOR_ID_MASK          0x07
#define WUSB3801_VERSION_ID_MASK         0xF8
#define WUSB3801_VENDOR_SUB_ID_MASK         0xA0
#define WUSB3801_POLARITY_CC_MASK        0x03
#define WUSB3801_CC_STS_MASK            0x03


/* WUSB3801 STATES MACHINES */
#define WUSB3801_STATE_DISABLED             0x00
#define WUSB3801_STATE_ERROR_RECOVERY       0x01
#define WUSB3801_STATE_UNATTACHED_SNK       0x02
#define WUSB3801_STATE_UNATTACHED_SRC       0x03
#define WUSB3801_STATE_ATTACHWAIT_SNK       0x04
#define WUSB3801_STATE_ATTACHWAIT_SRC       0x05
#define WUSB3801_STATE_ATTACHED_SNK         0x06
#define WUSB3801_STATE_ATTACHED_SRC         0x07
#define WUSB3801_STATE_AUDIO_ACCESSORY      0x08
#define WUSB3801_STATE_DEBUG_ACCESSORY      0x09
#define WUSB3801_STATE_TRY_SNK              0x0A
#define WUSB3801_STATE_TRYWAIT_SRC          0x0B
#define WUSB3801_STATE_TRY_SRC              0x0C
#define WUSB3801_STATE_TRYWAIT_SNK          0x0D

#define WUSB3801_CC2_CONNECTED 1
#define WUSB3801_CC1_CONNECTED 0

/* wake lock timeout in ms */
#define WUSB3801_WAKE_LOCK_TIMEOUT          1000
/*1.5 Seconds timeout for force detection*/
#define ROLE_SWITCH_TIMEOUT                   1500
#define DEBOUNCE_TIME_OUT   50

bool first_check_flag = false;
extern int g_TypeC_Src_Detect;

/*Private data*/
struct wusb3801_data
{
    uint32_t  int_gpio;
    uint8_t  init_mode;
    uint8_t  dfp_power;
    uint8_t  dttime;
};

struct wusb3801_chip
{
    struct i2c_client *client;
    struct wusb3801_data *pdata;
    struct workqueue_struct  *cc_wq;
    struct tcpc_device *tcpc;
    struct tcpc_desc *tcpc_desc;
    struct work_struct dwork;
    struct delayed_work first_check_typec_work;
    struct mutex mlock;
    struct semaphore suspend_lock;
    int irq_gpio;
    int irq_gpio_num;
    int ufp_power;
    int try_attcnt;
    uint8_t     mode;
    uint8_t     dev_id;
    uint8_t     dev_sub_id;
    uint8_t     type;
    uint8_t     state;
    uint8_t     bc_lvl;
    uint8_t     dfp_power;
    uint8_t     dttime;
    uint8_t     attached;
    uint8_t     init_state;
    uint8_t     defer_init; //work round the additional interrupt caused by mode change
} ;

#define wusb3801_update_state(chip, st)                      \
    if(chip) {                                                 \
        chip->state = st;                                        \
        dev_info(&chip->client->dev, "%s: %s\n", __func__, #st); \
        wake_up_interruptible(&wubs3801_mode_switch);                     \
    }

#define STR(s)    #s
#define STRV(s)   STR(s)
static int wusb3801_reset_device(struct wusb3801_chip *chip);
static void wusb3801_detach(struct wusb3801_chip *chip);
//static unsigned wusb3801_is_vbus_on(struct wusb3801_chip *chip);
DECLARE_WAIT_QUEUE_HEAD(wubs3801_mode_switch);

/******************************************************/

/************************************************************************
 *
 *       wusb3801_write_masked_byte
 *
 *  Description :
 *  -------------
 *  Read-Modified-Writeback operation through I2C communication.
 *
 *  Parameter     :
 *  -----------
 *  Client         :    Pointer to I2C client.
 *  Addr           :    Address of internal register of slave device
 *  Mask           :    Mask to prevent irrelevant bit changes
 *  val            :    Data wants to write.
 *  Return values  :    Zero if no error, else error code.
 *  ---------------
 *  None
 *
 ************************************************************************/

static int wusb3801_write_masked_byte(struct i2c_client *client,
                                      uint8_t addr, uint8_t mask, uint8_t val)
{
    int rc = 0;

    if (!mask)
    {
        /* no actual access */
        rc = -EINVAL;
        goto out;
    }
    rc = i2c_smbus_read_byte_data(client, addr);
    if (rc < 0)
        goto out;
    rc = i2c_smbus_write_byte_data(client,
                                   addr, BITS_SET((uint8_t)rc, mask, val));
out:
    return rc;
}

static int wusb3801_read_device_id(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_VERSION_ID);
    if (rc < 0)
        return rc;
    dev_info(cdev, "VendorID register: 0x%02x\n", rc);

    if ((rc & WUSB3801_VENDOR_ID_MASK) != WUSB3801_VENDOR_ID)
    {
        return -EINVAL;
    }

    chip->dev_id = rc;
    dev_info(cdev, "Vendor id: 0x%02x, Version id: 0x%02x\n", rc & WUSB3801_VENDOR_ID_MASK,
             (rc & WUSB3801_VERSION_ID_MASK) >> 3);
    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_TEST_01);
    if (rc < 0)
        return rc;

    chip->dev_sub_id = rc & WUSB3801_VENDOR_SUB_ID_MASK;
    dev_info(cdev, "VendorSUBID register: 0x%02x\n", rc & WUSB3801_VENDOR_SUB_ID_MASK);
    return 0;
}

static int wusb3801_update_status(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    /* Get control0 register */
    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_CONTROL0);
    if (rc < 0)
    {
        dev_err(cdev, "%s: fail to read mode\n", __func__);
        return rc;
    }

    chip->mode      =  rc & WUSB3801_MODE_MASK;
    chip->dfp_power =  BITS_GET(rc, WUSB3801_HOST_CUR_MASK);
    chip->dttime    =  WUSB3801_TGL_40MS;

    return 0;
}


static int wusb3801_check_modes(uint8_t mode)
{
    switch (mode)
    {
        case WUSB3801_DRP_ACC            :
        case WUSB3801_DRP                :
        case WUSB3801_SNK_ACC            :
        case WUSB3801_SNK                :
        case WUSB3801_SRC_ACC            :
        case WUSB3801_SRC                :
        case WUSB3801_DRP_PREFER_SRC_ACC :
        case WUSB3801_DRP_PREFER_SRC     :
        case WUSB3801_DRP_PREFER_SNK_ACC :
        case WUSB3801_DRP_PREFER_SNK     :
            return 0;
            break;
        default:
            break;
    }
    return -EINVAL;
}

/*
 *  Spec lets transitioning to below states from any state
 *  WUSB3801_STATE_DISABLED
 *  WUSB3801_STATE_ERROR_RECOVERY
 */
static int wusb3801_set_chip_state(struct wusb3801_chip *chip, uint8_t state)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    if (state > WUSB3801_STATE_UNATTACHED_SRC)
        return -EINVAL;


    rc = i2c_smbus_write_byte_data(chip->client,
                                   WUSB3801_REG_CONTROL1,
                                   (state == WUSB3801_STATE_DISABLED) ? \
                                   WUSB3801_DISABLED :        \
                                   0);

    if (rc < 0)
    {
        dev_err(cdev, "failed to write state machine(%d)\n", rc);
    }

    chip->init_state = state;

    return rc;
}


static int wusb3801_set_mode(struct wusb3801_chip *chip, uint8_t mode)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    if (mode != chip->mode)
    {
        /*
        rc = wusb3801_write_masked_byte(chip->client,
                WUSB3801_REG_CONTROL0,
                WUSB3801_MODE_MASK,
                mode);
                */
        rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_CONTROL0);
        if (rc < 0)
        {
            dev_err(cdev, "%s: fail to read mode\n", __func__);
            return rc;
        }
        rc &= ~WUSB3801_MODE_MASK;
        //rc |= mode;
        rc |= (mode | WUSB3801_INT_MASK);//Disable the chip interrupt
        rc = i2c_smbus_write_byte_data(chip->client,
                                       WUSB3801_REG_CONTROL0, rc);

        if (rc < 0)
        {
            dev_err(cdev, "failed to write mode(%d)\n", rc);
            return rc;
        }

        //Clear the chip interrupt
        rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_INTERRUPT);
        if (rc < 0)
        {
            dev_err(cdev, "%s: fail to clear chip interrupt\n", __func__);
            return rc;
        }

        rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_CONTROL0);
        if (rc < 0)
        {
            dev_err(cdev, "%s: fail to read chip interrupt\n", __func__);
            return rc;
        }
        rc &= ~WUSB3801_INT_MASK;//enable the chip interrupt
        rc = i2c_smbus_write_byte_data(chip->client,
                                       WUSB3801_REG_CONTROL0, rc);

        if (rc < 0)
        {
            dev_err(cdev, "failed to enable chip interrupt(%d)\n", rc);
            return rc;
        }

        chip->mode = mode;
    }

    dev_dbg(cdev, "%s: mode (0x%02x) (0x%02x)\n", __func__, chip->mode, mode);

    return rc;
}

static int wusb3801_set_dfp_power(struct wusb3801_chip *chip, uint8_t hcurrent)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    if (hcurrent == chip->dfp_power)
    {
        dev_dbg(cdev, "vaule is not updated(%d)\n",
                hcurrent);
        return rc;
    }

    rc = wusb3801_write_masked_byte(chip->client,
                                    WUSB3801_REG_CONTROL0,
                                    WUSB3801_HOST_CUR_MASK,
                                    hcurrent);
    if (rc < 0)
    {
        dev_err(cdev, "failed to write current(%d)\n", rc);
        return rc;
    }

    chip->dfp_power = hcurrent;

    dev_dbg(cdev, "%s: host current(%d)\n", __func__, hcurrent);

    return rc;
}

/**********************************************************************
 * When 3A capable DRP device is connected without VBUS,
 * DRP always detect it as SINK device erroneously.
 * Since USB Type-C specification 1.0 and 1.1 doesn't
 * consider this corner case, apply workaround for this case.
 * Set host mode current to 1.5A initially, and then change
 * it to default USB current right after detection SINK port.
 ***********************************************************************/
static int wusb3801_init_force_dfp_power(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    rc = wusb3801_write_masked_byte(chip->client,
                                    WUSB3801_REG_CONTROL0,
                                    WUSB3801_HOST_CUR_MASK,
                                    WUSB3801_HOST_3000MA);
    if (rc < 0)
    {
        dev_err(cdev, "failed to write current\n");
        return rc;
    }

    chip->dfp_power = WUSB3801_HOST_3000MA;

    dev_dbg(cdev, "%s: host current (%d)\n", __func__, rc);

    return rc;
}
/************************************************************************
 *
 *       wusb3801_set_toggle_time
 *
 *  Description :
 *  -------------
 *  Wusb3801 varints only support for fixed duty cycles periods (40ms:40ms)
 *
 ************************************************************************/
#if 0
static int wusb3801_set_toggle_time(struct wusb3801_chip *chip, uint8_t toggle_time)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    if (toggle_time != WUSB3801_TGL_40MS)
    {
        dev_err(cdev, "toggle_time(%d) is unavailable\n", toggle_time);
        return -EINVAL;
    }

    chip->dttime = WUSB3801_TGL_40MS;

    dev_dbg(cdev, "%s: Fixed toggle time (%d)\n", __func__, chip->dttime);

    return rc;
}
#endif

static int wusb3801_init_reg(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    /* change current */
    rc = wusb3801_init_force_dfp_power(chip);
    if (rc < 0)
        dev_err(cdev, "%s: failed to force dfp power\n", __func__);

    /* change mode */
    rc = wusb3801_set_mode(chip, chip->pdata->init_mode);
    if (rc < 0)
        dev_err(cdev, "%s: failed to set mode\n", __func__);

    rc = wusb3801_set_chip_state(chip,
                                 WUSB3801_STATE_ERROR_RECOVERY);
    if (rc < 0)
        dev_err(cdev, "%s: Reset state failed.\n", __func__);

    return rc;
}

static int wusb3801_reset_device(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    rc = i2c_smbus_write_byte_data(chip->client,
                                   WUSB3801_REG_CONTROL1, BIT_REG_CTRL1_SM_RST);
    if (rc < 0)
    {
        dev_err(cdev, "reset fails\n");
        return rc;
    }

    rc = wusb3801_update_status(chip);
    if (rc < 0)
        dev_err(cdev, "fail to read status\n");

    rc = wusb3801_init_reg(chip);
    if (rc < 0)
        dev_err(cdev, "fail to init reg\n");

    wusb3801_detach(chip);

    /* clear global interrupt mask */
    rc = wusb3801_write_masked_byte(chip->client,
                                    WUSB3801_REG_CONTROL0,
                                    WUSB3801_INT_MASK,
                                    WUSB3801_INT_ENABLE);
    if (rc < 0)
    {
        dev_err(cdev, "%s: fail to init\n", __func__);
        return rc;
    }

    dev_info(cdev, "mode[0x%02x], host_cur[0x%02x], dttime[0x%02x]\n",
             chip->mode, chip->dfp_power, chip->dttime);

    return rc;
}

static void wusb3801_bclvl_changed(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0, limit, hcurrent;
    uint8_t status, type;

    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_STATUS);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read\n", __func__);
        rc = wusb3801_reset_device(chip);
        if (rc < 0)
            dev_err(cdev, "%s: failed to reset\n", __func__);
        return;
    }

    status = (rc & WUSB3801_TYPE_MASK) ? 1 : 0;
    type = status  ? rc & WUSB3801_TYPE_MASK : WUSB3801_TYPE_INVALID;
    hcurrent = rc & WUSB3801_BCLVL_MASK;
    dev_dbg(cdev, "%s: sts[0x%02x], type[0x%02x]\n", __func__, status, type);

    if (type == WUSB3801_TYPE_SNK)
    {
        /* do nothing */
    }
    else if (type == WUSB3801_TYPE_SRC)
    {
        limit = (chip->bc_lvl == WUSB3801_SNK_3000MA ? 3000 :
                 (chip->bc_lvl == WUSB3801_SNK_1500MA ? 1500 : 0));
        dev_dbg(cdev, "%s: limit = %d\n", __func__, limit);
    }
    else
    {
        /* do nothing */
    }
}


static void wusb3801_src_detected(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    dev_info(cdev, "%s \n", __func__);
    if ((chip->mode & WUSB3801_ROLE_MASK) == BIT_REG_CTRL0_RLE_SRC)
    {
        dev_err(cdev, "not support in source mode\n");
        rc = wusb3801_reset_device(chip);
        if (rc < 0)
            dev_err(cdev, "%s: failed to reset\n", __func__);
        return;
    }
    wusb3801_update_state(chip, WUSB3801_STATE_ATTACHED_SNK);

    //if(wusb3801_is_vbus_on(chip))
    //{
    //  dev_err(cdev, "USB2.0 detected %s.\n", __func__);
    //  wusb3801_usb20_host_detected(chip);
    //}
    /**
    *  TODO: Add you code here if you want doing something special during
    *        TYPEC source has detected.
    */
    chip->type = WUSB3801_TYPE_SRC;
}

static void wusb3801_snk_detected(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    dev_info(cdev, "%s \n", __func__);
    if ((chip->mode & WUSB3801_ROLE_MASK) == BIT_REG_CTRL0_RLE_SNK)
    {
        dev_err(cdev, "not support in sink mode\n");
        rc = wusb3801_reset_device(chip);
        if (rc < 0)
            dev_err(cdev, "%s: failed to reset\n", __func__);
        return;
    }

    /**
    *  TODO: Add you code here if you want doing something special during
    *        TYPEC Sink has detected.
    */
    /*
     * mode == WUSB3801_SRC/WUSB3801_SRC_ACC
     */
    wusb3801_set_dfp_power(chip, chip->pdata->dfp_power);
    wusb3801_update_state(chip, WUSB3801_STATE_ATTACHED_SRC);
    chip->type = WUSB3801_TYPE_SNK;
}

static void wusb3801_dbg_acc_detected(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    dev_info(cdev, "%s \n", __func__);
    if (chip->mode & BIT_REG_CTRL0_DIS_ACC)
    {
        dev_err(cdev, "not support accessory mode\n");
        rc = wusb3801_reset_device(chip);
        if (rc < 0)
            dev_err(cdev, "%s: failed to reset\n", __func__);
        return;
    }

    /**
    *  TODO: Add you code here if you want doing something special during
    *        TYPEC Debug accessory has detected.
    */
    wusb3801_update_state(chip, WUSB3801_STATE_DEBUG_ACCESSORY);
}

static void wusb3801_aud_acc_detected(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    dev_info(cdev, "%s \n", __func__);
    if (chip->mode & BIT_REG_CTRL0_DIS_ACC)
    {
        dev_err(cdev, "not support accessory mode\n");
        rc = wusb3801_reset_device(chip);
        if (rc < 0)
            dev_err(cdev, "%s: failed to reset\n", __func__);
        return;
    }

    /**
    *  TODO: Add you code here if you want doing something special during
    *        TYPEC Audio accessory has detected.
    */
    wusb3801_update_state(chip, WUSB3801_STATE_AUDIO_ACCESSORY);
}

static void wusb3801_aud_acc_detach(struct wusb3801_chip *chip)
{
    //struct device *cdev = &chip->client->dev;
    /**
        *  TODO: Add you code here if you want doing something special during
        *        TYPEC Audio accessory pull out.
        */
}

static void wusb3801_detach(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    dev_err(cdev, "%s: type[0x%02x] chipstate[0x%02x]\n",
            __func__, chip->type, chip->state);

    switch (chip->state)
    {
        case WUSB3801_STATE_ATTACHED_SRC:
            wusb3801_init_force_dfp_power(chip);
            break;
        case WUSB3801_STATE_ATTACHED_SNK:
            break;
        case WUSB3801_STATE_DEBUG_ACCESSORY:
            break;
        case WUSB3801_STATE_AUDIO_ACCESSORY:
            wusb3801_aud_acc_detach(chip);
            break;
        case WUSB3801_STATE_DISABLED:
        case WUSB3801_STATE_ERROR_RECOVERY:
            break;
        default:
            dev_err(cdev, "%s: Invaild chipstate[0x%02x]\n",
                    __func__, chip->state);
            break;
    }
    chip->type = WUSB3801_TYPE_INVALID;
    chip->bc_lvl = WUSB3801_SNK_0MA;
    chip->ufp_power  = 0;
    chip->try_attcnt = 0;
    chip->attached   = 0;
    chip->tcpc->typec_attach_new = TYPEC_UNATTACHED;
    tcpci_notify_typec_state(chip->tcpc);
    if (chip->tcpc->typec_attach_old == TYPEC_ATTACHED_SRC)
    {
        tcpci_source_vbus(chip->tcpc,
                          TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_0V, 0);
    }
    chip->tcpc->typec_attach_old = TYPEC_UNATTACHED;
    g_TypeC_Src_Detect = 0;
    //Excute the defered init mode into drp+try.snk when detached
    if (chip->defer_init == 1)
    {
        dev_err(cdev, "%s: reset to init mode\n", __func__);
        rc = wusb3801_set_mode(chip, chip->pdata->init_mode);
        if (rc < 0)
            dev_err(cdev, "%s: failed to set init mode\n", __func__);
        chip->defer_init = 0;
    }
    wusb3801_update_state(chip, WUSB3801_STATE_ERROR_RECOVERY);
}

#if 0
static unsigned wusb3801_is_vbus_off(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    rc = i2c_smbus_read_byte_data(chip->client,
                                  WUSB3801_REG_STATUS);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read status\n", __func__);
        return false;
    }

    return !((rc & WUSB3801_TYPE_MASK) && (rc & WUSB3801_VBUS_OK));
}
#endif /*if 0*/
#if 0
static unsigned wusb3801_is_vbus_on(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;

    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_STATUS);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read status\n", __func__);
        return false;
    }

    return !!(rc & WUSB3801_VBUS_OK);
}
#endif

static void wusb3801_attach(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int rc = 0;
    uint8_t status, type;

    /* get status */
    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_STATUS);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read status\n", __func__);
        return;
    }

    status = (rc & WUSB3801_ATTACH) ? true : false;
    type = status ? rc & WUSB3801_TYPE_MASK : WUSB3801_TYPE_INVALID;
    dev_info(cdev, "%s:sts[0x%02x], type[0x%02x]\n", __func__, status, type);

    if (chip->state != WUSB3801_STATE_ERROR_RECOVERY)
    {
        rc = wusb3801_set_chip_state(chip,
                                     WUSB3801_STATE_ERROR_RECOVERY);
        if (rc < 0)
            dev_err(cdev, "%s: failed to set error recovery\n", __func__);
        wusb3801_detach(chip);
        dev_err(cdev, "%s: Invaild chipstate[0x%02x]\n", __func__, chip->state);
        return;
    }
    /*chip->bc_lvl = rc & WUSB3801_BCLVL_MASK;
    if (chip->bc_lvl == 0)
    {
        chip->bc_lvl = WUSB3801_SNK_DEFAULT;
        dev_err(cdev, "%s: chip current is 0, set default\n",
                __func__);
    }*/
    switch (type)
    {
        case WUSB3801_TYPE_SRC:
            dev_info(cdev, "%s: WUSB3801_TYPE_SRC\n", __func__);
            wusb3801_src_detected(chip);
            chip->tcpc_desc->rp_lvl = TYPEC_CC_RP_3_0;
            if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SRC)
            {
                chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SRC;
                tcpci_source_vbus(chip->tcpc,
                                  TCP_VBUS_CTRL_TYPEC, TCPC_VBUS_SOURCE_5V, 3000);
                tcpci_notify_typec_state(chip->tcpc);
                chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SRC;
            }
            g_TypeC_Src_Detect = 1;
            break;
        case WUSB3801_TYPE_SNK:
            dev_info(cdev, "%s: WUSB3801_TYPE_SNK\n", __func__);
            wusb3801_snk_detected(chip);
            if (chip->tcpc->typec_attach_new != TYPEC_ATTACHED_SNK)
            {
                chip->tcpc->typec_attach_new = TYPEC_ATTACHED_SNK;
                tcpci_notify_typec_state(chip->tcpc);
                chip->tcpc->typec_attach_old = TYPEC_ATTACHED_SNK;
            }
            g_TypeC_Src_Detect = 0;
            break;
        case WUSB3801_TYPE_DBG_ACC:
            wusb3801_dbg_acc_detected(chip);
            chip->type = type;
            break;
        case WUSB3801_TYPE_AUD_ACC:
            wusb3801_aud_acc_detected(chip);
            chip->type = type;
            break;
        case WUSB3801_TYPE_INVALID:
            wusb3801_detach(chip);
            dev_err(cdev, "%s: Invaild type[0x%02x]\n", __func__, type);
            break;
        default:
            rc = wusb3801_set_chip_state(chip,
                                         WUSB3801_STATE_ERROR_RECOVERY);
            if (rc < 0)
                dev_err(cdev, "%s: failed to set error recovery\n",
                        __func__);

            wusb3801_detach(chip);
            dev_err(cdev, "%s: Unknwon type[0x%02x]\n", __func__, type);
            break;
    }
    chip->attached = true;
}
static void wusb3801_work_handler(struct work_struct *work)
{
    struct wusb3801_chip *chip =
        container_of(work, struct wusb3801_chip, dwork);
    struct device *cdev = &chip->client->dev;
    int rc = 0;
    uint8_t int_sts;

    dev_info(cdev, "%s \n", __func__);

    if (first_check_flag == true)
        return;

    mutex_lock(&chip->mlock);
    /* get interrupt */
    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_INTERRUPT);
    mutex_unlock(&chip->mlock);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read interrupt\n", __func__);
        goto work_unlock;
    }
    int_sts = rc & WUSB3801_INT_STS_MASK;
    dev_info(cdev, "%s: int_sts[0x%02x]\n", __func__, int_sts);

    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_STATUS);
    if (rc < 0)
    {
        dev_err(cdev, "%s: failed to read reg status\n", __func__);
        goto work_unlock;
    }
    dev_info(cdev, "%s WUSB3801_REG_STATUS : 0x%02x\n", __func__, rc);

    if (int_sts & WUSB3801_INT_DETACH)
    {
        wusb3801_detach(chip);
    }
    if (int_sts & WUSB3801_INT_ATTACH)
    {
        if (chip->attached)
        {
            wusb3801_bclvl_changed(chip);
        }
        else
        {
            wusb3801_attach(chip);
        }
    }
work_unlock:
    enable_irq(chip->irq_gpio);
}

static irqreturn_t wusb3801_interrupt(int irq, void *data)
{
    struct wusb3801_chip *chip = (struct wusb3801_chip *)data;
    if (!chip)
    {
        dev_err(&chip->client->dev, "%s : called before init.\n", __func__);
        return IRQ_HANDLED;
    }

    disable_irq_nosync(irq);

    /*
     * wake_lock_timeout, prevents multiple suspend entries
     * before charger gets chance to trigger usb core for device
     */
    if (!queue_work(chip->cc_wq, &chip->dwork))
    {
        dev_err(&chip->client->dev, "%s: can't alloc work\n", __func__);
        enable_irq(irq);
    }

    return IRQ_HANDLED;
}

static int wusb3801_init_gpio(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    int ret = 0;

    chip->irq_gpio_num = of_get_named_gpio(cdev->of_node,
                                           "wusb3801,int-gpio", 0);
    dev_info(cdev, "%s: int_gpio: %d\n", __func__, chip->irq_gpio_num);

    ret = devm_gpio_request(&chip->client->dev,
                            chip->irq_gpio_num, "type_c_port0");
    if (ret < 0)
    {
        dev_err(cdev, "Error: failed to request GPIO %d (ret = %d)\n",
                chip->irq_gpio_num, ret);
    }

    ret = gpio_direction_input(chip->irq_gpio_num);
    if (ret < 0)
    {
        dev_err(cdev,
                "Error: failed to set GPIO %d as input pin(ret = %d)\n",
                chip->irq_gpio_num, ret);
    }

    chip->irq_gpio = gpio_to_irq(chip->irq_gpio_num);
    if (chip->irq_gpio  <= 0)
    {
        dev_err(cdev, "gpio to irq fail, chip->irq(%d)\n",
                chip->irq_gpio);
    }

    ret = request_irq(chip->irq_gpio, wusb3801_interrupt,
                      IRQF_TRIGGER_FALLING | IRQF_ONESHOT,
                      "wusb3801_irq", chip);
    if (ret)
        dev_err(cdev, "unable to request int_gpio %d\n",
                chip->irq_gpio_num);

    dev_info(cdev, "%s: name=%s, gpio=%d, IRQ number=%d\n",
             __func__, chip->tcpc_desc->name,
             chip->irq_gpio_num, chip->irq_gpio);
    return ret;
}

static void wusb3801_free_gpio(struct wusb3801_chip *chip)
{
    if (gpio_is_valid(chip->irq_gpio_num))
        gpio_free(chip->irq_gpio_num);
}

static int wusb3801_parse_dt(struct wusb3801_chip *chip)
{
    struct device *cdev = &chip->client->dev;
    struct device_node *dev_node = cdev->of_node;
    struct wusb3801_data *data = chip->pdata;
    u32 val = 0;
    int rc = 0;

    dev_err(cdev, "dev_node->name=%s.\n", dev_node->name);

    rc = of_property_read_u32(dev_node,
                              "wusb3801,init-mode", &val);
    data->init_mode = (u8)val;
    dev_err(cdev, "data->init_mode=%d.\n", data->init_mode);
    if (rc || wusb3801_check_modes(data->init_mode))
    {
        dev_err(cdev, "init mode is not available and set default\n");
        data->init_mode = WUSB3801_INIT_MODE; //WUSB3801_DRP_ACC;
        rc = 0;
    }

    rc = of_property_read_u32(dev_node,
                              "wusb3801,host-current", &val);
    data->dfp_power = (u8)val;
    dev_err(cdev, "data->dfp_power=%d.\n", data->dfp_power);
    if (rc || (data->dfp_power > WUSB3801_HOST_3000MA))
    {
        dev_err(cdev, "host current is not available and set default\n");
        data->dfp_power = WUSB3801_HOST_DEFAULT;
        rc = 0;
    }

    rc = of_property_read_u32(dev_node,
                              "wusb3801,drp-toggle-time", &val);
    data->dttime = (u8)val;
    if (rc || (data->dttime != WUSB3801_TGL_40MS))
    {
        dev_err(cdev, "Fixed drp time and set default (40ms:40ms)\n");
        data->dttime = WUSB3801_TGL_40MS;
        rc = 0;
    }

    dev_dbg(cdev, "init_mode:%d dfp_power:%d toggle_time:%d\n",
            data->init_mode, data->dfp_power, data->dttime);

    return rc;
}

/**
int register_typec_switch_callback(struct typec_switch_data *new_driver)
{
    return 0;
}
EXPORT_SYMBOL_GPL(register_typec_switch_callback);
*/
int wusb3801_alert_status_clear(struct tcpc_device *tcpc,
                                uint32_t mask)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_tcpc_init(struct tcpc_device *tcpc, bool sw_reset)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int wusb3801_fault_status_clear(struct tcpc_device *tcpc,
                                uint8_t status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int wusb3801_get_alert_mask(struct tcpc_device *tcpc, uint32_t *mask)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int wusb3801_get_alert_status(struct tcpc_device *tcpc,
                              uint32_t *alert)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_get_power_status(struct tcpc_device *tcpc,
                                     uint16_t *pwr_status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
int wusb3801_get_fault_status(struct tcpc_device *tcpc,
                              uint8_t *status)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_get_cc(struct tcpc_device *tcpc, int *cc1, int *cc2)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_set_cc(struct tcpc_device *tcpc, int pull)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_set_polarity(struct tcpc_device *tcpc, int polarity)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_set_low_rp_duty(struct tcpc_device *tcpc,
                                    bool low_rp)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_set_vconn(struct tcpc_device *tcpc, int enable)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}
static int wusb3801_tcpc_deinit(struct tcpc_device *tcpc_dev)
{
    pr_info("%s: enter \n", __func__);
    return 0;
}

static struct tcpc_ops wusb3801_tcpc_ops =
{
    .init = wusb3801_tcpc_init,
    .alert_status_clear = wusb3801_alert_status_clear,
    .fault_status_clear = wusb3801_fault_status_clear,
    .get_alert_mask = wusb3801_get_alert_mask,
    .get_alert_status = wusb3801_get_alert_status,
    .get_power_status = wusb3801_get_power_status,
    .get_fault_status = wusb3801_get_fault_status,
    .get_cc = wusb3801_get_cc,
    .set_cc = wusb3801_set_cc,
    .set_polarity = wusb3801_set_polarity,
    .set_low_rp_duty = wusb3801_set_low_rp_duty,
    .set_vconn = wusb3801_set_vconn,
    //.set_role = wusb3801_set_role,
    //.get_mode = wusb3801_get_mode,
    .deinit = wusb3801_tcpc_deinit,
};

static void wusb3801_first_check_typec_work(struct work_struct *work)
{
    struct wusb3801_chip *chip = container_of(work,
                                 struct wusb3801_chip, first_check_typec_work.work);
    struct device *cdev = &chip->client->dev;
    int rc = 0;
    u8 int_sts = 0;

    /* get interrupt */
    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_INTERRUPT);
    if (rc < 0)
        dev_err(cdev, "%s: failed to read interrupt\n", __func__);

    int_sts = rc & WUSB3801_INT_STS_MASK;

    rc = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_STATUS);
    if (rc < 0)
        dev_err(cdev, "%s: failed to read reg status\n", __func__);
    dev_info(cdev, "%s WUSB3801_REG_STATUS : 0x%02x\n", __func__, rc);

    dev_info(cdev, "%s: int_sts[0x%02x]\n", __func__, int_sts);
    if (int_sts & WUSB3801_INT_DETACH)
    {
        wusb3801_detach(chip);
    }
    if (int_sts & WUSB3801_INT_ATTACH)
    {
        wusb3801_attach(chip);
    }

    first_check_flag = false;
}


static int wusb3801_probe(struct i2c_client *client,
                          const struct i2c_device_id *id)
{
    struct wusb3801_chip *chip;
    struct device *cdev = &client->dev;
    struct wusb3801_data *data;
    struct tcpc_desc *desc;
    int ret = 0;

    dev_info(cdev, "wusb3801_probe start \n");
    if (!i2c_check_functionality(client->adapter,
                                 I2C_FUNC_SMBUS_BYTE_DATA |
                                 I2C_FUNC_SMBUS_WORD_DATA))
    {
        dev_err(cdev, "smbus data not supported!\n");
        return -EIO;
    }

    chip = devm_kzalloc(cdev, sizeof(struct wusb3801_chip), GFP_KERNEL);
    if (!chip)
    {
        dev_err(cdev, "can't alloc wusb3801_chip\n");
        return -ENOMEM;
    }

    chip->client = client;
    i2c_set_clientdata(client, chip);
    first_check_flag = true;

    ret = wusb3801_read_device_id(chip);
    if (ret)
    {
        dev_err(cdev, "wusb3801 doesn't find, try again\n");
        ret = wusb3801_read_device_id(chip);
        if (ret)
        {
            dev_err(cdev, "wusb3801 doesn't find, stop find\n");
            goto err1;
        }
    }

    data = devm_kzalloc(cdev, sizeof(struct wusb3801_data), GFP_KERNEL);
    if (!data)
    {
        dev_err(cdev, "can't alloc wusb3801_data\n");
        ret = -ENOMEM;
        goto err1;
    }

    chip->pdata = data;

    ret = wusb3801_parse_dt(chip);
    if (ret)
    {
        dev_err(cdev, "can't parse dt\n");
        goto err2;
    }


    desc = devm_kzalloc(cdev, sizeof(*desc), GFP_KERNEL);
    if (!desc)
        return -ENOMEM;

    desc->name = kzalloc(13, GFP_KERNEL);
    if (!desc->name)
        return -ENOMEM;

    strcpy((char *)desc->name, "type_c_port0");
    desc->role_def = TYPEC_ROLE_TRY_SRC;
    desc->rp_lvl = TYPEC_CC_RP_1_5;
    chip->tcpc_desc = desc;

    dev_info(cdev, "%s: type_c_port0, role=%d\n", __func__, desc->role_def);

    chip->tcpc = tcpc_device_register(cdev, desc, &wusb3801_tcpc_ops, chip);
    chip->tcpc->typec_attach_old = TYPEC_UNATTACHED;
    chip->tcpc->typec_attach_new = TYPEC_UNATTACHED;

    ret = wusb3801_init_gpio(chip);
    if (ret)
    {
        dev_err(cdev, "fail to init gpio\n");
        goto err2;
    }

    chip->type      = WUSB3801_TYPE_INVALID;
    chip->state     = WUSB3801_STATE_ERROR_RECOVERY;
    chip->attached  = 0;
    chip->bc_lvl    = WUSB3801_SNK_0MA;
    chip->ufp_power = 0;
    chip->defer_init = 0;   // Do the mode init when chip init

    chip->cc_wq = alloc_ordered_workqueue("wusb3801-wq", WQ_HIGHPRI);
    if (!chip->cc_wq)
    {
        dev_err(cdev, "unable to create workqueue wusb3801-wq\n");
        goto err3;
    }

    INIT_WORK(&chip->dwork, wusb3801_work_handler);
    INIT_DELAYED_WORK(&chip->first_check_typec_work,
                      wusb3801_first_check_typec_work);
    mutex_init(&chip->mlock);

    schedule_delayed_work(&chip->first_check_typec_work, msecs_to_jiffies(3000));
    ret = wusb3801_reset_device(chip);
    if (ret)
    {
        dev_err(cdev, "failed to initialize\n");
        goto err4;
    }
    enable_irq_wake(chip->irq_gpio);

    /*ret = i2c_smbus_read_byte_data(chip->client, WUSB3801_REG_CONTROL0);
    if (ret < 0)
    {
        dev_err(cdev, "%s: failed to read reg status\n", __func__);
    }
    dev_err(cdev, "%s WUSB3801_REG_CONTROL0 : 0x%02x\n", __func__, ret);*/


    i2c_smbus_write_byte_data(chip->client,
                              WUSB3801_REG_CONTROL1, BIT_REG_CTRL1_SM_RST);
    dev_info(cdev, "wusb3801_probe end \n");

    return 0;
err4:
    cancel_delayed_work_sync(&chip->first_check_typec_work);
    destroy_workqueue(chip->cc_wq);
    mutex_destroy(&chip->mlock);
err3:
    wusb3801_free_gpio(chip);
    devm_kfree(cdev, chip->tcpc_desc);
err2:
    devm_kfree(cdev, chip->pdata);
err1:
    i2c_set_clientdata(client, NULL);
    devm_kfree(cdev, chip);

    return ret;
}

static int wusb3801_remove(struct i2c_client *client)
{
    struct wusb3801_chip *chip = i2c_get_clientdata(client);
    struct device *cdev = &client->dev;

    if (!chip)
    {
        dev_err(cdev, "%s : chip is null\n", __func__);
        return -ENODEV;
    }

    if (chip->irq_gpio > 0)
        devm_free_irq(cdev, chip->irq_gpio, chip);

    cancel_delayed_work_sync(&chip->first_check_typec_work);
    destroy_workqueue(chip->cc_wq);
    mutex_destroy(&chip->mlock);
    wusb3801_free_gpio(chip);

    devm_kfree(cdev, chip->pdata);

    i2c_set_clientdata(client, NULL);
    devm_kfree(cdev, chip);

    return 0;
}

static void wusb3801_shutdown(struct i2c_client *client)
{
    struct wusb3801_chip *chip = i2c_get_clientdata(client);
    struct device *cdev = &client->dev;
    int rc = 0;

    rc |= wusb3801_set_mode(chip, WUSB3801_SNK);
    rc |= wusb3801_set_chip_state(chip, WUSB3801_STATE_ERROR_RECOVERY);
    if (rc < 0)
        dev_err(cdev, "%s: failed to set sink mode\n", __func__);
}

#ifdef CONFIG_PM
static int wusb3801_suspend(struct device *dev)
{
    return 0;
}

static int wusb3801_resume(struct device *dev)
{
    return 0;
}

static const struct dev_pm_ops wusb3801_dev_pm_ops =
{
    .suspend = wusb3801_suspend,
    .resume  = wusb3801_resume,
};
#endif

static const struct i2c_device_id wusb3801_id_table[] =
{
    {"wusb3801", 0},
    {},
};
MODULE_DEVICE_TABLE(i2c, wusb3801_id_table);

#ifdef CONFIG_OF

static struct of_device_id wusb3801_match_table[] =
{
    { .compatible = "semi,wusb3801",},
    { },
};
#else /*!CONFIG_OF*/
#define wusb3801_match_table NULL
#endif /*!CONFIG_OF*/

static struct i2c_driver wusb3801_i2c_driver =
{
    .driver = {
        .name = "wusb3801",
        .owner = THIS_MODULE,
        .of_match_table = wusb3801_match_table,
#ifdef CONFIG_PM
        .pm = &wusb3801_dev_pm_ops,
#endif
    },
    .probe = wusb3801_probe,
    .remove = wusb3801_remove,
    .shutdown = wusb3801_shutdown,
    .id_table = wusb3801_id_table,
};

static __init int wusb3801_i2c_init(void)
{
    return i2c_add_driver(&wusb3801_i2c_driver);
}

static __exit void wusb3801_i2c_exit(void)
{
    i2c_del_driver(&wusb3801_i2c_driver);
}

subsys_initcall(wusb3801_i2c_init);
module_exit(wusb3801_i2c_exit);

MODULE_AUTHOR("zhang.tao@tinno.com");
MODULE_DESCRIPTION("WUSB3801 USB Type-C driver for Mediatek Platform");
MODULE_LICENSE("GPL v2");
