// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (C) 2019 MediaTek Inc.
 *
 * Author: Andy Teng <andy.teng@mediatek.com>
 *
 */

#include <linux/module.h>
#include "pinctrl-mtk-mt6761.h"
#include "pinctrl-paris.h"

/* mt6761 have multiple bases to program pin configuration listed as the below:
 * gpio:0x10005000,     iocfg_lt:0x10002000, iocfg_lm:0x10002200,
 * iocfg_lb:0x10002400, iocfg_bl:0x10002600, iocfg_rr:0x10002800,
 * iocfg_rb:0x10002A00, iocfg_rt:0x10002C00
 * _i_based could be used to indicate what base the pin should be mapped into.
 */

#define PIN_FIELD_BASE(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits) \
	PIN_FIELD_CALC(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits, \
		       32, 0)

#define PINS_FIELD_BASE(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits) \
	PIN_FIELD_CALC(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits,  \
		       32, 1)

static const struct mtk_pin_field_calc mt6761_pin_mode_range[] = {
	PIN_FIELD_BASE(0, 7, 0, 0x0300, 0x10, 0, 4),
	PIN_FIELD_BASE(8, 15, 0, 0x0310, 0x10, 0, 4),
	PIN_FIELD_BASE(16, 23, 0, 0x0320, 0x10, 0, 4),
	PIN_FIELD_BASE(24, 31, 0, 0x0330, 0x10, 0, 4),
	PIN_FIELD_BASE(32, 39, 0, 0x0340, 0x10, 0, 4),
	PIN_FIELD_BASE(40, 47, 0, 0x0350, 0x10, 0, 4),
	PIN_FIELD_BASE(48, 55, 0, 0x0360, 0x10, 0, 4),
	PIN_FIELD_BASE(56, 63, 0, 0x0370, 0x10, 0, 4),
	PIN_FIELD_BASE(64, 71, 0, 0x0380, 0x10, 0, 4),
	PIN_FIELD_BASE(72, 79, 0, 0x0390, 0x10, 0, 4),
	PIN_FIELD_BASE(80, 87, 0, 0x03A0, 0x10, 0, 4),
	PIN_FIELD_BASE(88, 95, 0, 0x03B0, 0x10, 0, 4),
	PIN_FIELD_BASE(96, 103, 0, 0x03C0, 0x10, 0, 4),
	PIN_FIELD_BASE(104, 111, 0, 0x03D0, 0x10, 0, 4),
	PIN_FIELD_BASE(112, 119, 0, 0x03E0, 0x10, 0, 4),
	PIN_FIELD_BASE(120, 127, 0, 0x03F0, 0x10, 0, 4),
	PIN_FIELD_BASE(128, 135, 0, 0x0400, 0x10, 0, 4),
	PIN_FIELD_BASE(136, 143, 0, 0x0410, 0x10, 0, 4),
	PIN_FIELD_BASE(144, 151, 0, 0x0420, 0x10, 0, 4),
	PIN_FIELD_BASE(152, 159, 0, 0x0430, 0x10, 0, 4),
	PIN_FIELD_BASE(160, 167, 0, 0x0440, 0x10, 0, 4),
	PIN_FIELD_BASE(168, 175, 0, 0x0450, 0x10, 0, 4),
	PIN_FIELD_BASE(176, 178, 0, 0x0460, 0x10, 0, 4),
};

static const struct mtk_pin_field_calc mt6761_pin_dir_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0000, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0010, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 178, 0, 0x0050, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_di_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0200, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0210, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0220, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0230, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0240, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 178, 0, 0x0250, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_do_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0100, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0110, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0120, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0130, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0140, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 178, 0, 0x0150, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_ies_range[] = {
	PIN_FIELD_BASE(0, 0, 5, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(1, 1, 5, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(2, 2, 5, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(3, 3, 5, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(4, 4, 5, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(5, 5, 5, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(6, 6, 5, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(7, 7, 5, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(8, 8, 6, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(9, 9, 5, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(10, 10, 5, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(11, 11, 5, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(12, 12, 2, 0x0020, 0x10, 9, 1),
	PIN_FIELD_BASE(13, 13, 3, 0x0020, 0x10, 26, 1),
	PIN_FIELD_BASE(14, 14, 3, 0x0020, 0x10, 25, 1),
	PIN_FIELD_BASE(15, 15, 3, 0x0020, 0x10, 27, 1),
	PIN_FIELD_BASE(16, 16, 3, 0x0020, 0x10, 24, 1),
	PIN_FIELD_BASE(17, 17, 3, 0x0020, 0x10, 19, 1),
	PIN_FIELD_BASE(18, 18, 3, 0x0020, 0x10, 16, 1),
	PIN_FIELD_BASE(19, 19, 3, 0x0020, 0x10, 18, 1),
	PIN_FIELD_BASE(20, 20, 3, 0x0020, 0x10, 17, 1),
	PIN_FIELD_BASE(21, 21, 3, 0x0020, 0x10, 22, 1),
	PIN_FIELD_BASE(22, 22, 3, 0x0020, 0x10, 21, 1),
	PIN_FIELD_BASE(23, 23, 3, 0x0020, 0x10, 23, 1),
	PIN_FIELD_BASE(24, 24, 3, 0x0020, 0x10, 20, 1),
	PIN_FIELD_BASE(25, 25, 3, 0x0020, 0x10, 14, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0020, 0x10, 13, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0020, 0x10, 15, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0020, 0x10, 12, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0020, 0x10, 1, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0020, 0x10, 5, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0020, 0x10, 2, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0020, 0x10, 4, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0020, 0x10, 3, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0020, 0x10, 10, 1),
	PIN_FIELD_BASE(36, 36, 3, 0x0020, 0x10, 11, 1),
	PIN_FIELD_BASE(37, 37, 3, 0x0020, 0x10, 9, 1),
	PIN_FIELD_BASE(38, 38, 3, 0x0020, 0x10, 6, 1),
	PIN_FIELD_BASE(39, 39, 3, 0x0020, 0x10, 8, 1),
	PIN_FIELD_BASE(40, 40, 3, 0x0020, 0x10, 7, 1),
	PIN_FIELD_BASE(41, 41, 4, 0x0040, 0x10, 19, 1),
	PIN_FIELD_BASE(42, 42, 4, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(43, 43, 4, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(44, 44, 4, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(45, 45, 4, 0x0040, 0x10, 22, 1),
	PIN_FIELD_BASE(46, 46, 4, 0x0040, 0x10, 21, 1),
	PIN_FIELD_BASE(47, 47, 4, 0x0040, 0x10, 20, 1),
	PIN_FIELD_BASE(48, 48, 4, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(49, 49, 4, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(50, 50, 4, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(51, 51, 4, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(52, 52, 4, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 4, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(54, 54, 4, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(55, 55, 4, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(56, 56, 4, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(57, 57, 4, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(58, 58, 4, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(59, 59, 4, 0x0040, 0x10, 31, 1),
	PIN_FIELD_BASE(60, 60, 4, 0x0040, 0x10, 30, 1),
	PIN_FIELD_BASE(61, 61, 6, 0x0030, 0x10, 18, 1),
	PIN_FIELD_BASE(62, 62, 6, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(63, 63, 6, 0x0030, 0x10, 17, 1),
	PIN_FIELD_BASE(64, 64, 6, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(65, 65, 6, 0x0030, 0x10, 20, 1),
	PIN_FIELD_BASE(66, 66, 6, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(67, 67, 6, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(68, 68, 6, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(69, 69, 6, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(70, 70, 6, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(71, 71, 6, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(72, 72, 6, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(73, 73, 6, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(74, 74, 6, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 6, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(76, 76, 6, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(77, 77, 6, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(78, 78, 6, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(79, 79, 6, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(80, 80, 6, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(81, 81, 6, 0x0030, 0x10, 25, 1),
	PIN_FIELD_BASE(82, 82, 6, 0x0030, 0x10, 24, 1),
	PIN_FIELD_BASE(83, 83, 6, 0x0030, 0x10, 22, 1),
	PIN_FIELD_BASE(84, 84, 6, 0x0030, 0x10, 23, 1),
	PIN_FIELD_BASE(85, 85, 4, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(86, 86, 4, 0x0040, 0x10, 29, 1),
	PIN_FIELD_BASE(87, 87, 4, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(88, 88, 4, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(89, 89, 5, 0x0030, 0x10, 25, 1),
	PIN_FIELD_BASE(90, 90, 6, 0x0030, 0x10, 21, 1),
	PIN_FIELD_BASE(91, 91, 5, 0x0030, 0x10, 20, 1),
	PIN_FIELD_BASE(92, 92, 5, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(93, 93, 5, 0x0030, 0x10, 17, 1),
	PIN_FIELD_BASE(94, 94, 5, 0x0030, 0x10, 18, 1),
	PIN_FIELD_BASE(95, 95, 5, 0x0030, 0x10, 26, 1),
	PIN_FIELD_BASE(96, 96, 5, 0x0030, 0x10, 27, 1),
	PIN_FIELD_BASE(97, 97, 5, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(98, 98, 5, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(99, 99, 5, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(100, 100, 5, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(101, 101, 5, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(102, 102, 5, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(103, 103, 5, 0x0030, 0x10, 21, 1),
	PIN_FIELD_BASE(104, 104, 5, 0x0030, 0x10, 23, 1),
	PIN_FIELD_BASE(105, 105, 5, 0x0030, 0x10, 22, 1),
	PIN_FIELD_BASE(106, 106, 5, 0x0030, 0x10, 24, 1),
	PIN_FIELD_BASE(107, 107, 7, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(108, 108, 7, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(109, 109, 7, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(110, 110, 7, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(111, 111, 7, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(112, 112, 7, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(113, 113, 7, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(114, 114, 7, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(115, 115, 7, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(116, 116, 7, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(117, 117, 7, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 7, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(119, 119, 7, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(120, 120, 7, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(121, 121, 7, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(122, 122, 1, 0x0010, 0x10, 9, 1),
	PIN_FIELD_BASE(123, 123, 1, 0x0010, 0x10, 10, 1),
	PIN_FIELD_BASE(124, 124, 1, 0x0010, 0x10, 8, 1),
	PIN_FIELD_BASE(125, 125, 1, 0x0010, 0x10, 12, 1),
	PIN_FIELD_BASE(126, 126, 1, 0x0010, 0x10, 14, 1),
	PIN_FIELD_BASE(127, 127, 1, 0x0010, 0x10, 16, 1),
	PIN_FIELD_BASE(128, 128, 1, 0x0010, 0x10, 11, 1),
	PIN_FIELD_BASE(129, 129, 1, 0x0010, 0x10, 15, 1),
	PIN_FIELD_BASE(130, 130, 1, 0x0010, 0x10, 17, 1),
	PIN_FIELD_BASE(131, 131, 1, 0x0010, 0x10, 18, 1),
	PIN_FIELD_BASE(132, 132, 1, 0x0010, 0x10, 13, 1),
	PIN_FIELD_BASE(133, 133, 1, 0x0010, 0x10, 19, 1),
	PIN_FIELD_BASE(134, 134, 2, 0x0020, 0x10, 14, 1),
	PIN_FIELD_BASE(135, 135, 2, 0x0020, 0x10, 17, 1),
	PIN_FIELD_BASE(136, 136, 2, 0x0020, 0x10, 1, 1),
	PIN_FIELD_BASE(137, 137, 2, 0x0020, 0x10, 7, 1),
	PIN_FIELD_BASE(138, 138, 2, 0x0020, 0x10, 4, 1),
	PIN_FIELD_BASE(139, 139, 2, 0x0020, 0x10, 5, 1),
	PIN_FIELD_BASE(140, 140, 2, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(141, 141, 2, 0x0020, 0x10, 6, 1),
	PIN_FIELD_BASE(142, 142, 2, 0x0020, 0x10, 2, 1),
	PIN_FIELD_BASE(143, 143, 2, 0x0020, 0x10, 3, 1),
	PIN_FIELD_BASE(144, 144, 2, 0x0020, 0x10, 12, 1),
	PIN_FIELD_BASE(145, 145, 2, 0x0020, 0x10, 11, 1),
	PIN_FIELD_BASE(146, 146, 2, 0x0020, 0x10, 13, 1),
	PIN_FIELD_BASE(147, 147, 2, 0x0020, 0x10, 10, 1),
	PIN_FIELD_BASE(148, 148, 2, 0x0020, 0x10, 15, 1),
	PIN_FIELD_BASE(149, 149, 2, 0x0020, 0x10, 16, 1),
	PIN_FIELD_BASE(150, 150, 4, 0x0040, 0x10, 23, 1),
	PIN_FIELD_BASE(151, 151, 4, 0x0040, 0x10, 24, 1),
	PIN_FIELD_BASE(152, 152, 4, 0x0040, 0x10, 25, 1),
	PIN_FIELD_BASE(153, 153, 4, 0x0040, 0x10, 26, 1),
	PIN_FIELD_BASE(154, 154, 4, 0x0040, 0x10, 28, 1),
	PIN_FIELD_BASE(155, 155, 6, 0x0030, 0x10, 28, 1),
	PIN_FIELD_BASE(156, 156, 6, 0x0030, 0x10, 27, 1),
	PIN_FIELD_BASE(157, 157, 6, 0x0030, 0x10, 29, 1),
	PIN_FIELD_BASE(158, 158, 6, 0x0030, 0x10, 26, 1),
	PIN_FIELD_BASE(159, 159, 4, 0x0040, 0x10, 27, 1),
	PIN_FIELD_BASE(160, 160, 2, 0x0020, 0x10, 8, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(163, 163, 1, 0x0010, 0x10, 0, 1),
	PIN_FIELD_BASE(164, 164, 1, 0x0010, 0x10, 1, 1),
	PIN_FIELD_BASE(165, 165, 1, 0x0010, 0x10, 2, 1),
	PIN_FIELD_BASE(166, 166, 1, 0x0010, 0x10, 3, 1),
	PIN_FIELD_BASE(167, 167, 1, 0x0010, 0x10, 4, 1),
	PIN_FIELD_BASE(168, 168, 1, 0x0010, 0x10, 5, 1),
	PIN_FIELD_BASE(169, 169, 1, 0x0010, 0x10, 6, 1),
	PIN_FIELD_BASE(170, 170, 1, 0x0010, 0x10, 7, 1),
	PIN_FIELD_BASE(171, 171, 4, 0x0040, 0x10, 17, 1),
	PIN_FIELD_BASE(172, 172, 4, 0x0040, 0x10, 18, 1),
	PIN_FIELD_BASE(173, 173, 4, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(174, 174, 4, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 4, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(176, 176, 4, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(177, 177, 4, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(178, 178, 4, 0x0040, 0x10, 16, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_smt_range[] = {
	PIN_FIELD_BASE(0, 0, 5, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(1, 1, 5, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(2, 2, 5, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(3, 3, 5, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(4, 4, 5, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(5, 5, 5, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(6, 6, 5, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(7, 7, 5, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(8, 8, 6, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(9, 9, 5, 0x00b0, 0x10, 6, 1),
	PIN_FIELD_BASE(10, 10, 5, 0x00b0, 0x10, 6, 1),
	PIN_FIELD_BASE(11, 11, 5, 0x00b0, 0x10, 6, 1),
	PIN_FIELD_BASE(12, 12, 2, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(13, 13, 3, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(14, 14, 3, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(15, 15, 3, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(16, 16, 3, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(17, 17, 3, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(18, 18, 3, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(19, 19, 3, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(20, 20, 3, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(21, 21, 3, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(22, 22, 3, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(23, 23, 3, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(24, 24, 3, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(25, 25, 3, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x00b0, 0x10, 0, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x00b0, 0x10, 1, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(36, 36, 3, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(37, 37, 3, 0x00b0, 0x10, 6, 1),
	PIN_FIELD_BASE(38, 38, 3, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(39, 39, 3, 0x00b0, 0x10, 3, 1),
	PIN_FIELD_BASE(40, 40, 3, 0x00b0, 0x10, 3, 1),
	PIN_FIELD_BASE(41, 41, 4, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(42, 42, 4, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(43, 43, 4, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(44, 44, 4, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(45, 45, 4, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(46, 46, 4, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(47, 47, 4, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(48, 48, 4, 0x00c0, 0x10, 15, 1),
	PIN_FIELD_BASE(49, 49, 4, 0x00c0, 0x10, 17, 1),
	PIN_FIELD_BASE(50, 50, 4, 0x00c0, 0x10, 14, 1),
	PIN_FIELD_BASE(51, 51, 4, 0x00c0, 0x10, 16, 1),
	PIN_FIELD_BASE(52, 52, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(53, 53, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(54, 54, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(55, 55, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(56, 56, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(57, 57, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(58, 58, 4, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(59, 59, 4, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(60, 60, 4, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(61, 61, 6, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(62, 62, 6, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(63, 63, 6, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(64, 64, 6, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(65, 65, 6, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(66, 66, 6, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(67, 67, 6, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(68, 68, 6, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(69, 69, 6, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(70, 70, 6, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(71, 71, 6, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(72, 72, 6, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(73, 73, 6, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(74, 74, 6, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(75, 75, 6, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(76, 76, 6, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(77, 77, 6, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(78, 78, 6, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(79, 79, 6, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(80, 80, 6, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(81, 81, 6, 0x0080, 0x10, 12, 1),
	PIN_FIELD_BASE(82, 82, 6, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(83, 83, 6, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(84, 84, 6, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(85, 85, 4, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(86, 86, 4, 0x00c0, 0x10, 13, 1),
	PIN_FIELD_BASE(87, 87, 4, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(88, 88, 4, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(89, 89, 5, 0x00b0, 0x10, 13, 1),
	PIN_FIELD_BASE(90, 90, 6, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(91, 91, 5, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(92, 92, 5, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(93, 93, 5, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(94, 94, 5, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(95, 95, 5, 0x00b0, 0x10, 14, 1),
	PIN_FIELD_BASE(96, 96, 5, 0x00b0, 0x10, 14, 1),
	PIN_FIELD_BASE(97, 97, 5, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(98, 98, 5, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(99, 99, 5, 0x00b0, 0x10, 0, 1),
	PIN_FIELD_BASE(100, 100, 5, 0x00b0, 0x10, 1, 1),
	PIN_FIELD_BASE(101, 101, 5, 0x00b0, 0x10, 3, 1),
	PIN_FIELD_BASE(102, 102, 5, 0x00b0, 0x10, 3, 1),
	PIN_FIELD_BASE(103, 103, 5, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(104, 104, 5, 0x00b0, 0x10, 11, 1),
	PIN_FIELD_BASE(105, 105, 5, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(106, 106, 5, 0x00b0, 0x10, 12, 1),
	PIN_FIELD_BASE(107, 107, 7, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(108, 108, 7, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(109, 109, 7, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(110, 110, 7, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(111, 111, 7, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(112, 112, 7, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(113, 113, 7, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(114, 114, 7, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(115, 115, 7, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(116, 116, 7, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(117, 117, 7, 0x0080, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 7, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(119, 119, 7, 0x0080, 0x10, 14, 1),
	PIN_FIELD_BASE(120, 120, 7, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(121, 121, 7, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(122, 122, 1, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(123, 123, 1, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(124, 124, 1, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(125, 125, 1, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(126, 126, 1, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(127, 127, 1, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(128, 128, 1, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(129, 129, 1, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(130, 130, 1, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(131, 131, 1, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(132, 132, 1, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(133, 133, 1, 0x0080, 0x10, 12, 1),
	PIN_FIELD_BASE(134, 134, 2, 0x0060, 0x10, 11, 1),
	PIN_FIELD_BASE(135, 135, 2, 0x0060, 0x10, 13, 1),
	PIN_FIELD_BASE(136, 136, 2, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(137, 137, 2, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(138, 138, 2, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(139, 139, 2, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(140, 140, 2, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(141, 141, 2, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(142, 142, 2, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(143, 143, 2, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(144, 144, 2, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(145, 145, 2, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(146, 146, 2, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(147, 147, 2, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(148, 148, 2, 0x0060, 0x10, 12, 1),
	PIN_FIELD_BASE(149, 149, 2, 0x0060, 0x10, 12, 1),
	PIN_FIELD_BASE(150, 150, 4, 0x00c0, 0x10, 9, 1),
	PIN_FIELD_BASE(151, 151, 4, 0x00c0, 0x10, 9, 1),
	PIN_FIELD_BASE(152, 152, 4, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(153, 153, 4, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(154, 154, 4, 0x00c0, 0x10, 11, 1),
	PIN_FIELD_BASE(155, 155, 6, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(156, 156, 6, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(157, 157, 6, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(158, 158, 6, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(159, 159, 4, 0x00c0, 0x10, 11, 1),
	PIN_FIELD_BASE(160, 160, 2, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0080, 0x10, 15, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0080, 0x10, 16, 1),
	PIN_FIELD_BASE(163, 163, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(164, 164, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(165, 165, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(166, 166, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(167, 167, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(168, 168, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(169, 169, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(170, 170, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(171, 171, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(172, 172, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(173, 173, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(174, 174, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(175, 175, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(176, 176, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(177, 177, 4, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(178, 178, 4, 0x00c0, 0x10, 5, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_pu_range[] = {
	PIN_FIELD_BASE(0, 0, 5, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(1, 1, 5, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(2, 2, 5, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(3, 3, 5, 0x0060, 0x10, 11, 1),
	PIN_FIELD_BASE(4, 4, 5, 0x0060, 0x10, 12, 1),
	PIN_FIELD_BASE(5, 5, 5, 0x0060, 0x10, 13, 1),
	PIN_FIELD_BASE(6, 6, 5, 0x0060, 0x10, 14, 1),
	PIN_FIELD_BASE(7, 7, 5, 0x0060, 0x10, 15, 1),
	PIN_FIELD_BASE(8, 8, 6, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(9, 9, 5, 0x0060, 0x10, 16, 1),
	PIN_FIELD_BASE(10, 10, 5, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(11, 11, 5, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(12, 12, 2, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(13, 13, 3, 0x0060, 0x10, 14, 1),
	PIN_FIELD_BASE(14, 14, 3, 0x0060, 0x10, 13, 1),
	PIN_FIELD_BASE(15, 15, 3, 0x0060, 0x10, 15, 1),
	PIN_FIELD_BASE(16, 16, 3, 0x0060, 0x10, 12, 1),
	PIN_FIELD_BASE(17, 17, 3, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(18, 18, 3, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(19, 19, 3, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(20, 20, 3, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(21, 21, 3, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(22, 22, 3, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(23, 23, 3, 0x0060, 0x10, 11, 1),
	PIN_FIELD_BASE(24, 24, 3, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(25, 25, 3, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 4, 0x0080, 0x10, 19, 1),
	PIN_FIELD_BASE(42, 42, 4, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(43, 43, 4, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(44, 44, 4, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(45, 45, 4, 0x0080, 0x10, 22, 1),
	PIN_FIELD_BASE(46, 46, 4, 0x0080, 0x10, 21, 1),
	PIN_FIELD_BASE(47, 47, 4, 0x0080, 0x10, 20, 1),
	PIN_FIELD_BASE(48, 48, 4, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(49, 49, 4, 0x0090, 0x10, 5, 1),
	PIN_FIELD_BASE(50, 50, 4, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(51, 51, 4, 0x0090, 0x10, 4, 1),
	PIN_FIELD_BASE(52, 52, 4, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 4, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(54, 54, 4, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(55, 55, 4, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(56, 56, 4, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(57, 57, 4, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(58, 58, 4, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(59, 59, 4, 0x0080, 0x10, 31, 1),
	PIN_FIELD_BASE(60, 60, 4, 0x0080, 0x10, 30, 1),
	PIN_FIELD_BASE(61, 61, 6, 0x0050, 0x10, 18, 1),
	PIN_FIELD_BASE(62, 62, 6, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(63, 63, 6, 0x0050, 0x10, 17, 1),
	PIN_FIELD_BASE(64, 64, 6, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(65, 65, 6, 0x0050, 0x10, 20, 1),
	PIN_FIELD_BASE(66, 66, 6, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(67, 67, 6, 0x0050, 0x10, 19, 1),
	PIN_FIELD_BASE(68, 68, 6, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(69, 69, 6, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(70, 70, 6, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(71, 71, 6, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(72, 72, 6, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(73, 73, 6, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(74, 74, 6, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 6, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(76, 76, 6, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(77, 77, 6, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(78, 78, 6, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(79, 79, 6, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(80, 80, 6, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(81, 81, 6, 0x0050, 0x10, 25, 1),
	PIN_FIELD_BASE(82, 82, 6, 0x0050, 0x10, 24, 1),
	PIN_FIELD_BASE(83, 83, 6, 0x0050, 0x10, 22, 1),
	PIN_FIELD_BASE(84, 84, 6, 0x0050, 0x10, 23, 1),
	PIN_FIELD_BASE(85, 85, 4, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(86, 86, 4, 0x0080, 0x10, 29, 1),
	PIN_FIELD_BASE(87, 87, 4, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(88, 88, 4, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(89, 89, 5, 0x0060, 0x10, 21, 1),
	PIN_FIELD_BASE(90, 90, 6, 0x0050, 0x10, 21, 1),
	PIN_FIELD_BASE(95, 95, 5, 0x0060, 0x10, 22, 1),
	PIN_FIELD_BASE(96, 96, 5, 0x0060, 0x10, 23, 1),
	PIN_FIELD_BASE(97, 97, 5, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(98, 98, 5, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(99, 99, 5, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(100, 100, 5, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(101, 101, 5, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(102, 102, 5, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(103, 103, 5, 0x0060, 0x10, 17, 1),
	PIN_FIELD_BASE(104, 104, 5, 0x0060, 0x10, 19, 1),
	PIN_FIELD_BASE(105, 105, 5, 0x0060, 0x10, 18, 1),
	PIN_FIELD_BASE(106, 106, 5, 0x0060, 0x10, 20, 1),
	PIN_FIELD_BASE(107, 107, 7, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(108, 108, 7, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(109, 109, 7, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(110, 110, 7, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(111, 111, 7, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(112, 112, 7, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(113, 113, 7, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(114, 114, 7, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(115, 115, 7, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(116, 116, 7, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(117, 117, 7, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 7, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(119, 119, 7, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(120, 120, 7, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(121, 121, 7, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(134, 134, 2, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(135, 135, 2, 0x0040, 0x10, 19, 1),
	PIN_FIELD_BASE(136, 136, 2, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(137, 137, 2, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(138, 138, 2, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(139, 139, 2, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(140, 140, 2, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(141, 141, 2, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(142, 142, 2, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(143, 143, 2, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(144, 144, 2, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(145, 145, 2, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(146, 146, 2, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(147, 147, 2, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(148, 148, 2, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(149, 149, 2, 0x0040, 0x10, 16, 1),
	PIN_FIELD_BASE(150, 150, 4, 0x0080, 0x10, 23, 1),
	PIN_FIELD_BASE(151, 151, 4, 0x0080, 0x10, 24, 1),
	PIN_FIELD_BASE(152, 152, 4, 0x0080, 0x10, 25, 1),
	PIN_FIELD_BASE(153, 153, 4, 0x0080, 0x10, 26, 1),
	PIN_FIELD_BASE(154, 154, 4, 0x0080, 0x10, 28, 1),
	PIN_FIELD_BASE(155, 155, 6, 0x0050, 0x10, 28, 1),
	PIN_FIELD_BASE(156, 156, 6, 0x0050, 0x10, 27, 1),
	PIN_FIELD_BASE(157, 157, 6, 0x0050, 0x10, 29, 1),
	PIN_FIELD_BASE(158, 158, 6, 0x0050, 0x10, 26, 1),
	PIN_FIELD_BASE(159, 159, 4, 0x0080, 0x10, 27, 1),
	PIN_FIELD_BASE(160, 160, 2, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(163, 163, 1, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(164, 164, 1, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(165, 165, 1, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(166, 166, 1, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(167, 167, 1, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(168, 168, 1, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(169, 169, 1, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(170, 170, 1, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(171, 171, 4, 0x0080, 0x10, 17, 1),
	PIN_FIELD_BASE(172, 172, 4, 0x0080, 0x10, 18, 1),
	PIN_FIELD_BASE(173, 173, 4, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(174, 174, 4, 0x0080, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 4, 0x0080, 0x10, 13, 1),
	PIN_FIELD_BASE(176, 176, 4, 0x0080, 0x10, 14, 1),
	PIN_FIELD_BASE(177, 177, 4, 0x0080, 0x10, 15, 1),
	PIN_FIELD_BASE(178, 178, 4, 0x0080, 0x10, 16, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_pd_range[] = {
	PIN_FIELD_BASE(0, 0, 5, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(1, 1, 5, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(2, 2, 5, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(3, 3, 5, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(4, 4, 5, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(5, 5, 5, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(6, 6, 5, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(7, 7, 5, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(8, 8, 6, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(9, 9, 5, 0x0040, 0x10, 16, 1),
	PIN_FIELD_BASE(10, 10, 5, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(11, 11, 5, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(12, 12, 2, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(13, 13, 3, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(14, 14, 3, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(15, 15, 3, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(16, 16, 3, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(17, 17, 3, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(18, 18, 3, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(19, 19, 3, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(20, 20, 3, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(21, 21, 3, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(22, 22, 3, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(23, 23, 3, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(24, 24, 3, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(25, 25, 3, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 4, 0x0060, 0x10, 19, 1),
	PIN_FIELD_BASE(42, 42, 4, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(43, 43, 4, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(44, 44, 4, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(45, 45, 4, 0x0060, 0x10, 22, 1),
	PIN_FIELD_BASE(46, 46, 4, 0x0060, 0x10, 21, 1),
	PIN_FIELD_BASE(47, 47, 4, 0x0060, 0x10, 20, 1),
	PIN_FIELD_BASE(48, 48, 4, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(49, 49, 4, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(50, 50, 4, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(51, 51, 4, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(52, 52, 4, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 4, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(54, 54, 4, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(55, 55, 4, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(56, 56, 4, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(57, 57, 4, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(58, 58, 4, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(59, 59, 4, 0x0060, 0x10, 31, 1),
	PIN_FIELD_BASE(60, 60, 4, 0x0060, 0x10, 30, 1),
	PIN_FIELD_BASE(61, 61, 6, 0x0040, 0x10, 18, 1),
	PIN_FIELD_BASE(62, 62, 6, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(63, 63, 6, 0x0040, 0x10, 17, 1),
	PIN_FIELD_BASE(64, 64, 6, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(65, 65, 6, 0x0040, 0x10, 20, 1),
	PIN_FIELD_BASE(66, 66, 6, 0x0040, 0x10, 16, 1),
	PIN_FIELD_BASE(67, 67, 6, 0x0040, 0x10, 19, 1),
	PIN_FIELD_BASE(68, 68, 6, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(69, 69, 6, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(70, 70, 6, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(71, 71, 6, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(72, 72, 6, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(73, 73, 6, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(74, 74, 6, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 6, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(76, 76, 6, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(77, 77, 6, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(78, 78, 6, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(79, 79, 6, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(80, 80, 6, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(81, 81, 6, 0x0040, 0x10, 25, 1),
	PIN_FIELD_BASE(82, 82, 6, 0x0040, 0x10, 24, 1),
	PIN_FIELD_BASE(83, 83, 6, 0x0040, 0x10, 22, 1),
	PIN_FIELD_BASE(84, 84, 6, 0x0040, 0x10, 23, 1),
	PIN_FIELD_BASE(85, 85, 4, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(86, 86, 4, 0x0060, 0x10, 29, 1),
	PIN_FIELD_BASE(87, 87, 4, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(88, 88, 4, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(89, 89, 5, 0x0040, 0x10, 21, 1),
	PIN_FIELD_BASE(90, 90, 6, 0x0040, 0x10, 21, 1),
	PIN_FIELD_BASE(95, 95, 5, 0x0040, 0x10, 22, 1),
	PIN_FIELD_BASE(96, 96, 5, 0x0040, 0x10, 23, 1),
	PIN_FIELD_BASE(97, 97, 5, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(98, 98, 5, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(99, 99, 5, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(100, 100, 5, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(101, 101, 5, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(102, 102, 5, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(103, 103, 5, 0x0040, 0x10, 17, 1),
	PIN_FIELD_BASE(104, 104, 5, 0x0040, 0x10, 19, 1),
	PIN_FIELD_BASE(105, 105, 5, 0x0040, 0x10, 18, 1),
	PIN_FIELD_BASE(106, 106, 5, 0x0040, 0x10, 20, 1),
	PIN_FIELD_BASE(107, 107, 7, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(108, 108, 7, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(109, 109, 7, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(110, 110, 7, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(111, 111, 7, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(112, 112, 7, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(113, 113, 7, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(114, 114, 7, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(115, 115, 7, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(116, 116, 7, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(117, 117, 7, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 7, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(119, 119, 7, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(120, 120, 7, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(121, 121, 7, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(134, 134, 2, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(135, 135, 2, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(136, 136, 2, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(137, 137, 2, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(138, 138, 2, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(139, 139, 2, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(140, 140, 2, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(141, 141, 2, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(142, 142, 2, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(143, 143, 2, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(144, 144, 2, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(145, 145, 2, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(146, 146, 2, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(147, 147, 2, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(148, 148, 2, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(149, 149, 2, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(150, 150, 4, 0x0060, 0x10, 23, 1),
	PIN_FIELD_BASE(151, 151, 4, 0x0060, 0x10, 24, 1),
	PIN_FIELD_BASE(152, 152, 4, 0x0060, 0x10, 25, 1),
	PIN_FIELD_BASE(153, 153, 4, 0x0060, 0x10, 26, 1),
	PIN_FIELD_BASE(154, 154, 4, 0x0060, 0x10, 28, 1),
	PIN_FIELD_BASE(155, 155, 6, 0x0040, 0x10, 28, 1),
	PIN_FIELD_BASE(156, 156, 6, 0x0040, 0x10, 27, 1),
	PIN_FIELD_BASE(157, 157, 6, 0x0040, 0x10, 29, 1),
	PIN_FIELD_BASE(158, 158, 6, 0x0040, 0x10, 26, 1),
	PIN_FIELD_BASE(159, 159, 4, 0x0060, 0x10, 27, 1),
	PIN_FIELD_BASE(160, 160, 2, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0040, 0x10, 16, 1),
	PIN_FIELD_BASE(163, 163, 1, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(164, 164, 1, 0x0020, 0x10, 1, 1),
	PIN_FIELD_BASE(165, 165, 1, 0x0020, 0x10, 2, 1),
	PIN_FIELD_BASE(166, 166, 1, 0x0020, 0x10, 3, 1),
	PIN_FIELD_BASE(167, 167, 1, 0x0020, 0x10, 4, 1),
	PIN_FIELD_BASE(168, 168, 1, 0x0020, 0x10, 5, 1),
	PIN_FIELD_BASE(169, 169, 1, 0x0020, 0x10, 6, 1),
	PIN_FIELD_BASE(170, 170, 1, 0x0020, 0x10, 7, 1),
	PIN_FIELD_BASE(171, 171, 4, 0x0060, 0x10, 17, 1),
	PIN_FIELD_BASE(172, 172, 4, 0x0060, 0x10, 18, 1),
	PIN_FIELD_BASE(173, 173, 4, 0x0060, 0x10, 11, 1),
	PIN_FIELD_BASE(174, 174, 4, 0x0060, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 4, 0x0060, 0x10, 13, 1),
	PIN_FIELD_BASE(176, 176, 4, 0x0060, 0x10, 14, 1),
	PIN_FIELD_BASE(177, 177, 4, 0x0060, 0x10, 15, 1),
	PIN_FIELD_BASE(178, 178, 4, 0x0060, 0x10, 16, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_pupd_range[] = {
	PIN_FIELD_BASE(29, 29, 3, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(36, 36, 3, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(37, 37, 3, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(38, 38, 3, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(39, 39, 3, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(40, 40, 3, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(91, 91, 5, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(92, 92, 5, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(93, 93, 5, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(94, 94, 5, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(122, 122, 1, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(123, 123, 1, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(124, 124, 1, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(125, 125, 1, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(126, 126, 1, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(127, 127, 1, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(128, 128, 1, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(129, 129, 1, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(130, 130, 1, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(131, 131, 1, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(132, 132, 1, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(133, 133, 1, 0x0030, 0x10, 11, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_r0_range[] = {
	PIN_FIELD_BASE(29, 29, 3, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0070, 0x10, 10, 1),
	PIN_FIELD_BASE(36, 36, 3, 0x0070, 0x10, 11, 1),
	PIN_FIELD_BASE(37, 37, 3, 0x0070, 0x10, 9, 1),
	PIN_FIELD_BASE(38, 38, 3, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(39, 39, 3, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(40, 40, 3, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(91, 91, 5, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(92, 92, 5, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(93, 93, 5, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(94, 94, 5, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(122, 122, 1, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(123, 123, 1, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(124, 124, 1, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(125, 125, 1, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(126, 126, 1, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(127, 127, 1, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(128, 128, 1, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(129, 129, 1, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(130, 130, 1, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(131, 131, 1, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(132, 132, 1, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(133, 133, 1, 0x0050, 0x10, 11, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_r1_range[] = {
	PIN_FIELD_BASE(29, 29, 3, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(36, 36, 3, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(37, 37, 3, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(38, 38, 3, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(39, 39, 3, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(40, 40, 3, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(91, 91, 5, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(92, 92, 5, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(93, 93, 5, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(94, 94, 5, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(122, 122, 1, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(123, 123, 1, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(124, 124, 1, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(125, 125, 1, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(126, 126, 1, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(127, 127, 1, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(128, 128, 1, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(129, 129, 1, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(130, 130, 1, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(131, 131, 1, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(132, 132, 1, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(133, 133, 1, 0x0060, 0x10, 11, 1),
};

static const struct mtk_pin_field_calc mt6761_pin_drv_range[] = {
	PIN_FIELD_BASE(0, 0, 5, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(1, 1, 5, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(2, 2, 5, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(3, 3, 5, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(4, 4, 5, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(5, 5, 5, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(6, 6, 5, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(7, 7, 5, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(8, 8, 6, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(9, 9, 5, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(10, 10, 5, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(11, 11, 5, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(12, 12, 2, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(13, 13, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(14, 14, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(15, 15, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(16, 16, 3, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(17, 17, 3, 0x0000, 0x10, 23, 3),
	PIN_FIELD_BASE(18, 18, 3, 0x0000, 0x10, 26, 3),
	PIN_FIELD_BASE(19, 19, 3, 0x0000, 0x10, 23, 3),
	PIN_FIELD_BASE(20, 20, 3, 0x0000, 0x10, 23, 3),
	PIN_FIELD_BASE(21, 21, 3, 0x0000, 0x10, 29, 3),
	PIN_FIELD_BASE(22, 22, 3, 0x0000, 0x10, 29, 3),
	PIN_FIELD_BASE(23, 23, 3, 0x0000, 0x10, 29, 3),
	PIN_FIELD_BASE(24, 24, 3, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(25, 25, 3, 0x0000, 0x10, 17, 3),
	PIN_FIELD_BASE(26, 26, 3, 0x0000, 0x10, 17, 3),
	PIN_FIELD_BASE(27, 27, 3, 0x0000, 0x10, 17, 3),
	PIN_FIELD_BASE(28, 28, 3, 0x0000, 0x10, 20, 3),
	PIN_FIELD_BASE(29, 29, 3, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(30, 30, 3, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(31, 31, 3, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(32, 32, 3, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(33, 33, 3, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(34, 34, 3, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(35, 35, 3, 0x0000, 0x10, 13, 2),
	PIN_FIELD_BASE(36, 36, 3, 0x0000, 0x10, 13, 2),
	PIN_FIELD_BASE(37, 37, 3, 0x0000, 0x10, 15, 2),
	PIN_FIELD_BASE(38, 38, 3, 0x0000, 0x10, 11, 2),
	PIN_FIELD_BASE(39, 39, 3, 0x0000, 0x10, 9, 2),
	PIN_FIELD_BASE(40, 40, 3, 0x0000, 0x10, 9, 2),
	PIN_FIELD_BASE(41, 41, 4, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(42, 42, 4, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(43, 43, 4, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(44, 44, 4, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(45, 45, 4, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(46, 46, 4, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(47, 47, 4, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(48, 48, 4, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(49, 49, 4, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(50, 50, 4, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(51, 51, 4, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(52, 52, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(53, 53, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(54, 54, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(55, 55, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(56, 56, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(57, 57, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(58, 58, 4, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(59, 59, 4, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(60, 60, 4, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(61, 61, 6, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(62, 62, 6, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(63, 63, 6, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(64, 64, 6, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(65, 65, 6, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(66, 66, 6, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(67, 67, 6, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(68, 68, 6, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(69, 69, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(70, 70, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(71, 71, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(72, 72, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(73, 73, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(74, 74, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(75, 75, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(76, 76, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(77, 77, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(78, 78, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(79, 79, 6, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(80, 80, 6, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(81, 81, 6, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(82, 82, 6, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(83, 83, 6, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(84, 84, 6, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(85, 85, 4, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(86, 86, 4, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(87, 87, 4, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(88, 88, 4, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(89, 89, 5, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(90, 90, 6, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(91, 91, 5, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(92, 92, 5, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(93, 93, 5, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(94, 94, 5, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(95, 95, 5, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(96, 96, 5, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(97, 97, 5, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(98, 98, 5, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(99, 99, 5, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(100, 100, 5, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(101, 101, 5, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(102, 102, 5, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(103, 103, 5, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(104, 104, 5, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(105, 105, 5, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(106, 106, 5, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(107, 107, 7, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(108, 108, 7, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(109, 109, 7, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(110, 110, 7, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(111, 111, 7, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(112, 112, 7, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(113, 113, 7, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(114, 114, 7, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(115, 115, 7, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(116, 116, 7, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(117, 117, 7, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(118, 118, 7, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(119, 119, 7, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(120, 120, 7, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(121, 121, 7, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(122, 122, 1, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(123, 123, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(124, 124, 1, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(125, 125, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(126, 126, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(127, 127, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(128, 128, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(129, 129, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(130, 130, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(131, 131, 1, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(132, 132, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(133, 133, 1, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(134, 134, 2, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(135, 135, 2, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(136, 136, 2, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(137, 137, 2, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(138, 138, 2, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(139, 139, 2, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(140, 140, 2, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(141, 141, 2, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(142, 142, 2, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(143, 143, 2, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(144, 144, 2, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(145, 145, 2, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(146, 146, 2, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(147, 147, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(148, 148, 2, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(149, 149, 2, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(150, 150, 4, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(151, 151, 4, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(152, 152, 4, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(153, 153, 4, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(154, 154, 4, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(155, 155, 6, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(156, 156, 6, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(157, 157, 6, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(158, 158, 6, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(159, 159, 4, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(160, 160, 2, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(161, 161, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(162, 162, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(163, 163, 1, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(164, 164, 1, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(165, 165, 1, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(166, 166, 1, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(167, 167, 1, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(168, 168, 1, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(169, 169, 1, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(170, 170, 1, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(171, 171, 4, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(172, 172, 4, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(173, 173, 4, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(174, 174, 4, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(175, 175, 4, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(176, 176, 4, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(177, 177, 4, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(178, 178, 4, 0x0000, 0x10, 15, 3),
};

static const struct mtk_pin_reg_calc mt6761_reg_cals[PINCTRL_PIN_REG_MAX] = {
	[PINCTRL_PIN_REG_MODE] = MTK_RANGE(mt6761_pin_mode_range),
	[PINCTRL_PIN_REG_DIR] = MTK_RANGE(mt6761_pin_dir_range),
	[PINCTRL_PIN_REG_DI] = MTK_RANGE(mt6761_pin_di_range),
	[PINCTRL_PIN_REG_DO] = MTK_RANGE(mt6761_pin_do_range),
	[PINCTRL_PIN_REG_SR] = MTK_RANGE(mt6761_pin_dir_range),
	[PINCTRL_PIN_REG_SMT] = MTK_RANGE(mt6761_pin_smt_range),
	[PINCTRL_PIN_REG_IES] = MTK_RANGE(mt6761_pin_ies_range),
	[PINCTRL_PIN_REG_PU] = MTK_RANGE(mt6761_pin_pu_range),
	[PINCTRL_PIN_REG_PD] = MTK_RANGE(mt6761_pin_pd_range),
	[PINCTRL_PIN_REG_DRV] = MTK_RANGE(mt6761_pin_drv_range),
	[PINCTRL_PIN_REG_PUPD] = MTK_RANGE(mt6761_pin_pupd_range),
	[PINCTRL_PIN_REG_R0] = MTK_RANGE(mt6761_pin_r0_range),
	[PINCTRL_PIN_REG_R1] = MTK_RANGE(mt6761_pin_r1_range),
};

static const struct mtk_eint_hw mt6761_eint_hw = {
	.port_mask = 7,
	.ports     = 6,
	.ap_num    = 160,
	.db_cnt    = 16,
};

static const struct mtk_pin_soc mt6761_data = {
	.reg_cal = mt6761_reg_cals,
	.pins = mtk_pins_mt6761,
	.npins = ARRAY_SIZE(mtk_pins_mt6761),
	.ngrps = ARRAY_SIZE(mtk_pins_mt6761),
	.nfuncs = 8,
	.eint_hw = &mt6761_eint_hw,
	.gpio_m = 0,
	.bias_set_combo = mtk_pinconf_bias_set_combo,
	.bias_get_combo = mtk_pinconf_bias_get_combo,
	.drive_set = mtk_pinconf_drive_set_raw,
	.drive_get = mtk_pinconf_drive_get_raw,
};

static const struct of_device_id mt6761_pinctrl_of_match[] = {
	{ .compatible = "mediatek,mt6761-pinctrl", },
	{ }
};

static int mt6761_pinctrl_probe(struct platform_device *pdev)
{
	return mtk_paris_pinctrl_probe(pdev, &mt6761_data);
}

static struct platform_driver mt6761_pinctrl_driver = {
	.driver = {
		.name = "mt6761-pinctrl",
		.of_match_table = mt6761_pinctrl_of_match,
		.pm = &mtk_eint_pm_ops_v2,
	},
	.probe = mt6761_pinctrl_probe,
};

static int __init mt6761_pinctrl_init(void)
{
	return platform_driver_register(&mt6761_pinctrl_driver);
}
arch_initcall(mt6761_pinctrl_init);

MODULE_LICENSE("GPL v2");
MODULE_DESCRIPTION("MediaTek MT6761 Pinctrl Driver");
