// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (C) 2019 MediaTek Inc.
 * Author: Andy Teng <andy.teng@mediatek.com>
 *
 */

#include "pinctrl-mtk-mt6873.h"
#include "pinctrl-paris.h"

/* MT6873 have multiple bases to program pin configuration listed as the below:
 * gpio:0x10005000,     iocfg_rm:0x11C20000, iocfg_bm:0x11D10000,
 * iocfg_bl:0x11D30000, iocfg_br:0x11D40000, iocfg_lm:0x11E20000,
 * iocfg_lb:0x11E70000, iocfg_rt:0x11EA0000, iocfg_lt:0x11F20000,
 * iocfg_tl:0x11F30000
 * _i_based could be used to indicate what base the pin should be mapped into.
 */

#define PIN_FIELD_BASE(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits) \
	PIN_FIELD_CALC(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits, \
		       32, 0)

#define PINS_FIELD_BASE(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits) \
	PIN_FIELD_CALC(s_pin, e_pin, i_base, s_addr, x_addrs, s_bit, x_bits,  \
		       32, 1)

static const struct mtk_pin_field_calc mt6873_pin_mode_range[] = {
	PIN_FIELD_BASE(0, 7, 0, 0x0300, 0x10, 0, 4),
	PIN_FIELD_BASE(8, 15, 0, 0x0310, 0x10, 0, 4),
	PIN_FIELD_BASE(16, 23, 0, 0x0320, 0x10, 0, 4),
	PIN_FIELD_BASE(24, 31, 0, 0x0330, 0x10, 0, 4),
	PIN_FIELD_BASE(32, 39, 0, 0x0340, 0x10, 0, 4),
	PIN_FIELD_BASE(40, 47, 0, 0x0350, 0x10, 0, 4),
	PIN_FIELD_BASE(48, 55, 0, 0x0360, 0x10, 0, 4),
	PIN_FIELD_BASE(56, 63, 0, 0x0370, 0x10, 0, 4),
	PIN_FIELD_BASE(64, 71, 0, 0x0380, 0x10, 0, 4),
	PIN_FIELD_BASE(72, 79, 0, 0x0390, 0x10, 0, 4),
	PIN_FIELD_BASE(80, 87, 0, 0x03A0, 0x10, 0, 4),
	PIN_FIELD_BASE(88, 95, 0, 0x03B0, 0x10, 0, 4),
	PIN_FIELD_BASE(96, 103, 0, 0x03C0, 0x10, 0, 4),
	PIN_FIELD_BASE(104, 111, 0, 0x03D0, 0x10, 0, 4),
	PIN_FIELD_BASE(112, 119, 0, 0x03E0, 0x10, 0, 4),
	PIN_FIELD_BASE(120, 127, 0, 0x03F0, 0x10, 0, 4),
	PIN_FIELD_BASE(128, 135, 0, 0x0400, 0x10, 0, 4),
	PIN_FIELD_BASE(136, 143, 0, 0x0410, 0x10, 0, 4),
	PIN_FIELD_BASE(144, 151, 0, 0x0420, 0x10, 0, 4),
	PIN_FIELD_BASE(152, 159, 0, 0x0430, 0x10, 0, 4),
	PIN_FIELD_BASE(160, 167, 0, 0x0440, 0x10, 0, 4),
	PIN_FIELD_BASE(168, 175, 0, 0x0450, 0x10, 0, 4),
	PIN_FIELD_BASE(176, 183, 0, 0x0460, 0x10, 0, 4),
	PIN_FIELD_BASE(184, 191, 0, 0x0470, 0x10, 0, 4),
	PIN_FIELD_BASE(192, 199, 0, 0x0480, 0x10, 0, 4),
	PIN_FIELD_BASE(200, 207, 0, 0x0490, 0x10, 0, 4),
	PIN_FIELD_BASE(208, 215, 0, 0x04A0, 0x10, 0, 4),
	PIN_FIELD_BASE(216, 219, 0, 0x04B0, 0x10, 0, 4),
};

static const struct mtk_pin_field_calc mt6873_pin_dir_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0000, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0010, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 191, 0, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(192, 219, 0, 0x0060, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_di_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0200, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0210, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0220, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0230, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0240, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 191, 0, 0x0250, 0x10, 0, 1),
	PIN_FIELD_BASE(192, 219, 0, 0x0260, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_do_range[] = {
	PIN_FIELD_BASE(0, 31, 0, 0x0100, 0x10, 0, 1),
	PIN_FIELD_BASE(32, 63, 0, 0x0110, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 95, 0, 0x0120, 0x10, 0, 1),
	PIN_FIELD_BASE(96, 127, 0, 0x0130, 0x10, 0, 1),
	PIN_FIELD_BASE(128, 159, 0, 0x0140, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 191, 0, 0x0150, 0x10, 0, 1),
	PIN_FIELD_BASE(192, 219, 0, 0x0160, 0x10, 0, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_smt_range[] = {
	PIN_FIELD_BASE(0, 0, 4, 0x00f0, 0x10, 8, 1),
	PIN_FIELD_BASE(1, 1, 4, 0x00f0, 0x10, 8, 1),
	PIN_FIELD_BASE(2, 2, 4, 0x00f0, 0x10, 8, 1),
	PIN_FIELD_BASE(3, 3, 4, 0x00f0, 0x10, 8, 1),
	PIN_FIELD_BASE(4, 4, 4, 0x00f0, 0x10, 8, 1),
	PIN_FIELD_BASE(5, 5, 4, 0x00f0, 0x10, 9, 1),
	PIN_FIELD_BASE(6, 6, 4, 0x00f0, 0x10, 9, 1),
	PIN_FIELD_BASE(7, 7, 4, 0x00f0, 0x10, 9, 1),
	PIN_FIELD_BASE(8, 8, 4, 0x00f0, 0x10, 9, 1),
	PIN_FIELD_BASE(9, 9, 4, 0x00f0, 0x10, 5, 1),
	PIN_FIELD_BASE(10, 10, 6, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(11, 11, 6, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(12, 12, 6, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(13, 13, 6, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(14, 14, 6, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(15, 15, 6, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(16, 16, 8, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(17, 17, 8, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(18, 18, 7, 0x0100, 0x10, 4, 1),
	PIN_FIELD_BASE(19, 19, 7, 0x0100, 0x10, 4, 1),
	PIN_FIELD_BASE(20, 20, 7, 0x0100, 0x10, 5, 1),
	PIN_FIELD_BASE(21, 21, 7, 0x0100, 0x10, 5, 1),
	PIN_FIELD_BASE(22, 22, 2, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(23, 23, 2, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(24, 24, 2, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(25, 25, 2, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x00a0, 0x10, 10, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x00a0, 0x10, 10, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x00a0, 0x10, 11, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x00a0, 0x10, 11, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x00a0, 0x10, 11, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x00a0, 0x10, 11, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x00a0, 0x10, 12, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x00a0, 0x10, 12, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x00a0, 0x10, 12, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x00a0, 0x10, 12, 1),
	PIN_FIELD_BASE(36, 36, 2, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(37, 37, 2, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(38, 38, 2, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(39, 39, 2, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(40, 40, 8, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 8, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(42, 42, 8, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(43, 43, 7, 0x0100, 0x10, 4, 1),
	PIN_FIELD_BASE(44, 44, 7, 0x0100, 0x10, 4, 1),
	PIN_FIELD_BASE(45, 45, 1, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(46, 46, 1, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(47, 47, 1, 0x00c0, 0x10, 12, 1),
	PIN_FIELD_BASE(48, 48, 1, 0x00c0, 0x10, 13, 1),
	PIN_FIELD_BASE(49, 49, 1, 0x00c0, 0x10, 13, 1),
	PIN_FIELD_BASE(50, 50, 1, 0x00c0, 0x10, 13, 1),
	PIN_FIELD_BASE(51, 51, 1, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(52, 52, 1, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(53, 53, 1, 0x00c0, 0x10, 9, 1),
	PIN_FIELD_BASE(54, 54, 1, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(55, 55, 1, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(56, 56, 1, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(57, 57, 3, 0x00a0, 0x10, 8, 1),
	PIN_FIELD_BASE(58, 58, 3, 0x00a0, 0x10, 8, 1),
	PIN_FIELD_BASE(59, 59, 3, 0x00a0, 0x10, 9, 1),
	PIN_FIELD_BASE(60, 60, 3, 0x00a0, 0x10, 9, 1),
	PIN_FIELD_BASE(61, 61, 3, 0x00a0, 0x10, 10, 1),
	PIN_FIELD_BASE(62, 62, 3, 0x00a0, 0x10, 10, 1),
	PIN_FIELD_BASE(63, 63, 3, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 64, 3, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(65, 65, 3, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(66, 66, 3, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(67, 67, 3, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(68, 68, 3, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(69, 69, 3, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(70, 70, 3, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(71, 71, 3, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(72, 72, 3, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(73, 73, 3, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(74, 74, 3, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(75, 75, 3, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(76, 76, 3, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(77, 77, 3, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(78, 78, 3, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(79, 79, 3, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(80, 80, 3, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(81, 81, 3, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(82, 82, 3, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(83, 83, 3, 0x00a0, 0x10, 5, 1),
	PIN_FIELD_BASE(84, 84, 3, 0x00a0, 0x10, 5, 1),
	PIN_FIELD_BASE(85, 85, 3, 0x00a0, 0x10, 7, 1),
	PIN_FIELD_BASE(86, 86, 3, 0x00a0, 0x10, 7, 1),
	PIN_FIELD_BASE(87, 87, 3, 0x00a0, 0x10, 6, 1),
	PIN_FIELD_BASE(88, 88, 3, 0x00a0, 0x10, 6, 1),
	PIN_FIELD_BASE(89, 89, 2, 0x00c0, 0x10, 9, 1),
	PIN_FIELD_BASE(90, 90, 2, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(91, 91, 2, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(92, 92, 2, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(93, 93, 2, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(94, 94, 2, 0x00c0, 0x10, 4, 1),
	PIN_FIELD_BASE(95, 95, 2, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(96, 96, 2, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(97, 97, 2, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(98, 98, 2, 0x00c0, 0x10, 5, 1),
	PIN_FIELD_BASE(99, 99, 2, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(100, 100, 2, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(101, 101, 2, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(102, 102, 2, 0x00c0, 0x10, 6, 1),
	PIN_FIELD_BASE(103, 103, 2, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(104, 104, 2, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(105, 105, 2, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(106, 106, 2, 0x00c0, 0x10, 7, 1),
	PIN_FIELD_BASE(107, 107, 2, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(108, 108, 2, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(109, 109, 2, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(110, 110, 2, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(111, 111, 2, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(112, 112, 2, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(113, 113, 2, 0x00c0, 0x10, 8, 1),
	PIN_FIELD_BASE(114, 114, 2, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(115, 115, 2, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(116, 116, 2, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(117, 117, 2, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x00f0, 0x10, 12, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x00f0, 0x10, 18, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x00f0, 0x10, 17, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x00f0, 0x10, 23, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x00f0, 0x10, 16, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x00f0, 0x10, 22, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x00f0, 0x10, 15, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x00f0, 0x10, 21, 1),
	PIN_FIELD_BASE(126, 126, 4, 0x00f0, 0x10, 6, 1),
	PIN_FIELD_BASE(127, 127, 4, 0x00f0, 0x10, 7, 1),
	PIN_FIELD_BASE(128, 128, 4, 0x00f0, 0x10, 10, 1),
	PIN_FIELD_BASE(129, 129, 4, 0x00f0, 0x10, 10, 1),
	PIN_FIELD_BASE(130, 130, 4, 0x00f0, 0x10, 3, 1),
	PIN_FIELD_BASE(131, 131, 4, 0x00f0, 0x10, 4, 1),
	PIN_FIELD_BASE(132, 132, 4, 0x00f0, 0x10, 11, 1),
	PIN_FIELD_BASE(133, 133, 4, 0x00f0, 0x10, 10, 1),
	PIN_FIELD_BASE(134, 134, 4, 0x00f0, 0x10, 10, 1),
	PIN_FIELD_BASE(135, 135, 4, 0x00f0, 0x10, 11, 1),
	PIN_FIELD_BASE(136, 136, 4, 0x00f0, 0x10, 0, 1),
	PIN_FIELD_BASE(137, 137, 4, 0x00f0, 0x10, 1, 1),
	PIN_FIELD_BASE(138, 138, 4, 0x00f0, 0x10, 2, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x00f0, 0x10, 14, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x00f0, 0x10, 20, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x00f0, 0x10, 13, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x00f0, 0x10, 19, 1),
	PIN_FIELD_BASE(143, 143, 1, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(144, 144, 1, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(145, 145, 1, 0x00c0, 0x10, 11, 1),
	PIN_FIELD_BASE(146, 146, 1, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(147, 147, 1, 0x00c0, 0x10, 10, 1),
	PIN_FIELD_BASE(148, 148, 1, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(149, 149, 1, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(150, 150, 1, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(151, 151, 1, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(152, 152, 7, 0x0100, 0x10, 6, 1),
	PIN_FIELD_BASE(153, 153, 7, 0x0100, 0x10, 6, 1),
	PIN_FIELD_BASE(154, 154, 7, 0x0100, 0x10, 6, 1),
	PIN_FIELD_BASE(155, 155, 7, 0x0100, 0x10, 6, 1),
	PIN_FIELD_BASE(156, 156, 7, 0x0100, 0x10, 7, 1),
	PIN_FIELD_BASE(157, 157, 7, 0x0100, 0x10, 7, 1),
	PIN_FIELD_BASE(158, 158, 7, 0x0100, 0x10, 7, 1),
	PIN_FIELD_BASE(159, 159, 7, 0x0100, 0x10, 7, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x0100, 0x10, 12, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0100, 0x10, 13, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0100, 0x10, 0, 1),
	PIN_FIELD_BASE(163, 163, 7, 0x0100, 0x10, 1, 1),
	PIN_FIELD_BASE(164, 164, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(165, 165, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(166, 166, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(167, 167, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(168, 168, 7, 0x0100, 0x10, 2, 1),
	PIN_FIELD_BASE(169, 169, 7, 0x0100, 0x10, 3, 1),
	PIN_FIELD_BASE(170, 170, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(171, 171, 7, 0x0100, 0x10, 8, 1),
	PIN_FIELD_BASE(172, 172, 7, 0x0100, 0x10, 9, 1),
	PIN_FIELD_BASE(173, 173, 7, 0x0100, 0x10, 10, 1),
	PIN_FIELD_BASE(174, 174, 7, 0x0100, 0x10, 9, 1),
	PIN_FIELD_BASE(175, 175, 7, 0x0100, 0x10, 10, 1),
	PIN_FIELD_BASE(176, 176, 7, 0x0100, 0x10, 9, 1),
	PIN_FIELD_BASE(177, 177, 7, 0x0100, 0x10, 9, 1),
	PIN_FIELD_BASE(178, 178, 7, 0x0100, 0x10, 10, 1),
	PIN_FIELD_BASE(179, 179, 7, 0x0100, 0x10, 10, 1),
	PIN_FIELD_BASE(180, 180, 7, 0x0100, 0x10, 11, 1),
	PIN_FIELD_BASE(181, 181, 7, 0x0100, 0x10, 11, 1),
	PIN_FIELD_BASE(182, 182, 7, 0x0100, 0x10, 11, 1),
	PIN_FIELD_BASE(183, 183, 9, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(184, 184, 9, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(185, 185, 9, 0x0090, 0x10, 4, 1),
	PIN_FIELD_BASE(186, 186, 9, 0x0090, 0x10, 6, 1),
	PIN_FIELD_BASE(187, 187, 9, 0x0090, 0x10, 8, 1),
	PIN_FIELD_BASE(188, 188, 9, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(189, 189, 9, 0x0090, 0x10, 7, 1),
	PIN_FIELD_BASE(190, 190, 9, 0x0090, 0x10, 9, 1),
	PIN_FIELD_BASE(191, 191, 9, 0x0090, 0x10, 10, 1),
	PIN_FIELD_BASE(192, 192, 9, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(193, 193, 9, 0x0090, 0x10, 5, 1),
	PIN_FIELD_BASE(194, 194, 9, 0x0090, 0x10, 11, 1),
	PIN_FIELD_BASE(195, 195, 5, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(196, 196, 5, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(197, 197, 5, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(198, 198, 5, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(199, 199, 5, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(206, 206, 5, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(207, 207, 5, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(208, 208, 5, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(209, 209, 5, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(210, 210, 5, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(211, 211, 5, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(212, 212, 5, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(213, 213, 5, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(214, 214, 5, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(215, 215, 5, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(216, 216, 5, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(217, 217, 5, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(218, 218, 5, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(219, 219, 5, 0x0080, 0x10, 4, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_ies_range[] = {
	PIN_FIELD_BASE(0, 0, 4, 0x0070, 0x10, 9, 1),
	PIN_FIELD_BASE(1, 1, 4, 0x0070, 0x10, 10, 1),
	PIN_FIELD_BASE(2, 2, 4, 0x0070, 0x10, 11, 1),
	PIN_FIELD_BASE(3, 3, 4, 0x0070, 0x10, 12, 1),
	PIN_FIELD_BASE(4, 4, 4, 0x0070, 0x10, 13, 1),
	PIN_FIELD_BASE(5, 5, 4, 0x0070, 0x10, 14, 1),
	PIN_FIELD_BASE(6, 6, 4, 0x0070, 0x10, 15, 1),
	PIN_FIELD_BASE(7, 7, 4, 0x0070, 0x10, 16, 1),
	PIN_FIELD_BASE(8, 8, 4, 0x0070, 0x10, 17, 1),
	PIN_FIELD_BASE(9, 9, 4, 0x0070, 0x10, 18, 1),
	PIN_FIELD_BASE(10, 10, 6, 0x0010, 0x10, 0, 1),
	PIN_FIELD_BASE(11, 11, 6, 0x0010, 0x10, 1, 1),
	PIN_FIELD_BASE(12, 12, 6, 0x0010, 0x10, 2, 1),
	PIN_FIELD_BASE(13, 13, 6, 0x0010, 0x10, 3, 1),
	PIN_FIELD_BASE(14, 14, 6, 0x0010, 0x10, 4, 1),
	PIN_FIELD_BASE(15, 15, 6, 0x0010, 0x10, 5, 1),
	PIN_FIELD_BASE(16, 16, 8, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(17, 17, 8, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(18, 18, 7, 0x0050, 0x10, 21, 1),
	PIN_FIELD_BASE(19, 19, 7, 0x0050, 0x10, 22, 1),
	PIN_FIELD_BASE(20, 20, 7, 0x0050, 0x10, 23, 1),
	PIN_FIELD_BASE(21, 21, 7, 0x0050, 0x10, 24, 1),
	PIN_FIELD_BASE(22, 22, 2, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(23, 23, 2, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(24, 24, 2, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(25, 25, 2, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0030, 0x10, 27, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0030, 0x10, 24, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0030, 0x10, 26, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0030, 0x10, 23, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0030, 0x10, 25, 1),
	PIN_FIELD_BASE(36, 36, 2, 0x0050, 0x10, 20, 1),
	PIN_FIELD_BASE(37, 37, 2, 0x0050, 0x10, 21, 1),
	PIN_FIELD_BASE(38, 38, 2, 0x0050, 0x10, 22, 1),
	PIN_FIELD_BASE(39, 39, 2, 0x0050, 0x10, 23, 1),
	PIN_FIELD_BASE(40, 40, 8, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 8, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(42, 42, 8, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(43, 43, 7, 0x0050, 0x10, 25, 1),
	PIN_FIELD_BASE(44, 44, 7, 0x0050, 0x10, 26, 1),
	PIN_FIELD_BASE(45, 45, 1, 0x0030, 0x10, 18, 1),
	PIN_FIELD_BASE(46, 46, 1, 0x0030, 0x10, 20, 1),
	PIN_FIELD_BASE(47, 47, 1, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(48, 48, 1, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(49, 49, 1, 0x0030, 0x10, 17, 1),
	PIN_FIELD_BASE(50, 50, 1, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(51, 51, 1, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(52, 52, 1, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(53, 53, 1, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(54, 54, 1, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(55, 55, 1, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(56, 56, 1, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(57, 57, 3, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(58, 58, 3, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(59, 59, 3, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(60, 60, 3, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(61, 61, 3, 0x0030, 0x10, 28, 1),
	PIN_FIELD_BASE(62, 62, 3, 0x0030, 0x10, 22, 1),
	PIN_FIELD_BASE(63, 63, 3, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 64, 3, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(65, 65, 3, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(66, 66, 3, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(67, 67, 3, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(68, 68, 3, 0x0030, 0x10, 17, 1),
	PIN_FIELD_BASE(69, 69, 3, 0x0030, 0x10, 18, 1),
	PIN_FIELD_BASE(70, 70, 3, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(71, 71, 3, 0x0030, 0x10, 20, 1),
	PIN_FIELD_BASE(72, 72, 3, 0x0030, 0x10, 21, 1),
	PIN_FIELD_BASE(73, 73, 3, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(74, 74, 3, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 3, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(76, 76, 3, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(77, 77, 3, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(78, 78, 3, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(79, 79, 3, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(80, 80, 3, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(81, 81, 3, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(82, 82, 3, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(83, 83, 3, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(84, 84, 3, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(85, 85, 3, 0x0030, 0x10, 31, 1),
	PIN_FIELD_BASE(86, 86, 3, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(87, 87, 3, 0x0030, 0x10, 29, 1),
	PIN_FIELD_BASE(88, 88, 3, 0x0030, 0x10, 30, 1),
	PIN_FIELD_BASE(89, 89, 2, 0x0050, 0x10, 24, 1),
	PIN_FIELD_BASE(90, 90, 2, 0x0050, 0x10, 25, 1),
	PIN_FIELD_BASE(91, 91, 2, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(92, 92, 2, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(93, 93, 2, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(94, 94, 2, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(95, 95, 2, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(96, 96, 2, 0x0050, 0x10, 31, 1),
	PIN_FIELD_BASE(97, 97, 2, 0x0050, 0x10, 26, 1),
	PIN_FIELD_BASE(98, 98, 2, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(99, 99, 2, 0x0050, 0x10, 27, 1),
	PIN_FIELD_BASE(100, 100, 2, 0x0050, 0x10, 28, 1),
	PIN_FIELD_BASE(101, 101, 2, 0x0050, 0x10, 29, 1),
	PIN_FIELD_BASE(102, 102, 2, 0x0050, 0x10, 30, 1),
	PIN_FIELD_BASE(103, 103, 2, 0x0050, 0x10, 18, 1),
	PIN_FIELD_BASE(104, 104, 2, 0x0050, 0x10, 17, 1),
	PIN_FIELD_BASE(105, 105, 2, 0x0050, 0x10, 19, 1),
	PIN_FIELD_BASE(106, 106, 2, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(107, 107, 2, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(108, 108, 2, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(109, 109, 2, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(110, 110, 2, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(111, 111, 2, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(112, 112, 2, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(113, 113, 2, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(114, 114, 2, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(115, 115, 2, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(116, 116, 2, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(117, 117, 2, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x0070, 0x10, 23, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x0070, 0x10, 29, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x0070, 0x10, 28, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x0070, 0x10, 27, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x0070, 0x10, 26, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(126, 126, 4, 0x0070, 0x10, 19, 1),
	PIN_FIELD_BASE(127, 127, 4, 0x0070, 0x10, 20, 1),
	PIN_FIELD_BASE(128, 128, 4, 0x0070, 0x10, 21, 1),
	PIN_FIELD_BASE(129, 129, 4, 0x0070, 0x10, 22, 1),
	PIN_FIELD_BASE(130, 130, 4, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(131, 131, 4, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(132, 132, 4, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(133, 133, 4, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(134, 134, 4, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(135, 135, 4, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(136, 136, 4, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(137, 137, 4, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(138, 138, 4, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x0070, 0x10, 25, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x0070, 0x10, 31, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x0070, 0x10, 24, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x0070, 0x10, 30, 1),
	PIN_FIELD_BASE(143, 143, 1, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(144, 144, 1, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(145, 145, 1, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(146, 146, 1, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(147, 147, 1, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(148, 148, 1, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(149, 149, 1, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(150, 150, 1, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(151, 151, 1, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(152, 152, 7, 0x0050, 0x10, 30, 1),
	PIN_FIELD_BASE(153, 153, 7, 0x0050, 0x10, 29, 1),
	PIN_FIELD_BASE(154, 154, 7, 0x0050, 0x10, 27, 1),
	PIN_FIELD_BASE(155, 155, 7, 0x0050, 0x10, 28, 1),
	PIN_FIELD_BASE(156, 156, 7, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(157, 157, 7, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(158, 158, 7, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(159, 159, 7, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x0050, 0x10, 31, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(163, 163, 7, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(164, 164, 7, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(165, 165, 7, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(166, 166, 7, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(167, 167, 7, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(168, 168, 7, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(169, 169, 7, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(170, 170, 7, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(171, 171, 7, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(172, 172, 7, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(173, 173, 7, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(174, 174, 7, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 7, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(176, 176, 7, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(177, 177, 7, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(178, 178, 7, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(179, 179, 7, 0x0050, 0x10, 17, 1),
	PIN_FIELD_BASE(180, 180, 7, 0x0050, 0x10, 18, 1),
	PIN_FIELD_BASE(181, 181, 7, 0x0050, 0x10, 19, 1),
	PIN_FIELD_BASE(182, 182, 7, 0x0050, 0x10, 20, 1),
	PIN_FIELD_BASE(183, 183, 9, 0x0020, 0x10, 1, 1),
	PIN_FIELD_BASE(184, 184, 9, 0x0020, 0x10, 2, 1),
	PIN_FIELD_BASE(185, 185, 9, 0x0020, 0x10, 4, 1),
	PIN_FIELD_BASE(186, 186, 9, 0x0020, 0x10, 6, 1),
	PIN_FIELD_BASE(187, 187, 9, 0x0020, 0x10, 8, 1),
	PIN_FIELD_BASE(188, 188, 9, 0x0020, 0x10, 3, 1),
	PIN_FIELD_BASE(189, 189, 9, 0x0020, 0x10, 7, 1),
	PIN_FIELD_BASE(190, 190, 9, 0x0020, 0x10, 9, 1),
	PIN_FIELD_BASE(191, 191, 9, 0x0020, 0x10, 10, 1),
	PIN_FIELD_BASE(192, 192, 9, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(193, 193, 9, 0x0020, 0x10, 5, 1),
	PIN_FIELD_BASE(194, 194, 9, 0x0020, 0x10, 11, 1),
	PIN_FIELD_BASE(195, 195, 5, 0x0030, 0x10, 16, 1),
	PIN_FIELD_BASE(196, 196, 5, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(197, 197, 5, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(198, 198, 5, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(199, 199, 5, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0030, 0x10, 15, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0030, 0x10, 17, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(206, 206, 5, 0x0030, 0x10, 18, 1),
	PIN_FIELD_BASE(207, 207, 5, 0x0030, 0x10, 19, 1),
	PIN_FIELD_BASE(208, 208, 5, 0x0030, 0x10, 20, 1),
	PIN_FIELD_BASE(209, 209, 5, 0x0030, 0x10, 12, 1),
	PIN_FIELD_BASE(210, 210, 5, 0x0030, 0x10, 11, 1),
	PIN_FIELD_BASE(211, 211, 5, 0x0030, 0x10, 13, 1),
	PIN_FIELD_BASE(212, 212, 5, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(213, 213, 5, 0x0030, 0x10, 14, 1),
	PIN_FIELD_BASE(214, 214, 5, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(215, 215, 5, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(216, 216, 5, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(217, 217, 5, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(218, 218, 5, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(219, 219, 5, 0x0030, 0x10, 2, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_pu_range[] = {
	PIN_FIELD_BASE(0, 0, 4, 0x00b0, 0x10, 9, 1),
	PIN_FIELD_BASE(1, 1, 4, 0x00b0, 0x10, 10, 1),
	PIN_FIELD_BASE(2, 2, 4, 0x00b0, 0x10, 11, 1),
	PIN_FIELD_BASE(3, 3, 4, 0x00b0, 0x10, 12, 1),
	PIN_FIELD_BASE(4, 4, 4, 0x00b0, 0x10, 13, 1),
	PIN_FIELD_BASE(5, 5, 4, 0x00b0, 0x10, 14, 1),
	PIN_FIELD_BASE(6, 6, 4, 0x00b0, 0x10, 15, 1),
	PIN_FIELD_BASE(7, 7, 4, 0x00b0, 0x10, 16, 1),
	PIN_FIELD_BASE(8, 8, 4, 0x00b0, 0x10, 17, 1),
	PIN_FIELD_BASE(9, 9, 4, 0x00b0, 0x10, 18, 1),
	PIN_FIELD_BASE(16, 16, 8, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(17, 17, 8, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(18, 18, 7, 0x00a0, 0x10, 21, 1),
	PIN_FIELD_BASE(19, 19, 7, 0x00a0, 0x10, 22, 1),
	PIN_FIELD_BASE(20, 20, 7, 0x00a0, 0x10, 23, 1),
	PIN_FIELD_BASE(21, 21, 7, 0x00a0, 0x10, 24, 1),
	PIN_FIELD_BASE(22, 22, 2, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(23, 23, 2, 0x0090, 0x10, 4, 1),
	PIN_FIELD_BASE(24, 24, 2, 0x0090, 0x10, 5, 1),
	PIN_FIELD_BASE(25, 25, 2, 0x0090, 0x10, 6, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0070, 0x10, 27, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0070, 0x10, 24, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0070, 0x10, 26, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0070, 0x10, 23, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0070, 0x10, 25, 1),
	PIN_FIELD_BASE(36, 36, 2, 0x0090, 0x10, 20, 1),
	PIN_FIELD_BASE(37, 37, 2, 0x0090, 0x10, 21, 1),
	PIN_FIELD_BASE(38, 38, 2, 0x0090, 0x10, 22, 1),
	PIN_FIELD_BASE(39, 39, 2, 0x0090, 0x10, 23, 1),
	PIN_FIELD_BASE(40, 40, 8, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 8, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(42, 42, 8, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(43, 43, 7, 0x00a0, 0x10, 25, 1),
	PIN_FIELD_BASE(44, 44, 7, 0x00a0, 0x10, 26, 1),
	PIN_FIELD_BASE(57, 57, 3, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(58, 58, 3, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(59, 59, 3, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(60, 60, 3, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(61, 61, 3, 0x0070, 0x10, 28, 1),
	PIN_FIELD_BASE(62, 62, 3, 0x0070, 0x10, 22, 1),
	PIN_FIELD_BASE(63, 63, 3, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 64, 3, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(65, 65, 3, 0x0070, 0x10, 12, 1),
	PIN_FIELD_BASE(66, 66, 3, 0x0070, 0x10, 15, 1),
	PIN_FIELD_BASE(67, 67, 3, 0x0070, 0x10, 16, 1),
	PIN_FIELD_BASE(68, 68, 3, 0x0070, 0x10, 17, 1),
	PIN_FIELD_BASE(69, 69, 3, 0x0070, 0x10, 18, 1),
	PIN_FIELD_BASE(70, 70, 3, 0x0070, 0x10, 19, 1),
	PIN_FIELD_BASE(71, 71, 3, 0x0070, 0x10, 20, 1),
	PIN_FIELD_BASE(72, 72, 3, 0x0070, 0x10, 21, 1),
	PIN_FIELD_BASE(73, 73, 3, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(74, 74, 3, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 3, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(76, 76, 3, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(77, 77, 3, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(78, 78, 3, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(79, 79, 3, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(80, 80, 3, 0x0070, 0x10, 9, 1),
	PIN_FIELD_BASE(81, 81, 3, 0x0070, 0x10, 10, 1),
	PIN_FIELD_BASE(82, 82, 3, 0x0070, 0x10, 11, 1),
	PIN_FIELD_BASE(83, 83, 3, 0x0070, 0x10, 13, 1),
	PIN_FIELD_BASE(84, 84, 3, 0x0070, 0x10, 14, 1),
	PIN_FIELD_BASE(85, 85, 3, 0x0070, 0x10, 31, 1),
	PIN_FIELD_BASE(86, 86, 3, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(87, 87, 3, 0x0070, 0x10, 29, 1),
	PIN_FIELD_BASE(88, 88, 3, 0x0070, 0x10, 30, 1),
	PIN_FIELD_BASE(89, 89, 2, 0x0090, 0x10, 24, 1),
	PIN_FIELD_BASE(90, 90, 2, 0x0090, 0x10, 25, 1),
	PIN_FIELD_BASE(91, 91, 2, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(92, 92, 2, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(93, 93, 2, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(94, 94, 2, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(95, 95, 2, 0x00a0, 0x10, 5, 1),
	PIN_FIELD_BASE(96, 96, 2, 0x0090, 0x10, 31, 1),
	PIN_FIELD_BASE(97, 97, 2, 0x0090, 0x10, 26, 1),
	PIN_FIELD_BASE(98, 98, 2, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(99, 99, 2, 0x0090, 0x10, 27, 1),
	PIN_FIELD_BASE(100, 100, 2, 0x0090, 0x10, 28, 1),
	PIN_FIELD_BASE(101, 101, 2, 0x0090, 0x10, 29, 1),
	PIN_FIELD_BASE(102, 102, 2, 0x0090, 0x10, 30, 1),
	PIN_FIELD_BASE(103, 103, 2, 0x0090, 0x10, 18, 1),
	PIN_FIELD_BASE(104, 104, 2, 0x0090, 0x10, 17, 1),
	PIN_FIELD_BASE(105, 105, 2, 0x0090, 0x10, 19, 1),
	PIN_FIELD_BASE(106, 106, 2, 0x0090, 0x10, 16, 1),
	PIN_FIELD_BASE(107, 107, 2, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(108, 108, 2, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(109, 109, 2, 0x0090, 0x10, 10, 1),
	PIN_FIELD_BASE(110, 110, 2, 0x0090, 0x10, 7, 1),
	PIN_FIELD_BASE(111, 111, 2, 0x0090, 0x10, 9, 1),
	PIN_FIELD_BASE(112, 112, 2, 0x0090, 0x10, 11, 1),
	PIN_FIELD_BASE(113, 113, 2, 0x0090, 0x10, 8, 1),
	PIN_FIELD_BASE(114, 114, 2, 0x0090, 0x10, 14, 1),
	PIN_FIELD_BASE(115, 115, 2, 0x0090, 0x10, 13, 1),
	PIN_FIELD_BASE(116, 116, 2, 0x0090, 0x10, 15, 1),
	PIN_FIELD_BASE(117, 117, 2, 0x0090, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x00b0, 0x10, 23, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x00b0, 0x10, 29, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x00b0, 0x10, 28, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x00b0, 0x10, 27, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x00b0, 0x10, 26, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(126, 126, 4, 0x00b0, 0x10, 19, 1),
	PIN_FIELD_BASE(127, 127, 4, 0x00b0, 0x10, 20, 1),
	PIN_FIELD_BASE(128, 128, 4, 0x00b0, 0x10, 21, 1),
	PIN_FIELD_BASE(129, 129, 4, 0x00b0, 0x10, 22, 1),
	PIN_FIELD_BASE(130, 130, 4, 0x00b0, 0x10, 6, 1),
	PIN_FIELD_BASE(131, 131, 4, 0x00b0, 0x10, 7, 1),
	PIN_FIELD_BASE(132, 132, 4, 0x00b0, 0x10, 8, 1),
	PIN_FIELD_BASE(133, 133, 4, 0x00b0, 0x10, 3, 1),
	PIN_FIELD_BASE(134, 134, 4, 0x00b0, 0x10, 4, 1),
	PIN_FIELD_BASE(135, 135, 4, 0x00b0, 0x10, 5, 1),
	PIN_FIELD_BASE(136, 136, 4, 0x00b0, 0x10, 0, 1),
	PIN_FIELD_BASE(137, 137, 4, 0x00b0, 0x10, 1, 1),
	PIN_FIELD_BASE(138, 138, 4, 0x00b0, 0x10, 2, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x00b0, 0x10, 25, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x00b0, 0x10, 31, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x00b0, 0x10, 24, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x00b0, 0x10, 30, 1),
	PIN_FIELD_BASE(143, 143, 1, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(144, 144, 1, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(145, 145, 1, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(146, 146, 1, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(147, 147, 1, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(148, 148, 1, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(149, 149, 1, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(150, 150, 1, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(151, 151, 1, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(156, 156, 7, 0x00a0, 0x10, 29, 1),
	PIN_FIELD_BASE(157, 157, 7, 0x00a0, 0x10, 30, 1),
	PIN_FIELD_BASE(158, 158, 7, 0x00a0, 0x10, 31, 1),
	PIN_FIELD_BASE(159, 159, 7, 0x00b0, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x00a0, 0x10, 27, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x00a0, 0x10, 28, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(163, 163, 7, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(164, 164, 7, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(165, 165, 7, 0x00a0, 0x10, 3, 1),
	PIN_FIELD_BASE(166, 166, 7, 0x00a0, 0x10, 4, 1),
	PIN_FIELD_BASE(167, 167, 7, 0x00a0, 0x10, 5, 1),
	PIN_FIELD_BASE(168, 168, 7, 0x00a0, 0x10, 6, 1),
	PIN_FIELD_BASE(169, 169, 7, 0x00a0, 0x10, 7, 1),
	PIN_FIELD_BASE(170, 170, 7, 0x00a0, 0x10, 8, 1),
	PIN_FIELD_BASE(171, 171, 7, 0x00a0, 0x10, 9, 1),
	PIN_FIELD_BASE(172, 172, 7, 0x00a0, 0x10, 13, 1),
	PIN_FIELD_BASE(173, 173, 7, 0x00a0, 0x10, 14, 1),
	PIN_FIELD_BASE(174, 174, 7, 0x00a0, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 7, 0x00a0, 0x10, 15, 1),
	PIN_FIELD_BASE(176, 176, 7, 0x00a0, 0x10, 10, 1),
	PIN_FIELD_BASE(177, 177, 7, 0x00a0, 0x10, 11, 1),
	PIN_FIELD_BASE(178, 178, 7, 0x00a0, 0x10, 16, 1),
	PIN_FIELD_BASE(179, 179, 7, 0x00a0, 0x10, 17, 1),
	PIN_FIELD_BASE(180, 180, 7, 0x00a0, 0x10, 18, 1),
	PIN_FIELD_BASE(181, 181, 7, 0x00a0, 0x10, 19, 1),
	PIN_FIELD_BASE(182, 182, 7, 0x00a0, 0x10, 20, 1),
	PIN_FIELD_BASE(195, 195, 5, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(196, 196, 5, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(197, 197, 5, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(198, 198, 5, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(199, 199, 5, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0050, 0x10, 17, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(206, 206, 5, 0x0050, 0x10, 18, 1),
	PIN_FIELD_BASE(207, 207, 5, 0x0050, 0x10, 19, 1),
	PIN_FIELD_BASE(208, 208, 5, 0x0050, 0x10, 20, 1),
	PIN_FIELD_BASE(209, 209, 5, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(210, 210, 5, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(211, 211, 5, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(212, 212, 5, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(213, 213, 5, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(214, 214, 5, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(215, 215, 5, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(216, 216, 5, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(217, 217, 5, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(218, 218, 5, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(219, 219, 5, 0x0050, 0x10, 2, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_pd_range[] = {
	PIN_FIELD_BASE(0, 0, 4, 0x0090, 0x10, 9, 1),
	PIN_FIELD_BASE(1, 1, 4, 0x0090, 0x10, 10, 1),
	PIN_FIELD_BASE(2, 2, 4, 0x0090, 0x10, 11, 1),
	PIN_FIELD_BASE(3, 3, 4, 0x0090, 0x10, 12, 1),
	PIN_FIELD_BASE(4, 4, 4, 0x0090, 0x10, 13, 1),
	PIN_FIELD_BASE(5, 5, 4, 0x0090, 0x10, 14, 1),
	PIN_FIELD_BASE(6, 6, 4, 0x0090, 0x10, 15, 1),
	PIN_FIELD_BASE(7, 7, 4, 0x0090, 0x10, 16, 1),
	PIN_FIELD_BASE(8, 8, 4, 0x0090, 0x10, 17, 1),
	PIN_FIELD_BASE(9, 9, 4, 0x0090, 0x10, 18, 1),
	PIN_FIELD_BASE(16, 16, 8, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(17, 17, 8, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(18, 18, 7, 0x0070, 0x10, 21, 1),
	PIN_FIELD_BASE(19, 19, 7, 0x0070, 0x10, 22, 1),
	PIN_FIELD_BASE(20, 20, 7, 0x0070, 0x10, 23, 1),
	PIN_FIELD_BASE(21, 21, 7, 0x0070, 0x10, 24, 1),
	PIN_FIELD_BASE(22, 22, 2, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(23, 23, 2, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(24, 24, 2, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(25, 25, 2, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(26, 26, 3, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(27, 27, 3, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(28, 28, 3, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(29, 29, 3, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(30, 30, 3, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(31, 31, 3, 0x0050, 0x10, 27, 1),
	PIN_FIELD_BASE(32, 32, 3, 0x0050, 0x10, 24, 1),
	PIN_FIELD_BASE(33, 33, 3, 0x0050, 0x10, 26, 1),
	PIN_FIELD_BASE(34, 34, 3, 0x0050, 0x10, 23, 1),
	PIN_FIELD_BASE(35, 35, 3, 0x0050, 0x10, 25, 1),
	PIN_FIELD_BASE(36, 36, 2, 0x0070, 0x10, 20, 1),
	PIN_FIELD_BASE(37, 37, 2, 0x0070, 0x10, 21, 1),
	PIN_FIELD_BASE(38, 38, 2, 0x0070, 0x10, 22, 1),
	PIN_FIELD_BASE(39, 39, 2, 0x0070, 0x10, 23, 1),
	PIN_FIELD_BASE(40, 40, 8, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(41, 41, 8, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(42, 42, 8, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(43, 43, 7, 0x0070, 0x10, 25, 1),
	PIN_FIELD_BASE(44, 44, 7, 0x0070, 0x10, 26, 1),
	PIN_FIELD_BASE(57, 57, 3, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(58, 58, 3, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(59, 59, 3, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(60, 60, 3, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(61, 61, 3, 0x0050, 0x10, 28, 1),
	PIN_FIELD_BASE(62, 62, 3, 0x0050, 0x10, 22, 1),
	PIN_FIELD_BASE(63, 63, 3, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(64, 64, 3, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(65, 65, 3, 0x0050, 0x10, 12, 1),
	PIN_FIELD_BASE(66, 66, 3, 0x0050, 0x10, 15, 1),
	PIN_FIELD_BASE(67, 67, 3, 0x0050, 0x10, 16, 1),
	PIN_FIELD_BASE(68, 68, 3, 0x0050, 0x10, 17, 1),
	PIN_FIELD_BASE(69, 69, 3, 0x0050, 0x10, 18, 1),
	PIN_FIELD_BASE(70, 70, 3, 0x0050, 0x10, 19, 1),
	PIN_FIELD_BASE(71, 71, 3, 0x0050, 0x10, 20, 1),
	PIN_FIELD_BASE(72, 72, 3, 0x0050, 0x10, 21, 1),
	PIN_FIELD_BASE(73, 73, 3, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(74, 74, 3, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(75, 75, 3, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(76, 76, 3, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(77, 77, 3, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(78, 78, 3, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(79, 79, 3, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(80, 80, 3, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(81, 81, 3, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(82, 82, 3, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(83, 83, 3, 0x0050, 0x10, 13, 1),
	PIN_FIELD_BASE(84, 84, 3, 0x0050, 0x10, 14, 1),
	PIN_FIELD_BASE(85, 85, 3, 0x0050, 0x10, 31, 1),
	PIN_FIELD_BASE(86, 86, 3, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(87, 87, 3, 0x0050, 0x10, 29, 1),
	PIN_FIELD_BASE(88, 88, 3, 0x0050, 0x10, 30, 1),
	PIN_FIELD_BASE(89, 89, 2, 0x0070, 0x10, 24, 1),
	PIN_FIELD_BASE(90, 90, 2, 0x0070, 0x10, 25, 1),
	PIN_FIELD_BASE(91, 91, 2, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(92, 92, 2, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(93, 93, 2, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(94, 94, 2, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(95, 95, 2, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(96, 96, 2, 0x0070, 0x10, 31, 1),
	PIN_FIELD_BASE(97, 97, 2, 0x0070, 0x10, 26, 1),
	PIN_FIELD_BASE(98, 98, 2, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(99, 99, 2, 0x0070, 0x10, 27, 1),
	PIN_FIELD_BASE(100, 100, 2, 0x0070, 0x10, 28, 1),
	PIN_FIELD_BASE(101, 101, 2, 0x0070, 0x10, 29, 1),
	PIN_FIELD_BASE(102, 102, 2, 0x0070, 0x10, 30, 1),
	PIN_FIELD_BASE(103, 103, 2, 0x0070, 0x10, 18, 1),
	PIN_FIELD_BASE(104, 104, 2, 0x0070, 0x10, 17, 1),
	PIN_FIELD_BASE(105, 105, 2, 0x0070, 0x10, 19, 1),
	PIN_FIELD_BASE(106, 106, 2, 0x0070, 0x10, 16, 1),
	PIN_FIELD_BASE(107, 107, 2, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(108, 108, 2, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(109, 109, 2, 0x0070, 0x10, 10, 1),
	PIN_FIELD_BASE(110, 110, 2, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(111, 111, 2, 0x0070, 0x10, 9, 1),
	PIN_FIELD_BASE(112, 112, 2, 0x0070, 0x10, 11, 1),
	PIN_FIELD_BASE(113, 113, 2, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(114, 114, 2, 0x0070, 0x10, 14, 1),
	PIN_FIELD_BASE(115, 115, 2, 0x0070, 0x10, 13, 1),
	PIN_FIELD_BASE(116, 116, 2, 0x0070, 0x10, 15, 1),
	PIN_FIELD_BASE(117, 117, 2, 0x0070, 0x10, 12, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x0090, 0x10, 23, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x0090, 0x10, 29, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x0090, 0x10, 28, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x00a0, 0x10, 2, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x0090, 0x10, 27, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x00a0, 0x10, 1, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x0090, 0x10, 26, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x00a0, 0x10, 0, 1),
	PIN_FIELD_BASE(126, 126, 4, 0x0090, 0x10, 19, 1),
	PIN_FIELD_BASE(127, 127, 4, 0x0090, 0x10, 20, 1),
	PIN_FIELD_BASE(128, 128, 4, 0x0090, 0x10, 21, 1),
	PIN_FIELD_BASE(129, 129, 4, 0x0090, 0x10, 22, 1),
	PIN_FIELD_BASE(130, 130, 4, 0x0090, 0x10, 6, 1),
	PIN_FIELD_BASE(131, 131, 4, 0x0090, 0x10, 7, 1),
	PIN_FIELD_BASE(132, 132, 4, 0x0090, 0x10, 8, 1),
	PIN_FIELD_BASE(133, 133, 4, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(134, 134, 4, 0x0090, 0x10, 4, 1),
	PIN_FIELD_BASE(135, 135, 4, 0x0090, 0x10, 5, 1),
	PIN_FIELD_BASE(136, 136, 4, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(137, 137, 4, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(138, 138, 4, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x0090, 0x10, 25, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x0090, 0x10, 31, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x0090, 0x10, 24, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x0090, 0x10, 30, 1),
	PIN_FIELD_BASE(143, 143, 1, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(144, 144, 1, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(145, 145, 1, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(146, 146, 1, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(147, 147, 1, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(148, 148, 1, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(149, 149, 1, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(150, 150, 1, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(151, 151, 1, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(156, 156, 7, 0x0070, 0x10, 29, 1),
	PIN_FIELD_BASE(157, 157, 7, 0x0070, 0x10, 30, 1),
	PIN_FIELD_BASE(158, 158, 7, 0x0070, 0x10, 31, 1),
	PIN_FIELD_BASE(159, 159, 7, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x0070, 0x10, 27, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x0070, 0x10, 28, 1),
	PIN_FIELD_BASE(162, 162, 7, 0x0070, 0x10, 0, 1),
	PIN_FIELD_BASE(163, 163, 7, 0x0070, 0x10, 1, 1),
	PIN_FIELD_BASE(164, 164, 7, 0x0070, 0x10, 2, 1),
	PIN_FIELD_BASE(165, 165, 7, 0x0070, 0x10, 3, 1),
	PIN_FIELD_BASE(166, 166, 7, 0x0070, 0x10, 4, 1),
	PIN_FIELD_BASE(167, 167, 7, 0x0070, 0x10, 5, 1),
	PIN_FIELD_BASE(168, 168, 7, 0x0070, 0x10, 6, 1),
	PIN_FIELD_BASE(169, 169, 7, 0x0070, 0x10, 7, 1),
	PIN_FIELD_BASE(170, 170, 7, 0x0070, 0x10, 8, 1),
	PIN_FIELD_BASE(171, 171, 7, 0x0070, 0x10, 9, 1),
	PIN_FIELD_BASE(172, 172, 7, 0x0070, 0x10, 13, 1),
	PIN_FIELD_BASE(173, 173, 7, 0x0070, 0x10, 14, 1),
	PIN_FIELD_BASE(174, 174, 7, 0x0070, 0x10, 12, 1),
	PIN_FIELD_BASE(175, 175, 7, 0x0070, 0x10, 15, 1),
	PIN_FIELD_BASE(176, 176, 7, 0x0070, 0x10, 10, 1),
	PIN_FIELD_BASE(177, 177, 7, 0x0070, 0x10, 11, 1),
	PIN_FIELD_BASE(178, 178, 7, 0x0070, 0x10, 16, 1),
	PIN_FIELD_BASE(179, 179, 7, 0x0070, 0x10, 17, 1),
	PIN_FIELD_BASE(180, 180, 7, 0x0070, 0x10, 18, 1),
	PIN_FIELD_BASE(181, 181, 7, 0x0070, 0x10, 19, 1),
	PIN_FIELD_BASE(182, 182, 7, 0x0070, 0x10, 20, 1),
	PIN_FIELD_BASE(195, 195, 5, 0x0040, 0x10, 16, 1),
	PIN_FIELD_BASE(196, 196, 5, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(197, 197, 5, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(198, 198, 5, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(199, 199, 5, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0040, 0x10, 15, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0040, 0x10, 17, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(206, 206, 5, 0x0040, 0x10, 18, 1),
	PIN_FIELD_BASE(207, 207, 5, 0x0040, 0x10, 19, 1),
	PIN_FIELD_BASE(208, 208, 5, 0x0040, 0x10, 20, 1),
	PIN_FIELD_BASE(209, 209, 5, 0x0040, 0x10, 12, 1),
	PIN_FIELD_BASE(210, 210, 5, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(211, 211, 5, 0x0040, 0x10, 13, 1),
	PIN_FIELD_BASE(212, 212, 5, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(213, 213, 5, 0x0040, 0x10, 14, 1),
	PIN_FIELD_BASE(214, 214, 5, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(215, 215, 5, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(216, 216, 5, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(217, 217, 5, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(218, 218, 5, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(219, 219, 5, 0x0040, 0x10, 2, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_drv_range[] = {
	PIN_FIELD_BASE(0, 0, 4, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(1, 1, 4, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(2, 2, 4, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(3, 3, 4, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(4, 4, 4, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(5, 5, 4, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(6, 6, 4, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(7, 7, 4, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(8, 8, 4, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(9, 9, 4, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(10, 10, 6, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(11, 11, 6, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(12, 12, 6, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(13, 13, 6, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(14, 14, 6, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(15, 15, 6, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(16, 16, 8, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(17, 17, 8, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(18, 18, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(19, 19, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(20, 20, 7, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(21, 21, 7, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(22, 22, 2, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(23, 23, 2, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(24, 24, 2, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(25, 25, 2, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(26, 26, 3, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(27, 27, 3, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(28, 28, 3, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(29, 29, 3, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(30, 30, 3, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(31, 31, 3, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(32, 32, 3, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(33, 33, 3, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(34, 34, 3, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(35, 35, 3, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(36, 36, 2, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(37, 37, 2, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(38, 38, 2, 0x0010, 0x10, 27, 3),
	PIN_FIELD_BASE(39, 39, 2, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(40, 40, 8, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(41, 41, 8, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(42, 42, 8, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(43, 43, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(44, 44, 7, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(45, 45, 1, 0x0010, 0x10, 6, 2),
	PIN_FIELD_BASE(46, 46, 1, 0x0010, 0x10, 6, 2),
	PIN_FIELD_BASE(47, 47, 1, 0x0010, 0x10, 6, 2),
	PIN_FIELD_BASE(48, 48, 1, 0x0010, 0x10, 8, 2),
	PIN_FIELD_BASE(49, 49, 1, 0x0010, 0x10, 8, 2),
	PIN_FIELD_BASE(50, 50, 1, 0x0010, 0x10, 8, 2),
	PIN_FIELD_BASE(51, 51, 1, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(52, 52, 1, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(53, 53, 1, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(54, 54, 1, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(55, 55, 1, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(56, 56, 1, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(57, 57, 3, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(58, 58, 3, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(59, 59, 3, 0x0010, 0x10, 27, 3),
	PIN_FIELD_BASE(60, 60, 3, 0x0010, 0x10, 27, 3),
	PIN_FIELD_BASE(61, 61, 3, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(62, 62, 3, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(63, 63, 3, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(64, 64, 3, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(65, 65, 3, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(66, 66, 3, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(67, 67, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(68, 68, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(69, 69, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(70, 70, 3, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(71, 71, 3, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(72, 72, 3, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(73, 73, 3, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(74, 74, 3, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(75, 75, 3, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(76, 76, 3, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(77, 77, 3, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(78, 78, 3, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(79, 79, 3, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(80, 80, 3, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(81, 81, 3, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(82, 82, 3, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(83, 83, 3, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(84, 84, 3, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(85, 85, 3, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(86, 86, 3, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(87, 87, 3, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(88, 88, 3, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(89, 89, 2, 0x0020, 0x10, 3, 3),
	PIN_FIELD_BASE(90, 90, 2, 0x0020, 0x10, 6, 3),
	PIN_FIELD_BASE(91, 91, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(92, 92, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(93, 93, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(94, 94, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(95, 95, 2, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(96, 96, 2, 0x0020, 0x10, 24, 3),
	PIN_FIELD_BASE(97, 97, 2, 0x0020, 0x10, 9, 3),
	PIN_FIELD_BASE(98, 98, 2, 0x0020, 0x10, 27, 3),
	PIN_FIELD_BASE(99, 99, 2, 0x0020, 0x10, 12, 3),
	PIN_FIELD_BASE(100, 100, 2, 0x0020, 0x10, 15, 3),
	PIN_FIELD_BASE(101, 101, 2, 0x0020, 0x10, 18, 3),
	PIN_FIELD_BASE(102, 102, 2, 0x0020, 0x10, 21, 3),
	PIN_FIELD_BASE(103, 103, 2, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(104, 104, 2, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(105, 105, 2, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(106, 106, 2, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(107, 107, 2, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(108, 108, 2, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(109, 109, 2, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(110, 110, 2, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(111, 111, 2, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(112, 112, 2, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(113, 113, 2, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(114, 114, 2, 0x0010, 0x10, 15, 3),
	PIN_FIELD_BASE(115, 115, 2, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(116, 116, 2, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(117, 117, 2, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(118, 118, 4, 0x0020, 0x10, 3, 3),
	PIN_FIELD_BASE(119, 119, 4, 0x0020, 0x10, 21, 3),
	PIN_FIELD_BASE(120, 120, 4, 0x0020, 0x10, 18, 3),
	PIN_FIELD_BASE(121, 121, 4, 0x0030, 0x10, 6, 3),
	PIN_FIELD_BASE(122, 122, 4, 0x0020, 0x10, 15, 3),
	PIN_FIELD_BASE(123, 123, 4, 0x0030, 0x10, 3, 3),
	PIN_FIELD_BASE(124, 124, 4, 0x0020, 0x10, 12, 3),
	PIN_FIELD_BASE(125, 125, 4, 0x0030, 0x10, 0, 3),
	PIN_FIELD_BASE(126, 126, 4, 0x0010, 0x10, 18, 3),
	PIN_FIELD_BASE(127, 127, 4, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(128, 128, 4, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(129, 129, 4, 0x0010, 0x10, 27, 3),
	PIN_FIELD_BASE(130, 130, 4, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(131, 131, 4, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(132, 132, 4, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(133, 133, 4, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(134, 134, 4, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(135, 135, 4, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(136, 136, 4, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(137, 137, 4, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(138, 138, 4, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(139, 139, 4, 0x0020, 0x10, 9, 3),
	PIN_FIELD_BASE(140, 140, 4, 0x0020, 0x10, 27, 3),
	PIN_FIELD_BASE(141, 141, 4, 0x0020, 0x10, 6, 3),
	PIN_FIELD_BASE(142, 142, 4, 0x0020, 0x10, 24, 3),
	PIN_FIELD_BASE(143, 143, 1, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(144, 144, 1, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(145, 145, 1, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(146, 146, 1, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(147, 147, 1, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(148, 148, 1, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(149, 149, 1, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(150, 150, 1, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(151, 151, 1, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(152, 152, 7, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(153, 153, 7, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(154, 154, 7, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(155, 155, 7, 0x0010, 0x10, 21, 3),
	PIN_FIELD_BASE(156, 156, 7, 0x0020, 0x10, 3, 3),
	PIN_FIELD_BASE(157, 157, 7, 0x0020, 0x10, 6, 3),
	PIN_FIELD_BASE(158, 158, 7, 0x0020, 0x10, 9, 3),
	PIN_FIELD_BASE(159, 159, 7, 0x0020, 0x10, 12, 3),
	PIN_FIELD_BASE(160, 160, 7, 0x0010, 0x10, 27, 3),
	PIN_FIELD_BASE(161, 161, 7, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(162, 162, 7, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(163, 163, 7, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(164, 164, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(165, 165, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(166, 166, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(167, 167, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(168, 168, 7, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(169, 169, 7, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(170, 170, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(171, 171, 7, 0x0010, 0x10, 24, 3),
	PIN_FIELD_BASE(172, 172, 7, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(173, 173, 7, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(174, 174, 7, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(175, 175, 7, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(176, 176, 7, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(177, 177, 7, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(178, 178, 7, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(179, 179, 7, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(180, 180, 7, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(181, 181, 7, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(182, 182, 7, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(183, 183, 9, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(184, 184, 9, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(185, 185, 9, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(186, 186, 9, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(187, 187, 9, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(188, 188, 9, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(189, 189, 9, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(190, 190, 9, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(191, 191, 9, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(192, 192, 9, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(193, 193, 9, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(194, 194, 9, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(195, 195, 5, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(196, 196, 5, 0x0000, 0x10, 18, 3),
	PIN_FIELD_BASE(197, 197, 5, 0x0000, 0x10, 24, 3),
	PIN_FIELD_BASE(198, 198, 5, 0x0000, 0x10, 21, 3),
	PIN_FIELD_BASE(199, 199, 5, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(200, 200, 8, 0x0000, 0x10, 9, 3),
	PIN_FIELD_BASE(201, 201, 8, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(202, 202, 5, 0x0010, 0x10, 6, 3),
	PIN_FIELD_BASE(203, 203, 5, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(204, 204, 8, 0x0000, 0x10, 6, 3),
	PIN_FIELD_BASE(205, 205, 8, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(206, 206, 5, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(207, 207, 5, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(208, 208, 5, 0x0010, 0x10, 12, 3),
	PIN_FIELD_BASE(209, 209, 5, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(210, 210, 5, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(211, 211, 5, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(212, 212, 5, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(213, 213, 5, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(214, 214, 5, 0x0000, 0x10, 0, 3),
	PIN_FIELD_BASE(215, 215, 5, 0x0000, 0x10, 27, 3),
	PIN_FIELD_BASE(216, 216, 5, 0x0000, 0x10, 12, 3),
	PIN_FIELD_BASE(217, 217, 5, 0x0000, 0x10, 15, 3),
	PIN_FIELD_BASE(218, 218, 5, 0x0000, 0x10, 3, 3),
	PIN_FIELD_BASE(219, 219, 5, 0x0000, 0x10, 6, 3),
};

static const struct mtk_pin_field_calc mt6873_pin_pupd_range[] = {
	PIN_FIELD_BASE(10, 10, 6, 0x0020, 0x10, 0, 1),
	PIN_FIELD_BASE(11, 11, 6, 0x0020, 0x10, 1, 1),
	PIN_FIELD_BASE(12, 12, 6, 0x0020, 0x10, 2, 1),
	PIN_FIELD_BASE(13, 13, 6, 0x0020, 0x10, 3, 1),
	PIN_FIELD_BASE(14, 14, 6, 0x0020, 0x10, 4, 1),
	PIN_FIELD_BASE(15, 15, 6, 0x0020, 0x10, 5, 1),
	PIN_FIELD_BASE(45, 45, 1, 0x0060, 0x10, 9, 1),
	PIN_FIELD_BASE(46, 46, 1, 0x0060, 0x10, 11, 1),
	PIN_FIELD_BASE(47, 47, 1, 0x0060, 0x10, 10, 1),
	PIN_FIELD_BASE(48, 48, 1, 0x0060, 0x10, 7, 1),
	PIN_FIELD_BASE(49, 49, 1, 0x0060, 0x10, 8, 1),
	PIN_FIELD_BASE(50, 50, 1, 0x0060, 0x10, 6, 1),
	PIN_FIELD_BASE(51, 51, 1, 0x0060, 0x10, 0, 1),
	PIN_FIELD_BASE(52, 52, 1, 0x0060, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 1, 0x0060, 0x10, 5, 1),
	PIN_FIELD_BASE(54, 54, 1, 0x0060, 0x10, 2, 1),
	PIN_FIELD_BASE(55, 55, 1, 0x0060, 0x10, 4, 1),
	PIN_FIELD_BASE(56, 56, 1, 0x0060, 0x10, 3, 1),
	PIN_FIELD_BASE(152, 152, 7, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(153, 153, 7, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(154, 154, 7, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(155, 155, 7, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(183, 183, 9, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(184, 184, 9, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(185, 185, 9, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(186, 186, 9, 0x0030, 0x10, 6, 1),
	PIN_FIELD_BASE(187, 187, 9, 0x0030, 0x10, 8, 1),
	PIN_FIELD_BASE(188, 188, 9, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(189, 189, 9, 0x0030, 0x10, 7, 1),
	PIN_FIELD_BASE(190, 190, 9, 0x0030, 0x10, 9, 1),
	PIN_FIELD_BASE(191, 191, 9, 0x0030, 0x10, 10, 1),
	PIN_FIELD_BASE(192, 192, 9, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(193, 193, 9, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(194, 194, 9, 0x0030, 0x10, 11, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_r0_range[] = {
	PIN_FIELD_BASE(10, 10, 6, 0x0030, 0x10, 0, 1),
	PIN_FIELD_BASE(11, 11, 6, 0x0030, 0x10, 1, 1),
	PIN_FIELD_BASE(12, 12, 6, 0x0030, 0x10, 2, 1),
	PIN_FIELD_BASE(13, 13, 6, 0x0030, 0x10, 3, 1),
	PIN_FIELD_BASE(14, 14, 6, 0x0030, 0x10, 4, 1),
	PIN_FIELD_BASE(15, 15, 6, 0x0030, 0x10, 5, 1),
	PIN_FIELD_BASE(45, 45, 1, 0x0080, 0x10, 9, 1),
	PIN_FIELD_BASE(46, 46, 1, 0x0080, 0x10, 11, 1),
	PIN_FIELD_BASE(47, 47, 1, 0x0080, 0x10, 10, 1),
	PIN_FIELD_BASE(48, 48, 1, 0x0080, 0x10, 7, 1),
	PIN_FIELD_BASE(49, 49, 1, 0x0080, 0x10, 8, 1),
	PIN_FIELD_BASE(50, 50, 1, 0x0080, 0x10, 6, 1),
	PIN_FIELD_BASE(51, 51, 1, 0x0080, 0x10, 0, 1),
	PIN_FIELD_BASE(52, 52, 1, 0x0080, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 1, 0x0080, 0x10, 5, 1),
	PIN_FIELD_BASE(54, 54, 1, 0x0080, 0x10, 2, 1),
	PIN_FIELD_BASE(55, 55, 1, 0x0080, 0x10, 4, 1),
	PIN_FIELD_BASE(56, 56, 1, 0x0080, 0x10, 3, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x00e0, 0x10, 0, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x00e0, 0x10, 12, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x00e0, 0x10, 10, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x00e0, 0x10, 22, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x00e0, 0x10, 8, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x00e0, 0x10, 20, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x00e0, 0x10, 6, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x00e0, 0x10, 18, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x00e0, 0x10, 4, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x00e0, 0x10, 16, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x00e0, 0x10, 2, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x00e0, 0x10, 14, 1),
	PIN_FIELD_BASE(152, 152, 7, 0x00c0, 0x10, 3, 1),
	PIN_FIELD_BASE(153, 153, 7, 0x00c0, 0x10, 2, 1),
	PIN_FIELD_BASE(154, 154, 7, 0x00c0, 0x10, 0, 1),
	PIN_FIELD_BASE(155, 155, 7, 0x00c0, 0x10, 1, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x00f0, 0x10, 0, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x00f0, 0x10, 2, 1),
	PIN_FIELD_BASE(183, 183, 9, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(184, 184, 9, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(185, 185, 9, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(186, 186, 9, 0x0040, 0x10, 6, 1),
	PIN_FIELD_BASE(187, 187, 9, 0x0040, 0x10, 8, 1),
	PIN_FIELD_BASE(188, 188, 9, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(189, 189, 9, 0x0040, 0x10, 7, 1),
	PIN_FIELD_BASE(190, 190, 9, 0x0040, 0x10, 9, 1),
	PIN_FIELD_BASE(191, 191, 9, 0x0040, 0x10, 10, 1),
	PIN_FIELD_BASE(192, 192, 9, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(193, 193, 9, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(194, 194, 9, 0x0040, 0x10, 11, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0030, 0x70, 2, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0030, 0x70, 6, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0030, 0x70, 0, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0030, 0x70, 2, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0020, 0x70, 0, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0020, 0x70, 4, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_r1_range[] = {
	PIN_FIELD_BASE(10, 10, 6, 0x0040, 0x10, 0, 1),
	PIN_FIELD_BASE(11, 11, 6, 0x0040, 0x10, 1, 1),
	PIN_FIELD_BASE(12, 12, 6, 0x0040, 0x10, 2, 1),
	PIN_FIELD_BASE(13, 13, 6, 0x0040, 0x10, 3, 1),
	PIN_FIELD_BASE(14, 14, 6, 0x0040, 0x10, 4, 1),
	PIN_FIELD_BASE(15, 15, 6, 0x0040, 0x10, 5, 1),
	PIN_FIELD_BASE(45, 45, 1, 0x0090, 0x10, 9, 1),
	PIN_FIELD_BASE(46, 46, 1, 0x0090, 0x10, 11, 1),
	PIN_FIELD_BASE(47, 47, 1, 0x0090, 0x10, 10, 1),
	PIN_FIELD_BASE(48, 48, 1, 0x0090, 0x10, 7, 1),
	PIN_FIELD_BASE(49, 49, 1, 0x0090, 0x10, 8, 1),
	PIN_FIELD_BASE(50, 50, 1, 0x0090, 0x10, 6, 1),
	PIN_FIELD_BASE(51, 51, 1, 0x0090, 0x10, 0, 1),
	PIN_FIELD_BASE(52, 52, 1, 0x0090, 0x10, 1, 1),
	PIN_FIELD_BASE(53, 53, 1, 0x0090, 0x10, 5, 1),
	PIN_FIELD_BASE(54, 54, 1, 0x0090, 0x10, 2, 1),
	PIN_FIELD_BASE(55, 55, 1, 0x0090, 0x10, 4, 1),
	PIN_FIELD_BASE(56, 56, 1, 0x0090, 0x10, 3, 1),
	PIN_FIELD_BASE(118, 118, 4, 0x00e0, 0x10, 1, 1),
	PIN_FIELD_BASE(119, 119, 4, 0x00e0, 0x10, 13, 1),
	PIN_FIELD_BASE(120, 120, 4, 0x00e0, 0x10, 11, 1),
	PIN_FIELD_BASE(121, 121, 4, 0x00e0, 0x10, 23, 1),
	PIN_FIELD_BASE(122, 122, 4, 0x00e0, 0x10, 9, 1),
	PIN_FIELD_BASE(123, 123, 4, 0x00e0, 0x10, 21, 1),
	PIN_FIELD_BASE(124, 124, 4, 0x00e0, 0x10, 7, 1),
	PIN_FIELD_BASE(125, 125, 4, 0x00e0, 0x10, 19, 1),
	PIN_FIELD_BASE(139, 139, 4, 0x00e0, 0x10, 5, 1),
	PIN_FIELD_BASE(140, 140, 4, 0x00e0, 0x10, 17, 1),
	PIN_FIELD_BASE(141, 141, 4, 0x00e0, 0x10, 3, 1),
	PIN_FIELD_BASE(142, 142, 4, 0x00e0, 0x10, 15, 1),
	PIN_FIELD_BASE(152, 152, 7, 0x00d0, 0x10, 3, 1),
	PIN_FIELD_BASE(153, 153, 7, 0x00d0, 0x10, 2, 1),
	PIN_FIELD_BASE(154, 154, 7, 0x00d0, 0x10, 0, 1),
	PIN_FIELD_BASE(155, 155, 7, 0x00d0, 0x10, 1, 1),
	PIN_FIELD_BASE(160, 160, 7, 0x00f0, 0x10, 1, 1),
	PIN_FIELD_BASE(161, 161, 7, 0x00f0, 0x10, 3, 1),
	PIN_FIELD_BASE(183, 183, 9, 0x0050, 0x10, 1, 1),
	PIN_FIELD_BASE(184, 184, 9, 0x0050, 0x10, 2, 1),
	PIN_FIELD_BASE(185, 185, 9, 0x0050, 0x10, 4, 1),
	PIN_FIELD_BASE(186, 186, 9, 0x0050, 0x10, 6, 1),
	PIN_FIELD_BASE(187, 187, 9, 0x0050, 0x10, 8, 1),
	PIN_FIELD_BASE(188, 188, 9, 0x0050, 0x10, 3, 1),
	PIN_FIELD_BASE(189, 189, 9, 0x0050, 0x10, 7, 1),
	PIN_FIELD_BASE(190, 190, 9, 0x0050, 0x10, 9, 1),
	PIN_FIELD_BASE(191, 191, 9, 0x0050, 0x10, 10, 1),
	PIN_FIELD_BASE(192, 192, 9, 0x0050, 0x10, 0, 1),
	PIN_FIELD_BASE(193, 193, 9, 0x0050, 0x10, 5, 1),
	PIN_FIELD_BASE(194, 194, 9, 0x0050, 0x10, 11, 1),
	PIN_FIELD_BASE(200, 200, 8, 0x0030, 0x70, 3, 1),
	PIN_FIELD_BASE(201, 201, 8, 0x0030, 0x70, 7, 1),
	PIN_FIELD_BASE(202, 202, 5, 0x0030, 0x70, 1, 1),
	PIN_FIELD_BASE(203, 203, 5, 0x0030, 0x70, 3, 1),
	PIN_FIELD_BASE(204, 204, 8, 0x0020, 0x70, 1, 1),
	PIN_FIELD_BASE(205, 205, 8, 0x0020, 0x70, 5, 1),
};

static const struct mtk_pin_field_calc mt6873_pin_eh_range[] = {
	PIN_FIELD_BASE(89, 89, 2, 0x0040, 0x10, 0, 5),
	PIN_FIELD_BASE(90, 90, 2, 0x0040, 0x10, 5, 5),
	PIN_FIELD_BASE(118, 118, 4, 0x0040, 0x10, 0, 3),
	PIN_FIELD_BASE(119, 119, 4, 0x0040, 0x10, 18, 3),
	PIN_FIELD_BASE(120, 120, 4, 0x0040, 0x10, 15, 3),
	PIN_FIELD_BASE(121, 121, 4, 0x0050, 0x10, 3, 3),
	PIN_FIELD_BASE(122, 122, 4, 0x0040, 0x10, 12, 3),
	PIN_FIELD_BASE(123, 123, 4, 0x0050, 0x10, 0, 3),
	PIN_FIELD_BASE(124, 124, 4, 0x0040, 0x10, 9, 3),
	PIN_FIELD_BASE(125, 125, 4, 0x0040, 0x10, 27, 3),
	PIN_FIELD_BASE(139, 139, 4, 0x0040, 0x10, 6, 3),
	PIN_FIELD_BASE(140, 140, 4, 0x0040, 0x10, 24, 3),
	PIN_FIELD_BASE(141, 141, 4, 0x0040, 0x10, 3, 3),
	PIN_FIELD_BASE(142, 142, 4, 0x0040, 0x10, 21, 3),
	PIN_FIELD_BASE(160, 160, 7, 0x0030, 0x10, 0, 3),
	PIN_FIELD_BASE(161, 161, 7, 0x0030, 0x10, 3, 3),
	PIN_FIELD_BASE(200, 200, 8, 0x0010, 0x10, 3, 3),
	PIN_FIELD_BASE(201, 201, 8, 0x0010, 0x10, 9, 3),
	PIN_FIELD_BASE(202, 202, 5, 0x0020, 0x10, 0, 3),
	PIN_FIELD_BASE(203, 203, 5, 0x0020, 0x10, 3, 3),
	PIN_FIELD_BASE(204, 204, 8, 0x0010, 0x10, 0, 3),
	PIN_FIELD_BASE(205, 205, 8, 0x0010, 0x10, 6, 3),
};

static const struct mtk_eh_pin_pinmux mt6873_eh_pin_pinmux_list[] = {
	/* pin number, pinmux number */
	{89, 2},
	{90, 2},
	{118, 1},
	{119, 1},
	{120, 1},
	{121, 1},
	{122, 1},
	{123, 1},
	{124, 1},
	{125, 1},
	{139, 1},
	{140, 1},
	{141, 1},
	{142, 1},
	{160, 1},
	{161, 1},
	{200, 1},
	{201, 1},
	{202, 1},
	{203, 1},
	{204, 1},
	{205, 1},
	{0xffff, 0}, /* indicate end of array */
};

static const struct mtk_eint_hw mt6873_eint_hw = {
	.port_mask = 7,
	.ports     = 7,
	.ap_num    = 224,
	.db_cnt    = 32,
};


static const struct mtk_pin_reg_calc mt6873_reg_cals[PINCTRL_PIN_REG_MAX] = {
	[PINCTRL_PIN_REG_MODE] = MTK_RANGE(mt6873_pin_mode_range),
	[PINCTRL_PIN_REG_DIR] = MTK_RANGE(mt6873_pin_dir_range),
	[PINCTRL_PIN_REG_DI] = MTK_RANGE(mt6873_pin_di_range),
	[PINCTRL_PIN_REG_DO] = MTK_RANGE(mt6873_pin_do_range),
	[PINCTRL_PIN_REG_SR] = MTK_RANGE(mt6873_pin_dir_range),
	[PINCTRL_PIN_REG_SMT] = MTK_RANGE(mt6873_pin_smt_range),
	[PINCTRL_PIN_REG_IES] = MTK_RANGE(mt6873_pin_ies_range),
	[PINCTRL_PIN_REG_PU] = MTK_RANGE(mt6873_pin_pu_range),
	[PINCTRL_PIN_REG_PD] = MTK_RANGE(mt6873_pin_pd_range),
	[PINCTRL_PIN_REG_DRV] = MTK_RANGE(mt6873_pin_drv_range),
	[PINCTRL_PIN_REG_PUPD] = MTK_RANGE(mt6873_pin_pupd_range),
	[PINCTRL_PIN_REG_R0] = MTK_RANGE(mt6873_pin_r0_range),
	[PINCTRL_PIN_REG_R1] = MTK_RANGE(mt6873_pin_r1_range),
	[PINCTRL_PIN_REG_DRV_EH] = MTK_RANGE(mt6873_pin_eh_range),
};

static const struct mtk_pin_soc mt6873_data = {
	.reg_cal = mt6873_reg_cals,
	.pins = mtk_pins_mt6873,
	.npins = ARRAY_SIZE(mtk_pins_mt6873),
	.ngrps = ARRAY_SIZE(mtk_pins_mt6873),
	.eint_hw = &mt6873_eint_hw,
	.nfuncs = 8,
	.gpio_m = 0,
	.race_free_access = true,
	.eh_pin_pinmux = mt6873_eh_pin_pinmux_list,
	.bias_set_combo = mtk_pinconf_bias_set_combo,
	.bias_get_combo = mtk_pinconf_bias_get_combo,
	.drive_set = mtk_pinconf_drive_set_raw,
	.drive_get = mtk_pinconf_drive_get_raw,
	.adv_pull_get = mtk_pinconf_adv_pull_get,
	.adv_pull_set = mtk_pinconf_adv_pull_set,
	.adv_drive_get = mtk_pinconf_adv_drive_get,
	.adv_drive_set = mtk_pinconf_adv_drive_set,
};

static const struct of_device_id mt6873_pinctrl_of_match[] = {
	{ .compatible = "mediatek,mt6873-pinctrl", },
	{ }
};

static int mt6873_pinctrl_probe(struct platform_device *pdev)
{
	return mtk_paris_pinctrl_probe(pdev, &mt6873_data);
}

static struct platform_driver mt6873_pinctrl_driver = {
	.driver = {
		.name = "mt6873-pinctrl",
		.of_match_table = mt6873_pinctrl_of_match,
		.pm = &mtk_eint_pm_ops_v2,
	},
	.probe = mt6873_pinctrl_probe,
};

static int __init mt6873_pinctrl_init(void)
{
	return platform_driver_register(&mt6873_pinctrl_driver);
}
arch_initcall(mt6873_pinctrl_init);
