/*
* Copyright (C) 2016 MediaTek Inc.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See http://www.gnu.org/licenses/gpl-2.0.html for more details.
*/

#include <linux/init.h>		/* For init/exit macros */
#include <linux/module.h>	/* For MODULE_ marcros*/
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
#include <linux/platform_device.h>
#include <linux/device.h>
#include <linux/kdev_t.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/delay.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/wait.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/poll.h>
#include <linux/power_supply.h>
#include <linux/time.h>
#include <linux/mutex.h>
#include <linux/kthread.h>
#include <linux/proc_fs.h>
#include <linux/platform_device.h>
#include <linux/seq_file.h>
#include <linux/scatterlist.h>
#include <linux/suspend.h>
#include <linux/version.h>
#include <linux/i2c.h>
#include <linux/pinctrl/consumer.h>

#ifdef CONFIG_OF
#include <linux/of.h>
#include <linux/of_irq.h>
#include <linux/of_address.h>
#include <linux/of_device.h>
#endif
#include "psc5415e.h"

#include <charger_class.h>
#include <mtk_charger.h>
#include <linux/regulator/driver.h>
#include <linux/regulator/machine.h>
#ifdef CONFIG_T_PRODUCT_INFO
#include <dev_info.h>
#endif

#if (defined(CONFIG_PROJECT_U5020F) || defined(CONFIG_PROJECT_U5021F))
#include "cust_charger_init.h"
#endif

static const unsigned int CSTH[] = {//Rsense = 68 mohm
	482000, 578000, 771000, 867000,
	1060000, 1156000, 1349000, 1446000
};

/*psc5415e REG00 IINLIM[5:0]*/
static const unsigned int INPUT_CSTH[] = {
	100000, 500000, 800000, 1000000
};

/* psc5415e REG0A BOOST_LIM[2:0], mA */
static const unsigned int BOOST_CURRENT_LIMIT[] = {
	500, 750, 1200, 1400, 1650, 1875, 2150,
};

#ifdef CONFIG_OF
#else
#define PSC5415E_SLAVE_ADDR_WRITE	0xD4
#define PSC5415E_SLAVE_ADDR_Read	0xD5
#ifdef  I2C_SWITHING_CHARGER_CHANNEL
#define PSC5415E_BUSNUM I2C_SWITHING_CHARGER_CHANNEL
#else
#define PSC5415E_BUSNUM 1
#endif
#endif

static struct psc5415e_info {
	struct charger_device *chg_dev;
	struct power_supply *psy;
	struct charger_properties chg_props;
	struct device *dev;
	struct hrtimer otg_timer;
	struct workqueue_struct *otg_boost_workq;
	struct work_struct kick_work;
	unsigned int polling_interval;
	bool polling_enabled;
	const char *chg_dev_name;
	const char *eint_name;
	int irq;
	struct regulator_dev *otg_rdev;	

	struct pinctrl *pinctrl;
	struct pinctrl_state *pins_default;
	struct pinctrl_state *psc_chg_en_low;
	struct pinctrl_state *psc_chg_en_high;
} *g_psc5415e_info;

static struct i2c_client *new_client;
static const struct i2c_device_id psc5415e_i2c_id[] = { {"psc5415e", 1}, {} };

static void enable_boost_polling(bool poll_en);
static void usbotg_boost_kick_work(struct work_struct *work);
static enum hrtimer_restart usbotg_gtimer_func(struct hrtimer *timer);

static unsigned int charging_value_to_parameter(const unsigned int *parameter, const unsigned int array_size,
					const unsigned int val)
{
	if (val < array_size)
		return parameter[val];
	pr_err("Can't find the parameter\n");
	return parameter[0];
}

static unsigned int charging_parameter_to_value(const unsigned int *parameter, const unsigned int array_size,
					const unsigned int val)
{
	unsigned int i;

	pr_err_ratelimited("array_size = %d\n", array_size);

	for (i = 0; i < array_size; i++) {
		if (val == *(parameter + i))
			return i;
	}

	pr_err("NO register value match\n");
	/* TODO: ASSERT(0);	// not find the value */
	return 0;
}

static unsigned int bmt_find_closest_level(const unsigned int *pList, unsigned int number,
					 unsigned int level)
{
	unsigned int i;
	unsigned int max_value_in_last_element;

	if (pList[0] < pList[1])
		max_value_in_last_element = 1;
	else
		max_value_in_last_element = 0;

	if (max_value_in_last_element == 1) {
		for (i = (number - 1); i != 0; i--) {	/* max value in the last element */
			if (pList[i] <= level) {
				pr_err_ratelimited("zzf_%d<=%d, i=%d\n", pList[i], level, i);
				return pList[i];
			}
		}
		pr_err("Can't find closest level\n");
		return pList[0];
		/* return 000; */
	} else {
		for (i = 0; i < number; i++) {	/* max value in the first element */
			if (pList[i] <= level)
				return pList[i];
		}
		pr_err("Can't find closest level\n");
		return pList[number - 1];
		/* return 000; */
	}
}

unsigned char psc5415e_reg[PSC5415E_REG_NUM] = { 0 };
static DEFINE_MUTEX(psc5415e_i2c_access);
static DEFINE_MUTEX(psc5415e_access_lock);

static int psc5415e_read_byte(u8 reg_addr, u8 *rd_buf, int rd_len)
{
	int ret = 0;
	struct i2c_adapter *adap = new_client->adapter;
	struct i2c_msg msg[2];
	u8 *w_buf = NULL;
	u8 *r_buf = NULL;

	memset(msg, 0, 2 * sizeof(struct i2c_msg));

	w_buf = kzalloc(1, GFP_KERNEL);
	if (w_buf == NULL)
		return -1;
	r_buf = kzalloc(rd_len, GFP_KERNEL);
	if (r_buf == NULL)
		return -1;

	*w_buf = reg_addr;

	msg[0].addr = new_client->addr;
	msg[0].flags = 0;
	msg[0].len = 1;
	msg[0].buf = w_buf;

	msg[1].addr = new_client->addr;
	msg[1].flags = 1;
	msg[1].len = rd_len;
	msg[1].buf = r_buf;

	mutex_lock(&psc5415e_access_lock);
	ret = i2c_transfer(adap, msg, 2);
	if(ret < 0){
		chr_err("%s: read 0x%x register failed\n",__func__,reg_addr);
	}
	mutex_unlock(&psc5415e_access_lock);
	memcpy(rd_buf, r_buf, rd_len);

	kfree(w_buf);
	kfree(r_buf);
	return ret;
}

int psc5415e_write_byte(unsigned char reg_num, u8 *wr_buf, int wr_len)
{
	int ret = 0;
	struct i2c_adapter *adap = new_client->adapter;
	struct i2c_msg msg;
	u8 *w_buf = NULL;

	memset(&msg, 0, sizeof(struct i2c_msg));

	w_buf = kzalloc(wr_len, GFP_KERNEL);
	if (w_buf == NULL)
		return -1;

	w_buf[0] = reg_num;
	memcpy(w_buf + 1, wr_buf, wr_len);

	msg.addr = new_client->addr;
	msg.flags = 0;
	msg.len = wr_len;
	msg.buf = w_buf;

	mutex_lock(&psc5415e_access_lock);

	ret = i2c_transfer(adap, &msg, 1);

	mutex_unlock(&psc5415e_access_lock);

	kfree(w_buf);
	return ret;
}

unsigned int psc5415e_read_interface(unsigned char reg_num, unsigned char *val, unsigned char MASK,
				unsigned char SHIFT)
{
	unsigned char psc5415e_reg = 0;
	unsigned int ret = 0;

	ret = psc5415e_read_byte(reg_num, &psc5415e_reg, 1);
	pr_err_ratelimited("[psc5415e_read_interface] Reg[%x]=0x%x\n", reg_num, psc5415e_reg);
	psc5415e_reg &= (MASK << SHIFT);
	*val = (psc5415e_reg >> SHIFT);
	pr_err_ratelimited("[psc5415e_read_interface] val=0x%x\n", *val);

	return ret;
}

int psc5415e_config_interface(unsigned char reg_num, unsigned char val, unsigned char MASK,
					unsigned char SHIFT)
{
	unsigned char psc5415e_reg = 0;
	unsigned char psc5415e_reg_ori = 0;
	int ret = 0;

	ret = psc5415e_read_byte(reg_num, &psc5415e_reg, 1);
	if (ret < 0){
		return ret;
	}
	psc5415e_reg_ori = psc5415e_reg;
	psc5415e_reg &= ~(MASK << SHIFT);
	psc5415e_reg |= (val << SHIFT);
	if (reg_num == PSC5415E_CON4)
		psc5415e_reg &= ~(1 << CON4_RESET_SHIFT);

	ret = psc5415e_write_byte(reg_num, &psc5415e_reg, 2);
	pr_err_ratelimited("[psc5415e_config_interface] write Reg[%x]=0x%x from 0x%x\n", reg_num,
			psc5415e_reg, psc5415e_reg_ori);
	/* Check */
	/* psc5415e_read_byte(reg_num, &psc5415e_reg, 1); */
	/* printk("[psc5415e_config_interface] Check Reg[%x]=0x%x\n", reg_num, psc5415e_reg); */

	return ret;
}

/* write one register directly */
unsigned int psc5415e_reg_config_interface(unsigned char reg_num, unsigned char val)
{
	unsigned char psc5415e_reg = val;

	return psc5415e_write_byte(reg_num, &psc5415e_reg, 2);
}

void psc5415e_set_tmr_rst(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char)(val),
				(unsigned char)(CON0_TMR_RST_MASK),
				(unsigned char)(CON0_TMR_RST_SHIFT)
				);
}

unsigned int psc5415e_get_otg_status(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char *)(&val),
				(unsigned char)(CON0_OTG_MASK),
				(unsigned char)(CON0_OTG_SHIFT)
				);
	return val;
}

void psc5415e_set_en_stat(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char)(val),
				(unsigned char)(CON0_EN_STAT_MASK),
				(unsigned char)(CON0_EN_STAT_SHIFT)
				);
}

unsigned int psc5415e_get_chip_status(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char *)(&val),
				(unsigned char)(CON0_STAT_MASK),
				(unsigned char)(CON0_STAT_SHIFT)
				);
	return val;
}

unsigned int psc5415e_get_boost_status(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char *)(&val),
				(unsigned char)(CON0_BOOST_MASK),
				(unsigned char)(CON0_BOOST_SHIFT)
				);
	return val;

}

unsigned int psc5415e_get_fault_status(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON0),
				(unsigned char *)(&val),
				(unsigned char)(CON0_FAULT_MASK),
				(unsigned char)(CON0_FAULT_SHIFT)
				);
	return val;
}

void psc5415e_set_input_charging_current(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_LIN_LIMIT_MASK),
				(unsigned char)(CON1_LIN_LIMIT_SHIFT)
				);
}

unsigned int psc5415e_get_input_charging_current(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char *)(&val),
				(unsigned char)(CON1_LIN_LIMIT_MASK),
				(unsigned char)(CON1_LIN_LIMIT_SHIFT)
				);

	return val;
}

void psc5415e_set_v_low(unsigned int val)
{

	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_LOW_V_MASK),
				(unsigned char)(CON1_LOW_V_SHIFT)
				);
}

void psc5415e_set_te(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_TE_MASK),
				(unsigned char)(CON1_TE_SHIFT)
				);
}

void psc5415e_set_ce(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_CE_MASK),
				(unsigned char)(CON1_CE_SHIFT)
				);
}

unsigned int psc5415e_get_ce(void)
{
	unsigned int ret = 0;
	unsigned int val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char *)(&val),
				(unsigned char)(CON1_CE_MASK),
				(unsigned char)(CON1_CE_SHIFT)
				);
	return val;
}

void psc5415e_set_hz_mode(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_HZ_MODE_MASK),
				(unsigned char)(CON1_HZ_MODE_SHIFT)
				);
}

void psc5415e_set_opa_mode(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON1),
				(unsigned char)(val),
				(unsigned char)(CON1_OPA_MODE_MASK),
				(unsigned char)(CON1_OPA_MODE_SHIFT)
				);
}

void psc5415e_set_oreg(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON2),
				(unsigned char)(val),
				(unsigned char)(CON2_OREG_MASK),
				(unsigned char)(CON2_OREG_SHIFT)
				);
}
void psc5415e_set_otg_pl(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON2),
				(unsigned char)(val),
				(unsigned char)(CON2_OTG_PL_MASK),
				(unsigned char)(CON2_OTG_PL_SHIFT)
				);
}
void psc5415e_set_otg_en(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON2),
				(unsigned char)(val),
				(unsigned char)(CON2_OTG_EN_MASK),
				(unsigned char)(CON2_OTG_EN_SHIFT)
				);
}

unsigned int psc5415e_get_vender_code(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON3),
				(unsigned char *)(&val),
				(unsigned char)(CON3_VENDER_CODE_MASK),
				(unsigned char)(CON3_VENDER_CODE_SHIFT)
				);
	return val;
}
unsigned int psc5415e_get_pn(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON3),
				(unsigned char *)(&val),
				(unsigned char)(CON3_PIN_MASK),
				(unsigned char)(CON3_PIN_SHIFT)
				);
	return val;
}

unsigned int psc5415e_get_revision(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON3),
				(unsigned char *)(&val),
				(unsigned char)(CON3_REVISION_MASK),
				(unsigned char)(CON3_REVISION_SHIFT)
				);
	return val;
}

void psc5415e_set_reset(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON4),
				(unsigned char)(val),
				(unsigned char)(CON4_RESET_MASK),
				(unsigned char)(CON4_RESET_SHIFT)
				);
}

void psc5415e_set_iocharge(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON4),
				(unsigned char)(val),
				(unsigned char)(CON4_I_CHR_MASK),
				(unsigned char)(CON4_I_CHR_SHIFT)
				);
}

void psc5415e_set_iterm(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON4),
				(unsigned char)(val),
				(unsigned char)(CON4_I_TERM_MASK),
				(unsigned char)(CON4_I_TERM_SHIFT)
				);
}

void psc5415e_set_dis_vreg(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char)(val),
				(unsigned char)(CON5_DIS_VREG_MASK),
				(unsigned char)(CON5_DIS_VREG_SHIFT)
				);
}

void psc5415e_set_add20mV_vreg(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char)(val),
				(unsigned char)(CON5_ADD20MV_VREG_MASK),
				(unsigned char)(CON5_ADD20MV_VREG_SHIFT)
				);
}

void psc5415e_set_io_level(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char)(val),
				(unsigned char)(CON5_IO_LEVEL_MASK),
				(unsigned char)(CON5_IO_LEVEL_SHIFT)
				);
}

unsigned int psc5415e_get_sp_status(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char *)(&val),
				(unsigned char)(CON5_SP_STATUS_MASK),
				(unsigned char)(CON5_SP_STATUS_SHIFT)
				);
	return val;
}

unsigned int psc5415e_get_en_level(void)
{
	unsigned char val = 0;

	psc5415e_read_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char *)(&val),
				(unsigned char)(CON5_EN_LEVEL_MASK),
				(unsigned char)(CON5_EN_LEVEL_SHIFT)
				);
	return val;
}

void psc5415e_set_vsp(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON5),
				(unsigned char)(val),
				(unsigned char)(CON5_VSP_MASK),
				(unsigned char)(CON5_VSP_SHIFT)
				);
}

void psc5415e_set_i_safe(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON6),
				(unsigned char)(val),
				(unsigned char)(CON6_ISAFE_MASK),
				(unsigned char)(CON6_ISAFE_SHIFT)
				);
}

void psc5415e_set_v_safe(unsigned int val)
{
	psc5415e_config_interface((unsigned char)(PSC5415E_CON6),
				(unsigned char)(val),
				(unsigned char)(CON6_VSAFE_MASK),
				(unsigned char)(CON6_VSAFE_SHIFT)
				);
}

static int psc5415e_dump_register(struct charger_device *chg_dev)
{
	int i;

	for (i = 0; i < PSC5415E_REG_NUM; i++) {
		psc5415e_read_byte(i, &psc5415e_reg[i], 1);
		pr_err("psc5415e cz [0x%x]=0x%x ", i, psc5415e_reg[i]);
	}
	pr_err("\n");

	return 0;
}


static int psc5415e_parse_dt(struct psc5415e_info *info, struct device *dev)
{
	struct device_node *np = dev->of_node;
	int ret = -1;

	pr_err("%s\n", __func__);

	if (!np) {
		pr_err("%s: no of node\n", __func__);
		return -ENODEV;
	}

	if (of_property_read_string(np, "charger_name", &info->chg_dev_name) < 0) {
		info->chg_dev_name = "primary_chg";
		pr_err("%s: no charger name\n", __func__);
	}

	if (of_property_read_string(np, "alias_name", &(info->chg_props.alias_name)) < 0) {
		info->chg_props.alias_name = "psc5415e";
		pr_err("%s: no alias name\n", __func__);
	}

	info->pinctrl = devm_pinctrl_get(dev);
	if (IS_ERR(info->pinctrl)) {
		ret = PTR_ERR(info->pinctrl);
		pr_err("Cannot find psc5415e info->pinctrl! ret=%d\n", ret);
		goto parse_err;
	}

	info->pins_default = pinctrl_lookup_state(info->pinctrl, "default");
	if (IS_ERR(info->pins_default)) {
		ret = PTR_ERR(info->pins_default);
		pr_err("psc Cannot find accdet pinctrl default!\n");
	}

	info->psc_chg_en_high = pinctrl_lookup_state(info->pinctrl,"psc_chg_en_high");
	if (IS_ERR(info->psc_chg_en_high))
	{
		ret = PTR_ERR(info->psc_chg_en_high);
		pr_err("info->psc_chg_en_high ret = %d\n",ret);
		goto parse_err;
	}

	info->psc_chg_en_low = pinctrl_lookup_state(info->pinctrl,"psc_chg_en_low");
	if (IS_ERR(info->psc_chg_en_low))
	{
		ret = PTR_ERR(info->psc_chg_en_low);
		pr_err("info->psc_chg_en_low ret = %d\n",ret);
		goto parse_err;
	}


	return 0;
	
	parse_err:
		pr_err("parse dts failed!\n");

	return ret;
}

static int psc5415e_do_event(struct charger_device *chg_dev, unsigned int event, unsigned int args)
{
	struct psc5415e_info *info = charger_get_data(chg_dev);
	struct power_supply *chg_psy = NULL;
	
	if (chg_dev == NULL)
		return -EINVAL;

	chg_psy = devm_power_supply_get_by_phandle(info->dev,"charger");

	if (!chg_psy) {
		dev_notice(info->dev, "%s: cannot get psy\n", __func__);
		return -ENODEV;
	}

	switch (event) {
		case EVENT_FULL:
		case EVENT_RECHARGE:
		case EVENT_DISCHARGE:
			power_supply_changed(chg_psy);
			break;
		default:
			break;
	}
	return 0;
}

static int psc5415e_enable_charging(struct charger_device *chg_dev, bool en)
{
	unsigned int status = 0;
	struct psc5415e_info *info = g_psc5415e_info;

	if (true == en) {
		psc5415e_set_ce(0);
		//psc5415e_set_hz_mode(0);
		//psc5415e_set_opa_mode(0);
		//enable charging: psc_chg_en_low
		pinctrl_select_state(info->pinctrl, info->psc_chg_en_low);
		pr_err("psc5415e calm enable charging\n");
	} else {
		psc5415e_set_ce(1);
		//disable charging: psc_chg_en_high
		pinctrl_select_state(info->pinctrl, info->psc_chg_en_high);
		pr_err("psc5415e calm disable charging\n");
	}

	return status;
}

static int psc5415e_set_cv_voltage(struct charger_device *chg_dev, u32 cv)
{
	unsigned int set_cv_voltage;
	unsigned short int register_value;
	
	if(cv < 4200000){
		set_cv_voltage = 4100000;
		register_value = 0x0;
	}else if(cv < 4350000){
		set_cv_voltage = 4200000;
		register_value = 0x2;
	}else if(cv < 4400000){
		set_cv_voltage = 4350000;
		register_value = 0x24;
	}else{
		set_cv_voltage = 4400000;
		register_value = 0x2d;
	}

#if defined(TINNO_BUILD_GCF)
	set_cv_voltage = 4350000;
	register_value = 0x24;
	psc5415e_set_add20mV_vreg(1);//4.35 + 0.02 =4.37mV
 	pr_err("TINNO_BUILD_GCF:charging_set_cv_voltage register_value=0x%x %d %d \n",
	 register_value, cv, set_cv_voltage + 20000);	
#else
	pr_err("charging_set_cv_voltage register_value=0x%x %d %d\n",
	 register_value, cv, set_cv_voltage);
#endif

	psc5415e_set_oreg(register_value);

	return 0;
}

static int psc5415e_get_current(struct charger_device *chg_dev, u32 *ichg)
{
	int status = 0;
	unsigned int array_size;
	unsigned char reg_value;

	array_size = ARRAY_SIZE(CSTH);
	psc5415e_read_interface(0x1, &reg_value, 0x3, 0x6);	/* IINLIM */
	*ichg = charging_value_to_parameter(CSTH, array_size, reg_value);

	return status;
}

static int psc5415e_set_current(struct charger_device *chg_dev, u32 current_value)
{
	unsigned int status = 0;
	unsigned int set_chr_current;
	unsigned int array_size;
	unsigned int register_value;

	if (current_value <= 350000) {
		psc5415e_set_io_level(1);
	} else {
		psc5415e_set_io_level(0);
		array_size = ARRAY_SIZE(CSTH);
		set_chr_current = bmt_find_closest_level(CSTH, array_size, current_value);
		register_value = charging_parameter_to_value(CSTH, array_size, set_chr_current);
		psc5415e_set_iocharge(register_value);
	}
	psc5415e_set_iterm(0x06);
	psc5415e_set_te(0x01);

	return status;
}


static int psc5415e_get_input_current(struct charger_device *chg_dev, u32 *aicr)
{
	unsigned int status = 0;
	unsigned int array_size;
	unsigned int register_value;

	array_size = ARRAY_SIZE(INPUT_CSTH);
	register_value = psc5415e_get_input_charging_current();
	*aicr = charging_parameter_to_value(INPUT_CSTH, array_size, register_value);

	return status;
}

static int psc5415e_set_input_current(struct charger_device *chg_dev, u32 current_value)
{
	unsigned int status = 0;
	unsigned int set_chr_current;
	unsigned int array_size;
	unsigned int register_value;

	if (current_value > 500000) {
		register_value = 0x3;
	} else {
		array_size = ARRAY_SIZE(INPUT_CSTH);
		set_chr_current = bmt_find_closest_level(INPUT_CSTH, array_size, current_value);
		register_value =
	 charging_parameter_to_value(INPUT_CSTH, array_size, set_chr_current);
	}

   if(register_value == 0x0)
   register_value = 0x1;
	psc5415e_set_input_charging_current(register_value);

	return status;
}
int psc5415e_get_charging_status(void)
{
	unsigned char reg_val = 0;
	bool chg_en = false;
	int ret;
	
	reg_val = psc5415e_get_chip_status();
	chg_en = psc5415e_get_ce();

	switch (reg_val) {
	case PSC5415E_CHG_STATUS_READY:
				/* fallthrough */
	case PSC5415E_CHG_STATUS_PROGESS:
		if (!chg_en)
			ret = POWER_SUPPLY_STATUS_CHARGING;
		else
			ret = POWER_SUPPLY_STATUS_NOT_CHARGING;
		break;
	case PSC5415E_CHG_STATUS_DONE:
		ret = POWER_SUPPLY_STATUS_FULL;
		break;
	case PSC5415E_CHG_STATUS_FAULT:
		ret = POWER_SUPPLY_STATUS_DISCHARGING;
		break;
	default:
		ret = -ENODATA;
	}

	return ret;
}
EXPORT_SYMBOL_GPL(psc5415e_get_charging_status);

static int psc5415e_get_charging_done(struct charger_device *chg_dev, bool *is_done)
{
	unsigned int status = 0;
	unsigned int ret_val;

	ret_val = psc5415e_get_chip_status();

	if (ret_val == 0x2)
		*is_done = true;
	else
		*is_done = false;

	return status;
}

static int psc5415e_reset_watch_dog_timer(struct charger_device *chg_dev)
{
	psc5415e_set_tmr_rst(1);
	return 0;
}

static int psc5415e_charger_enable_otg(struct charger_device *chg_dev, bool en)
{
	if(en){
        	psc5415e_set_opa_mode(1);
        	enable_boost_polling(1);
	}else{
        	psc5415e_set_opa_mode(0);
        	enable_boost_polling(0);
	}

	return 0;
}

static void enable_boost_polling(bool poll_en)
{
	if (g_psc5415e_info) {
		if (poll_en) {
			hrtimer_start(&g_psc5415e_info->otg_timer, 
				ktime_set(g_psc5415e_info->polling_interval, 0),
				HRTIMER_MODE_REL);
			g_psc5415e_info->polling_enabled = true;
		} else {
			g_psc5415e_info->polling_enabled = false;
			hrtimer_cancel(&g_psc5415e_info->otg_timer);
		}
	}
}

static void usbotg_boost_kick_work(struct work_struct *work)
{

	struct psc5415e_info *boost_manager =
		container_of(work, struct psc5415e_info, kick_work);

	pr_debug_ratelimited("usbotg_boost_kick_work\n");

	psc5415e_set_tmr_rst(1);

	if (boost_manager->polling_enabled == true)
		hrtimer_start(&boost_manager->otg_timer, 
			ktime_set(boost_manager->polling_interval, 0),
			HRTIMER_MODE_REL);
}

static enum hrtimer_restart usbotg_gtimer_func(struct hrtimer *timer)
{
	struct psc5415e_info *boost_manager =
		container_of(timer, struct psc5415e_info,otg_timer);

	queue_work(boost_manager->otg_boost_workq,
		   &boost_manager->kick_work);

	return HRTIMER_NORESTART;
}

static int psc5415e_enable_vbus(struct regulator_dev *rdev)
{
	psc5415e_set_opa_mode(1);
	enable_boost_polling(1);

	return 0;
}

static int psc5415e_disable_vbus(struct regulator_dev *rdev)
{
	psc5415e_set_opa_mode(0);
	enable_boost_polling(0);

	return 0;
}

static int psc5415e_is_enabled_vbus(struct regulator_dev *rdev)
{
	return psc5415e_get_otg_status();
}

static const struct regulator_init_data psc5415e_vbus_init_data = {
	.constraints = {
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
};

static const struct regulator_ops psc5415e_vbus_ops = {
	.enable = psc5415e_enable_vbus,
	.disable = psc5415e_disable_vbus,
	.is_enabled = psc5415e_is_enabled_vbus,
};

static const struct regulator_desc psc5415e_otg_rdesc = {
	.of_match = "usb-otg-vbus",
	.name = "usb-otg-vbus",
	.ops = &psc5415e_vbus_ops,
	.owner = THIS_MODULE,
	.type = REGULATOR_VOLTAGE,
	.fixed_uV = 5000000,
	.n_voltages = 1,
};

static struct charger_ops psc5415e_chg_ops = {

	/* Normal charging */
	.dump_registers = psc5415e_dump_register,
	.enable = psc5415e_enable_charging,
	.get_charging_current = psc5415e_get_current,
	.set_charging_current = psc5415e_set_current,
	.get_input_current = psc5415e_get_input_current,
	.set_input_current = psc5415e_set_input_current,
	/*.get_constant_voltage = psc5415e_get_battery_voreg,*/
	.set_constant_voltage = psc5415e_set_cv_voltage,
	.kick_wdt = psc5415e_reset_watch_dog_timer,
	.is_charging_done = psc5415e_get_charging_done,
	.enable_otg = psc5415e_charger_enable_otg,
	.event = psc5415e_do_event,
};

static void psc5415e_hw_init(void)
{
#if 1 // defined(HIGH_BATTERY_VOLTAGE_SUPPORT)
	psc5415e_reg_config_interface(0x06, 0x7f);	/* ISAFE = 1250mA, VSAFE = 4.34V */
#else
	psc5415e_reg_config_interface(0x06, 0x70);	//和FAE确认，该值芯片内部写死为最大值，软件无需操作。
#endif
	psc5415e_reg_config_interface(0x00, 0xC0);	/* kick chip watch dog */
	psc5415e_reg_config_interface(0x01, 0xb8);	/* TE=1, CE=0, HZ_MODE=0, OPA_MODE=0 */
	psc5415e_reg_config_interface(0x05, 0x03);	//VSP = 0b011 ,4.442

	psc5415e_reg_config_interface(0x04, 0x36);	/* 0b110 = 147mA */
}

static int psc5415e_i2c_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	int ret = 0;
	int count = 0;
	struct psc5415e_info *info = NULL;
	struct regulator_config config = { };

	pr_err("%s entry\n", __func__);
	info = devm_kzalloc(&client->dev, sizeof(struct psc5415e_info), GFP_KERNEL);

	if (!info)
		return -ENOMEM;

	
	new_client = client;
	info->dev = &client->dev;
	ret = psc5415e_parse_dt(info, &client->dev);

	if (ret < 0)
		return ret;

	while(ret != 2 && count < 5)//bq24157:2    psc5415:7
	{
		ret = psc5415e_get_vender_code();
		count++;
		if (ret != 7) {
			pr_err("%s: psc5415e_get_vender_code failed,cnt = %d\n", __func__,count);
			}
		else
			break;
	}
	
	if(count == 5 || ret == 2)//bq24157:2 
	{
		pr_err("%s: psc5415e is not exist\n", __func__);
		return -ENODEV;		
	}

	/* Register charger device */
	info->chg_dev = charger_device_register(info->chg_dev_name,
		&client->dev, info, &psc5415e_chg_ops, &info->chg_props);

	if (IS_ERR_OR_NULL(info->chg_dev)) {
		pr_err("%s: register charger device failed\n", __func__);
		ret = PTR_ERR(info->chg_dev);
		return ret;
	}

	psc5415e_hw_init();

	psc5415e_dump_register(info->chg_dev);
	
	hrtimer_init(&info->otg_timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
	info->otg_timer.function = usbotg_gtimer_func;

	info->otg_boost_workq = create_singlethread_workqueue("otg_boost_workq");
	INIT_WORK(&info->kick_work, usbotg_boost_kick_work);
	info->polling_interval = 20;

	/* otg regulator */
	config.init_data = &psc5415e_vbus_init_data;
	config.dev = info->dev;
	config.driver_data = info;
	info->otg_rdev = devm_regulator_register(info->dev,
						&psc5415e_otg_rdesc, &config);
	if (IS_ERR(info->otg_rdev)) {
		ret = PTR_ERR(info->otg_rdev);
		pr_info("%s: register otg regulator failed (%d)\n", __func__, ret);
		return ret;
	}

	g_psc5415e_info = info;
	g_primary_chg_type = PSC5415E;

#ifdef CONFIG_T_PRODUCT_INFO
	FULL_PRODUCT_DEVICE_INFO(ID_CHARGECHIP, "PSC5415E");
#endif

	return 0;
}

static int psc5415e_i2c_remove(struct i2c_client *client)
{
	psc5415e_set_reset(1);
	return 0;
}

static void psc5415e_shutdown(struct i2c_client *i2c)
{
	psc5415e_set_reset(1);
	pr_info("%s\n", __func__);
}

#ifdef CONFIG_OF
static const struct of_device_id psc5415e_of_match[] = {
	{.compatible = "prisemi,psc5415e"},
	{},
};
#else
static struct i2c_board_info i2c_psc5415e __initdata = {
	I2C_BOARD_INFO("psc5415e", (psc5415e_SLAVE_ADDR_WRITE >> 1))
};
#endif

static struct i2c_driver psc5415e_driver = {
	.driver = {
		.name = "psc5415e",
		.owner = THIS_MODULE,
#ifdef CONFIG_OF
		.of_match_table = psc5415e_of_match,
#endif
		},
	.probe = psc5415e_i2c_probe,
	.remove = psc5415e_i2c_remove,
	.shutdown = psc5415e_shutdown,
	.id_table = psc5415e_i2c_id,
};

static int __init psc5415e_init(void)
{
	pr_err("%s entry.\n",__func__);
	if (i2c_add_driver(&psc5415e_driver) != 0)
		pr_err("Failed to register psc5415e i2c driver.\n");
	else
		pr_info("Success to register psc5415e i2c driver.\n");

	return 0;
}

static void __exit psc5415e_exit(void)
{
	psc5415e_set_reset(1);
	i2c_del_driver(&psc5415e_driver);
}

module_init(psc5415e_init);
module_exit(psc5415e_exit);
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("I2C psc5415e Driver");
MODULE_AUTHOR("Henry Chen<henryc.chen@mediatek.com>");
